/************************************************************************
* Modulo : manta_in                     Archivo: manta_in.c       *
* Fecha creacion : 17/Jan/92                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Este archivo implanta funciones de interfaz de los modulos            *
*************************************************************************
* Autores: Cesar A. Munoz H.                                            *
************************************************************************/

       /*-------------------------------------------------------+
       | Inclusion de las librerias estandar                    |
       +-------------------------------------------------------*/
 
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <malloc.h>
#include "manta.h"
#include "manta_in.h"

       /*-------------------------------------------------------+
       | Inclusion de Constantes, Tipos, Macros y de prototipos |
       | de las funciones externas del proyecto.                |
       +-------------------------------------------------------*/
 
#include "manta_in.h"

        /*------------------------------------------------------+
        |  Variable global objs, con todos los objetos creados  |
        +------------------------------------------------------*/

objetos            objs      = NULL; 
funciones          func      = NULL;
manta_menu               ambiente;  
manta_menu               interfaces;  
int                tad_error;

        /*------------------------------------------------------+
        |      Definicion de las funciones de este modulo       |
        +------------------------------------------------------*/

/************************************************************************
* manta_opcion  manta_crear_opcion(char nombre[])                                   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE: True                                                             *
* POST: manta_crear_opcion es una nueva opcion con nombre,posx,posy y longitud*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/
 
manta_opcion  manta_crear_opcion(char nombre[])
{/*--------------------------------------------------------------------*/
    manta_opcion op;
    
    op = (manta_opcion)malloc(sizeof(struct smanta_opcion));
    strcpy(op->nombre,nombre);
    op->longitud = strlen(nombre)+2;
    return(op);
/*--------------------------------------------------------------------*/}
 
/************************************************************************
* menu  manta_crear_menu()                                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE: True                                                             *
* POST: crear_menu es un nuevo menu con posxs,posys,tipo,def_op         *
*       ademas : posxid == posxsi, posyid == posysi,num_op = 0          *
*       Si def_opcion = NO_OPCION, entonces act_op = 0. Si no           *
*       act_op = def_op                                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/
 
manta_menu  manta_crear_menu()          
{/*--------------------------------------------------------------------*/
    manta_menu m;
    int  i;

    m = (manta_menu) malloc (sizeof(struct smanta_menu));
    m->num_op = 0;
    m->act_op = 0;
    strcpy(m->titulo,"");
    for(i=0;i!= MAX_OP; i++) 
        m->opciones[i] = NULL;
    return(m);
/*--------------------------------------------------------------------*/}
 
/************************************************************************
* void manta_adic_opcion(menu m, manta_opcion o)                                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE: m = un menu, o = una opcion                                      *
* POST: el menu m tiene la opcion o, todos sus campos se actualizan     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/
 
void manta_adic_opcion(manta_menu m, manta_opcion o)                                   
{/*--------------------------------------------------------------------*/
    m->opciones[(m->num_op)++] = o;
/*--------------------------------------------------------------------*/}
 

/************************************************************************
* void manta_pintar_menu(menu m)                                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE:  m = un menu                                                     *
* POST: Se ha pintado en pantalla el menu m, con la opcion actual       *
*       en modo reverso.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/
 
void manta_pintar_menu(manta_menu m)
{/*--------------------------------------------------------------------*/
    int   op; 
    int   longitud;
    
    longitud=0;
    printf("\n____________________________________________________________________\n");
    if (strcmp(m->titulo,""))
      printf("%s\n", m->titulo);
    for (op=0; op < m->num_op; op++) {
      longitud = longitud + strlen(m->opciones[op]->nombre) + 1;
      if (longitud < 68)
	printf("%s ",m->opciones[op]->nombre);
      else {
        longitud = strlen(m->opciones[op]->nombre) + 1;
	printf("\n%s ",m->opciones[op]->nombre);
      }
    }
    printf("\n--------------------------------------------------------------------\n\n");

/*--------------------------------------------------------------------*/}


/**************************************************************************
* short manta_es_prefijo(char comando[],char cad_opcion[])  			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Verificar si <comando> es prefijo de <cad_opcion>.         *
**************************************************************************/

short manta_es_prefijo(char comando[],char cad_opcion[])  			 
{/*----------------------------------------------------------------------*/
  int i;

    for (i=0; (comando[i] != '\0') && (cad_opcion[i] != 0) &&
              (toupper(comando[i]) == toupper(cad_opcion[i])) ; i++)
    ;
    if (comando[i] == '\0')
	return(1);
      else
	return(0)
    ;
/*----------------------------------------------------------------------*/}

/**************************************************************************
* int manta_opcion_valida(menu m, char comando[])                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Verificar si <comando> es una opcion valida del menu m     *
* DESCRIPCION: Verifica que <comando> sea un prefijo de alguna de las op- *
*   ciones del menu <num_menu>.                                           *
* POST : opcion_valida = NO_OPCION, si m no tiene la opcion comando       *
*        opcion_valida = Numero de la opcion del menu m, que corresponde  *
*                        a comando.                                       *
**************************************************************************/

int manta_opcion_valida(manta_menu m, char comando[])
{/*----------------------------------------------------------------------*/
    int i;
    
    if (!strcmp(comando,""))
        return(NO_OPCION);
    for (i = 0; i < m->num_op && 
                !manta_es_prefijo(comando,m->opciones[i]->nombre);i++);
    if ( i == m->num_op )
        return(NO_OPCION);
    return(i);
/*----------------------------------------------------------------------*/}


/**************************************************************************
* void manta_escriba_cadena(char cadena[], int n)                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO: presentar en pantalla el string que viene en <cadena> y      *
*            salta <n> lineas.                                            *
*                                                                         *
**************************************************************************/

void  manta_escriba_cadena(char cadena[],int n)
{/*----------------------------------------------------------------------*/
    int i;

    printf("%s ", cadena);
    for(i=n;i>0;i--)
      printf("\n");
/*----------------------------------------------------------------------*/}


/************************************************************************
* int manta_admon_menu(menu m)                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE:  m = un menu                                                     *
* POST: manta_admon_menu = opcion escogida por el usuario del menu m          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/
 
int manta_admon_menu(manta_menu m)
{/*--------------------------------------------------------------------*/
    int   op;
    char comando[LONG_OP];

    manta_pintar_menu(m);
    op = m->act_op;
    strcpy(comando,"");
    leer_cadena(MEN8,comando);
    op = manta_opcion_valida(m,comando);
    if (op >= 0)
      m->act_op = op; 
    return(op); 
/*--------------------------------------------------------------------*/}


/************************************************************************
* void imprimir_obj_prim(ptr o,char nombre[])                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : TRUE                                                            *
* POS : Se ha impreso el objeto primitivo o,en la pantalla.             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

void imprimir_obj_prim(ptr o,char nombre[])
{/*--------------------------------------------------------------------*/
    char cadena[LONG_OP];
    int  *obj_int;
    
    strcpy(cadena,"");
    if (!strcmp(nombre,"NAT")) {
        obj_int = (int *)o;
        sprintf(cadena,"%d",*obj_int);
      }
      else if (!strcmp(nombre,"BOOL")) {
        obj_int = (int *)o;
        if (*obj_int == FALSE)
            strcpy(cadena,"FALSE");
        else
            strcpy(cadena,"TRUE");
      }
    manta_escriba_cadena(cadena,0);
/*--------------------------------------------------------------------*/}


/************************************************************************
* void leer_guardar_obj(ptr o, char tit[],char tipo[])                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : tit es el titulo del recuadro donde se lee el objeto del tipo   *
* POS : Se ha guardado el objeto o, del tipo, en la lista objs.         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

void leer_guardar_obj(ptr o, char tit[],char tipo[])                 
{/*--------------------------------------------------------------------*/
    char    titulo[LONG_CAD];
    char    cadena[LONG_OP];

    if ((!strcmp(tipo,"NAT")) || (!strcmp(tipo,"BOOL"))) 
        return;
    sprintf(titulo,"%s -> [%s]? ",tit,tipo);
    leer_cadena(titulo,cadena);
    if (!strcmp(cadena,""))
        return;
    guardar_obj(o,tipo,cadena);
/*--------------------------------------------------------------------*/}

/************************************************************************
* objetos nuevo_obj(ptr o, char tipo[], char nombre[])                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : TRUE                                                            *
* POS : Se ha creado un nuevo objeto con el tipo y con el nombre.       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

objetos nuevo_obj(ptr o, char tipo[], char nombre[])                 
{/*--------------------------------------------------------------------*/
    objetos ob;

    ob = (objetos) malloc (sizeof(struct objetos));
    ob->u.obj = o;
    strcpy(ob->nom_tipo,tipo);
    strcpy(ob->nom_var,nombre);
    ob->sig = NULL;
    return(ob);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void guardar_obj(ptr o, char tipo[], char nombre[])                   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : TRUE                                                            *
* POS : Se ha guardado el objeto o, del tipo, en la lista objs, con el  *
*       nombre.                                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

void guardar_obj(ptr o, char tipo[], char nombre[])                  
{/*--------------------------------------------------------------------*/
    objetos     p;
    objetos     ob;
    manta_tad         t_o;
    tipo_fborr  f;

    if ((!strcmp(tipo,"NAT")) || (!strcmp(tipo,"BOOL")) ||
	(!strcmp(nombre,"")))
        return;
    for (p = objs; p!= NULL && (strcmp(p->nom_tipo,tipo) ||
                                strcmp(p->nom_var,nombre));p=p->sig);
    if (p != NULL) {
        f = (tipo_fborr)buscar_funcion(tipo,4);
        if (f != NULL)
            f(&(p->u.obj));
        p->u.obj = o;
    } else {
        ob = nuevo_obj(o,tipo,nombre);
        ob->sig = objs;
        objs = ob;
        listar_objs();
    }
    t_o = (manta_tad)o;
    inc_uso(t_o);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void borrar_obj(objetos *o)                                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : o esta en la lista de objs                                      *
* POS : Se ha borrado el objeto o de la lista de objs.                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

void borrar_obj(objetos *o)                            
{/*--------------------------------------------------------------------*/
    objetos p;

    if (objs == NULL)
        return;
    if ((*o) == objs) {
        objs = objs->sig;
        free(*o);
	(*o) = NULL;
        return;
    }
    for (p = objs; p->sig != NULL && p->sig != (*o);p=p->sig);
    if (p->sig != NULL) { 
       p->sig = p->sig->sig;
       free(*o);
       (*o) = NULL;
    }
/*--------------------------------------------------------------------*/}

/************************************************************************
* objetos leer_obj_prim(char tit[], char tipo[])                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : tit = es el titulo del recuadro donde se lee, tipo es el tipo   *
*       del objeto leido.                                               *
* POS : leer_obj_prim = Objeto del tipo primitivo que aparece en la     *
*       lista objs con un nombre leido por pantalla.                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

objetos leer_obj_prim(char tit[], char tipo[])
{/*--------------------------------------------------------------------*/
    char    titulo[LONG_CAD];
    char    cadena[LONG_OP];
    static  struct  objetos  obj_prim;
    char    *p;

    strcpy(obj_prim.nom_tipo,tipo);
    sprintf(titulo,"%s [%s]? ",tit,tipo);
    leer_cadena(titulo,cadena);
    if (!strcmp(cadena,"")) {
        return(NULL);
    }
    if (!strcmp(tipo,"BOOL")) {
        if (!strcmp(cadena,"TRUE"))
            obj_prim.u.BOOL = TRUE;
        else if (!strcmp(cadena,"FALSE"))
            obj_prim.u.BOOL = FALSE;
        else {
            sprintf(titulo,MEN10,tipo);
            manta_escriba_cadena(titulo,1);
            return(NULL);
	}
    }
    else if (!strcmp(tipo,"NAT")) {
        obj_prim.u.NAT = strtol(cadena,&p,10);
        if (p == cadena) {
            sprintf(titulo,MEN10,tipo);
            manta_escriba_cadena(titulo,1);
            return(NULL);
	}
    }
    else {
        sprintf(titulo,MEN10,MEN11);
        manta_escriba_cadena(titulo,1);
        return(NULL);
    }
    return(&obj_prim);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void salvar_escribir(char tit[],char tipo[])                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : tit es el titulo de la ventana, tipo es el tipo del objeto que  *
*       se escribira.                                                   *
* POS : Se han salvado las coordenadas (POSX1_VENT,POSY1_VENT,          *
*       POSX2_VENT,POSY2_VENT) de pantalla                              *
*       Ademas, coloca las variables posx_imp y posy_imp en 1.          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

void salvar_escribir(char tit[],char tipo[])                          
{/*--------------------------------------------------------------------*/
    char titulo[LONG_CAD];

    sprintf(titulo,"%s -> %s",tit,tipo);
    manta_escriba_cadena(titulo,1);
/*--------------------------------------------------------------------*/}


/************************************************************************
* void leer_cadena(char mensaje[],char cadena[])                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : True                                                            *
* POS : Se ha colocado el mensaje y leido la cadena.                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

void leer_cadena(char mensaje[],char cadena[])                       
{/*--------------------------------------------------------------------*/
    char     men[LONG_CAD];

    sprintf(men,"%s: ",mensaje);
    manta_escriba_cadena(men,0);
    fgets(cadena,LONG_CAD,stdin);
    if (cadena[strlen(cadena)-1]=='\n')
      { cadena[strlen(cadena)-1]='\0'; }
/*--------------------------------------------------------------------*/}

/************************************************************************
* objetos leer_buscar_obj(char tit[],char tipo[])                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : Tit= titulo del recuadro donde se lee, tipo es el tipo que se   *
*       de objeto que se debe leer.                                     *
* POS : Se ha leido una cadena y se retorna un objeto con el nombre de  *
*       la cadena, y el tipo que entra como parametro. Si no existe,    *
*       retorna NULL.                                                   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

objetos leer_buscar_obj(char tit[],char tipo[])                      
{/*--------------------------------------------------------------------*/
    char    titulo[LONG_CAD];
    char    cadena[LONG_OP];
    objetos p;

    if (!strcmp(tipo,"BOOL") || !strcmp(tipo,"NAT"))
        return( leer_obj_prim(tit,tipo) );    
    sprintf(titulo,"%s [%s]? ",tit,tipo);
    leer_cadena(titulo,cadena);
    if (!strcmp(cadena,"")) {
        return(NULL);
    }
    for(p=objs; (p != NULL) && 
                (strcmp(p->nom_var,cadena) || strcmp(p->nom_tipo,tipo));
        p=p->sig);
    if (p==NULL) {
        sprintf(titulo,MEN10,tipo);
        manta_escriba_cadena(titulo,1);
    }
    return(p);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void    listar_objs(void)                                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : True                                                            *
* POS : Se ha listado la lista objs                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

void    listar_objs(void)
{/*--------------------------------------------------------------------*/
    objetos p;

    manta_escriba_cadena(MEN5,1);
    for(p=objs; p!=NULL; p = p->sig) {
        manta_escriba_cadena(p->nom_var,0);
        manta_escriba_cadena(p->nom_tipo,1);
    }
    manta_escriba_cadena("",1);

/*--------------------------------------------------------------------*/}

/************************************************************************
* void    borrar_lista_objs(void)                                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : True                                                            *
* POS : Se ha borrardo la lista objs                                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
* MODIF : Hernan Aponte                                                 *
************************************************************************/

void    borrar_lista_objs(void)                                      
{/*--------------------------------------------------------------------*/
    objetos p;

    while (objs != NULL) {
        p = objs;
        borrar_obj(&p);
    }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void	adicionar_funcion(char tipo[], ptr f_impr, ptr f_salv,          *
*			  ptr f_carg, ptr f_copi, ptr f_borr)           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Adicionar al tipo :                                       *
*             La funcion de impresion f_impr                            *
*             La funcion de salvar    f_falv                            *
*             La funcion de cargar    f_carg                            *
*             La funcion de copiar    f_copi                            *
*             La funcion de borrar    f_borr                            *
************************************************************************/
 
void	adicionar_funcion(char tipo[], ptr f_impr, ptr f_salv,
			  ptr f_carg, ptr f_copi, ptr f_borr)
{/*--------------------------------------------------------------------*/
    funciones ff;

    ff = (funciones)malloc(sizeof(struct funciones));
    strcpy(ff->tipo,tipo);
    ff->f_impr = f_impr;
    ff->f_salv = f_salv;
    ff->f_carg = f_carg;
    ff->f_copi = f_copi;
    ff->f_borr = f_borr;
    ff->sig = func;
    func = ff;
/*--------------------------------------------------------------------*/}

/************************************************************************
* ptr buscar_funcion(char tipo[], int clase)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Buscar una funcion dentro de la lista funciones, que      *
*             corresponde al tipo.                                      *
* Si clase = 0. Retorna la funcion de impresion de tipo.                *
* Si clase = 1. Retorna la funcion de salvar de tipo                    *
* Si clase = 2. Retorna la funcion de cargar de tipo                    *
* Si clase = 3. Retorna la funcion de copiar de tipo                    *
* Si clase = 4. Retorna la funcion de borrar de tipo                    *
************************************************************************/
 
ptr buscar_funcion(char tipo[], int clase)                            
{/*--------------------------------------------------------------------*/
    funciones f;

    for(f = func; f != NULL && strcmp(f->tipo,tipo); f= f->sig);
    if (f == NULL)
        return(NULL); 
    else
        switch (clase) {
	    case 0: return(f->f_impr);
	    case 1: return(f->f_salv);
	    case 2: return(f->f_carg);
	    case 3: return(f->f_copi);
	    case 4: return(f->f_borr);
            default: return(NULL);
	}
/*--------------------------------------------------------------------*/}
 
/************************************************************************
* void	leer_impr_obj(char tipo[])                                      * 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Leer un objeto del tipo e imprimirlo por pantalla.        *
* PRE: tipo = tipo del objeto.                                          *
************************************************************************/
 
void	leer_impr_obj(char tipo[]) 
{/*--------------------------------------------------------------------*/
    objetos	o;
    tipo_fimpr  f;
    char        titulo[LONG_CAD];

    sprintf(titulo,"%s(",LIST);
    o = leer_buscar_obj(titulo,tipo);
    if ( o == NULL ) return;
    f = (tipo_fimpr)buscar_funcion(tipo,0);
    if (f == NULL) {
        sprintf(titulo,MEN1,tipo);
        manta_escriba_cadena(titulo,1);
    } else {
        sprintf(titulo,"%s(%s)",LIST,o->nom_var);
        salvar_escribir(titulo,tipo);
        f(o->u.obj);
      }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void	leer_salv_obj(char tipo[])                                      * 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Leer un objeto del tipo y lo salva en un archivo del      *
*             mismo nombre del objeto                                   *
* PRE: tipo = tipo del objeto.                                          *
************************************************************************/
 
void	leer_salv_obj(char tipo[])                                      
{/*--------------------------------------------------------------------*/
    objetos	o;
    tipo_fsalv  f;
    char        titulo[LONG_CAD];
    FILE        *fsalv;

    sprintf(titulo,"%s(",SAVE);
    o = leer_buscar_obj(titulo,tipo);
    if ( o == NULL ) return;
    f = (tipo_fsalv)buscar_funcion(tipo,1);
    if (f == NULL) {
        sprintf(titulo,MEN12,tipo);
        manta_escriba_cadena(titulo,1);
    } else {
        sprintf(titulo,"%s/%s.obj",DIR_ENV,o->nom_var);
        fsalv = fopen(titulo,"w");
        fprintf(fsalv,"%s ",o->nom_var);
        fprintf(fsalv,"%s ",o->nom_tipo);
        f(fsalv,o->u.obj);
        manta_escriba_cadena(MEN14,1);
        fclose(fsalv);
      }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void	salvar_objs(void)                                               * 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Salva la lista de objetos del ambiente en un archivo.     *
************************************************************************/
 
void	salvar_objs(void)                                               
{/*--------------------------------------------------------------------*/
    objetos	o;
    tipo_fsalv  f;
    char        nombre[LONG_OP];
    char        nombre_arch[LONG_CAD];
    char        titulo[LONG_CAD];
    FILE        *fsalv;

   
    if ( objs == NULL ) {
        manta_escriba_cadena(MEN19,1);   /* lista de objetos vacia */
        return;
    }
    leer_cadena(MEN13,nombre);    /* Nombre archivo: */
    if (!strcmp(nombre,"")) return;
    sprintf(nombre_arch,"%s/%s.env",DIR_ENV,nombre);
    fsalv = fopen(nombre_arch,"w");
    for (o = objs; o != NULL; o = o->sig) {
        f = (tipo_fsalv)buscar_funcion(o->nom_tipo,1);
        if (f == NULL) {
	    sprintf(titulo,MEN12,o->nom_tipo);
	    manta_escriba_cadena(titulo,1);   /* el tipo no tiene fun. de salvar */
	} else  {
            fprintf(fsalv,"%s ",o->nom_var);
            fprintf(fsalv,"%s ",o->nom_tipo);
	    f(fsalv,o->u.obj);
	  }
    }
    fclose(fsalv);
    manta_escriba_cadena(MEN14,1);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void	leer_carg_obj(char tipo[])                                      * 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Carga un objeto de tipo, del archivo que entre el usuario *
*             Lo deja en el ambiente.                                   *
* PRE: tipo = tipo del objeto.                                          *
************************************************************************/
 
void	leer_carg_obj(char tipo[])                                      
{/*--------------------------------------------------------------------*/
    tipo_fcarg  f;
    char        nombre[LONG_OP];
    char        nombre_arch[LONG_CAD];
    char        tipo_f[LONG_OP];
    char        titulo[LONG_CAD];
    FILE        *fcarg;

    leer_cadena(MEN13,nombre);
    if (!strcmp(nombre,""))
        return;
    sprintf(nombre_arch,"%s/%s.obj",DIR_ENV,nombre);
    fcarg = fopen(nombre_arch,"r");
    if (fcarg == NULL){
        manta_escriba_cadena(MEN15,1);
        return;
    }
    fscanf(fcarg,"%s ",nombre);
    fscanf(fcarg,"%s ",tipo_f);
    if (strcmp(tipo,tipo_f)) {
        manta_escriba_cadena(MEN18,1);
        fclose(fcarg);
        return;        
    }
    f = (tipo_fcarg)buscar_funcion(tipo,2);
    if (f == NULL) {
        sprintf(titulo,MEN16,tipo);
        manta_escriba_cadena(titulo,1);
    } else  
        guardar_obj(f(fcarg),tipo,nombre);
    manta_escriba_cadena(MEN17,1);
    fclose(fcarg);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void	cargar_objs(void)                                               * 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Carga la lista de objetos del ambiente en un archivo.     *
************************************************************************/
 
void	cargar_objs(void)                                              
{/*--------------------------------------------------------------------*/
    tipo_fcarg  f;
    char        nombre[LONG_OP];
    char        nombre_arch[LONG_CAD];
    char        tipo[LONG_OP];
    char        titulo[LONG_CAD];
    FILE        *fcarg;

   
    leer_cadena(MEN13,nombre);  /* Nombre de archivo: */
    if (!strcmp(nombre,""))
        return;
    sprintf(nombre_arch,"%s/%s.env",DIR_ENV,nombre);
    fcarg = fopen(nombre_arch,"r");
    if (fcarg == NULL){
        manta_escriba_cadena(MEN15,1);  /* archivo inexistente */
        return;
    }
    while (!feof(fcarg)) {
        fscanf(fcarg,"%s ",nombre);
        fscanf(fcarg,"%s ",tipo);
        f = (tipo_fcarg)buscar_funcion(tipo,2);
        if (f == NULL) {
	    sprintf(titulo,MEN16,tipo);   /* el tipo no tiene fun. de cargar */
	    manta_escriba_cadena(titulo,1);
            fclose(fcarg);
            return;
	} else  
	  guardar_obj(f(fcarg),tipo,nombre);
    }
    manta_escriba_cadena(MEN17,1);      /* archivo cargado */
    fclose(fcarg);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void	leer_copi_obj(char tipo[])                                      * 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Copia un objeto del tipo en otro.                         *
*             Deja el objeto en el ambiente.                            *
* PRE: tipo = tipo del objeto.                                          *
************************************************************************/
 
void	leer_copi_obj(char tipo[])                                      
{/*--------------------------------------------------------------------*/
    objetos     o;
    tipo_fcopi  f;
    char        nombre[LONG_OP];
    char        titulo[LONG_CAD];

    sprintf(titulo,"%s(",COPY);
    o = leer_buscar_obj(titulo,tipo);
    if ( o == NULL ) return;
    f = (tipo_fcopi)buscar_funcion(tipo,3);
    if (f == NULL) {
        sprintf(titulo,MEN20,tipo);
        manta_escriba_cadena(titulo,1);
    } else {
        sprintf(titulo,"%s(%s) -> %s",COPY,o->nom_var,tipo);
        leer_cadena(titulo,nombre);
        guardar_obj(f(o->u.obj),tipo,nombre);
      }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void	leer_borr_obj(char tipo[])                                      * 
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Borra un objeto del tipo. Lo saca del ambiente.           *
* PRE: tipo = tipo del objeto.                                          *
************************************************************************/
 
void	leer_borr_obj(char tipo[])                                     
{/*--------------------------------------------------------------------*/
    objetos     o;
    tipo_fborr  f;
    char        titulo[LONG_CAD];

    sprintf(titulo,"%s(",DELETE);
    o = leer_buscar_obj(titulo,tipo);
    if ( o == NULL ) return;
    f = (tipo_fborr)buscar_funcion(tipo,4);
    if (f == NULL) {
        sprintf(titulo,MEN21,tipo);   /* el tipo no tiene fun. de borrar */
        manta_escriba_cadena(titulo,1);
    } else {
        f(&(o->u.obj));
        borrar_obj(&o);
        listar_objs();
      }
/*--------------------------------------------------------------------*/}
 
/************************************************************************
* ptr ERROR(char mensaje[])                                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Informa mensaje de error                                              *
* Modifica la variable global tad_error con el valor del error.         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* Fecha creacion : 17/Jan/92                                            *
************************************************************************/

ptr ERROR(char mensaje[])
{/*--------------------------------------------------------------------*/
    char titulo[LONG_CAD];

    tad_error = 1;
    sprintf(titulo,MEN3,mensaje);
    manta_escriba_cadena(titulo,1);
    return(NULL);
/*--------------------------------------------------------------------*/}


/**************************************************************************
* void crear_menu_ambiente()                                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Crear un menu con opciones para salvar, cargar y borrar el *
*              ambiente.                                                  *
**************************************************************************/

void crear_menu_ambiente()
{/*----------------------------------------------------------------------*/
    ambiente = manta_crear_menu();
    manta_adic_opcion(ambiente,manta_crear_opcion(SAVE));
    manta_adic_opcion(ambiente,manta_crear_opcion(LOAD));
    manta_adic_opcion(ambiente,manta_crear_opcion(DELETE));
    manta_adic_opcion(ambiente,manta_crear_opcion(EXIT));
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void	admon_menu_ambiente(void)                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Controlar el menu que permite salvar, cargar,borrar el     *
*              ambiente.                                                  *
**************************************************************************/

void	admon_menu_ambiente(void)                                        
{/*----------------------------------------------------------------------*/
    int  opcion;

    opcion = manta_admon_menu(ambiente);
    while (opcion != 3) {
      switch (opcion) {
        case 0  : 
          salvar_objs();
          break;
        case 1  : 
          cargar_objs();
          break;
        case 2  : 
          borrar_lista_objs();
          listar_objs();
          break;
        case NO_OPCION :
          return;
        default :
          manta_escriba_cadena(MEN22,1);
          break;
      }
      opcion = manta_admon_menu(ambiente);
    }
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void crear_menu_interfaces()                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Crear un menu con opciones para salvar, cargar, borrar  y  *
*              copiar objetos                                             *
**************************************************************************/

void crear_menu_interfaces()      
{/*----------------------------------------------------------------------*/
    interfaces = manta_crear_menu();
    manta_adic_opcion(interfaces,manta_crear_opcion(LIST));
    manta_adic_opcion(interfaces,manta_crear_opcion(SAVE));
    manta_adic_opcion(interfaces,manta_crear_opcion(LOAD));
    manta_adic_opcion(interfaces,manta_crear_opcion(COPY));
    manta_adic_opcion(interfaces,manta_crear_opcion(DELETE));
    manta_adic_opcion(interfaces,manta_crear_opcion(EXIT));
/*----------------------------------------------------------------------*/}

/**************************************************************************
* void	admon_menu_interfaces(char tipo[])                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Controlar el menu que permite salvar, cargar,copiar y      *
*              borrar objetos del tipo.                                   *
**************************************************************************/

void	admon_menu_interfaces(char tipo[]) 
{/*----------------------------------------------------------------------*/
    int  opcion;

    strcpy(interfaces->titulo,tipo);
    opcion = manta_admon_menu(interfaces);
    while (opcion != 5) {
      switch (opcion) {
        case 0  : 
          leer_impr_obj(tipo);     /* imprimir */
          break;
        case 1  : 
          leer_salv_obj(tipo);     /* salvar */
          break;
        case 2  : 
          leer_carg_obj(tipo);     /* cargar */
          break;
        case 3  : 
          leer_copi_obj(tipo);     /* copiar */
          break;
        case 4  : 
          leer_borr_obj(tipo);     /* borrar */
          break;
        case NO_OPCION :
          return;
        default :
          manta_escriba_cadena(MEN22,1);
          break;
      }
      opcion = manta_admon_menu(interfaces);
    }
    strcpy(interfaces->titulo,"");
/*----------------------------------------------------------------------*/}


