/**
 * PROYECTO MANTA
 * DERECHOS RESERVADOS A LA UNIVERSIDAD DE LOS ANDES  (c) 1985-1999
 *
 * API PARA EL KERNEL DE MANTA
 * Descripcion: Contiene funciones para accesar el kernel de ManTa
 * @author   Vladimir Tamara Patino
 * @version 2.0
 * Santafe de Bogota D.C. (COLOMBIA)  1997-1999
 **/

#include "apimanta.h"
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "os_dep.h"


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   Del API para el API
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Inicializa API de ManTa
 * Inicializa ManTa empleando archivo de configuracion nom.
 *       Los codigos de retorno son:
 *       0 Exito
 *       -1 No pueden retornarse mensajes, memoria insuficiente
 *       -2 Archivo de configuracion incorrecto
 *       -3 Problemas al cargar tabla de simbolos
 *       -4 Problemas al inicializar motor de reescritura
 * @param nom Nombre del archivo de configuracion
 **/
int iniMantaAPI(char *nom)
{
  FILE *fp;
  char nombre_usuario[100];
  char merr[1000];

  assert(nom!=NULL);

  /* Inicializa manejo de Mensajes */
  /*  mensaje_retornado=(char *)malloc(sizeof(char)*MAX_MENS_RET);
  if (mensaje_retornado==NULL)
    { return -1; } No puede retornarse mensajes (memoria insuficiente)

    mensaje_retornado[0]='\0'; */
  fp=fopen(nom,"r");
  if (fp==NULL)
    { return -2;  }
  fclose(fp);
  iniciar_editor_direct(nom); /* Emplea archivo de configuracion*/
  iniciar_mensajes(); /* Inicializa mensajes */
  iniciar_claves();
  nombre_autor(nombre_usuario);
  cargue_tads_primitivos();
  symtable=NULL;
  if (loadSymbols(&symtable,merr)!=0)
    {
      printf("%s",merr);
      return -3;
    }
  if (init_rewriting(merr)!=0)
    {
      printf("%s",merr);
      return -4;
    }
  proved=NULL;
  unproved=NULL;
  
  return 0;
}


/**
 * Finaliza API de ManTa.
 * Debe ser invocado despues de haber inicializado el API con iniMantaAPI
 **/
int finMantaAPI()
{
  /*  if (mensaje_retornado!=NULL)
    {
      free(mensaje_retornado);
      mensaje_retornado=NULL;
      }*/
  end_rewriting();
  if (symtable!=NULL)
    {
      freeSymbols(&symtable);
    }
  return 0;
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Nombre
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Crea un nodo nodo_nombre inicializado con 0
 * @param n En este parametro retorna el nuevo nodo (la funcion lozaliza mem)
 * @param merr Buffer para mensajes de error
 * @return 0 sii no hay error
 **/
int nuevoNodoNombre(nodo_nombre **n,char *merr)
{
  assert(n!=NULL);
  *n=(nodo_nombre *)malloc(sizeof(nodo_nombre));
  if (*n==NULL)
    {
      strcat(merr,mensajes[166]);
      strcat(merr,"\n");
      return -1;
    }
  (*n)->nombre[0]='\0';
  (*n)->sig=NULL;
  return 0;
}


/**
 * Retorna el nombre de un nodo_nombre
 * @param n nodo del cual retornar nombre
 * @param nom espacio ya localizado donde se retornar el nombre
 **/
void nombreNodoNombre(nodo_nombre *n, char *nom)
{
  assert(nom!=NULL);
  if (n==NULL)
    {
      nom[0]='\0';
      return;
    }
  sprintf(nom,"%s",n->nombre);
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LNombre (Lista de Nombres)
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Inicializa la lista que recibe como una lista vacia.
 * @param ln Cabeza de la lista de nombres por inicializar
 **/
int inicLNombre(nodo_nombre **ln)
{
  assert(ln!=NULL);
  *ln=NULL;
  return 0;
}


/**
 * Saca una copia de una lista de nombres
 * @param n Cabeza de la lista por copiar 
 * @param nf Sera la cabeza de la nueva lista (esta funcion localiza memoria)
 * @param merr Buffer para mensajes de error
 **/
int copiaLNombre(nodo_nombre **n,nodo_nombre *nf,char *merr)
{
  nodo_nombre *naux,**ant;
  assert(n!=NULL);
  assert(merr!=NULL);

  for (*n=NULL,ant=n;nf!=NULL;nf=nf->sig)
    {
      naux=NULL;
      if (nuevoNodoNombre(&naux,merr)!=0)
	{
	  elimLNombre(n);
	  *n=NULL;
	  return -1;
	}
      *ant=naux;
      strcpy(naux->nombre,nf->nombre);
      naux->sig=NULL;
      ant=&(naux->sig);
    }
  return 0;
}


const int ADIC_ADD=0,  /* Agrega siempre */
          ADIC_REP=1;  /* Si ya existe reemplaza */

/**
 * Agrega en orden alfabetico el nombre nuevo, Reemplaza
 * o agrega segun el valor remp (ADIC_REP o ADIC_ADD)
 * @param ln Cabeza de la lista de nombres
 * @param nuevo Nuevo nodo por agregar
 * @param remp Forma de agregar el nodo
 *             ADIC_ADD  si existe otro del mismo nombre agrega el nuevo antes
 *             ADIC_REP si existe otro con el mismo nombre lo reemplaza
 *                     (libera la memoria del otro)
 **/
void adicElemLNombre(nodo_nombre **ln,nodo_nombre *nuevo, int remp)
{
  nodo_nombre *ac,**ant;
  char nomt[255],nom[255];
  int rc;
  
  assert(ln!=NULL && nuevo!=NULL);
  
  nombreNodoNombre(nuevo,nom);
  ac=*ln;
  ant=ln;
  nombreNodoNombre(ac,nomt);
  for (;ac!=NULL && (rc=strcmp_nocase(nom,nomt))>0;
       ant=&(ac->sig),ac = ac->sig,nombreNodoNombre(ac,nomt))
    ;
  if (remp==ADIC_REP && rc==0 && ac!=NULL)
    {
      nuevo->sig=ac->sig;
      *ant=nuevo;
      ac->sig=NULL;
      free(ac);
    }
  else
    {
      nuevo->sig=ac;
      *ant=nuevo;
    }
}


/**
 * Retorna la cantidad de nombres en la lista
 * @param ln Cabeza de la lista de nombres
 **/
int cantElemLNombre(nodo_nombre *ln)
{
  nodo_nombre *p_aux;
  int i;

  for (i=0,p_aux =ln;(p_aux != NULL);
       p_aux = p_aux->sig,i++);
  return(i);
}


/**
 * Retorna un apuntador al noodo_nombre cuyo nombre es nom
 * Si no existe retorna NULL
 * @param n cabeza de la lista de nombres
 * @param nom Nombre buscado
 **/
nodo_nombre *buscaNombreLNombre(nodo_nombre *n,char *nom)
{
  nodo_nombre *p_aux;
  char nt[255];
  
  assert(nom!=NULL);
  for (p_aux = n,nombreNodoNombre(p_aux,nt);p_aux!=NULL &&
         strcmp(nt,nom)!=0;
       p_aux = p_aux->sig,nombreNodoNombre(p_aux,nt));
  
  return p_aux;
}


/**
 * Retorna el nombre del i-esimo nombre de la lista n
 * @param n Cabeza de la lista
 * @param i Indice del cual sacar nombre.  Debe ser menor que la cantidad
 *          de elementos de la lista
 * @param nom En este parametro deja el nombre del i-esimo nodo
 **/
void nombreElemLNombre(nodo_nombre *n,int i, char* nom)
{
  nodo_nombre *p_aux;
  int k;
  assert(0<=i && i<cantElemLNombre(n) && nom!=NULL);
  for (k=0,p_aux = n;k<i;
       p_aux = p_aux->sig,k++);
  nombreNodoNombre(p_aux,nom);
}


/**
 * Reemplaza toda ocurrencia del nombre old por new
 * @param n Cabeza de la lista
 * @param old Nombre antiguo
 * @param new Nombre nuevo
 **/
void reemplazaLNombre(nodo_nombre *n,char *old,char *new)
{
  assert(old!=NULL);
  assert(new!=NULL);
  
  while (n!=NULL)
    {
      if (strcmp(n->nombre,old)==0)
	{ strcpy(n->nombre,new); }
      n=n->sig;
    }
}


/**
 * Libera una lista de nombres
 * @param n cabeza de la lista 
 **/
void elimLNombre(nodo_nombre **n)
{
  nodo_nombre *naux;
  assert(n!=NULL);
  if (*n==NULL)
    return;
  for (;(*n)!=NULL;*n=naux)
    {
      naux=(*n)->sig;
      (*n)->nombre[0]='\0';
      free(*n);
    }
}



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Dominio
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Crea un nodo nuevo para un dominio
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoDominio(nodo_dom **n,char *merr)
{
  assert(n!=NULL);
  *n=(nodo_dom *)malloc(sizeof(nodo_dom));
  if (*n==NULL)
    {
      strcat(merr,mensajes[166]);
      strcat(merr,"\n");
      return -1;
    }
  (*n)->nombre[0]='\0';
  (*n)->induccion=0;
  (*n)->sig=NULL;
  return 0;
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LDominio (Lista de Dominios)
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Reemplaza toda ocurrencia del nombre old por new
 * @param n Cabeza de la lista
 * @param old Nombre antiguo
 * @param new Nombre nuevo
 **/
void reemplazaLDominio(nodo_dom *n,char *old,char *new)
{
  assert(old!=NULL);
  assert(new!=NULL);
  
  while (n!=NULL)
    {
      if (strcmp(n->nombre,old)==0)
	{ strcpy(n->nombre,new); }
      n=n->sig;
    }
}

/**
 * Crea una lista de dominios nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLDominio(nodo_dom **n,nodo_dom *nf,char *merr)
{
  nodo_dom *naux,**ant;
  assert(n!=NULL);
  assert(merr!=NULL);

  for (*n=NULL,ant=n;nf!=NULL;nf=nf->sig)
    {
      naux=NULL;
      if (nuevoNodoDominio(&naux,merr)!=0)
	{
	  elimLDominio(n);
	  *n=NULL;
	  return -1;
	}
      *ant=naux;
      strcpy(naux->nombre,nf->nombre);
      naux->induccion=nf->induccion;
      naux->sig=NULL;
      ant=&(naux->sig);
    }
  return 0;
 }


/**
 * Libera una lista de dominios
 * @param n cabeza de la lista 
 **/
void elimLDominio(nodo_dom **n)
{
  nodo_dom *naux;
  assert(n!=NULL);
  while (*n!=NULL)
    {
      naux=(*n)->sig;
      (*n)->nombre[0]='\0';
      free(*n);
      *n=naux;
    }
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Generadora
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para una generadora
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoGeneradora(nodo_generadora **n,char *merr)
{
  assert(n!=NULL);
  *n=(nodo_generadora *)malloc(sizeof(nodo_generadora));
  if (*n==NULL)
    {
      strcat(merr,mensajes[166]);
      strcat(merr,"\n");
      return -1;
    }
  (*n)->nombre[0]='\0';
  (*n)->nombre_fte[0]='\0';
  (*n)->dominio=NULL;
  (*n)->sig=NULL;
  return 0;
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LGeneradora
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea una lista de generadoras nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLGeneradora(nodo_generadora **n,nodo_generadora *nf,char *merr)
{
  nodo_generadora *naux,**ant;
  assert(n!=NULL);
  assert(merr!=NULL);

  for (*n=NULL,ant=n;nf!=NULL;nf=nf->sig)
    {
      naux=NULL;
      if (nuevoNodoGeneradora(&naux,merr)!=0)
	{
	  elimLGeneradora(n);
	  *n=NULL;
	  return -1;
	}
      *ant=naux;
      strcpy(naux->nombre,nf->nombre);
      strcpy(naux->nombre_fte,nf->nombre_fte);
      if (copiaLNombre(&(naux->dominio),nf->dominio,merr)!=0)
	{
	  elimLGeneradora(n);
	  *n=NULL;
	  return -1;
	}
      naux->sig=NULL;
      ant=&(naux->sig);
    }
  return 0;
}


/**
 * Libera una lista de generadoras
 * @param n cabeza de la lista 
 **/
void elimLGeneradora(nodo_generadora **n)
{
  nodo_generadora *naux;
  assert(n!=NULL);
  while (*n!=NULL)
    {
      naux=(*n)->sig;
      (*n)->nombre[0]='\0';
      (*n)->nombre_fte[0]='\0';
      elimLNombre(&(*n)->dominio);
      free(*n);
      *n=naux;
    }
 }

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Expresion
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para una expresin
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoExpresion(nodo_expresion **n,char *merr)
{
  assert(n!=NULL);
  *n=(nodo_expresion *)malloc(sizeof(nodo_expresion));
  if (*n==NULL)
    {
      strcat(merr,mensajes[166]);
      strcat(merr,"\n");
      return -1;
    }
  (*n)->nombre[0]='\0';
  (*n)->tipo[0]='\0';
  (*n)->clase[0]='\0';
  (*n)->argumentos=NULL;
  return 0;
}


/**
 * Crea una expresin nueva copiando otra existente
 * @param n expresin existente
 * @param nf Sera la nueva expresin (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaNodoExpresion(nodo_expresion **n,nodo_expresion *nf,char *merr)
{
  assert(n!=NULL);
  assert(merr!=NULL);

  *n=NULL;
  if (nf==NULL)
    { return 0; }
  if (nuevoNodoExpresion(n,merr)!=0)
    {
      *n=NULL;
      return -1;
    }
  strcpy((*n)->nombre,nf->nombre);
  strcpy((*n)->tipo,nf->tipo);
  strcpy((*n)->clase,nf->clase);
  if (copiaLArgumento(&((*n)->argumentos),nf->argumentos,merr)!=0)
    {
      elimNodoExpresion(n);
      *n=NULL;
      return -1;
    }
  return 0;
 }


/**
 * Libera una expresion
 * @param n cabeza de la lista 
 **/
void elimNodoExpresion(nodo_expresion **n)
{
  assert(n!=NULL);
  if (*n!=NULL)
    {
      (*n)->nombre[0]='\0';
      (*n)->tipo[0]='\0';
      (*n)->clase[0]='\0';
      elimLArgumento(&(*n)->argumentos);
      free(*n);
      *n=NULL;
    }
}

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Argumento
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para un argumento
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoArgumento(nodo_argum **n,char *merr)
{
  assert(n!=NULL);
  *n=(nodo_argum *)malloc(sizeof(nodo_argum));
  if (*n==NULL)
    {
      strcat(merr,mensajes[166]);
      strcat(merr,"\n");
      return -1;
    }
  (*n)->expr=NULL;
  (*n)->sig=NULL;
  return 0;
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LArgumento
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Crea una lista de argumentos nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLArgumento(nodo_argum **n,nodo_argum *nf,char *merr)
{
  nodo_argum *naux,**ant;
  assert(n!=NULL);
  assert(merr!=NULL);

  for (*n=NULL,ant=n;nf!=NULL;nf=nf->sig)
    {
      naux=NULL;
      if (nuevoNodoArgumento(&naux,merr)!=0)
	{
	  elimLArgumento(n);
	  *n=NULL;
	  return -1;
	}
      *ant=naux;
      if (copiaNodoExpresion(&(naux->expr),nf->expr,merr)!=0)
	{
	  elimLArgumento(n);
	  *n=NULL;
	  return -1;
	}
      naux->sig=NULL;
      ant=&(naux->sig);
    }
  return 0;
 }


/**
 * Libera una lista de argumentos
 * @param n cabeza de la lista 
 **/
void elimLArgumento(nodo_argum **n)
{
  nodo_argum *naux;
  assert(n!=NULL);
  while (*n!=NULL)
    {
      naux=(*n)->sig;
      (*n)->sig=NULL;
      elimNodoExpresion(&(*n)->expr);
      free(*n);
      *n=naux;
    }
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Igualdad
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para una igualdad
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoIgualdad(nodo_igualdad **n,char *merr)
{
  assert(n!=NULL);
  *n=(nodo_igualdad *)malloc(sizeof(nodo_igualdad));
  if (*n==NULL)
    {
      strcat(merr,mensajes[166]);
      strcat(merr,"\n");
      return -1;
    }
  (*n)->li_str[0]='\0';
  (*n)->ld_str[0]='\0';
  (*n)->lado_izq=NULL;
  (*n)->lado_der=NULL;
  (*n)->sig=NULL;
  return 0;
}

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LIgualdad
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea una lista de igualdades nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLIgualdad(nodo_igualdad **n,nodo_igualdad *nf,char *merr)
{
  nodo_igualdad *naux,**ant;
  assert(n!=NULL);
  assert(merr!=NULL);

  for (*n=NULL,ant=n;nf!=NULL;nf=nf->sig)
    {
      naux=NULL;
      if (nuevoNodoIgualdad(&naux,merr)!=0)
	{
	  elimLIgualdad(n);
	  *n=NULL;
	  return -1;
	}
      *ant=naux;
      strcpy(naux->li_str,nf->li_str);
      strcpy(naux->ld_str,nf->ld_str);
      if (copiaNodoExpresion(&(naux->lado_izq),nf->lado_izq,merr)!=0)
	{
	  elimLIgualdad(n);
	  *n=NULL;
	  return -1;
	}
      if (copiaNodoExpresion(&(naux->lado_der),nf->lado_der,merr)!=0)
	{
	  elimLIgualdad(n);
	  *n=NULL;
	  return -1;
	}
      naux->sig=NULL;
      ant=&(naux->sig);
    }
  return 0;
 }
  

/**
 * Libera una lista de igualdades
 * @param n cabeza de la lista 
 **/
void elimLIgualdad(nodo_igualdad **n)
{
  nodo_igualdad *naux;
  assert(n!=NULL);
  while (*n!=NULL)
    {
      naux=(*n)->sig;
      (*n)->li_str[0]='\0';
      (*n)->ld_str[0]='\0';
      elimNodoExpresion(&(*n)->lado_izq);
      elimNodoExpresion(&(*n)->lado_der);
      free(*n);
      *n=naux;
    }
}



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Selectora
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nodo nuevo para una selectora
 * @param n Sera el nuevo nodo (memoria localizada por esta funcion)
 * @param merr Buffer para mensajes de error
 **/
int nuevoNodoSelectora(nodo_select **n,char *merr)
{
  assert(n!=NULL);
  *n=(nodo_select *)malloc(sizeof(nodo_select));
  if (*n==NULL)
    {
      strcat(merr,mensajes[166]);
      strcat(merr,"\n");
      return -1;
    }
  (*n)->nombre[0]='\0';
  (*n)->nombre_fte[0]='\0';
  (*n)->dominio=NULL;
  (*n)->codominio[0]='\0';
  (*n)->caracterizadora=0;
  (*n)->axiomas=NULL;
  (*n)->sig=NULL;
  return 0;
}

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LSelectora
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea una lista de selectoras nueva copiando otra existente
 * @param n cabeza de la lista existente
 * @param nf Sera la cabeza de la nueva lista (localizada por esta funcion)
 * @para merr Buffer para manejar mensajes de error
 **/
int copiaLSelectora(nodo_select **n,nodo_select *nf,char *merr)
{
  nodo_select *naux,**ant;
  assert(n!=NULL);
  assert(merr!=NULL);

  for (*n=NULL,ant=n;nf!=NULL;nf=nf->sig)
    {
      naux=NULL;
      if (nuevoNodoSelectora(&naux,merr)!=0)
	{
	  elimLSelectora(n);
	  *n=NULL;
	  return -1;
	}
      *ant=naux;
      strcpy(naux->nombre,nf->nombre);
      strcpy(naux->nombre_fte,nf->nombre_fte);
      strcpy(naux->codominio,nf->codominio);
      naux->caracterizadora=nf->caracterizadora;
      if (copiaLDominio(&(naux->dominio),nf->dominio,merr)!=0)
	{
	  elimLSelectora(n);
	  *n=NULL;
	  return -1;
	}
      if (copiaLIgualdad(&(naux->axiomas),nf->axiomas,merr)!=0)
	{
	  elimLSelectora(n);
	  *n=NULL;
	  return -1;
	}
      naux->sig=NULL;
      ant=&(naux->sig);
    }
  return 0;
 }


/**
 * Busca una selectora por nombre en una lista
 * @param sel Cabeza de la lista
 * @paam nombre Nombre de la selectora buscada
 * @return Un apuntado al nodo con la selectora o NULL si no esta
 **/
nodo_select *buscaNombreLSelectora(nodo_select *sel,char *nombre)
{
  assert(nombre!=NULL);
  while (sel!=NULL && strcmp(sel->nombre,nombre)!=0)
    {
      sel=sel->sig;
    }
  return sel;
}


/**
 * Busca una selectora por el nombre de la fuente en una lista
 * @param sel Cabeza de la lista
 * @paam nombre Nombre de la fuente
 * @return Un apuntado al nodo con la selectora o NULL si no esta
 **/
nodo_select *buscaFuenteLSelectora(nodo_select *sel,char *nombre_fte)
{
  assert(nombre_fte!=NULL);
  while (sel!=NULL && strcmp(sel->nombre_fte,nombre_fte)!=0)
    {
      sel=sel->sig;
    }
  return sel;
}


/**
 * Libera una lista de selectoras
 * @param n cabeza de la lista 
 **/
void elimLSelectora(nodo_select **n)
{
  nodo_select *naux;
  assert(n!=NULL);
  while (*n!=NULL)
    {
      naux=(*n)->sig;
      (*n)->nombre[0]='\0';
      (*n)->nombre_fte[0]='\0';
      elimLDominio(&(*n)->dominio);
      (*n)->codominio[0]='\0';
      elimLIgualdad(&(*n)->axiomas);
      free(*n);
      *n=naux;
    }
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Tipo
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Crea un nuevo nodo de un tipo
 * @param p Apuntara al nuevo tipo (localizado por esta funcion)
 * @param merr Buffer de errores
 * @return 0 si y solo si no hay error
 **/
int nuevoNodoTipo(nodo_tad **p,char *merr)
{
    assert(p!=NULL);

    (*p)=(nodo_tad *)malloc(sizeof(nodo_tad));
    if (*p==NULL) {
      strcat(merr,mensajes[166]);
      strcat(merr,"\n");
      return -1;
    }
    memset(*p,0,sizeof(nodo_tad));
    return 0;
}


/**
 * Retorna el nombre del TAD t
 * Si t es NULL retorna un nombre vacio
 * @param t Tad del cual se obtendra el nombre
 * @param largo Indica si se debe retornar el nombre largo
 * @param nom En este buffer queda el nombre (debe venir localizado)
 **/
void nombreNodoTipo(nodo_tad *t, int largo, char *nom)
{
  assert(nom!=NULL);
  if (t==NULL)
    {
      nom[0]='\0';
      return;
    }
  if (!largo)
    { sprintf(nom,"%s",t->nombre);  }
  else
    {
      switch (t->tipo) {
      case PROD_CARTESIANO:
	sprintf(nom," %s = %s x %s ",
		t->nombre,
		t->info_tipo->nombre,
		t->info_tipo->sig->nombre);
	break;
      case RESTRICCION:
	sprintf(nom," %s = ( %s : %s ) ",
		t->nombre,
		t->info_tipo->nombre,
		t->info_tipo->sig->nombre);
	break;
      case COMPLETACION:
	sprintf(nom," %s = %s + { %s } ",
		t->nombre,
		t->info_tipo->nombre,
		t->info_tipo->sig->nombre);
	break;
      case NT_EXTENSION:
	sprintf(nom," %s = %s + %s ",
		t->nombre,
		t->info_tipo->nombre,
		t->info_tipo->sig->nombre);
	break;
      case VARIACION:
	sprintf(nom," %s = %s U %s ",
		t->nombre,
		t->info_tipo->nombre,
		t->info_tipo->sig->nombre);
	break;
      case REPRESENTACION:
	sprintf(nom," %s : %s -> %s ",t->nombre,
		t->info_tipo->nombre,
		t->info_tipo->sig->nombre);
	break;
      case NORMAL:
      default:
	sprintf(nom," %s ",t->nombre);
	break;
	
      }
    }
}



/**
 * Retorna la cantidad de parametros del tad que recibe
 * @param t tad por examinar
 * @return La cantidad de parametros del tipo
 **/
int numParamT(nodo_tad *t)
{
  assert(t!=NULL);
  return cantElemLNombre(t->parametros);
}


/**
 * Retorna un parametro del tad recibido
 * @param t Tad por examinar
 * @param i Numero de Parametro por devolver
 * @param nom En este buffer deja el nombre (debe venir localizado)
 **/
void nombreParamT(nodo_tad *t,int i,char *nom)
{
  assert(t!=NULL);
  assert(nom!=NULL);
  assert(i>=0 && i<numParamT(t));
  nombreElemLNombre(t->parametros,i,nom);
}



/**
 * Indica si el tipo es inservible
 * @param t El tipo por revisar
 * @return Verdadero si el tipo es inservible
 **/
int tipoMalo(nodo_tad *t,char *merr)
{
  assert(t!=NULL);
  assert(merr!=NULL);
  
  if (t->nombre==NULL || (t->nombre)[0]=='\0' ||
      t->selectoras==NULL)
    {
      strcat(merr,mensajes[193]);
      strcat(merr,"\n");
      return 1;
    }
  return 0;
}


/**
 * Indica si un tipo es basico
 * @param t1 Tipo por revisar
 * @rturn verdadero si y solo si es un tipo bsico
 **/
int esTipoBasico(nodo_tad *t1)
{
  return (t1->parametros==NULL && 
	  (t1->iniciales!=NULL || t1->constructoras!=NULL));
}


/**
 * Indica si un tipo es extensin (sin generadoras ni parametros)
 * @param t1 Tipo por revisar
 * @param merr Buffer para mensajes de error
 * @return 1 si es extension
 *         0 y un mensaje de error en el buffer si no es
 **/
int esTipoExtension(nodo_tad *t1,char *merr)
{
  texpr mens_error;
  if  (t1->parametros==NULL && 
       t1->iniciales==NULL && 
       t1->constructoras==NULL &&
       t1->selectoras!=NULL)
    { return 1; }
  sprintf(merr,mensajes[199],t1->nombre);
  strcat(merr,mens_error);
  strcat(merr,"\n");
  return 0;
}


/**
 * Indica si un tipo es una representacion de otro
 * @param r Tipo por verificar
 * @param merr Buffer con mensajes de error
 * @return 1 si es representacion
 *         0 si no lo es y un mensaje en el buffer merr
 **/
int esRepresentacion(nodo_tad *r,char *merr)
{
  nodo_tad *t1,*t2;
  texpr mens_error;
  
  if (tipoMalo(r,merr))
    { return 0; }
  if (!esTipoExtension(r,merr))
    { return 0; }
  /* Verificamos que lo que recibe sea una representacion */
  if (r->tipo!=REPRESENTACION ||
      r->info_tipo==NULL ||
      r->info_tipo->sig==NULL)
    {
      sprintf(mens_error,mensajes[200],r->nombre);
      goto badexit;
    }
  /* Verificamos que los tads de la representacion esten */
  t1=buscaNombreLTipo(tad,r->info_tipo->nombre);
  if (t1==NULL)
    {
      sprintf(mens_error,mensajes[194],r->info_tipo->nombre);
      goto badexit;
    }      
  t2=buscaNombreLTipo(tad,r->info_tipo->sig->nombre);
  if (t2==NULL)
    {
      sprintf(mens_error,mensajes[194],r->info_tipo->sig->nombre);
      goto badexit;
    }      
  return 1;
 badexit:
  strcat(merr,mens_error);
  strcat(merr,"\n");
  return 0;
}


/**
 * Elimina de memoria el TAD que recibe
 * OJO: Si es parte de una lista deja el resto de la lista, y solo
 *      elimina el nodo solicitado
 * @param nt TAD por eliminar
 **/
void elimNodoTipo(nodo_tad **nt)
{

  assert(nt!=NULL);
  if (*nt==NULL)
    return;
  (*nt)->nombre[0]='\0';
  (*nt)->status=-1;
  (*nt)->tipo=-1;
  (*nt)->modif=-1;
  elimLNombre(&(*nt)->info_tipo);
  elimLNombre(&(*nt)->parametros);
  elimLGeneradora(&(*nt)->iniciales);
  elimLGeneradora(&(*nt)->constructoras);
  elimLSelectora(&(*nt)->selectoras);
  (*nt)->sig=NULL;
  free(*nt);
  
}

/**
 * Crea una copia de un tipo
 * @param dest En este quedara la copia (memoria localizada por esta func)
 * @param fte  Tad fuente
 * @param merr Buffer para mensajes de error
 * @param Retorna 0 si y solo si no hay errores
 **/
int copiaNodoTipo(nodo_tad **dest,nodo_tad *fte,char *merr)
{
  assert(fte!=NULL);
  assert(dest!=NULL);
  assert(merr!=NULL);
  
  if (nuevoNodoTipo(dest,merr)!=0)
    {
      return -1;
    }
  strcpy((*dest)->nombre,fte->nombre);
  (*dest)->status=fte->status;
  (*dest)->tipo=fte->tipo;
  (*dest)->modif=fte->modif;
  if (copiaLNombre(&(*dest)->info_tipo,fte->info_tipo,merr)!=0)
    {
      elimNodoTipo(dest);
      return -1;
    }
  if (copiaLNombre(&(*dest)->parametros,fte->parametros,merr)!=0)
    {
      elimNodoTipo(dest);
      return -1;
    }
  if (copiaLGeneradora(&(*dest)->iniciales,fte->iniciales,merr)!=0)
    {
      elimNodoTipo(dest);
      return -1;
    }
  if (copiaLGeneradora(&(*dest)->constructoras,fte->constructoras,merr)!=0)
    {
      elimNodoTipo(dest);
      return -1;
    }
  if (copiaLSelectora(&(*dest)->selectoras,fte->selectoras,merr)!=0)
    {
      elimNodoTipo(dest);
      return -1;
    }
  (*dest)->sig=fte->sig;
  
  return 0; 
}


/**
 * Carga un tipo del archivo bytecode abierto que recibe 
 * @param fp Archivo abierto para lectura
 * @param nt En este sitio dejara el tipo cargado (localizado por esta func.)
 * @param merr Buffer para mensajes de error
 * @return 0 si y solo si no hay errores
 **/
int cargaTipo(FILE *fp,nodo_tad **nt,char *merr)
{
  texpr    linea;
  nodo_tad *p;
  
  assert(nt!=NULL && fp!=NULL);
  
  if (nuevoNodoTipo(nt,merr)==-1) {
    return -1;
  }
  p=*nt;
  p->modif = 0;
  if (fscanf(fp,"%s",p->nombre)!=1)
    { return -1;}
  if (fscanf(fp,"%s",linea)!=1)              /* STATUS        */
    {  return -1;}
  if (fscanf(fp,"%s",linea)!=1)
    { return -1;}
  p->status = (short) atoi(linea);
  if (fscanf(fp,"%s",linea)!=1)              /* TIPO          */
    { return -1;}
  if (fscanf(fp,"%s",linea)!=1)
    {  return -1;}
  p->tipo = (short) atoi(linea);
  if (fscanf(fp,"%s",linea)!=1)              /* INFO_TIPO     */
    { return -1;}
  lea_arch_nombres(fp,&(p->info_tipo));
  if (fscanf(fp,"%s",linea)!=1)              /* PARAMETROS    */
    { return -1;}
  lea_arch_nombres(fp,&(p->parametros));
  if (fscanf(fp,"%s",linea)!=1)             /* INICIALES     */
    { return -1;}
  lea_arch_generadoras(fp,&(p->iniciales));
  if (fscanf(fp,"%s",linea)!=1)             /* CONSTRUCTORAS */
    { return -1;}
  lea_arch_generadoras(fp,&(p->constructoras));
  if (fscanf(fp,"%s",linea)!=1)             /* SELECTORAS    */
    { return -1;}
  lea_arch_selectoras(fp,&(p->selectoras));
  
  return 0;
}


/**
 * Salva un tipo en un archivo en formato bytecode
 * @param fp Archivo abierto para escritura
 * @param nt TAD por escribir
 * @return 0 si y solo si no hay errores
 **/
int salvaTipo(FILE *fp,nodo_tad *nt)
{
  assert(fp!=NULL);
  assert(nt!=NULL);
  
  fprintf (fp, "%s\n", nt->nombre);
  fprintf (fp, "STATUS\n");
  fprintf (fp, "%d\n", nt->status);
  fprintf (fp, "TIPO\n");
  fprintf (fp, "%d\n", nt->tipo);
  fprintf (fp, "INFO_TIPO\n");
  escriba_arch_nombres (fp, nt->info_tipo, "");
  fprintf (fp, "PARAMETROS\n");
  escriba_arch_nombres (fp, nt->parametros, "");
  fprintf (fp, "INICIALES\n");
  escriba_arch_generadoras (fp, nt->iniciales);
  fprintf (fp, "CONSTRUCTORAS\n");
  escriba_arch_generadoras (fp, nt->constructoras);
  fprintf (fp, "SELECTORAS\n");
  escriba_arch_selectoras (fp, nt->selectoras);
  fprintf (fp, "FIN\n");
  
  return 0;
}


/**
 * Carga del archivo tipo texto abierto que recibe un TAD
 * (hace analisis sintactico) y llena con este el tad que recibe
 * @param fp Archivo tipo texto abierto
 * @param tad En este sitio dejara el tipo cargado (localizado por esta func.)
 * @param merr Buffer para mensajes de error
 * @return 0 si y solo si no hay errores
 **/
int cargaTipoTexto(FILE *fp, nodo_tad **tad,char *merr)
{
  int e;
  char mens_error[1000];
  char mret[1000];
  
  assert(fp!=NULL);
  assert(tad!=NULL);
  
  num_ed=1; /* Se supone que estamos al comienzo del archivo */
  mens_error[0]='\0';
  if (nuevoNodoTipo(&tad_ed,merr)==-1) 
    {  return -1;  }
  deme_token (fp);
  e=analice_nombre (fp, mens_error);
  if (mens_error[0]!='\0')
    {
      sprintf(mret,":%i: %s\n",num_ed,mens_error);
      strcat(merr,mret);
    }
  if (!e)
    {
      liberar_tad(&tad_ed);
      return -1;
    }
  e=analice_iniciales (fp, mens_error);
  if (mens_error[0]!='\0')
    {
      sprintf(mret,":%i: %s\n",num_ed,mens_error);
      strcat(merr,mret);
    }
  if (!e)
    {
      liberar_tad(&tad_ed);
      return -1;
    }
  e=analice_constructoras(fp, mens_error);
  if (mens_error[0]!='\0')
    {
      sprintf(mret,":%i: %s\n",num_ed,mens_error);
      strcat(merr,mret);
    }
  if (!e)
    {
      liberar_tad(&tad_ed);
      return -1;
    }
  e=analice_selectoras(fp,mens_error,1);
  if (mens_error[0]!='\0')
    {
      sprintf(mret,":%i: %s\n",num_ed,mens_error);
      strcat(merr,mret);
    }
  if (!e)
    {
      liberar_tad(&tad_ed);
      return -1;
    }
  e=analice_axiomas_m32(fp,mens_error,tad_ed);
  if (mens_error[0]!='\0')
    {
      sprintf(mret,":%i: %s\n",num_ed,mens_error);
      strcat(merr,mret);
    }
  if (!e)
    {
      liberar_tad(&tad_ed);
      return -1;
    }
  if (tad_ed->selectoras==NULL && tad_ed->iniciales==NULL && 
      tad_ed->constructoras==NULL)
    {
      strcat(merr,mensajes[174]);
      strcat(merr,"\n");
      liberar_tad(&tad_ed);
      return -1;
    }
  *tad=tad_ed;
  
  return 0;
}


/**
 * Crea una archivo con la representacion del TAD tad en el
 * lenguaje de ManTa y la pone en el archivo abierto fp.
 * @param fp Archivo tipo texto abierto para escritura
 * @param tad Tad que se escribira
 **/
void salvaTipoTexto(FILE *fp,nodo_tad *tad)
{
  assert(tad!=NULL && fp!=NULL);
  ed_arch_signatura(fp,tad);
  fprintf(fp,"\n%s\n\n",claves[4]);
  ed_arch_axiomas_ops(fp,tad->selectoras);
}


/**** PARA DEPURAR ***/
void muestraTipo(nodo_tad *tad)
{
    printf("TAD %s, status=%i, tipo=%i, modif=%i\n",tad->nombre,tad->status,
           tad->tipo,tad->modif);
    printf("info_tipo=%p, parametros=%p, iniciales=%p, constructoras=%p\n",
           tad->info_tipo,tad->parametros,tad->iniciales,tad->constructoras);
    printf("selectoras=%p, sig=%p\n",tad->selectoras,tad->sig);
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD LTipo
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/** 
 * Inicializa el ambiente vacio
 **/
int inicLTipo(nodo_tad **L)
{
  assert(L!=NULL);
  *L=NULL;
  return 0;
}


/**
 * Agrega el elemento nuevo al final de la lista, si ya habia
 * uno con el nombre, el comportamiento depende del parametro remp.
 * @param tad  Cabeza de la lista de Tads
 * @param nuevo Tad por Agregar
 * @param remp  Forma de agregar:
 *              ADIC_ADD  si existe otro del mismo nombre agrega el nuevo antes
 *              ADIC_REP si existe otro con el mismo nombre lo reemplaza
 *                       (libera la memoria del otro)
 * @param merr  Buffer con mensajes de error
 **/
int adicElemLTipo(nodo_tad **tad,nodo_tad *nuevo, int remp,char *merr)
{
    nodo_tad *ac,**ant;
    char nomt[255],nom[255];
    int rc;

    assert(tad!=NULL && nuevo!=NULL);
    assert(merr!=NULL);

    nombreNodoTipo(nuevo,0,nom);
    ac=*tad;
    ant=tad;
    nombreNodoTipo(ac,0,nomt);
    for (;ac!=NULL && (rc=strcmp_nocase(nom,nomt))!=0;
         ant=&(ac->sig),ac = ac->sig,nombreNodoTipo(ac,0,nomt))
    ;
    if (remp==ADIC_REP && rc==0 && ac!=NULL)
      {
        nuevo->sig=ac->sig;
        *ant=nuevo;
        ac->sig=NULL;
        elimNodoTipo(&ac);
      }
    else
      {
        nuevo->sig=ac;
        *ant=nuevo;
      }
    if (ac==NULL) /* Por si funciones viejas usan esta */
      {
	ult_tad=nuevo;
      }
    return 0;
}


/**
 * Retorna la cantidad de Tipos en la lista iniciada con tad
 * @param tad Cabeza de la lista de TADs
 * @return Cantidad de elementos de la lista
 **/
int cantElemLTipo(nodo_tad *tad)
{
    int i;

    for (i=0;(tad != NULL);tad = tad->sig,i++);

    return i;
}


/**
 * Retorna un apuntador al TAD cuyo nombre es nomtad
 * @return apuntado al TAD o NULL si no existe
 **/
nodo_tad *buscaNombreLTipo(nodo_tad *tad,char *nomtad)
{
  nodo_tad *p_aux;
  char nt[255];
  
  assert(nomtad!=NULL);
  for (p_aux = tad,nombreNodoTipo(p_aux,0,nt);p_aux!=NULL &&
         strcmp(nt,nomtad)!=0;
       p_aux = p_aux->sig,nombreNodoTipo(p_aux,0,nt));
  
  return p_aux;
}


/**
 * Retorna el nombre del i-esimo Tipo de la lista tad
 * @param tad Cabeza de la lista de tads
 * @param i Indice del elemento
 * @param nom Apuntar al nombre del elemento que est en la pos. i
 **/
void nombreElemLTipo(nodo_tad *tad,int i, char* nom)
{
  nodo_tad *p_aux;
  
  int k;
  assert(0<=i && i<cantElemLTipo(tad) && nom!=NULL); /* VTP. 16/May/1997 */
  for (k=0,p_aux = tad;k<i;p_aux = p_aux->sig,k++);
  nombreNodoTipo(p_aux,0,nom);
}


/**
 * Si existe un tipo con el nombre recibido lo elimina
 * @param tad Cabeza de la lista de TADs
 * @param nom Nombre del tipo por eliminar
 **/
void elimElemLTipo(nodo_tad **tad,char *nom)
{
    nodo_tad *ac,**ant,*ta;
    char nomt[255];

    assert(tad!=NULL && nom!=NULL); /* VTP. 16/May/1997 */

    ac=*tad;
    ant=tad;
    nombreNodoTipo(ac,0,nomt);
    for (ta=NULL;ac!=NULL && strcmp(nomt,nom);
         ta=ac,ant=&(ac->sig),ac = ac->sig,nombreNodoTipo(ac,0,nomt))
    ;
    if (ac!=NULL) /* Encontro el nodo por eliminar */
    {
      if (ac->sig==NULL) /* Para que funcione con el kernel anterior */
	{
	  ult_tad=ta;
	}
      *ant=ac->sig;
      ac->sig=NULL;
      elimNodoTipo(&ac);
    }
}


/**** PARA DEPURAR ***/

/**
 * Muestra una lista de tipos en pantalla
 * @param t Cabeza de la lista 
 **/
void muestraLTipo(nodo_tad *t)
{
    printf("Mostrando lista, con %i elementos\n",cantElemLTipo(t));
    for (;t!=NULL;t=t->sig)
      { muestraTipo(t);  }
}



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Tabla de Simbolos.  Extension al TAD LTipos
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/* 
   Indica si el nombre de un tipo es valido
   nuevo  es verdadero si el tipo es nuevo 
          (en tal caso el nombre no puede estar definido)
*/
int nombreTipoValido(char *nom,int nuevo,char *merr)
{
  char mens_error[200];
  cad12 tadname;

  assert(nom!=NULL);
  assert(merr!=NULL);
  
  if (strcmp(nom,"EQ")==0)
    {
      sprintf(mens_error,mensajes[190],nom);
      strcat(merr,mens_error);
      strcat(merr,"\n");
      return 0;
    }
  if (nuevo && existsSymbol(symtable,nom,tadname))
    {
      sprintf(mens_error,mensajes[180],nom,tadname);
      strcat(merr,mens_error);
      strcat(merr,"\n");
      return 0;
    }
  if (existsSymbolParameter(symtable,nom,tadname))
    {
      sprintf(mens_error,mensajes[181],token,tadname);
      strcat(merr,mens_error);
      strcat(merr,"\n");
      return 0;
    }
  return 1;
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Ambiente.  Extension a TAD LTipos y Tabla de Simbolos
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Adiciona un tipo al ambiente y actualiza la tabla de simbolos
 * @param tad Cabeza de la lista de tads que conforman el ambiente
 * @param symtabl Cabeza de la tabla de simbolos
 * @param nuevo Nuevo TAD por adicionar
 * @param merr Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int adicTipoAmbiente(nodo_tad **tad,nodo_tad **symtable, nodo_tad *nuevo,
		     char *merr)
{
  int rc;
  rc=adicElemLTipo(tad,nuevo,ADIC_REP,merr);
  if (rc==0)
    { rc=updateSymbols(symtable,nuevo,merr); }

  return rc;
}


/**
 *  Carga un tipo en caso de que no este en el ambiente
 * @param tadAmb Cabeza del ambiente
 * @param nom    Nombre del tipo por cargar 
 * @param merr   Buffera para mensajes de error
 **/
nodo_tad* tipoAmbiente(nodo_tad **tadAmb,char *nom,char *merr)
{
  nodo_tad *t1,*tempAmb;
  texpr mens_error;
  assert(nom!=NULL);
  assert(merr!=NULL);
  
  if ((t1=buscaNombreLTipo(*tadAmb,nom))==NULL)
    {
      tempAmb=tad;
      tad=*tadAmb;
      busque_arch_tad(nom);
      tad=tempAmb;
      if ((t1=buscaNombreLTipo(*tadAmb,nom))==NULL)
	{
	  sprintf(mens_error,mensajes[194],nom);
	  strcat(merr,mens_error);
	  strcat(merr,"\n");
	  return NULL;
	}
    }
  return t1;
}

     
/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Prototipos en C.  Extension al TAD Tipo
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

const int CP_ENC=1,
  CP_CON=2,
  CP_SEL=4,
  CP_INT=8,
  CP_MAKE=16,  
  CP_MAIN=32,
  CP_TODO=63;

/**
 * Genera los archivos del prototipo en C del TAD que
 * recibe, los archivo generados se especifican en que
 * con las constantes CP_CON,CP_SEL,CP_INT,CP_MAKE,CP_MAIN o CP_TODO
 * @para t Tad por generar
 * @param que Constante que indica que generar 
 * @return 0 si y solo si no hay error
 **/
int generaCProt(nodo_tad *t,int que)
{
  assert(t!=NULL && !tipo_primitivo(t->nombre) && que!=0);
  if (que == CP_TODO)
    {
      generar_all_prot(t,1);
      return 0;
    }
  if (que & CP_ENC)
    { crear_headers(t); }
  if (que & CP_CON)
    { generar_constructoras(t,1); }
  if (que & CP_SEL)
    { generar_selectoras(t,1); }
  if (que & CP_INT)
    { crear_interfaz(t,1); }
  if (que & CP_MAIN)
    { crear_main(t); }
  if (que & CP_MAKE)
    { generar_make(t); }
  return 0;
}




/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Generacion Tipos.  Extension al TAD Tipo
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Crea un nuevo nodo de un tipo, por instanciacin
 * @param nombre 	Nombre del nuevo Tipo
 * @param p		Tad por instanciar
 * @param nomparam      Nombre del parametro de p por instanciar
 * @param p_inst	Tad con el cual instanciar
 * @param nuevo         Nuevo TAD (localiza memoria)
 * @param merr          Buffer para mensajes de error
 * @return Retorna 0 si y solo si no hay errores
 **/
int nuevoInstanciacionT(char *nombre,nodo_tad *p, char *nom_param,
                        nodo_tad *p_inst, nodo_tad **nuevo,char *merr)
{
  cad50 nuevo_nombre_param; /* Nuevo nombre del parametro de p */
  
  assert(nombre!=NULL && p!=NULL && nom_param!=NULL && p_inst!=NULL);
  
  if (!esTipoBasico(p_inst))
    {
      /* Only basic types can be instanced */
      strcat(merr,mensajes[178]);
      strcat(merr,"\n");
      return -1;
    }
  strcpy(nuevo_nombre_param,p_inst->nombre);
  if (nuevoNodoTipo(nuevo,merr)==-1)
    {
      return -1;
    }
  genere_copia_tad(p,p->nombre,*nuevo,nombre);
  cambiar_nombre(*nuevo,nom_param,nuevo_nombre_param);
  suprima_param_tad(*nuevo,nuevo_nombre_param);
  return 0;
}


/**
 * Crea un nuevo nodo de un tipo, por copia de otro
 * @param nom 	    Nombre del nuevo Tipo
 * @param fuente    Nodo del Tad Fuente
 * @param nuevo     Nuevo TAD (localiza memoria)
 * @param merr      Buffer para mensajes de error
 * @return Retorna 0 si y solo si no hay errores
 **/
int nuevoCopiaT(char *nom, nodo_tad *fuente, nodo_tad **nuevo,char *merr)
{
  assert(nom!=NULL);
  assert(fuente!=NULL);
  assert(nuevo!=NULL);
  assert(merr!=NULL);
  
  if (nuevoNodoTipo(nuevo,merr)==-1)
    {
      return -1;
    }
  
  genere_copia_tad (fuente, fuente->nombre, *nuevo, nom);
  (*nuevo)->status = 0;
  
  return 0;
}



/**
 * Crea un nuevo nodo de un tipo como producto cartesiano de 2
 * @param  nom 	     Nombre del nuevo Tipo
 * @param  t1        Primer Tipo para el producto
 * @param  t2        Segundo Tipo para el producto
 * @param  nuevo     Nuevo TAD (localiza memoria)
 * @param merr      Buffer para mensajes de error
 * @return Retorna 0 si y solo si no hay errores
 **/
int nuevoProdT(char *nom, nodo_tad *t1, nodo_tad *t2, 
	       nodo_tad **nuevo ,char *merr)
{
  
  nodo_tad *pre_ult_tad;
  
  assert(nom!=NULL);
  assert(t1!=NULL);
  assert(t2!=NULL);
  assert(nuevo!=NULL);
  assert(merr!=NULL);
  
  if (t1->parametros!=NULL || t2->parametros!=NULL ||
      (t1->iniciales==NULL && t1->constructoras==NULL)||
      (t2->iniciales==NULL && t2->constructoras==NULL))
    {
      strcat(merr,mensajes[179]);
      strcat(merr,"\n");
      return -1;
    }
  if (nuevoNodoTipo(nuevo,merr)==-1)
    {
      return -1;
    }

  strcpy ((*nuevo)->nombre, nom);
  
  /* Las funciones del nucleo 3.1 para producto cartesiano
     trabajan con ult_tad. */
  pre_ult_tad=ult_tad;
  ult_tad=*nuevo;
  
  informacion_tipo_prod ((*nuevo), t1->nombre, t2->nombre);
  genere_param_pc (t1, t2);
  genere_inicial_pc_implic (t1, t2);
  (*nuevo)->constructoras = NULL;
  genere_selectoras_pc_implic (t1, t2);
  adic_proyecciones_implic (t1, t2);
  adic_igualdad (t1, t2);
  (*nuevo)->status = 0;
  
  ult_tad=pre_ult_tad;
  
  return 0;
}


/**
 * Genera lados izquierdos de los axiomas para las selectoras de un 
 * tipo por restriccin
 * @param sel     Nodo de selectora
 * @param tr      Tipo que se esta restringiendo
 * @param li_str  Buffer donde retorna el lado izquierdo del axioma 
 *                (ya localizado)
 * @param trvar   Buffer para retornar una lista de variables del tipo que 
 *                se restringe separadas por | (ya localizado)
 **/
void genereAxiomaRestriccionIzquierdo(nodo_select *sel,
				      char *tr,
				      char *li_str,
				      char *trvar)
{
  nodo_dom *d;
  int nv;
  char var[20];
  
  assert(sel!=NULL);
  assert(tr!=NULL);
  assert(li_str!=NULL);
  assert(trvar!=NULL);
  
  sprintf(li_str,"%s(",sel->nombre);
  for (nv=1,d=sel->dominio;
       d!=NULL;
       d=d->sig,nv++)
    {
      sprintf(var,"X%i",nv);
      if (nv>1)
	{
	  strcat(li_str,",");
	}
      strcat(li_str,var);
      if (strcmp(d->nombre,tr)==0)
	{
	  strcat(trvar,"|");
	  strcat(trvar,var);
	}
    }
  strcat(li_str,")");
}


/**
 * Genera lados derechos de los axiomas para las selectoras de un 
 * tipo por restriccin
 * @param sel     Nodo de selectora 
 * @param trvar   Lista de variables del tipo que se restringe separadas por |
 * @param npred   Nombre de la funcion de restriccion
 * @param li_str  Lado izquierdo lleno
 * @param ld_str  Buffer para guardar el lado derecho calculado (ya localizado)
 **/
void genereAxiomaRestriccionDerecho(nodo_select *sel,
				    char *trvar,
				    char *npred,
				    char *li_str,
				    char *ld_str)
{
  char *ts,*ts2;
  texpr lss,cond,sc;
  int pi; /* Numero de parentesis izquierdos */
  
  assert(sel!=NULL);
  assert(trvar!=NULL);
  assert(npred!=NULL);
  assert(li_str!=NULL);
  assert(ld_str!=NULL);
  
  /* Generamos subexpresiones para el lado derecho a partir del lado
     izquierdo */
  ts=strstr(li_str,"(");
  assert(ts!=NULL); 
  /* Ya debio llenarse el lado izquiedo que debe ser SEL(X1,X2,...Xn) */
  sprintf(lss,"%s%s",sel->nombre_fte,ts);
  
  /* Ahora de acuerdo a los parametros que sean del tipo restringido
     generamos la expresion */
  
  /* Si no hay variables del tipo restringido no hay que hacer mas */
  if (trvar[0]=='\0') /* No hay variables del tipo restringido */
    {
      strcpy(ld_str,lss);
      return;
    }
  /* Si hay variables del tipo restringido debemos generar un if
     cuya condicion restringe tales variables */
  ts=strstr(trvar,"|");
  assert(ts!=NULL); /* Debe haber al menos una variable */
  ts++;
  cond[0]='\0';
  pi=0; /* Cantidad de parentesis izquierdos abiertos */
  do
    {
      ts2=strstr(ts,"|");
      if (ts2!=NULL)
	{
	  *ts2='\0';
	}
      sprintf(sc,"%s(%s)",npred,ts);
      if (ts2==NULL) /* Queda una sola variable */
	{
	  strcat(cond,sc);
	  while (pi>0)
	    {
	      strcat(cond,")");
	      pi--;
	    }
	}
      else /* Quedan mas variables */
	{
	  strcat(cond,"AND(");
	  strcat(cond,sc);
	  strcat(cond,",");
	  pi++;
	}
      ts=ts2==NULL?NULL:ts2+1;
    } while (ts!=NULL);
  
  /* Finalmente escribimos el if cond then E1 else ERROR fi */
  sprintf(ld_str,"%s %s %s %s %s %s %s",
	  claves[8],cond,claves[9],lss,claves[10],claves[11],claves[15]);
  return;
}



/**
 * Genera los axiomas para la selectora sel, sabiendo que
 * es una selectora de un tipo que restringe otro
 * @param tr    Es el tad que se restringe
 * @param sel   La selectora a la que se le crean (cambian) axiomas
 * @param npred Es el nombre del predicado de restriccion
 * @param merr  Buffer para errores
 * @return 0 si y solo no hay error
 **/
int genereAxiomaRestriccion(char *tr,nodo_select *sel,char *npred, char *merr)
{
  
  char trvar[100];
  texpr li_str,ld_str;
  int correcto;
  short tipo_error;
  FILE *fp2;
  
  assert(sel!=NULL);
  assert(tr!=NULL);
  assert(npred!=NULL);
  assert(merr!=NULL);
  
  if (sel->axiomas!=NULL)
    {
      elimLIgualdad(&sel->axiomas);
      sel->axiomas=NULL;
    }
  li_str[0]=ld_str[0]=trvar[0]='\0';
  genereAxiomaRestriccionIzquierdo(sel,tr,li_str,trvar);
  genereAxiomaRestriccionDerecho(sel,trvar,npred,li_str,ld_str);

  /* Ahora analizamos la expresion generada */
  fp2 = fopen ("axiom_ed.tad", "w");
  FileOpen (fp2, "axiom_ed.tad");
  fprintf(fp2,"%s=%s",li_str,ld_str);
  fclose(fp2);
  correcto = analice_axiomas (merr, &(sel->axiomas), sel->nombre,
			      &tipo_error);
  assert(correcto);
  unlink ("axiom_ed.tad");
  return 0;
}


/**
 * Cambia todos los parametros de la selectora sel a inductivos o no 
 * segun el parametro ind
 * @param sel  Selectora cuyo dominio se cambiara
 * @param ind  Valor que se pondra al campo induccion de todos 
 *             los elementos del dominio
 **/
void cambiaParametrosInduccion(nodo_select *sel, short ind)
{
  nodo_dom *d;
  if (sel==NULL)
    { return; }
  for (d=sel->dominio;d!=NULL;d=d->sig)
    { d->induccion=ind;  }
}


/**
 * Crea un nuevo nodo de un tipo como restriccion de otro por
 * un predicado
 * @param nom 	    Nombre del nuevo Tipo
 * @param tf        Tad por restringir
 * @param npred     Nombre del Predicado
 * @param pred      Predicado para restriccion
 * @param fp        Archivo abierto para escritura donde deja la
 *                  definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoResT(char *nom, nodo_tad *tf, char *npred, char *pred, 
              FILE *fp,nodo_tad **nuevo,char *merr)
{
  nodo_igualdad *cab_axiomas = NULL;
  nodo_select *s, *sel,**ant;
  nodo_dom *d;
  FILE *fp2;
  short correcto, tipo_error;
  texpr mens_error;
  cad12 nomtad;
  
  assert(nom!=NULL);
  assert(tf!=NULL);
  assert(npred!=NULL);
  assert(pred!=NULL);
  assert(fp!=NULL);
  assert(nuevo!=NULL);
  assert(merr!=NULL);
  
  
  /* Solo pueden restringirse tipos basicos */
  if (!esTipoBasico(tf))
    {
      sprintf(mens_error,mensajes[183]);
      strcat(merr,mens_error);
      strcat(merr,"\n");
      return -1;
    }
  
  /* Primero verificamos que el nombre del predicado no sea
     un simbolo ya definido */
  if (existsSymbol(symtable,npred,nomtad))
    {
      sprintf(mens_error,mensajes[180],npred,nomtad);
      strcat(merr,mens_error);
      strcat(merr,"\n");
      return -1;
    }
  
  /* Despues construimos el nodo que sera la restriccion */
  if (nuevoNodoTipo(nuevo,merr)==-1)
    {
      return -1;
    }
  tad_ed=*nuevo;
  /* Primero creamos informacion parcial para el
     nuevo nodo y la escribimos en el archivo*/
  genere_copia_tad (tf, "", tad_ed, "");
  strcpy (tad_ed->nombre, nom);
  informacion_tipo_restr (tad_ed, tf->nombre, npred);
  /* La restriccion no tiene generadoras */
  elimLGeneradora(&(tad_ed->iniciales));
  elimLGeneradora(&(tad_ed->constructoras));
  cambiar_nombre_sel (tad_ed, nom);
  
  /* Creamos una nueva selectora, que se trata del predicado restriccion */
  if (nuevoNodoSelectora(&s,merr)!=0)
    {
      elimNodoTipo(nuevo);
      return -1;
    }
  strcpy (s->nombre, npred);
  strcpy (s->nombre_fte, s->nombre);
  if (nuevoNodoDominio(&d,merr)!=0)
    {
      elimNodoTipo(nuevo);
      elimLSelectora(&s);
      return -1;
    }
  strcpy (d->nombre, tf->nombre);
  d->induccion = 1;
  d->sig = NULL;
  s->dominio = d;
  strcpy (s->codominio, "BOOL");
  s->caracterizadora = 1;
  s->axiomas=NULL;
  s->sig = tad_ed->selectoras;
  tad_ed->selectoras = s;
  
  /* Suprime la selectora de igualdad, pues es una extension */
  ant=&(tad_ed->selectoras);
  sel=tad_ed->selectoras;
  while (sel!=NULL && !es_prefijo("EQ_",sel->nombre))
    {
      ant=&(sel->sig);
      sel=sel->sig;
    }
  if (sel!=NULL)
    {
      *ant=sel->sig;
      elimLSelectora(&sel);
    }
  
  
  /* Ahora analizamos el predicado dado por el usuario */
  fp2 = fopen ("axiom_ed.tad", "w");
  FileOpen (fp2, "axiom_ed.tad");
  fprintf(fp2,"%s",pred);
  fclose(fp2);
  correcto = analice_axiomas (mens_error, &cab_axiomas, npred,
			      &tipo_error);
  unlink ("axiom_ed.tad");
  if (correcto)
    {
      s->axiomas = cab_axiomas;
    }
  else
    {
      elimNodoTipo(nuevo);
      /* No es necesario eliminar s, ni d pues ya estan pegados
	 a nuevo */
      strcat(merr,mens_error);
      strcat(merr,"\n");
      return -1;
    }
  
  
  /* Ahora modificamos todos los
     axiomas de selectoras del nuevo TAD, excepto el predicado */
  for (sel = s->sig; (sel != NULL) && correcto; sel = sel->sig)
    {
      /* Los parametros de cualquier selectora del tipo restringido
	 no son inductivos */
      cambiaParametrosInduccion(sel,0);
      if (genereAxiomaRestriccion(tf->nombre,sel,npred,merr)!=0)
	{
	  elimNodoTipo(nuevo);
	  return -1;
	}
    }
  tad_ed->status = 0;
  
  /* Escribimos el resultado en el archivo */
  salvaTipoTexto(fp,tad_ed);
  
  return 0;
}



/**
 * Crea un nuevo nodo de un tipo como suma directa de los dos
 * tipos que recibe
 * @param nom 	    Nombre del nuevo Tipo
 * @param t1        Primer TAD por sumar
 * @param t2        Segundo TAD por sumar
 * @param fp        Archivo abierto para escritura donde deja la
 *                  definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoSumT(char *nom, nodo_tad *t1, nodo_tad *t2,
              FILE *fp,nodo_tad **nuevo,char *merr)
{
  nodo_select	*s,
    *cab_select=NULL,
    *ult_select1=NULL,
    *ult_select2=NULL,
    *us,
    **ant;
  char *adic;
  texpr li_axioma,li_aux;
  int correcto;
  cad12 eqstr;
  
  assert(nom!=NULL);
  assert(t1!=NULL);
  assert(t2!=NULL);
  assert(fp!=NULL);
  assert(nuevo!=NULL);
  assert(merr!=NULL);
  
  /* Deben ser tipos diferentes */
  if (t1==t2)
    {
      strcat(merr,mensajes[188]);
      strcat(merr,"\n");
      return -1;
    }
  
  /* Deben ser tipos basicos */
  if (!esTipoBasico(t1) || !esTipoBasico(t2))
    {
      strcat(merr,mensajes[183]);
      strcat(merr,"\n");
      return -1;
    }
  
  if (nuevoNodoTipo(nuevo,merr)==-1)
    {  return -1;  }
  
  /* Primero creamos la signatura y la escribimos */
  tad_ed = *nuevo;
  strcpy(tad_ed->nombre,nom);
  informacion_tipo_extension(tad_ed,t1->nombre,t2->nombre);
  genere_param_sum_direc(t1,t2);
  genere_iniciales_sum_direc(t1,t2);
  genere_constructoras_sum_direc(t1,t2);
  
  /* Ahora se mezclan las selectoras */
  adic=(char *)malloc(sizeof(char)*(strlen(tad_ed->nombre)+3));
  HayMem(adic,sizeof(char)*(strlen(tad_ed->nombre)+3));
  sprintf(adic,"_%s",tad_ed->nombre);
  copie_lista_selectoras(t1->selectoras,&cab_select,&ult_select1,
			 t1->nombre,tad_ed->nombre,"",
			 t1,tad_ed->nombre);
  /* Suprimir la igualdad y nadie es caracterizador ahora*/
  s = cab_select;
  sprintf(eqstr,"EQ_%s",tad_ed->nombre);
  ant=&cab_select;
  us=NULL;
  while ((s != NULL) && strcmp(s->nombre,eqstr)!=0)
    {
      ant=&(s->sig);
      us=s;
      s=s->sig;
    }
  if (s!=NULL)
    {
      *ant=s->sig;
      if (s==ult_select1)
	{
	  ult_select1=us;
	}
      elimLSelectora(&s);
    }
  tad_ed->selectoras = cab_select;
  
  copie_lista_selectoras(t2->selectoras,&cab_select,&ult_select2,
			 t2->nombre,tad_ed->nombre,"",
			 t2,tad_ed->nombre);
  /* Suprimir la igualdad */
  s = cab_select;
  sprintf(eqstr,"EQ_%s",tad_ed->nombre);
  ant=&cab_select;
  us=NULL;
  while ((s != NULL) && strcmp(s->nombre,eqstr)!=0)
    {
      ant=&(s->sig);
      us=s;
      s=s->sig;
    }
  if (s!=NULL)
    {
      *ant=s->sig;
      if (s==ult_select2)
	{
	  ult_select2=us;
	}
      elimLSelectora(&s);
    }
  
  if (tad_ed->selectoras == NULL)
    tad_ed->selectoras = cab_select;
  else
    ult_select1->sig = cab_select
      ;
  
  /* Escribimos signatura y la signatura de la igualdad */
  ed_arch_signatura(fp,tad_ed);
  fprintf(fp,"EQ_%s (c) : %s (i) * %s (i) -> BOOL\n",tad_ed->nombre,
	  tad_ed->nombre,tad_ed->nombre);
  fprintf(fp,"\n%s\n\n",claves[4]);
  
  /* Escribimos axiomas */
  for (s = tad_ed->selectoras,correcto = 1; (s != cab_select) && correcto;
       s = s->sig) {
    correcto= construya_axs_selec_sum_direc(t2,s,adic,fp);
    /* Usamos ultimo parametro no NULL para que escriba en un archivo */
    fprintf(fp,"\n");
  }
  if (!correcto) {
    elimNodoTipo(nuevo);
    free(adic);
    return -1;
  }
  
  for (s = cab_select; (s != NULL) && correcto; s = s->sig) {
    correcto = construya_axs_selec_sum_direc(t1,s,adic,fp);
    fprintf(fp,"\n");
  }
  free(adic);
  if (!correcto) {
    elimNodoTipo(nuevo);
    free(adic);
    return -1;
  }
  /* Finalmente agregamos una selectora de igualdad */
  s=NULL;
  if (!tiene_igualdad_definida (tad_ed))
    {
      adic_select_eq (tad_ed, &s);
      strcpy (li_axioma, "(");
      sprintf (li_aux, "%s%s", s->nombre, li_axioma);
      strcpy (li_axioma, li_aux);
      genere_axiomas (fp, s->dominio, li_axioma, '1', s->axiomas);
    }
  tad_ed->status = 0;
  
  return 0;
}


/**
 * Crea un nuevo nodo de un tipo por completacion con un elmento
 * @param nom 	    Nombre del nuevo Tipo
 * @param tf        Tad por restringir
 * @param ne        Nombre del nuevo elemento
 * @param fp        Archivo abierto para escritura donde deja la
 *                  definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoComT(char *nom, nodo_tad *tf, char *ne,
              FILE *fp,nodo_tad **nuevo,char *merr)
{
  nodo_select *s, *ult_select=NULL;
  nodo_select **ant;
  short correcto;
  char *adic;
  texpr li_axioma,li_aux,mens_error;
  cad12 nomtad,eqstr;
  
  assert(nom!=NULL);
  assert(tf!=NULL);
  assert(ne!=NULL);
  assert(fp!=NULL);
  assert(nuevo!=NULL);
  assert(merr!=NULL);
  
  /* No debe ser una extension */
  if (tf->iniciales==NULL && tf->constructoras==NULL)
    {
      strcat(merr,mensajes[189]);
      strcat(merr,"\n");
      return -1;
    }
  /* El nombre del elemento para completar no debe estar definido */
  if (existsSymbol(symtable,ne,nomtad))
    {
      sprintf(mens_error,mensajes[180],ne,nomtad);
      strcat(merr,mens_error);
      strcat(merr,"\n");
      return -1;
    }
  
  if (nuevoNodoTipo(nuevo,merr)==-1)
    { return -1; }
  
  /* Primero creamos la signatura y la escribimos */
  tad_ed = *nuevo;
  genere_copia_tad (tf, tf->nombre, tad_ed, nom);
  informacion_tipo_completacion (tad_ed, tf->nombre, ne);
  genere_inicial_completacion (ne);
  ed_arch_signatura(fp,tad_ed);
  fprintf(fp,"\n%s\n\n",claves[4]);
  
  /* Se generan las nuevas selectoras y se escriben (excepto igualdad) */
  adic=(char *)malloc(sizeof(char)*(strlen(tad_ed->nombre)+3));
  HayMem(adic,sizeof(char)*(strlen(tad_ed->nombre)+3));
  sprintf(adic,"_%s",tad_ed->nombre);
  
  copie_lista_selectoras (tf->selectoras, &(tad_ed->selectoras),
			  &ult_select, tf->nombre, tad_ed->nombre, "",
			  tf, tad_ed->nombre);
  /* Suprimir la igualdad */
  s = tad_ed->selectoras;
  sprintf(eqstr,"EQ_%s",tad_ed->nombre);
  ant=&tad_ed->selectoras;
  while ((s != NULL) && strcmp(s->nombre,eqstr)!=0)
    {
      ant=&(s->sig);
      s=s->sig;
    }
  if (s!=NULL)
    {
      *ant=s->sig;
      elimLSelectora(&s);
    }
  
  for (s = tad_ed->selectoras, correcto = 1; (s != NULL) && correcto;
       s = s->sig)
    {
      correcto = construya_axs_selec_sum_direc (tf, s,adic,fp);
      fprintf(fp,"\n");
    }
  free(adic);
  if (!correcto)
    {
      elimNodoTipo(nuevo);
      return -1;
    }
  
  /* Finalmente agregamos una selectora de igualdad */
  s=NULL;
  if (!tiene_igualdad_definida (tad_ed))
    {
      adic_select_eq (tad_ed, &s);
      strcpy (li_axioma, "(");
      sprintf (li_aux, "%s%s", s->nombre, li_axioma);
      strcpy (li_axioma, li_aux);
      genere_axiomas (fp, s->dominio, li_axioma, '1', s->axiomas);
    }
  tad_ed->status = 0;
  
  return 0;
}


/**
 * Crea un nuevo nodo de un tipo como variacion de dos existentes
 * @param nom 	    Nombre del nuevo Tipo
 * @param t1        Primer TAD por sumar
 * @param t2        Segundo TAD por sumar
 * @param fp        Archivo abierto para escritura donde deja la
 *                  definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoVarT(char *nom, nodo_tad *t1, nodo_tad *t2,
              FILE *fp,nodo_tad **nuevo,char *merr)
{
  
  nodo_select *seq=NULL;
  
  assert(nom!=NULL);
  assert(t1!=NULL);
  assert(t2!=NULL);
  assert(fp!=NULL);
  assert(nuevo!=NULL);
  assert(merr!=NULL);
  
  /* Deben ser tipos diferentes */
  if (t1==t2)
    {
      strcat(merr,mensajes[188]);
      strcat(merr,"\n");
      return -1;
    }
  
  /* Deben ser tipos basicos */
  if (!esTipoBasico(t1) || !esTipoBasico(t2))
    {
      strcat(merr,mensajes[183]);
      strcat(merr,"\n");
      return -1;
    }
  
  if (nuevoNodoTipo(nuevo,merr)==-1)
    { return -1; }
  
  /* Primero creamos la signatura y la escribimos */
  tad_ed = *nuevo;
  strcpy (tad_ed->nombre, nom);
  informacion_tipo_variacion (tad_ed, t1->nombre, t2->nombre);
  genere_param_sum_direc (t1, t2);
  genere_iniciales_variacion(tad_ed,t1,t2);
  ed_arch_signatura(fp,tad_ed);
  fprintf(fp,"\n%s_%s     : %s -> %s\n",
	  t1->nombre,t2->nombre,t1->nombre,t2->nombre);
  fprintf(fp,"%s_%s     : %s -> %s\n",
	  t2->nombre,t1->nombre,t2->nombre,t1->nombre);
  fprintf(fp,"EQ_%s (c) : %s(i) * %s(i) -> BOOL\n\n",nom,nom,nom);
  fprintf(fp,"%s\n\n",claves[4]);
  genere_select_conversion (t1, t2,&(tad_ed->selectoras),fp);
  fprintf(fp,"\n");
  genere_select_conversion (t2, t1,&(tad_ed->selectoras->sig),fp);
  assert(tad_ed->iniciales!=NULL);
  adic_select_eq (tad_ed, &seq);
  fprintf(fp,"\nEQ_%s(%s_%s(X1),%s_%s(X2))=EQ_%s(X1,X2)\n",
	  nom,nom,t1->nombre,nom,t1->nombre,t1->nombre);
  fprintf(fp,"EQ_%s(%s_%s(X1),%s_%s(X2))=EQ_%s(X1,%s_%s(X2))\n",
	  nom,nom,t1->nombre,nom,t2->nombre,t1->nombre,
	  t2->nombre,t1->nombre);
  fprintf(fp,"EQ_%s(%s_%s(X1),%s_%s(X2))=EQ_%s(X1,%s_%s(X2))\n",
	  nom,nom,t2->nombre,nom,t1->nombre,t2->nombre,
	  t1->nombre,t2->nombre);
  fprintf(fp,"EQ_%s(%s_%s(X1),%s_%s(X2))=EQ_%s(X1,X2)\n",
	  nom,nom,t2->nombre,nom,t2->nombre,t2->nombre);
  fprintf(fp,"\n");
  tad_ed->status = 0;
  
  return 0;
}


/**
 * Crea un nuevo nodo de un tipo inductivo
 * @param nom 	Nombre del nuevo Tipo
 * @param fp        Archivo abierto para escritura donde deja la
 *                 definicion parcial del TAD
 * @param nuevo     Apuntara a un nuevo tad (definido parcialmente)
 * @param merr      Buffer para mensajes de error
 * @return 0 si y solo si no hay error
 **/
int nuevoIndT(char *nom, FILE *fp,nodo_tad **nuevo,char *merr)
{
  assert(nom!=NULL);
  assert(fp!=NULL);
  assert(nuevo!=NULL);
  assert(merr!=NULL);
  
  if (nuevoNodoTipo(nuevo,merr)==-1)
    { return -1; }
  
  /* Primero creamos la signatura y la escribimos */
  tad_ed = *nuevo;
  strcpy (tad_ed->nombre, nom);
  
  fprintf (fp, "%s %s [ /* T1,T2 */ ]\n\n", claves[0], nom);
  fprintf (fp, "%s\n", claves[1]);
  fprintf (fp, "/* ini :  T1    * T2    -> %s */\n\n", nom);
  fprintf (fp, "%s\n", claves[2]);
  fprintf (fp, "/* con : %s    * T1    * T2    -> %s */\n\n", nom, nom);
  fprintf (fp, "%s\n", claves[3]);
  fprintf (fp, "/* sel (%s): %s (%s)    * T1    * T2    -> T2 */\n\n",
	   claves[13], nom, claves[14]);
  fprintf (fp, "EQ_%s (%s): %s (%s)    * %s (%s)    -> BOOL\n\n",
	   nom, claves[13], nom, claves[14], nom, claves[14]);
  fprintf (fp, "%s\n", claves[4]);
  fprintf (fp, "/* sel = exp */\n\n");
  
  return 0;
}




/**
 * Las generadoras del tipo representado se copian como selectoras
 * en un tipo nuevo cambiando los nombres (agregando como prefijo 
 * el nombre de la representacion.)
 * Tambien cambiamos en todo dominio y codominio las ocurrencias 
 *  del tad representado con el tad que representa.
 *  Ningun dominio es inductivo en principio
 * @param rep Representacion que se construye
 * @param gen Generadoras del tad que se representa
 * @param antTipo nombre del TAD que se representa
 * @param nuevoTipo Nombre del TAD con el que se representa
 * @param merr Buffer para mensajes de error
 * @return codigo de error o 0 si no hay problemas
 **/
int generadorasRep(nodo_tad *rep,nodo_generadora *gen,
		   char *antTipo,char *nuevoTipo,char *merr)
{
  nodo_select **psel,*sel;
  nodo_dom **pdom;
  nodo_nombre *nn;

  /* Vamos al final de las  selectoras ya definidas */
  for (psel=&(rep->selectoras),sel=rep->selectoras;
       sel!=NULL;
       psel=&(sel->sig),sel=sel->sig);
  /* Agregamos las generadoras que recibimos */
  for (;
       gen!=NULL;
       gen=gen->sig,psel=&((*psel)->sig))
    {
      if (nuevoNodoSelectora(psel,merr)!=0)
	{
	  elimLSelectora(&rep->selectoras);
	  return -1;
	}
      sprintf((*psel)->nombre,"%s_%s",rep->nombre,gen->nombre);
      strcpy((*psel)->nombre_fte,gen->nombre);
      strcpy((*psel)->codominio,nuevoTipo);
      (*psel)->caracterizadora=0;
      (*psel)->axiomas=NULL;
      (*psel)->sig=NULL;
      /* Copiamos el dominio del uno en el otro, pero cambiando nombres */
      for (pdom=&((*psel)->dominio),nn=gen->dominio;
	   nn!=NULL;
	   nn=nn->sig, pdom=&((*pdom)->sig))
	{
	  if (nuevoNodoDominio(pdom,merr)!=0)
	    {
	      elimLSelectora(&rep->selectoras);
	      return -1;
	    }
	  if (strcmp(nn->nombre,antTipo)==0)
	    {
	      strcpy((*pdom)->nombre,nuevoTipo);
	    }
	  else
	    {
	      strcpy((*pdom)->nombre,nn->nombre);
	    }
	  (*pdom)->induccion=0;
	  (*pdom)->sig=NULL;
	}
      (*psel)->sig=NULL;
    }
  return 0;
}



/**
 * Un conjunto de selectoras sel se copia  como selectoras del nuevo tipo rep
 * cambiando los nombres (agregando como prefijo el nombre de la 
 * representacion.)
 * Tambien cambiamos en todo dominio y codominio las ocurrencias 
 * del tad antTipo por nuevoTipo.  Ningun dominio es inductivo en principio
 * @param rep TAD al que se agregan selectoras
 * @param s Nuevas selectoras
 * @param antTipo Nombre del tipo antiguo
 * @param nuevoTipo Nombre del nuevo tipo
 * @param merr Buffer para mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int selectorasRep(nodo_tad *rep,nodo_select *s,
		   char *antTipo,char *nuevoTipo,char *merr)
{
  nodo_select *sel,**psel;
  nodo_dom **pdom,*dom;
  cad12 eqstr;

  /* Vamos al final de las  selectoras ya definidas */
  for (psel=&(rep->selectoras),sel=rep->selectoras;
       sel!=NULL;
       psel=&(sel->sig),sel=sel->sig);
  sprintf(eqstr,"EQ_%s",antTipo);
  for (sel=s;
       sel!=NULL;
       sel=sel->sig,psel=&((*psel)->sig))
    {
      if (nuevoNodoSelectora(psel,merr)!=0)
	{
	  elimLSelectora(&rep->selectoras);
	  return -1;
	}
      sprintf((*psel)->nombre,"%s_%s",rep->nombre,sel->nombre);
      sprintf((*psel)->nombre_fte,"%s",sel->nombre);
      if (strcmp(sel->codominio,antTipo)==0 && strcmp(sel->nombre,eqstr)!=0)
	{
	  strcpy((*psel)->codominio,nuevoTipo);
	}
      else
	{
	  strcpy((*psel)->codominio,sel->codominio);
	}
      /* Copiamos el dominio del uno en el otro, pero cambiando nombres */
      for (pdom=&((*psel)->dominio),dom=sel->dominio;
	   dom!=NULL;
	   dom=dom->sig, pdom=&((*pdom)->sig))
	{
	  if (nuevoNodoDominio(pdom,merr)!=0)
	    {
	      elimLSelectora(&rep->selectoras);
	      return -1;
	    }
	  if (strcmp(dom->nombre,antTipo)==0)
	    {
	      strcpy((*pdom)->nombre,nuevoTipo);
	    }
	  else
	    {
	      strcpy((*pdom)->nombre,dom->nombre);
	    }
	  (*pdom)->induccion=0;
	  (*pdom)->sig=NULL;
	}
      (*psel)->caracterizadora=0;
      (*psel)->axiomas=NULL;
      (*psel)->sig=NULL;
    }
  return 0;
}


/**
 * Crea un TAD extensin para representar t1 sobre t2
 * @param nom Nombre del nuevo tipo
 * @param t1 TAD que se representa
 * @param t2 TAD con el que se representa
 * @param nuevo Apuntara al nuevo tad (localizado por esta funcion)
 * @param merr Buffer para mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int creaTADRep(char *nom,nodo_tad *t1,nodo_tad *t2,
	       nodo_tad **nuevo,char *merr)
{
  /* No puede ser un TAD aberrado */
  if (tipoMalo(t1,merr) || tipoMalo(t2,merr))
    {
      return -1;
    }
  if (!nombreTipoValido(nom,1,merr))
    {
      return -1;
    }
  /* Deben ser tipos diferentes */
  if (t1==t2 || strcmp(t1->nombre,t2->nombre)==0)
    {
      strcat(merr,mensajes[188]);
      strcat(merr,"\n");
      return -1;
    }
  /* Deben ser tipos basicos */
  if (!esTipoBasico(t1) || !esTipoBasico(t2))
    {
      strcat(merr,mensajes[183]);
      strcat(merr,"\n");
      return -1;
    }
  
  /* Creamos un tipo nuevo para copiar t1 con algunos cambios */
  if (nuevoNodoTipo(nuevo,merr)==-1)
    {
      return -1;
    }
  strcpy((*nuevo)->nombre,nom);
  (*nuevo)->tipo=REPRESENTACION;
  if (nuevoNodoNombre(&((*nuevo)->info_tipo),merr)==-1)
    {
      elimNodoTipo(nuevo);
      return -1;
    }
  strcpy((*nuevo)->info_tipo->nombre,t1->nombre);
  (*nuevo)->info_tipo->sig=NULL;
  if (nuevoNodoNombre(&((*nuevo)->info_tipo->sig),merr)==-1)
    {
      elimNodoTipo(nuevo);
      return -1;
    }
  strcpy((*nuevo)->info_tipo->sig->nombre,t2->nombre);
  (*nuevo)->info_tipo->sig->sig=NULL;
  (*nuevo)->parametros=NULL;

  /* No hay iniciales ni constructoras */
  (*nuevo)->iniciales=NULL;
  (*nuevo)->constructoras=NULL;

  /* Las iniciales de t1 seran ahora selectoras con los nombres cambiados 
     y el codominio t2 */
  if (generadorasRep(*nuevo,t1->iniciales,t1->nombre,t2->nombre,merr)!=0)
    {
      elimNodoTipo(nuevo);
      return -1;
    }

  /* Las constructoras tambien seran selectoras con los nombres cambiados y
     cambios en el domino (donde este t1 estara t2)*/
  if (generadorasRep(*nuevo,t1->constructoras,t1->nombre,t2->nombre,merr)!=0)
    {
      elimNodoTipo(nuevo);
      return -1;
    }

  /* Se copian las selectoras cambiando el nombre, el dominio y el codominio
     si es necesario */
  if (selectorasRep(*nuevo,t1->selectoras,t1->nombre,t2->nombre,merr)!=0)
    {
      elimNodoTipo(nuevo);
      return -1;
    }
  
  (*nuevo)->status = 0;
  return 0;
}


/**
 * Crea la definicion parcial de una representacion del TAD t1 en el TAD t2
 * @param nom Nombre del nuevo tipo
 * @param t1 TAD que se representa
 * @param t2 TAD con el que se representa
 * @param fp Archivo abierto para escritura donde dejara un TAD por completar
 * @param nuevo Apuntara al nuevo TAD
 * @param merr Buffer para mensajes de error
 **/
int nuevaRep(char *nom,nodo_tad *t1, nodo_tad *t2,
	     FILE *fp,nodo_tad **nuevo,char *merr)
{
  nodo_select *sel;
  texpr li_ax;
  
  assert(nom!=NULL);
  assert(t1!=NULL);
  assert(t2!=NULL);
  assert(fp!=NULL);
  assert(nuevo!=NULL);
  assert(merr!=NULL);

  if (creaTADRep(nom,t1,t2,nuevo,merr)!=0)
    { return -1; }
  tad_ed=*nuevo;
  /* Ya copiamos el TAD t1 haciendo los cambios apropiados
     ahora escribimos la signatura */
  ed_arch_signatura(fp,*nuevo);
  /* Ahora escribimos lo que se puede de los axiomas */
  fprintf(fp,"\n%s\n\n",claves[4]);
  for (sel=(*nuevo)->selectoras;sel!=NULL;sel=sel->sig)
    {
      sprintf (li_ax, "%s", sel->nombre);
      if (sel->dominio!=NULL)
	{
	  strcat(li_ax,"(");
	}
      genere_axiomas (fp, sel->dominio, li_ax, '1', sel->axiomas);
    }
  fprintf(fp,"\n");

  return 0;
} /* nuevaRep */


/**
 * Cambia toda ocurrencia de una selectora fuente 
 * por la asociada en nsel
 * @param expr Expresion donde se hara el cambio
 * @param nsel Selectoras con las que se remplazarn las antiguas
 * @param oldtype nombre del tipo anterior
 * @param newtype nombre del nuevo tipo
 **/
void cambiaSelectorasExpresion(nodo_expresion *expr,nodo_select *nsel,
			       char *oldtype, char *newtype)
{
  nodo_select *sf;
  nodo_argum *arg;
  assert(expr!=NULL);

  if ((sf=buscaFuenteLSelectora(nsel,expr->nombre))!=NULL)
    {
      strcpy(expr->nombre,sf->nombre);
      strcpy(expr->tipo,sf->codominio);
    }
  if (strcmp(expr->tipo,oldtype)==0)
    { strcpy(expr->tipo,newtype); }
  for (arg=expr->argumentos;arg!=NULL;arg=arg->sig)
    {  cambiaSelectorasExpresion(arg->expr,nsel,oldtype,newtype);   }
}



/**
 * Analiza cambios a una representacion (despues de intervencion del
 * usuario) y la actualiza con tales cambios
 * @param rep Representacion original
 * @param fp Archivo abierto para lectura con cambios del usuario
 * @param merr Buffer para mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int cambiosRep(nodo_tad **rep,FILE *fp,char *merr)
{
  nodo_tad *nrep; /* Representacion leida del archivo */
  nodo_select *sel,*sf;
  nodo_tad *t1,*t2;
  nodo_generadora *gen;
  texpr nomfun,mens_error;
  
  assert(rep!=NULL);
  assert(fp!=NULL);
  assert(merr!=NULL);
  
  /* Verificamos que lo que recibe sea una representacion */
  if (!esRepresentacion(*rep,merr))
    { return -1; }
  /* Los tipos de la representacion existen (lo verifica esRepresentacion) */
  t1=buscaNombreLTipo(tad,(*rep)->info_tipo->nombre);
  t2=buscaNombreLTipo(tad,(*rep)->info_tipo->sig->nombre);
  assert(t1!=NULL);
  assert(t2!=NULL);
  
  fseek(fp,0,SEEK_SET);
  nrep=NULL;
  if (cargaTipoTexto(fp, &nrep,merr)!=0)
    {
      return -1;
    }
  /* El nombre de la nueva representacion debe coincidir con la anterior */
  if (strcmp(nrep->nombre,(*rep)->nombre)!=0)
    {
      sprintf(mens_error,mensajes[198],(*rep)->nombre);
      strcat(merr,mens_error);
      strcat(merr,"\n");
      elimNodoTipo(&nrep);
      return -1;
    }
  
  /* Lo que esta en el archivo debe ser una extension (sin generadoras, 
     ni parametros) */
  if (!esTipoExtension(nrep,merr))
    {
      elimNodoTipo(&nrep);
      return -1;
    }
  
  /* Todas las generadoras heredadas deben estar como selectoras
     en el nuevo tipo, ademas cambiamos el origen de cada una */
  for (gen=t1->iniciales;gen!=NULL;gen=gen->sig)
    {
      sprintf(nomfun,"%s_%s",(*rep)->nombre,gen->nombre);
      if ((sf=buscaNombreLSelectora(nrep->selectoras,nomfun))==NULL)
	{
	  sprintf(mens_error,mensajes[197],nomfun,gen->nombre);
	  strcat(merr,mens_error);
	  strcat(merr,"\n");
	  elimNodoTipo(&nrep);
	  return -1;
	}
      strcpy(sf->nombre_fte,gen->nombre);
    }
  for (gen=t1->constructoras;gen!=NULL;gen=gen->sig)
    {
      sprintf(nomfun,"%s_%s",(*rep)->nombre,gen->nombre);
      if ((sf=buscaNombreLSelectora(nrep->selectoras,nomfun))==NULL)
	{
	  sprintf(mens_error,mensajes[197],nomfun,gen->nombre);
	  strcat(merr,mens_error);
	  strcat(merr,"\n");
	  elimNodoTipo(&nrep);
	  return -1;
	}
      strcpy(sf->nombre_fte,gen->nombre);
    }
  /* Todas las selectoras heredadas deben estar en el nuevo tipo */
  for (sel=t1->selectoras;sel!=NULL;sel=sel->sig)
    {
      sprintf(nomfun,"%s_%s",(*rep)->nombre,sel->nombre);
      if ((sf=buscaNombreLSelectora(nrep->selectoras,nomfun))==NULL)
	{
	  sprintf(mens_error,mensajes[197],nomfun,sel->nombre);
	  strcat(merr,mens_error);
	  strcat(merr,"\n");
	  elimNodoTipo(&nrep);
	  return -1;
	}
      strcpy(sf->nombre_fte,sel->nombre);
    }
  /* Ahora cambiamos la antigua representacion por la nueva */
  nrep->status=(*rep)->status;
  nrep->tipo=(*rep)->tipo;
  nrep->modif=1;
  nrep->info_tipo=(*rep)->info_tipo;
  nrep->sig=(*rep)->sig;
  (*rep)->info_tipo=NULL;
  elimNodoTipo(rep);
  *rep=nrep;
  
  return 0;
}


/**
 * Crea una nueva representacion por composicion (r2 o r1)
 * @param nom Nombre de la nueva representacion
 * @param r1 Representacion por componer
 * @param r2 Representacion por componer
 * @param nuevo Apuntara a la nueva representacion (localizada por esta fun)
 * @param merr Buffer para mensajes de error
 * @return 0 si y solo no hay problemas
 **/
int componeRep(char *nom,nodo_tad *r1, nodo_tad *r2,
	       nodo_tad **nuevo,char *merr)
{
  texpr mens_err;
  nodo_select *tsel,*sel,
    *nomsel; /* Para facilitar cambio de nombres  */
  char *r1cod,*r2cod;
  
  /* Verificamos que sean representaciones los tipos que recibe */
  if (!esRepresentacion(r1,merr) || !esRepresentacion(r2,merr))
    { return -1; }
  /* Verificamos que los dominios y codominios funcionen:
     r1: A -> B,    r2: B -> C,    r2 o r1: A -> C */
  r1cod=r1->info_tipo->sig->nombre;
  r2cod=r2->info_tipo->sig->nombre;
  if (strcmp(r1cod,r2->info_tipo->nombre)!=0)
    {
      sprintf(mens_err,mensajes[201],r1->nombre,r2->nombre);
      strcat(merr,mens_err);
      strcat(merr,"\n");
      return -1;
    }
  if (copiaNodoTipo(nuevo,r1,merr)!=0)
    { return -1; }
  strcpy((*nuevo)->nombre,nom);
  (*nuevo)->status=1;
  (*nuevo)->modif=1;
  strcpy((*nuevo)->info_tipo->sig->nombre,r2cod);
  (*nuevo)->sig=NULL;
  
  /* Unas selectoras temporales para facilitar el cambio de nombres */
  nomsel=NULL;
  if (copiaLSelectora(&nomsel,r1->selectoras,merr)!=0)
    {
      elimNodoTipo(nuevo);
      return -1;
    }
  /* Cambiamos los nombres de las selectoras en la nueva representacion y
     en la temporal*/
  for (tsel=nomsel,sel=(*nuevo)->selectoras;
       sel!=NULL;
       sel=sel->sig,tsel=tsel->sig)
    {
      assert(tsel!=NULL);
      sprintf(tsel->nombre,"%s_%s",nom,sel->nombre_fte);
      sprintf(tsel->nombre_fte,"%s",sel->nombre);
      sprintf(sel->nombre,"%s_%s",nom,sel->nombre_fte);
      reemplazaLDominio(tsel->dominio,r1cod,r2cod);
      reemplazaLDominio(sel->dominio,r1cod,r2cod);
      if (strcmp(tsel->codominio,r1cod)==0)
	{ strcpy(tsel->codominio,r2cod); }
      if (strcmp(sel->codominio,r1cod)==0)
	{ strcpy(sel->codominio,r2cod); }      
    }

  /* Cambiamos en los axiomas las ocurrencias de nombres de selectoras
     de r1 por los nombres equivalentes en la nueva representacion */
  for (sel=(*nuevo)->selectoras;
       sel!=NULL;
       sel=sel->sig)
    {
      cambiaSelectorasExpresion(sel->axiomas->lado_izq,nomsel,r1cod,r2cod);
      construya_exp(sel->axiomas->lado_izq,sel->axiomas->li_str,"");      
      cambiaSelectorasExpresion(sel->axiomas->lado_der,nomsel,r1cod,r2cod);
      construya_exp(sel->axiomas->lado_izq,sel->axiomas->li_str,"");
    }  
      
  /* Cambiamos en los axiomas las ocurrencias de nombres de selectoras
     del codominio de r1 por los nombres equivalentes de acuerdo a r2 */
  for (sel=(*nuevo)->selectoras;
       sel!=NULL;
       sel=sel->sig)
    {
      cambiaSelectorasExpresion(sel->axiomas->lado_izq,r2->selectoras,
				r1cod,r2cod);
      construya_exp(sel->axiomas->lado_izq,sel->axiomas->li_str,"");      
      /* Normalizamos y dejamos la normalizada */
      if (normalizaExpresion(&(sel->axiomas->lado_izq),merr)!=0)
	{ return -1; }
      construya_exp(sel->axiomas->lado_izq,sel->axiomas->li_str,"");
      cambiaSelectorasExpresion(sel->axiomas->lado_der,r2->selectoras,
				r1cod,r2cod);
      construya_exp(sel->axiomas->lado_der,sel->axiomas->ld_str,"");
      /* Normalizamos y dejamos la normalizada */
      if (normalizaExpresion(&(sel->axiomas->lado_der),merr)!=0)
	{ return -1; }
      construya_exp(sel->axiomas->lado_der,sel->axiomas->ld_str,"");
    }  

  elimLSelectora(&nomsel); /* Liberamos lista temporal */
  return 0;
}



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Entrada
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Libera una entrada
 * @param n cabeza de la lista 
 **/
void elimNodoEntrada(nodo_entrada **ne)
{
  assert(ne!=NULL);
  if (*ne==NULL)
    {return;}
  (*ne)->selector[0]='\0';
  elimNodoExpresion(&((*ne)->item));
  free(*ne);
  *ne=NULL;
}

/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Tabla
QQQQQQQQQQQQQQQQQQQQQQQQQ@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Libera una Tabla
 * @param n cabeza de la lista 
 **/
void elimLTabla(Ptr_tabla *t)
{
  assert(t!=NULL);
  if (*t==NULL)
    {return;}
  
  elimNodoEntrada(&((*t)->entrada));
  elimLTabla(&((*t)->sig));
  free(*t);
  *t=NULL;
}



/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   Tad Motor de Reescritura
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 * Normaliza la expresin que recibe
 * @param expr Expresion que se normaliza
 * @param merr Buffer para mensajes de error
 * @return 
 **/
int normalizaExpresion(nodo_expresion **expr,char *merr)
{
  int rc;
  nodo_demost *dem;
  dem = crear_demost ();
  fnlo (*expr,&(dem->his_li), &(dem->ult_li));
  elimNodoExpresion(expr);
  if ((rc=copiaNodoExpresion(expr,dem->ult_li->expr,merr))!=0)
    { return rc; }
  liberar_demost(&dem);
  return 0;
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   TAD Teoremas
@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */


/**
 *  Crea las conjeturas necesarias para probar una representacion
 * @param fp Archivo abierto donde dejara las conjeturas
 * @param rep Representacion por probar
 * @param merr Buffer con mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int conjeturasRep(FILE *fp,nodo_tad *rep,char *merr)
{
  nodo_select *sel, *sf,*nsel,**asf,*sfr;
  nodo_tad *t1; /* Tipo que se esta representando */
  texpr mens_error;
  nodo_igualdad **pax,*ax;
  
  assert(fp!=NULL);
  assert(rep!=NULL);
  assert(rep->tipo==REPRESENTACION);
  assert(rep->info_tipo!=NULL);
  assert(rep->info_tipo->sig!=NULL);
  assert(merr!=NULL);
  
  /* No puede ser un TAD aberrado */
  if (tipoMalo(rep,merr))
    { return -1; }
  
  /* Cargamos los tipos de la representacion en caso de que no esten */
  if ((t1=buscaNombreLTipo(tad,rep->info_tipo->nombre))==NULL)
    {
      busque_arch_tad(rep->info_tipo->nombre);
      if ((t1=buscaNombreLTipo(tad,rep->info_tipo->nombre))==NULL)
	{
	  sprintf(mens_error,mensajes[194],rep->info_tipo->nombre);
	  strcat(merr,mens_error);
	  strcat(merr,"\n");
	  return -1;
	}
    }
  if (buscaNombreLTipo(tad,rep->info_tipo->sig->nombre)==NULL)
    {
      busque_arch_tad(rep->info_tipo->sig->nombre);
      if (buscaNombreLTipo(tad,rep->info_tipo->sig->nombre)==NULL)
	{
	  sprintf(mens_error,mensajes[194],rep->info_tipo->sig->nombre);
	  strcat(merr,mens_error);
	  strcat(merr,"\n");
	  return -1;
	}
    }
  nsel=NULL;
  asf=&nsel;
  /* Copiamos las selectoras con axiomas haciendo los cambios apropiados */
  for (sel=t1->selectoras;sel!=NULL;sel=sel->sig)
    {
      if ((sfr=buscaFuenteLSelectora(rep->selectoras,sel->nombre))==NULL)
	{
	  sprintf(mens_error,mensajes[195],sel->nombre);
	  strcat(merr,mens_error);
	  strcat(merr,"\n");
	  elimLSelectora(&nsel);
	  return -1;
	}
      if (copiaLSelectora(&sf,sfr,merr)!=0)
	{
	  elimLSelectora(&nsel);
	}
      *asf=sf;
      asf=&(sf->sig);
      /* Copia el conjunto de axiomas pero haciendo los cambios apropiados */
      for (pax=&sf->axiomas,ax=sel->axiomas;
	   ax!=NULL;
	   ax=ax->sig,pax=&((*pax)->sig))
	{
	  if (copiaLIgualdad(pax,ax,merr)!=0)
	    {
	      elimLSelectora(&nsel);
	      return -1;
	    }
	  cambiaSelectorasExpresion((*pax)->lado_izq,rep->selectoras,
				    "","");
	  construya_exp((*pax)->lado_izq,(*pax)->li_str,"");
	  /* Las representaciones de BOOL en otros tipos requieren
	     que el codominio de la representacion de  EQ_BOOLL siga
	     teniendo como codominio BOOL (no el tipo representado)
	     y el axioma por probar debe tener valores de BOOL
	     al lado derecho */
	  if (strcmp(sel->nombre,"EQ_BOOL")!=0)
	    {
	      cambiaSelectorasExpresion((*pax)->lado_der,rep->selectoras,
					"","");
	      construya_exp((*pax)->lado_der,(*pax)->ld_str,"");
	      (*pax)->sig=NULL;
	    }
	}
    }
  
  /* Escribimos los axiomas en el archivo */
  ed_arch_axiomas_ops(fp,nsel);
  /* Eliminamos la memoria empleada */
  elimLSelectora(&nsel);
  return 0;
}


/**
 * Intenta probar las conjeturas que hay en un archivo
 * @param fp     Archivo con conjeturas
 * @param nprob  Numero de conjeturas probadas
 * @param prob   Apuntador a las conjeturas probadas
 * @param nnoprob Numero de conjeturas no probadas
 * @param noprob Apuntador a las conjeturas no probadas
 * @param merr   Buffer para mensajes de error
 * @return Codigo de error o 0 si no hay problemas
 **/
int probarConjeturas(FILE *fp,int *nprob,nodo_demost **prob,
                     int *nnoprob,nodo_demost **noprob,char *merr)
{
  nodo_igualdad *conj;
  
  assert(nprob!=NULL);
  assert(nnoprob!=NULL);
  assert(prob!=NULL);
  assert(noprob!=NULL);
  assert(merr!=NULL);
  
  *nprob=*nnoprob=0;
  *prob=*noprob=NULL;
  
  conj = analice_conjeturas(fp,merr);
  if (conj==NULL)
    { return -1; }
  /* Liberamos las pruebas que haya en memoria */
  if (proved!=NULL)
    {
      liberar_demost(&proved);
      proved=NULL;
    }
  if (unproved!=NULL)
    {
      liberar_demost(&unproved);
      unproved=NULL;
    }
  
  probar_conjeturas(nprob,nnoprob,&conj);
  if (*nprob>0 && copiar_demost(prob,proved,merr)!=0)
    {
      elimLIgualdad(&conj);
      return -1;
    }
  if (*nnoprob>0 && copiar_demost(noprob,unproved,merr))
    {
      elimLIgualdad(&conj);
      return -1;
    }
  elimLIgualdad(&conj);
  return 0;
}


/**
 * En un archivo escribe demostraciones de teoremas probados
 * @param fp     Archivo en el cual escribe
 * @param prob   Apuntador a los teoremas probados
 * @return Codigo de error o 0 si no hay problemas
 **/
int escribirPrueba(FILE *fp,nodo_demost *prob)
{
  assert(fp!=NULL);
  assert(prob!=NULL);
  escriba_arch(fp,prob,1);
  return 0;
}


/**
 * En un archivo escribe el intento de prueba de una conjetura
 * @param fp     Archivo en el cual escribe
 * @param noprob Apuntador a las conjeturas que no logro probar
 * @return Codigo de error o 0 si no hay problemas
 **/
int escribirIntentoPrueba(FILE *fp,nodo_demost *noprob)
{
  assert(fp!=NULL);
  assert(noprob!=NULL);
  escriba_arch(fp,noprob,0);
  return 0;
}


/* @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   Utilidades
   @@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@ */

/**
 * Dice si el nombre que recibe es sintacticamente valido para un TAD 
 * o nombre de funcion
 * @param nom Nombre por revisar
 * @return Verdadero si y solo es un nombre valido
 **/
int esNombreValido(char *nom)
{
  int i,l;
  assert(nom!=NULL);
  if ((l=strlen(nom))==0)
    {
      return 0;
    }
  for (i=0;i<l;i++)
    {
      if (!isalpha(nom[i]))
          return 0;
    }
  return 1;
}


/**
 * Compara dos cadenas sin tener en cuenta capitalizacion.
 * Este es un reemplaza para strcasecmp que no es ANSI
 * @param a Cadena  por comparar
 * @param b Cadena  por comparar
 * @return Un positivo si a>b, un negativoo si a<b y 0 si son iguales
 **/
int strcmp_nocase(char *a,char *b) {
  assert(a!=NULL);
  assert(b!=NULL);
  
  while (*a!='\0' && *b!='\0' && tolower(*a)==tolower(*b)) {
    a++;
    b++;
  }
  return (*a-*b);
}

/** 
 * Retorna una copia de la cadena que recibe.
 * Este es un remplazo para strdup que no es ANSI
 * @param c Cadena por duplicar
 * @return Un apuntador a un duplicado o NULL si no logra localizar memoria
 **/
char *duplicastr(char *c) 
{
  char *d=(char *)malloc(sizeof(char)*(strlen(c)+1));
  if (d!=NULL) 
    {
      strcpy(d,c);
      d[strlen(c)]='\0';
    }
  return d;
}

