  /* PROTOTIPOS.C	        						*
*  Este modulo se encarga de generar los siguientes archivos :          *
* <nombre_tad>_de.h  : Contiene las definiciones,macros y tipos       *
*                        del tad.                                       *
* <nombre_tad>.c       : Contiene el codigo de las funciones            *
* <nombre_tad>_co.inc: Contiene el codigo de las constructoras        *
* <nombre_tad>.h       : Contiene los prototipos de las funciones       *
*                        selectoras del tad.                            *
* <nombre_tad>_in.c : Contiene rutinas de interfaz del tad           *
* <nombre_tad>_in.h : Contiene los encabezados de las funciones de   *
*                        interfaz.                                      *
* <nobre_tad>_ma.c   : Contiene el programa principal del tad         *
* <nombre_tad>.mak    : Contiene un archivo de entrada al makefile     *
* Estos archivos se crean de acuerdo a los archivos de plantillas y     *
* se colocan en el directorio de la aplicacion.                         */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include "definic.h"
#include "apimanta.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* j-tamara 29/May/1997 */

/************************************************************************
* void copiar_plantilla_encab(FILE *fplant,FILE *fdef)                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario del encabezado del        *
* del archivo de definiciones (fdef), segun el archivo de plantilla     *
* fplant, para el tad t.                                                *
* DESCRIPCION : Lee el archivo de plantillas fplant y llena el          *
* archivo fdef con la informacion necesaria.                            *
* PRE : El archivo de plantilla tiene dos lineas especiales.            *
* La primera tiene dos %s para colocar el nombre del tad                *
* La segunda tiene un formato para colocar fechas.                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void copiar_plantilla_encab(FILE *fplant,FILE *fdef, nodo_tad *t)
{/*--------------------------------------------------------------------*/
    texpr linea1,linea2;
    int   l;

    assert(fplant!=NULL);               /* j-tamara 29/May/1997 */
    assert(fdef!=NULL);                 /* j-tamara 29/May/1997 */
    assert(t!=NULL);                    /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,fdef);
    fgets(linea1,sizeof(texpr),fplant);
    formatear_linea(linea1,linea2);
    l = strlen(linea2);
    sprintf(linea1,linea2,t->nombre,t->nombre);
    linea1[l-1] = linea2[l-1];
    linea1[l] = '\n';
    linea1[l+1] = '\0';
    fprintf(fdef,linea1);
    copiar_plantilla(fplant,fdef);
    linea_fecha(fplant,fdef);
    copiar_plantilla(fplant,fdef);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void copiar_plantilla(FILE *fplant,FILE *fdef)                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Copia la siguiente parte de la plantilla fplant en fdef   *
* DESCRIPCION : Lee el archivo de plantillas fplant y llena el          *
* archivo fdef con la informacion necesaria.                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void copiar_plantilla(FILE *fplant,FILE *fdef)
{/*--------------------------------------------------------------------*/

    texpr           linea1,linea2;
    int             continuar;

    assert(fplant!=NULL);                 /* j-tamara 29/May/1997 */
    assert(fdef!=NULL);
    fgets(linea1,sizeof(texpr),fplant);
    continuar = !feof(fplant);
    while (continuar) {
        formatear_linea(linea1,linea2);
        if (!strcmp(linea2,LIN_ESPERAR))
            continuar = 0;
          else {
            fprintf(fdef,"%s\n",linea2);
            fgets(linea1,sizeof(texpr),fplant);
            continuar = !feof(fplant);
	  }
    }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_defs_defines(FILE *fplant,FILE *fdef, nodo_tad *t)         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea las definiciones del archivo fdef,      *
* segun el archivo fplant, para el tad t.                               *
* DESCRIPCION : Lee el archivo de plantillas fplant y llena el          *
* archivo fdef con la informacion necesaria.                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_defs_defines(FILE *fplant,FILE *fdef, nodo_tad *t)
{/*--------------------------------------------------------------------*/
    nodo_generadora *gen;
    int             num;

    assert(fdef!=NULL);         /* j-tamara 29/May/1997 */
    assert(t!=NULL);            /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,fdef);
    for (num =0,gen = t -> iniciales; gen != NULL;
         gen = gen->sig,num++) {
        fprintf(fdef,"#define OP_%s  \t%d\n",gen->nombre,num);
    }
/*-- Se ha hecho un define por cada inicial --*/
    for (gen = t -> constructoras; gen != NULL;
         gen = gen->sig,num++) {
         fprintf(fdef,"#define OP_%s  \t%d\n",gen->nombre,num);
    }
/*-- Se ha hecho un define por cada constructora --*/
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_defs_typedef(FILE *fplant,FILE *fdef, nodo_tad *t,         *
*                         int autom)                                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea los typedef del archivo fdef, segun el  *
* archivo fplant, para el tad t.                                        *
* DESCRIPCION : Lee el archivo de plantillas fplant y llena el          *
* archivo fdef con la informacion necesaria.                            *
* POS : Si auto = 1, se genera el codigo automaticamente. Si no,        *
* solmente el esqueleto.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_defs_typedef(FILE *fplant,FILE *fdef, nodo_tad *t,
                        int autom)
{/*--------------------------------------------------------------------*/
    nodo_generadora *gen;
    nodo_nombre     *nom;
    int             num_nom;

    assert(t!=NULL);                    /* j-tamara 29/May/1997 */
    assert(fdef!=NULL);                 /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,fdef);
    for (nom=t->parametros;nom!=NULL;nom=nom->sig) {
        fprintf(fdef,"typedef void *%s;\n\n",nom->nombre);
    }
    fprintf(fdef,"typedef struct %s{\n",t->nombre);
    if (autom)  {
        fprintf(fdef,"\tint\t\tmanta_uso;\n");
        fprintf(fdef,"\tint\t\tmanta_op;\n");
        for(gen=t->iniciales;gen!=NULL;gen=gen->sig) {
            for(num_nom=0,nom=gen->dominio;nom!=NULL;nom= nom->sig,num_nom++)
                if (!strcmp(nom->nombre,"NAT"))
                    fprintf(fdef,"\tint\t\t%s%d;\n",gen->nombre,num_nom);
                else if (!strcmp(nom->nombre,"BOOL"))
                    fprintf(fdef,"\t%s\t\t%s%d;\n",
			    nom->nombre,gen->nombre,num_nom);
                else
                    fprintf(fdef,"\tstruct %s\t*%s%d;\n",
                            nom->nombre,gen->nombre,num_nom);
        }
        for(gen=t->constructoras;gen!=NULL;gen=gen->sig) {
            for(num_nom=0,nom=gen->dominio;nom!=NULL;nom= nom->sig,num_nom++)
                if (!strcmp(nom->nombre,"NAT"))
                    fprintf(fdef,"\tint\t\t%s%d;\n",gen->nombre,num_nom);
                else if (!strcmp(nom->nombre,"BOOL"))
                    fprintf(fdef,"\t%s\t\t%s%d;\n",
                            nom->nombre,gen->nombre,num_nom);
                else
                    fprintf(fdef,"\tstruct %s\t*%s%d;\n",
                            nom->nombre,gen->nombre,num_nom);
        }
    }
      else {
        fprintf(fdef,"\n#error %s\n\n",mensajes[125]);
      }
    fprintf(fdef,"} *%s;\n",t->nombre);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_defs_macros(FILE *fplant,FILE *fdef, nodo_tad *t,          *
*                        int autom)                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea macros del archivo fdef para el tad t.  *
* Se crean las siguiente macros :                                       *
* - Macros para decidir la operacion de construccion de una instancia   *
* - Macros que retornan las proyecciones de las constructoras.          *
* Son macros que deciden la operacion con la cual fue construido una    *
* instancia.                                                            *
* DESCRIPCION : Lee el archivo de plantillas fplant y llena el          *
* archivo fdef con la informacion necesaria.                            *
* POS : Si autom = 1, se genera el codigo automaticamente. Si no,       *
* solmente el esqueleto.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_defs_macros(FILE *fplant,FILE *fdef, nodo_tad *t,
                       int autom)
{/*--------------------------------------------------------------------*/
    nodo_generadora *gen;
    nodo_nombre     *nom;
    int             num_nom;

    assert(t!=NULL);            /* j-tamara 29/May/1997 */
    assert(fdef!=NULL);         /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,fdef);
    for(gen=t->iniciales;gen!=NULL;gen=gen->sig)
        if (autom)
            fprintf(fdef,"#define is_%s(OBJ)\t\t(OBJ->manta_op==OP_%s)\n",
                    gen->nombre,gen->nombre);
        else
            fprintf(fdef,"#define is_%s(OBJ)\n#error %s\n\n",gen->nombre,
                         mensajes[125]);
    for(gen=t->constructoras;gen!=NULL;gen=gen->sig)
        if (autom)
            fprintf(fdef,"#define is_%s(OBJ)\t\t(OBJ->manta_op==OP_%s)\n",
                    gen->nombre,gen->nombre);
        else
            fprintf(fdef,"#define is_%s(OBJ)\n#error %s\n\n",
                    gen->nombre,mensajes[125]);
    copiar_plantilla(fplant,fdef);
    for(gen=t->iniciales;gen!=NULL;gen=gen->sig)
        for(num_nom=0,nom=gen->dominio;nom!=NULL;nom= nom->sig,num_nom++)
            if (autom)
                fprintf(fdef,"#define pr%d%s(OBJ)\t\t(OBJ->%s%d)\n",
                        num_nom,gen->nombre,gen->nombre,num_nom);
            else
                fprintf(fdef,"#define pr%d%s(OBJ)\n#error %s\n\n",
                        num_nom,gen->nombre,mensajes[125]);
    for(gen=t->constructoras;gen!=NULL;gen=gen->sig)
        for(num_nom=0,nom=gen->dominio;nom!=NULL;nom= nom->sig,num_nom++)
            if (autom)
                fprintf(fdef,"#define pr%d%s(OBJ)\t\t(OBJ->%s%d)\n",
                        num_nom,gen->nombre,gen->nombre,num_nom);
            else
                fprintf(fdef,"#define pr%d%s(OBJ)\n#error %s\n\n",
                        num_nom,gen->nombre,mensajes[125]);

/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_func_includes(FILE *fplant,FILE *ffunc,                    *
*                          nodo_tad *t, int tip)                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea la secciones de include del             *
* archivo ffunc,segun el archivo fplant, para el tad t.                 *
* Si tip=0, las inclusiones son de los archivos de los TAD parametros   *
* Si tip=1, las inclusiones son de los archivos de interfaz de los TAD  *
* DESCRIPCION : Lee el archivo de plantillas fplant y llena el          *
* archivo ffunc con la informacion necesaria.                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_func_includes(FILE *fplant,FILE *ffunc,
                         nodo_tad *t, int tip)
{/*--------------------------------------------------------------------*/
    nodo_nombre *lista_nombres=NULL;
    nodo_nombre *p;
    char merr[1000];

    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,ffunc);
    merr[0]='\0';
    if (related_types_list(t,&lista_nombres,merr)!=0)
      { escriba_cadena(merr,2); }
    insertar_nombre_ord(&lista_nombres,t->nombre);
    for(p = lista_nombres;p != NULL; p=p->sig) 
      {
        if (!tipo_primitivo(p->nombre))
	  {
            if (tip == 0)
	      fprintf(ffunc,"#include \"%s_de.h\"\n",p->nombre);
            else
	      fprintf(ffunc,"#include \"%s_in.h\"\n",p->nombre);
	  }
      }
    if (tip == 0) {
        copiar_plantilla(fplant,ffunc);
        for(p = lista_nombres; p!= NULL; p=p->sig)
           if (!tipo_primitivo(p->nombre))
               fprintf(ffunc,"#include \"%s.h\"\n",p->nombre);
    }
    liberar_nombres(&lista_nombres);
/*--------------------------------------------------------------------*/}

/************************************************************************
 void crear_includes_header(FILE *ffunc, nodo_tad *t)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
  Crea seccion de includes en un encabezado
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
  AUTOR : Vladimir Tamara Patino	FECHA : 4-Oct-1998             
************************************************************************/

void crear_includes_header(FILE *ffunc, nodo_tad *t)
{/*--------------------------------------------------------------------*/
    nodo_nombre *lista_nombres=NULL;
    nodo_nombre *p;
    char merr[1000];

    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    merr[0]='\0';
    if (related_types_list(t,&lista_nombres,merr)!=0)
      { escriba_cadena(merr,2); }
    insertar_nombre_ord(&lista_nombres,t->nombre);
    fprintf(ffunc,"#include \"manta.h\"\n"); 

    for(p = lista_nombres;p != NULL; p=p->sig) 
      {
        if (!tipo_primitivo(p->nombre) && strcmp(p->nombre,t->nombre)!=0)
	  { fprintf(ffunc,"#include \"%s_de.h\"\n",p->nombre); }
      }
    liberar_nombres(&lista_nombres);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_defs(nodo_tad *t, int autom)                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo del archivo de definiciones   *
* del tad t.                                                            *
* POST : Si autom = 1, el codigo se crea automaticamente. Si no, solo   *
*        se crea el esqueleto del programa.                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_defs(nodo_tad *t, int autom)
{/*--------------------------------------------------------------------*/
    FILE     *fdef;
    FILE     *fplant;
    cad20    nombre_arch;

    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[17]);
    fplant = fopen(nombre_arch,"r");
    if (fplant == NULL)     /* verifica que la "plantilla de defs" existe */
        return;
    sprintf(nombre_arch,"%s/%s_de.h",direct[7],t->nombre);
    fdef=fopen(nombre_arch,"w");
    FileOpen(fdef,nombre_arch);         /* j-tamara 29/May/1997 */
    copiar_plantilla_encab(fplant,fdef,t);
    lineas_def(fplant,fdef,t->nombre);
    copiar_plantilla(fplant,fdef);
    crear_includes_header(fdef, t);
    /* VTP. 6/Oct/1998.  Mejores inclusiones en encabezados */
    copiar_plantilla(fplant,fdef);
    crear_defs_defines(fplant,fdef,t);
    crear_defs_typedef(fplant,fdef,t,autom);
    crear_defs_macros(fplant,fdef,t,autom);
    copiar_plantilla(fplant,fdef);
    fclose(fdef);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_main(nodo_tad *t)                                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo del archivo principal que     *
* permite probar el tad.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_main(nodo_tad *t)
{/*--------------------------------------------------------------------*/
    FILE     *fmain;
    FILE     *fplant;
    cad20    nombre_arch;

    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[20]);
    fplant = fopen(nombre_arch,"r");
    if (fplant == NULL)    /* verifica que la plantilla de ppal existe */
        return;
    sprintf(nombre_arch,"%s/%s_ma.c",direct[7],t->nombre);
    fmain=fopen(nombre_arch,"w");
    FileOpen(fmain,nombre_arch);        /* j-tamara 29/May/1997 */
    copiar_plantilla_encab(fplant,fmain,t);
    crear_func_includes(fplant,fmain,t,0);
    crear_func_includes(fplant,fmain,t,1);
    copiar_plantilla(fplant,fmain);
    crear_switch_main(fmain,t);
    copiar_plantilla(fplant,fmain);
    fclose(fmain);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_switch_main(FILE *fmain,nodo_tad *t)                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo del switch del main del       *
* programa principal, correspondiente al tad t.                         *
* PRE : t = es un tad, fmain es un archivo vacio                        *
* POST: fmain es un archivo con la funcion main definida.               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Ene-1992	*
************************************************************************/

void crear_switch_main(FILE *fmain,nodo_tad *t)
{/*--------------------------------------------------------------------*/
    nodo_nombre *lista_nom,*p;
    int         sangria;
    int         opcion_exit;
    int         opcion;
    int         espacios;

    assert(fmain!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    espacios = 1;
    sangria = 2;
    indente(fmain,sangria);
    fprintf(fmain,"int  opcion;\n");
    indente(fmain,sangria);
    fprintf(fmain,"manta_menu m;\n");
    lista_nom = lista_tipos(t);
    indente(fmain,sangria);
    fprintf(fmain,"manta_menu menu_%s;\n",t->nombre);
    for(p = lista_nom; p != NULL; p = p->sig)
        if (!tipo_primitivo(p->nombre)) {
            indente(fmain,sangria);
            fprintf(fmain,"manta_menu menu_%s;\n",p->nombre);
        }
/*-- Se han creado las definiciones de las varaiable de los menus --*/
    fprintf(fmain,"\n");
    indente(fmain,sangria);
    fprintf(fmain,"adicionar_funcion(\"%s\",(ptr)&(imprimir_obj_%s),\n",
	    t->nombre,t->nombre);
    indente(fmain,sangria);
    fprintf(fmain," (ptr)&(salvar_obj_%s), (ptr)&(cargar_obj_%s),\n",
	    t->nombre,t->nombre);
    indente(fmain,sangria);
    fprintf(fmain," (ptr)&(copiar_obj_%s), (ptr)&(borrar_obj_%s));\n",
	    t->nombre,t->nombre);
    for(p = lista_nom; p != NULL; p = p->sig)
        if (!tipo_primitivo(p->nombre)) {
	  indente(fmain,sangria);
	  fprintf(fmain,"adicionar_funcion(\"%s\",(ptr)&(imprimir_obj_%s),\n",
		  p->nombre,p->nombre);
	  indente(fmain,sangria);
	  fprintf(fmain," (ptr)&(salvar_obj_%s), (ptr)&(cargar_obj_%s),\n",
		  p->nombre,p->nombre);
	  indente(fmain,sangria);
	  fprintf(fmain," (ptr)&(copiar_obj_%s), (ptr)&(borrar_obj_%s));\n",
		  p->nombre,p->nombre);
	}
    indente(fmain,sangria);
    fprintf(fmain,"m = manta_crear_menu();\n");
    indente(fmain,sangria);
    fprintf(fmain,"manta_adic_opcion(m,manta_crear_opcion(OBJS));\n");
    indente(fmain,sangria);
    fprintf(fmain,
            "manta_adic_opcion(m,manta_crear_opcion(\"%s\"));\n",
	    t->nombre);
    for(opcion_exit = 1,p = lista_nom; p != NULL; p = p->sig)
        if (!tipo_primitivo(p->nombre)) {
            indente(fmain,sangria);
            fprintf(fmain,
             "manta_adic_opcion(m,manta_crear_opcion(\"%s\"));\n",
                                 p->nombre);
            opcion_exit++;
        }
    opcion_exit++;
    indente(fmain,sangria);
    fprintf(fmain,
                 "manta_adic_opcion(m,manta_crear_opcion(EXIT));\n");
/*-- Se ha creado el menu principal y se le adicionaron las opciones. --*/
    indente(fmain,sangria);
    fprintf(fmain,
       "crear_menu_interfaces();\n");
    indente(fmain,sangria);
    fprintf(fmain,"crear_menu_ambiente();\n");
    indente(fmain,sangria);
    fprintf(fmain,"menu_%s = crear_menu_%s();\n",t->nombre,t->nombre);
    for(p = lista_nom; p != NULL; p = p->sig)
        if (!tipo_primitivo(p->nombre)) {
            indente(fmain,sangria);
            fprintf(fmain,
                 "menu_%s = crear_menu_%s();\n",p->nombre,p->nombre);
        }
/*-- Se han creado los menus correspondientes a las opciones. --*/
    indente(fmain,sangria);
    fprintf(fmain,"listar_objs();\n");
    indente(fmain,sangria);
    fprintf(fmain,"opcion = manta_admon_menu(m);\n");
    indente(fmain,sangria++);
    fprintf(fmain,"while (opcion != %d) {\n",opcion_exit);
    indente(fmain,sangria++);
    fprintf(fmain,"switch (opcion) {\n");
    opcion = 0;
    indente(fmain,sangria);
    fprintf(fmain,"case %d  :\n",opcion);
    indente(fmain,sangria+espacios);
    fprintf(fmain,"admon_menu_ambiente();\n");
    indente(fmain,sangria+espacios);
    fprintf(fmain,"break;\n");
    opcion++;
    indente(fmain,sangria);
    fprintf(fmain,"case %d  :\n",opcion);
    indente(fmain,sangria+espacios);
    fprintf(fmain,"admon_menu_%s(menu_%s);\n",t->nombre,t->nombre);
    indente(fmain,sangria+espacios);
    fprintf(fmain,"break;\n");
    opcion++;
    for(p = lista_nom; p != NULL; p = p->sig)
        if (!tipo_primitivo(p->nombre)) {
            indente(fmain,sangria);
            fprintf(fmain,"case %d  :\n",opcion);
            indente(fmain,sangria+espacios);
            fprintf(fmain,"admon_menu_%s(menu_%s);\n",p->nombre,p->nombre);
            indente(fmain,sangria+espacios);
            fprintf(fmain,"break;\n");
            opcion++;
        }
    indente(fmain,sangria);
    fprintf(fmain,"case NO_OPCION :\n");
    indente(fmain,sangria+espacios);
    fprintf(fmain,"break;\n");
    indente(fmain,sangria);
    fprintf(fmain,"default :\n");
    indente(fmain,sangria+espacios);
    fprintf(fmain,"manta_escriba_cadena(MEN22,1);\n");
    indente(fmain,sangria+espacios);
    fprintf(fmain,"break;\n");
    sangria--;
    indente(fmain,sangria);
    fprintf(fmain,"}\n");
    indente(fmain,sangria);
    fprintf(fmain,"opcion = manta_admon_menu(m);\n");
    sangria--;
    indente(fmain,sangria);
    fprintf(fmain,"}\n");

/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_funciones(nodo_tad *t, int cons, int sel,                  *
*                      nodo_nombre *nom)                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo de la funciones selectoras    *
* y constructoraas del tad t.                                           *
* PRE : t = es un tad, ffunc es un archivo vacio, fplant es una         *
* plantilla de funciones.                                               *
* POST: ffunc es un archivo que implanta las funciones selectoras del   *
* tad t. fenc  es un archivo con los encabezados de las funciones.      *
* fgen es un archivo con las generadoras del tad.                       *
* OJO : Si cons == 0. No se generan las constructoras.                  *
*       Si cons == 1. Se generan automaticamente los codigos de las     *
*                     constructoras.                                    *
*       Si cons == 2. Se generan los esqueletos de las constructoras.   *
*       Si sel == 0.  No se generan las selectoras.                     *
*       Si sel == 1.                                                    *
*          Si nom == NULL: Se generan automaticamente los codigos de    *
*                     todas las selectoras.                             *
*          Si no:     Se generan los codigos de las selectoras que      *
*                     aparecen en nom, de las otras solo se generan     *
*                     los esqueletos.                                   *
*       Si sel == 2.                                                    *
*          Si nom == NULL: Se generan los esqueletos de todas las       *
*                     selectoras.                                       *
*          Si no:     Se generan los esqueletos de las selectoras que   *
*                     aparecen en nom, las otras se generan             *
*                     automaticamente.                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_funciones(nodo_tad *t, int cons, int sel)
{/*--------------------------------------------------------------------*/
    FILE            *ffunc = NULL;
    FILE            *fplant;
    FILE            *fplantenc;
    nodo_generadora *gen;
    nodo_select *sele;
    FILE            *fgen = NULL;
    cad20           nombre_arch;
    int             autom;

     /* Alvaro */

    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    if (sel != 0) {
        sprintf(nombre_arch,"%s/%s.c",direct[7],t->nombre);
        ffunc=fopen(nombre_arch,"w");
        FileOpen(ffunc,nombre_arch);    /* j-tamara 29/May/1997 */
        sprintf(nombre_arch,"%s/%s",direct[1],direct[13]);
        fplant = fopen(nombre_arch,"r");
        if (fplant == NULL)        /* verifica que la plantilla de selectoras
				      existe */
            return;
        copiar_plantilla_encab(fplant,ffunc,t);
        crear_func_includes(fplant,ffunc,t,0);
        copiar_plantilla(fplant,ffunc);
        fprintf(ffunc,"#include \"%s_co.inc\"\n",t->nombre);
        copiar_plantilla(fplant,ffunc);
        fclose(fplant);
/*-- Se ha creado el archivo <nombre_tad>.c con el comentario inicial --*/
    }
    if (cons != 0) {
        sprintf(nombre_arch,"%s/%s_co.inc",direct[7],t->nombre);
        fgen=fopen(nombre_arch,"w");
        FileOpen(fgen,nombre_arch);     /* j-tamara 29/May/1997 */
        sprintf(nombre_arch,"%s/%s",direct[1],direct[16]);
        fplant = fopen(nombre_arch,"r");
        if (fplant == NULL)     /*verifica que la plantilla de constructoras
				  existe */
            return;
        copiar_plantilla_encab(fplant,fgen,t);
        fclose(fplant);
/*-- Se ha creado el archivo <nombre_tad>_co.inc
     con el comentario inicial --*/
    }
    sprintf(nombre_arch,"%s/%s",direct[1],direct[12]);
    if (cons != 0) {
        if (cons == 1)
            autom = 1;
        else
            autom = 0;
        for (gen = t->iniciales;gen != NULL;gen=gen->sig) {
            fplantenc = fopen(nombre_arch,"r");
            FileOpen(fplantenc,nombre_arch);    /* j-tamara 29/May/1997 */
            crear_generadora(fplantenc,fgen,gen,t,autom);
            fclose(fplantenc);
        }
        for (gen = t->constructoras;gen != NULL;gen=gen->sig) {
            fplantenc = fopen(nombre_arch,"r");
            FileOpen(fplantenc,nombre_arch);    /* j-tamara 29/May/1997 */
            crear_generadora(fplantenc,fgen,gen,t,autom);
            fclose(fplantenc);
        }
    }
/*-- Se ha creado el archivo <nombre_tad>_co.inc
     con el codigo de las constructoras --*/
    if (sel != 0) {
        if (sel == 1)
            autom = 1;
        else
            autom = 0;
        for (sele = t->selectoras;sele != NULL; sele = sele->sig) {
            fplantenc = fopen(nombre_arch,"r");
            FileOpen(fplantenc,nombre_arch);    /* j-tamara 29/May/1997 */
            crear_selectora(fplantenc,ffunc,t,sele,autom);
            fclose(fplantenc);
        }
        crear_f_copiar_obj(ffunc,t,autom);
	crear_f_borrar_obj(ffunc,t,autom);
	/* VTP. 6/Oct/1998 The function copiar and borrar are now
	   generated in the tad.c and tad.h */
    }
/*-- Se ha creado el archivo <nombre_tad>.c
     con el codigo de las selectoras y funciones para copiar y borrar --*/
    if (sel !=0)
        fclose(ffunc);
    if (cons != 0)
        fclose(fgen);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_headers(nodo_tad *t)                                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion genera el archivo de encabezados del tad t   *
************************************************************************/

void crear_headers(nodo_tad *t)
{/*--------------------------------------------------------------------*/
    texpr           nombre_arch;
    FILE            *fenc;
    FILE            *fplant;
    nodo_generadora *gen;
    nodo_select     *sel;

    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[14]);
    fplant = fopen(nombre_arch,"r");
    if (fplant == NULL)           /* verifica que el arch. 'tad.h' existe */
        return;
    sprintf(nombre_arch,"%s/%s.h",direct[7],t->nombre);
    fenc=fopen(nombre_arch,"w");
    FileOpen(fenc,nombre_arch);         /* j-tamara 29/May/1997 */
    copiar_plantilla_encab(fplant,fenc,t);

    /* Agregadas para mejorar inclusion de archivos en sistemas grandes */
    /* VTP. 4/Oct/1998 */
    lineas_def(fplant,fenc,t->nombre);
    copiar_plantilla(fplant,fenc);
    fprintf(fenc,"#include \"%s_de.h\"\n",t->nombre);
    copiar_plantilla(fplant,fenc);

/*-- Se ha creado el archivo <nombre_tad>.h con el comentario inicial --*/

    for(gen = t->iniciales;gen!=NULL;gen=gen->sig) {
        fprintf(fenc,"extern \n");
        encabezado_gen(fenc,gen,t);
        fprintf(fenc,";\n\n");
    }
    for(gen = t->constructoras;gen!=NULL;gen=gen->sig){
        fprintf(fenc,"extern \n");
        encabezado_gen(fenc,gen,t);
        fprintf(fenc,";\n\n");
    }
    for(sel = t->selectoras;sel!=NULL;sel=sel->sig) {
        fprintf(fenc,"extern \n");
        encabezado_sel(fenc,sel);
        fprintf(fenc,";\n\n");
    }
    fprintf(fenc,"\nextern\n%s\tcopiar_obj_%s(%s o);\n\n",
	    t->nombre,t->nombre,t->nombre);
    fprintf(fenc,"extern\nvoid\tborrar_obj_%s(%s *o);\n\n",
	    t->nombre,t->nombre);
    /* VTP. 6/Oct/1998 The function copiar and borrar are now
       generated in the tad.c and tad.h */
    copiar_plantilla(fplant,fenc);
    fclose(fenc);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_interfaz(nodo_tad *t, int autom)                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo del archivo con las funciones *
* de interfaz del tad t.                                                *
* POS : Si autom = 1, se genera el codigo automaticamente. Si no,       *
* solmente el esqueleto.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_interfaz(nodo_tad *t, int autom)
{/*--------------------------------------------------------------------*/
    FILE     *finter;
    FILE     *fplant;
    cad20    nombre_arch;
    int      op_exit;

    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[19]);
    fplant = fopen(nombre_arch,"r");
    if (fplant == NULL)    /* verifica que la plantilla de encabezados de
			      interfaz existe */
        return;
    sprintf(nombre_arch,"%s/%s_in.h",direct[7],t->nombre);
    finter=fopen(nombre_arch,"w");
    FileOpen(finter,nombre_arch);       /* j-tamara 29/May/1997 */
    copiar_plantilla_encab(fplant,finter,t);
    lineas_def(fplant,finter,t->nombre);
    copiar_plantilla(fplant,finter);
   
    fprintf(finter,"extern\nmanta_menu\t");
    fprintf(finter,"crear_menu_%s();\n\n",t->nombre);
    fprintf(finter,"extern\nvoid\tadmon_menu_%s(manta_menu m);\n\n",t->nombre);
    fprintf(finter,"extern\nvoid\timprimir_obj_%s(%s o);\n\n",
                    t->nombre,t->nombre);
    fprintf(finter,"extern\nvoid\tsalvar_obj_%s(FILE *f,%s o);\n\n",
                    t->nombre,t->nombre);
    fprintf(finter,"extern\n%s\tcargar_obj_%s(FILE *f);\n\n",
                    t->nombre,t->nombre);

    /*    fprintf(finter,"extern\n%s\tcopiar_obj_%s(%s o);\n\n",
                    t->nombre,t->nombre,t->nombre);
    fprintf(finter,"extern\nvoid\tborrar_obj_%s(%s *o);\n\n",
    t->nombre,t->nombre); */
    /* VTP. 6/Oct/1998 The function copiar and borrar are now
       generated in the tad.c and tad.h */

    copiar_plantilla(fplant,finter);
    fclose(fplant);
    fclose(finter);
    sprintf(nombre_arch,"%s/%s",direct[1],direct[18]);
    fplant = fopen(nombre_arch,"r");
    if (fplant == NULL)
        return;
    sprintf(nombre_arch,"%s/%s_in.c",direct[7],t->nombre);
    finter=fopen(nombre_arch,"w");
    FileOpen(finter,nombre_arch);       /* j-tamara 29/May/1997 */
    copiar_plantilla_encab(fplant,finter,t);
    crear_func_includes(fplant,finter,t,0);
    crear_func_includes(fplant,finter,t,1);
    copiar_plantilla(fplant,finter);
    crear_f_crear_menu(finter,t,&op_exit);
    crear_f_admon_menu(finter,t,op_exit);
    crear_f_imprimir_obj(finter,t,autom);
    crear_f_salvar_obj(finter,t,autom);
    crear_f_cargar_obj(finter,t,autom);
    /*    crear_f_copiar_obj(finter,t,autom);
	  crear_f_borrar_obj(finter,t,autom); */
    /* VTP. 6/Oct/1998 The function copiar and borrar are now
       generated in the tad.c and tad.h */

    fclose(fplant);
    fclose(finter);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_f_crear_menu(FILE *finter,nodo_tad *t, int *op_exit)       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario y el codigo de la         *
* funcion de interfaz crear_menu.                                       *
* POST : *op_exit = ultima opcion del menu que se ha creado             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Ene-1992	*
************************************************************************/

void crear_f_crear_menu(FILE *finter,nodo_tad *t, int *op_exit)
{/*--------------------------------------------------------------------*/
    cad20           nombre_arch;
    FILE            *fplant;
    int             sangria;
    nodo_generadora *gen;
    nodo_select     *sel;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[12]);
    fplant = fopen(nombre_arch,"r");
    FileOpen(fplant,nombre_arch);       /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,finter);
    fprintf(finter,"manta_menu\t");
    fprintf(finter,"crear_menu_%s()\n",t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Colocar pre --*/
    copiar_plantilla(fplant,finter);
/*-- Colocar post --*/
    copiar_plantilla(fplant,finter);
    linea_fecha(fplant,finter);
    copiar_plantilla(fplant,finter);
    fprintf(finter,"manta_menu\t");
    fprintf(finter,"crear_menu_%s()\n",t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Se ha colocado la plantilla de la funcion --*/
    sangria = 2;
    indente(finter,sangria);
    fprintf(finter,"manta_menu  m;\n\n");
    indente(finter,sangria);
    fprintf(finter,"m = manta_crear_menu();\n");
    for((*op_exit) = 0,gen = t->iniciales; gen != NULL;
        gen = gen->sig,(*op_exit)++) {
        indente(finter,sangria);
        fprintf(finter,
                "manta_adic_opcion(m,manta_crear_opcion(\"%s\"));\n",gen->nombre);
    }
    for(gen = t->constructoras; gen != NULL;
        gen = gen->sig,(*op_exit)++) {
        indente(finter,sangria);
        fprintf(finter,
                "manta_adic_opcion(m,manta_crear_opcion(\"%s\"));\n",gen->nombre);
    }
    for(sel = t->selectoras; sel != NULL;
        sel = sel->sig,(*op_exit)++) {
        indente(finter,sangria);
        fprintf(finter,
                "manta_adic_opcion(m,manta_crear_opcion(\"%s\"));\n",sel->nombre);
    }
    (*op_exit)++;
    indente(finter,sangria);
    fprintf(finter,"manta_adic_opcion(m,manta_crear_opcion(INTER));\n");
    indente(finter,sangria);
    fprintf(finter,"manta_adic_opcion(m,manta_crear_opcion(EXIT));\n");
    indente(finter,sangria);
    fprintf(finter,"return(m);\n");
    copiar_plantilla(fplant,finter);
    fclose(fplant);

/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_f_admon_menu(FILE *finter,nodo_tad *t, int op_exit)        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario y el codigo de la         *
* funcion de interfaz admon_menu.                                       *
* PRE : op_exit= ultima opcion del menu, sobre el cual se crea el       *
*       administrador.                                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_f_admon_menu(FILE *finter,nodo_tad *t, int op_exit)
{/*--------------------------------------------------------------------*/
    cad20           nombre_arch;
    FILE            *fplant;
    int             sangria;
    nodo_generadora *gen;
    nodo_select     *sel;
    int             opcion;
    int             espacios;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    espacios = 1;
    sprintf(nombre_arch,"%s/%s",direct[1],direct[12]);
    fplant = fopen(nombre_arch,"r");
    FileOpen(fplant,nombre_arch);       /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,finter);
    fprintf(finter,"void\tadmon_menu_%s(manta_menu m)\n",t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Colocar pre --*/
    copiar_plantilla(fplant,finter);
/*-- Colocar post --*/
    copiar_plantilla(fplant,finter);
    linea_fecha(fplant,finter);
    copiar_plantilla(fplant,finter);
    fprintf(finter,"void\tadmon_menu_%s(manta_menu m)\n",t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Listo para empezar a generar codigo --*/
    sangria = 2;
    indente(finter,sangria);
    fprintf(finter,"objetos  obj;\n");
    indente(finter,sangria);
    fprintf(finter,"int  opcion;\n\n");
    indente(finter,sangria);
    fprintf(finter,"opcion = manta_admon_menu(m);\n");
    indente(finter,sangria++);
    fprintf(finter,"while (opcion != %d) {\n",op_exit);
    indente(finter,sangria);
    fprintf(finter,"tad_error = 0;\n");
    indente(finter,sangria++);
    fprintf(finter,"switch (opcion) {\n");
    for( opcion = 0,gen = t->iniciales; gen != NULL;
         gen = gen->sig, opcion++)
         crear_case_gen(finter,sangria,opcion,gen,t);
    for( gen = t->constructoras; gen != NULL;
         gen = gen->sig, opcion++)
         crear_case_gen(finter,sangria,opcion,gen,t);
    for(sel = t->selectoras; sel != NULL;
        sel = sel -> sig, opcion++)
         crear_case_sel(finter,sangria,opcion,sel);
    indente(finter,sangria);
    fprintf(finter,"case %d  :\n",opcion++);
    indente(finter,sangria+espacios);
    fprintf(finter,"admon_menu_interfaces(\"%s\");\n",t->nombre);
    indente(finter,sangria+espacios);
    fprintf(finter,"break;\n");
    indente(finter,sangria);
    fprintf(finter,"case NO_OPCION :\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"return;\n");
    indente(finter,sangria);
    fprintf(finter,"default :\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"manta_escriba_cadena(MEN22,1);\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"break;\n");
    sangria--;
    indente(finter,sangria);
    fprintf(finter,"}\n");
    indente(finter,sangria);
    fprintf(finter,"opcion = manta_admon_menu(m);\n");
    sangria--;
    indente(finter,sangria);
    fprintf(finter,"}\n");
    copiar_plantilla(fplant,finter);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_gen_imprimir(FILE *finter,int sangria, int es_else,        *
*                       nodo_generadora *gen)                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea codigo para imprimir la generadora gen, *
*             en el archivo finter.                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_gen_imprimir(FILE *finter,int sangria, int es_else,
			/*nodo_tad *t, */nodo_generadora *gen) /* 3.1b2 31/May/1997*/
{/*--------------------------------------------------------------------*/
    nodo_nombre     *n;
    int             num;

  /* Alvaro */
            assert(finter!=NULL);              /* j-tamara 29/May/1997 */
            assert(gen!=NULL);              /* j-tamara 29/May/1997 */
            indente(finter,sangria);
            if (es_else)
                fprintf(finter,"else if ");
            else {
                fprintf(finter,"if ");
	    }
            fprintf(finter,"( is_%s(o) ) {\n",gen->nombre);
            sangria++;
            indente(finter,sangria);
            fprintf(finter,"manta_escriba_cadena(\"%s(\",0);\n",gen->nombre);
            for(n=gen->dominio,num=0;n != NULL; n= n->sig,num++) {
                indente(finter,sangria);
                if (tipo_primitivo(n->nombre)) {
                    fprintf(finter,"prim = pr%d%s(o);\n",num,gen->nombre);
                    indente(finter,sangria);
                    fprintf(finter,"imprimir_obj_prim( &prim,\"%s\" );\n",
                            n->nombre);
		} else
                    fprintf(finter,"imprimir_obj_%s( pr%d%s(o) );\n",
                                   n->nombre,num,gen->nombre);
                if (n->sig != NULL) {
                    indente(finter,sangria);
                    fprintf(finter,"manta_escriba_cadena(\", \",0);\n");
                }
	    }
            indente(finter,sangria);
            fprintf(finter,"manta_escriba_cadena(\")\",0);\n");
            sangria--;
            indente(finter,sangria);
            fprintf(finter,"}\n");
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_f_imprimir_obj(FILE *finter,nodo_tad *t, int autom)        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario y el codigo de la         *
* funcion de interfaz imprimir_obj.                                     *
* POS : Si autom = 1, se genera el codigo automaticamente. Si no,       *
* solmente el esqueleto.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_f_imprimir_obj(FILE *finter,nodo_tad *t, int autom)
{/*--------------------------------------------------------------------*/
    cad20           nombre_arch;
    FILE            *fplant;
    int             sangria;
    nodo_generadora *gen;
    int             es_else,hay_prim;
    nodo_nombre    *n;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[12]);
    fplant = fopen(nombre_arch,"r");
    FileOpen(fplant,nombre_arch);       /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,finter);
    fprintf(finter,"void\timprimir_obj_%s(%s o)\n",t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Colocar pre --*/
    copiar_plantilla(fplant,finter);
/*-- Colocar post --*/
    copiar_plantilla(fplant,finter);
    linea_fecha(fplant,finter);
    copiar_plantilla(fplant,finter);
    fprintf(finter,"void\timprimir_obj_%s(%s o)\n",t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Listo para empezar a generar codigo --*/
    if (autom) {
        es_else = 0;
        sangria = 2;
	indente(finter,sangria);

	for (hay_prim=0,gen=t->iniciales;gen!=NULL;gen=gen->sig)
	  for(n=gen->dominio;n != NULL; n= n->sig) 
	    if (tipo_primitivo(n->nombre)) 
	      {  hay_prim++; }
	for (gen=t->constructoras;gen!=NULL;gen=gen->sig)
	  for(n=gen->dominio;n != NULL; n= n->sig) 
	    if (tipo_primitivo(n->nombre)) 
	      {  hay_prim++; }
	if (hay_prim)
	  { fprintf(finter,"int prim=0;\n\n"); }
        /* VTP. 6/OCT/98.  Codigo agreagado para evitar generacion
           de codigo inutil */
        for (gen = t->iniciales; gen != NULL; gen = gen->sig) {
	    crear_gen_imprimir(finter,sangria,es_else,gen);
            es_else = 1;
        }
    /*-- Se genero codigo para las iniciales --*/
        for (gen = t->constructoras; gen != NULL; gen = gen->sig)
	    crear_gen_imprimir(finter,sangria,es_else,gen);
    /*-- Se genero codigo para las constructoras --*/
     } else
        fprintf(finter,"\n#error %s\n\n",mensajes[125]);     /* !!!!! */
/*-- Final del codigo --*/
    copiar_plantilla(fplant,finter);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_gen_salvar(FILE *finter,int sangria, int es_else,          *
*                       nodo_tad *t, nodo_generadora *gen)              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea codigo para salvar la generadora gen,   *
*             en el archivo finter.                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_gen_salvar(FILE *finter,int sangria, int es_else,
                      /*nodo_tad *t,*/ nodo_generadora *gen) /*3.1b2 31/May/1997 */
{/*--------------------------------------------------------------------*/
    nodo_nombre     *n;
    int             num;

 /* Alvaro */

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(gen!=NULL);              /* j-tamara 29/May/1997 */
    indente(finter,sangria);
    if (es_else)
        fprintf(finter,"else if ");
    else
        fprintf(finter,"if ");
    fprintf(finter,"( is_%s(o) ) {\n",gen->nombre);
    sangria++;
    indente(finter,sangria);
    fprintf(finter,"fprintf(f,\"%%d \",OP_%s);\n",gen->nombre);
    for(n=gen->dominio,num=0;n != NULL; n= n->sig,num++) {
	indente(finter,sangria);
	if (tipo_primitivo(n->nombre))
	    fprintf(finter,"fprintf(f,\"%%d \",pr%d%s(o));\n",
		    num,gen->nombre);
	else
	    fprintf(finter,"salvar_obj_%s(f, pr%d%s(o) );\n",
		    n->nombre,num,gen->nombre);
    }
    sangria--;
    indente(finter,sangria);
    fprintf(finter,"}\n");
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_f_salvar_obj(FILE *finter,nodo_tad *t, int autom)          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario y el codigo de la         *
* funcion de interfaz salvar_obj.                                       *
* POS : Si autom = 1, se genera el codigo automaticamente. Si no,       *
* solmente el esqueleto.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_f_salvar_obj(FILE *finter,nodo_tad *t, int autom)
{/*--------------------------------------------------------------------*/
    cad20           nombre_arch;
    FILE            *fplant;
    int             sangria;
    nodo_generadora *gen;
    int             es_else;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[12]);
    fplant = fopen(nombre_arch,"r");
    FileOpen(fplant,nombre_arch);       /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,finter);
    fprintf(finter,"void\tsalvar_obj_%s(FILE *f, %s o)\n",
	           t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Colocar pre --*/
    copiar_plantilla(fplant,finter);
/*-- Colocar post --*/
    copiar_plantilla(fplant,finter);
    linea_fecha(fplant,finter);
    copiar_plantilla(fplant,finter);
    fprintf(finter,"void\tsalvar_obj_%s(FILE *f, %s o)\n",
	            t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Listo para empezar a generar codigo --*/
    if (autom) {
        es_else = 0;
        sangria = 2;
        for (gen = t->iniciales; gen != NULL; gen = gen->sig) {
            crear_gen_salvar(finter,sangria,es_else,gen);
            es_else = 1;
	}
    /*-- Se genero codigo para las iniciales --*/
        for (gen = t->constructoras; gen != NULL; gen = gen->sig)
            crear_gen_salvar(finter,sangria,es_else,gen);
    /*-- Se genero codigo para las constructoras --*/
    } else
        fprintf(finter,"\n#error %s\n\n",mensajes[125]);
/*-- Final del codigo --*/
    copiar_plantilla(fplant,finter);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_gen_cargar(FILE *finter,int sangria, int es_else,          *
*                       nodo_tad *t, nodo_generadora *gen)              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea codigo para cargar la generadora gen,   *
*             en el archivo finter.                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_gen_cargar(FILE *finter,int sangria, int es_else,
                      nodo_tad *t, nodo_generadora *gen)
{/*--------------------------------------------------------------------*/
    nodo_nombre     *n;
    int             num;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    assert(gen!=NULL);              /* j-tamara 29/May/1997 */
    indente(finter,sangria);
    if (es_else)
        fprintf(finter,"else if ");
    else
        fprintf(finter,"if ");
    fprintf(finter,"( tipo_op == OP_%s ) {\n",gen->nombre);
    sangria++;
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++) {
        indente(finter,sangria);
        if (!tipo_primitivo(n->nombre))
            fprintf(finter,"%s\t%c%d;\n",n->nombre,n->nombre[0],num);
        else if (!strcmp(n->nombre,"NAT"))
            fprintf(finter,"int\t%c%d;\n",n->nombre[0],num);
        else if (!strcmp(n->nombre,"BOOL"))
            fprintf(finter,"BOOL\t%c%d;\n",n->nombre[0],num);
    }
    fprintf(finter,"\n");
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++) {
        indente(finter,sangria);
        if (!tipo_primitivo(n->nombre))
            fprintf(finter,"%c%d = cargar_obj_%s(f);\n",
                    n->nombre[0],num,n->nombre);
        else
            fprintf(finter,"fscanf(f,\"%%d \",&%c%d);\n",n->nombre[0],num);
    }
    indente(finter,sangria);
    fprintf(finter,"%c_obj = %s(",t->nombre[0],gen->nombre);
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++)
        if (num == 0)
            fprintf(finter,"%c%d",n->nombre[0],num);
        else
            fprintf(finter,", %c%d",n->nombre[0],num);
    fprintf(finter,");\n");
    sangria--;
    indente(finter,sangria);
    fprintf(finter,"}\n");
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_f_cargar_obj(FILE *finter,nodo_tad *t, int autom)          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario y el codigo de la         *
* funcion de interfaz cargar_obj.                                       *
* POS : Si autom = 1, se genera el codigo automaticamente. Si no,       *
* solmente el esqueleto.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_f_cargar_obj(FILE *finter,nodo_tad *t, int autom)
{/*--------------------------------------------------------------------*/
    cad20           nombre_arch;
    FILE            *fplant;
    int             sangria;
    nodo_generadora *gen;
    int             es_else;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[12]);
    fplant = fopen(nombre_arch,"r");
    FileOpen(fplant,nombre_arch);       /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,finter);
    fprintf(finter,"%s\tcargar_obj_%s(FILE *f)\n",
                    t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Colocar pre --*/
    copiar_plantilla(fplant,finter);
/*-- Colocar post --*/
    copiar_plantilla(fplant,finter);
    linea_fecha(fplant,finter);
    copiar_plantilla(fplant,finter);
    fprintf(finter,"%s\tcargar_obj_%s(FILE *f)\n",
                    t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Listo para empezar a generar codigo --*/
    if (autom) {
        es_else = 0;
        sangria = 2;
        indente(finter,sangria);
        fprintf(finter,"int\ttipo_op;\n");
        indente(finter,sangria);
        fprintf(finter,"%s\t%c_obj;\n\n",t->nombre,t->nombre[0]);
        indente(finter,sangria);
        fprintf(finter,"fscanf(f,\"%%d \",&tipo_op);\n");
        for (gen = t->iniciales; gen != NULL; gen = gen->sig) {
            crear_gen_cargar(finter,sangria,es_else,t,gen);
            es_else = 1;
	}
    /*-- Se genero codigo para las iniciales --*/
        for (gen = t->constructoras; gen != NULL; gen = gen->sig)
            crear_gen_cargar(finter,sangria,es_else,t,gen);
    /*-- Se genero codigo para las constructoras --*/
        indente(finter,sangria);
        fprintf(finter,"return ( %c_obj );\n",t->nombre[0]);
    } else
        fprintf(finter,"\n#error %s\n\n",mensajes[125]);
/*-- Final del codigo --*/
    copiar_plantilla(fplant,finter);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_gen_copiar(FILE *finter,int sangria, int es_else,          *
*                       nodo_tad *t, nodo_generadora *gen)              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea codigo para copiar la generadora gen,   *
*             en el archivo finter.                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_gen_copiar(FILE *finter,int sangria, int es_else,
                      nodo_tad *t, nodo_generadora *gen)
{/*--------------------------------------------------------------------*/
    nodo_nombre     *n;
    int             num;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(gen!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    indente(finter,sangria);
    if (es_else)
        fprintf(finter,"else if ");
    else
        fprintf(finter,"if ");
    fprintf(finter,"( is_%s(o) ) {\n",gen->nombre);
    sangria++;
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++) {
        indente(finter,sangria);
        if (!tipo_primitivo(n->nombre))
            fprintf(finter,"%s\t%c%d;\n",n->nombre,n->nombre[0],num);
        else if (!strcmp(n->nombre,"NAT"))
            fprintf(finter,"int\t%c%d;\n",n->nombre[0],num);
        else if (!strcmp(n->nombre,"BOOL"))
            fprintf(finter,"BOOL\t%c%d;\n",n->nombre[0],num);
    }
    fprintf(finter,"\n");
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++) {
        indente(finter,sangria);
        if (!tipo_primitivo(n->nombre))
            fprintf(finter,"%c%d = copiar_obj_%s(pr%d%s(o));\n",
                    n->nombre[0],num,n->nombre,num,gen->nombre);
        else
            fprintf(finter,"%c%d = pr%d%s(o);\n",n->nombre[0],
		    num,num,gen->nombre);
    }
    indente(finter,sangria);
    fprintf(finter,"%c_obj = %s(",t->nombre[0],gen->nombre);
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++)
        if (num == 0)
            fprintf(finter,"%c%d",n->nombre[0],num);
        else
            fprintf(finter,", %c%d",n->nombre[0],num);
    fprintf(finter,");\n");
    sangria--;
    indente(finter,sangria);
    fprintf(finter,"}\n");
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_f_copiar_obj(FILE *finter,nodo_tad *t, int autom)          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario y el codigo de la         *
* funcion de interfaz copiar_obj.                                       *
* POS : Si autom = 1, se genera el codigo automaticamente. Si no,       *
* solmente el esqueleto.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_f_copiar_obj(FILE *finter,nodo_tad *t, int autom)
{/*--------------------------------------------------------------------*/
    cad20           nombre_arch;
    FILE            *fplant;
    int             sangria;
    nodo_generadora *gen;
    int             es_else;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[12]);
    fplant = fopen(nombre_arch,"r");
    FileOpen(fplant,nombre_arch);       /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,finter);
    fprintf(finter,"%s\tcopiar_obj_%s(%s o)\n",
                    t->nombre,t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Colocar pre --*/
    copiar_plantilla(fplant,finter);
/*-- Colocar post --*/
    copiar_plantilla(fplant,finter);
    linea_fecha(fplant,finter);
    copiar_plantilla(fplant,finter);
    fprintf(finter,"%s\tcopiar_obj_%s(%s o)\n",
                    t->nombre,t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Listo para empezar a generar codigo --*/
    if (autom) {
        es_else = 0;
        sangria = 2;
        indente(finter,sangria);
        fprintf(finter,"%s\t%c_obj;\n\n",t->nombre,t->nombre[0]);
        for (gen = t->iniciales; gen != NULL; gen = gen->sig) {
            crear_gen_copiar(finter,sangria,es_else,t,gen);
            es_else = 1;
	}
    /*-- Se genero codigo para las iniciales --*/
        for (gen = t->constructoras; gen != NULL; gen = gen->sig)
            crear_gen_copiar(finter,sangria,es_else,t,gen);
    /*-- Se genero codigo para las constructoras --*/
        indente(finter,sangria);
        fprintf(finter,"return ( %c_obj );\n",t->nombre[0]);
    } else
        fprintf(finter,"\n#error %s\n\n",mensajes[125]);
/*-- Final del codigo --*/
    copiar_plantilla(fplant,finter);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_gen_borrar(FILE *finter,int sangria, int es_else,          *
*                       nodo_tad *t, nodo_generadora *gen)              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea codigo para borrar la generadora gen,   *
*             en el archivo finter.                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_gen_borrar(FILE *finter,int sangria, int es_else,
                      /*nodo_tad *t,*/ nodo_generadora *gen)/* 3.1b2 31/May/1997 */
{/*--------------------------------------------------------------------*/
    nodo_nombre     *n;
    int             num;


  /* Alvaro */

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(gen!=NULL);              /* j-tamara 29/May/1997 */
    indente(finter,sangria);
    if (es_else)
        fprintf(finter,"else if ");
    else
        fprintf(finter,"if ");
    fprintf(finter,"( is_%s(obj) ) {\n",gen->nombre);
    sangria++;
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++)
        if (!tipo_primitivo(n->nombre)) {
            indente(finter,sangria);
            fprintf(finter,"%s\t%c%d;\n",n->nombre,n->nombre[0],num);
	}
    fprintf(finter,"\n");
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++) {
        if (!tipo_primitivo(n->nombre)) {
            indente(finter,sangria);
            fprintf(finter,"%c%d = pr%d%s(obj);\n",n->nombre[0],
		    num,num,gen->nombre);
            indente(finter,sangria);
            fprintf(finter,"borrar_obj_%s(&%c%d);\n",n->nombre,
		    n->nombre[0],num);
	}
    }
    indente(finter,sangria);
    fprintf(finter,"free(*o);\n");
    indente(finter,sangria);
    fprintf(finter,"(*o)=NULL;\n");
    sangria--;
    indente(finter,sangria);
    fprintf(finter,"}\n");
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_f_borrar_obj(FILE *finter,nodo_tad *t, int autom)          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario y el codigo de la         *
* funcion de interfaz borrar_obj.                                       *
* POS : Si autom = 1, se genera el codigo automaticamente. Si no,       *
* solmente el esqueleto.                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1992	*
************************************************************************/

void crear_f_borrar_obj(FILE *finter,nodo_tad *t, int autom)
{/*--------------------------------------------------------------------*/
    cad20           nombre_arch;
    FILE            *fplant;
    int             sangria;
    nodo_generadora *gen;
    int             es_else;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[12]);
    fplant = fopen(nombre_arch,"r");
    FileOpen(fplant,nombre_arch);       /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,finter);
    fprintf(finter,"void\tborrar_obj_%s(%s *o)\n",
                    t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Colocar pre --*/
    copiar_plantilla(fplant,finter);
/*-- Colocar post --*/
    copiar_plantilla(fplant,finter);
    linea_fecha(fplant,finter);
    copiar_plantilla(fplant,finter);
    fprintf(finter,"void\tborrar_obj_%s(%s *o)\n",
                    t->nombre,t->nombre);
    copiar_plantilla(fplant,finter);
/*-- Listo para empezar a generar codigo --*/
    if (autom) {
        es_else = 0;
        sangria = 2;
        indente(finter,sangria);
        fprintf(finter,"%s obj;\n\n",t->nombre);
        indente(finter,sangria);
        fprintf(finter,"obj = (*o);\n");
        indente(finter,sangria);
        fprintf(finter,"if (obj == NULL) return;\n");
        indente(finter,sangria);
        fprintf(finter,"if (uso_obj(obj) > 0) dec_uso(obj);\n");
        indente(finter,sangria);
        fprintf(finter,"if (uso_obj(obj) > 0) return;\n");
        for (gen = t->iniciales; gen != NULL; gen = gen->sig) {
            crear_gen_borrar(finter,sangria,es_else,gen);
            es_else = 1;
	}
    /*-- Se genero codigo para las iniciales --*/
        for (gen = t->constructoras; gen != NULL; gen = gen->sig)
            crear_gen_borrar(finter,sangria,es_else,gen);
    /*-- Se genero codigo para las constructoras --*/
    } else
        fprintf(finter,"\n#error %s\n\n",mensajes[125]);
/*-- Final del codigo --*/
    copiar_plantilla(fplant,finter);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_selectora(FILE *fplant,FILE *ffunc, nodo_tad *t,           *
*                      nodo_select *sel, int autom)                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario y el codigo de la         *
* selectora sel, y lo graba en el archivo ffunc (codigo).               *
* POST : Si autom = 1, el codigo se crar automaticamente. Si no, solo   *
*        se crea el esqueleto del programa.                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_selectora(FILE *fplant,FILE *ffunc, nodo_tad *t,
                     nodo_select *sel, int autom)
{/*--------------------------------------------------------------------*/
    nodo_igualdad *axio;

    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    assert(sel!=NULL);              /* j-tamara 29/May/1997 */
    selectora_comentario(fplant,ffunc,sel);
    encabezado_sel(ffunc,sel);
    fprintf(ffunc,"\n");
    copiar_plantilla(fplant,ffunc);
    if (autom)
      {
        for(axio = sel->axiomas;axio != NULL; axio = axio->sig)
	  codigo_axioma(ffunc,t,sel,axio);
	if (sel->dominio!=NULL)
          {
            if (strcmp(sel->codominio,"NAT")==0)
              { fprintf(ffunc,"\n  return 0;\n"); }
            else if (strcmp(sel->codominio,"BOOL")==0)
              { fprintf(ffunc,"\n  return FALSE;\n"); }
            else
              { fprintf(ffunc,"\n  return NULL;\n"); }
          }
      }
    else
        fprintf(ffunc,"\n#error %s\n\n",mensajes[125]);
    copiar_plantilla(fplant,ffunc);

/*--------------------------------------------------------------------*/}

/************************************************************************
* void codigo_axioma( FILE *ffunc, nodo_tad *t,nodo_select *sel,        *
*                     nodo_igualdad *axio )                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo del axioma axio de la         *
* selectora sel.                                                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void codigo_axioma(FILE *ffunc, nodo_tad *t, nodo_select *sel,
                   nodo_igualdad *axio)
{/*--------------------------------------------------------------------*/
  Ptr_tabla      sust;
  nodo_expresion *expr;
  short          aplico;
  cad20          tipo;
  int            sangria;
  
  assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
  assert(sel!=NULL);              /* j-tamara 29/May/1997 */
  sangria = 1;
  if ( axio == NULL ||
       axio->lado_izq == NULL ||
       axio->lado_der == NULL)
    return;
  sust = eps();
  sust_lado_izq(ffunc,axio->lado_izq,sust,sel->axiomas != axio,
		&sangria);
  expr = apl(axio->lado_der,sust,1,&aplico);
  strcpy(tipo,axio->lado_izq->tipo);
  if (!strcmp(tipo,"NAT"))
    strcpy(tipo,"int");
  if (sangria != 1)
    {
      /*-- Significa que en la parte izquierda hubo un if --*/
      fprintf(ffunc," ){\n");
    }
  codigo_lado_der(ffunc,t,sel,expr,tipo,sangria);
  if (sangria != 1)
    {
      /*-- Significa que en la parte izquierda hubo un que se debe cerrar --*/
      sangria--;
      indente(ffunc,sangria);
      fprintf(ffunc,"}\n");
    }
  elimLTabla(&sust);
  elimNodoExpresion(&expr);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void sust_lado_izq( FILE *ffunc,nodo_expresion *lado_izq,             *
*                     Ptr_tabla sust, int es_else, int *sangria)        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Sustitucion de los nombres de las variables de los        *
*             axiomas. Ademas, construye la guarda del if del lado      *
*             izquierdo en el archivo ffunc                             *
* PRE : sust = Sustitucion vacia                                        *
*       es_else = indica si el codigo debe empezar con else o no        *
* POST: sust = Una sustitucion de los nombres de las variables          *
*       ffunc = Codigo que evalua la parte izquierda del axioma         *
*       sangria = Valor de la sangria para escribir la parte derecha    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void sust_lado_izq( FILE *ffunc,nodo_expresion *lado_izq, Ptr_tabla sust,
                    int es_else, int *sangria)
{/*--------------------------------------------------------------------*/
    nodo_argum     *arg;
    nodo_expresion *expr;
    cad12          nombre;
    int            posi;

    assert(lado_izq!=NULL);              /* j-tamara 29/May/1997 */
    for(arg = lado_izq->argumentos,posi = 0;arg!=NULL;arg=arg->sig,posi++) {
        sprintf(nombre,"%c%d",arg->expr->tipo[0],posi);
        expr = crear_expresion(nombre,arg->expr->tipo,"VAR");
        sust_r_lado_izq(ffunc,arg->expr,expr,sust,es_else,sangria);
    }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void sust_r_lado_izq(FILE *ffunc,nodo_expresion *expo,                *
*                      nodo_expresion *exps,Ptr_tabla sust,             *
*                      int es_else, int *sangria)                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Construir recursivamente la sustitucion, y escribir en    *
*             el archivo ffunc, el codigo del lado izquierdo.           *
* PRE : sust = Una sustitucion                                          *
*       expo = Una expresion (origen)                                   *
*       exps = Es una parte de la expresion con la cual se sustituira   *
*       es_else = indica si el codigo debe empezar con else o no        *
* POS:  sust tiene la sustitucion para todas las variables que aparecen *
*       en la expresion exps.                                           *
*       ffunc = Codigo que evalua la parte izquierda del axioma         *
*       sangria = Valor de la sangria para escribir la parte derecha    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void sust_r_lado_izq(FILE *ffunc,nodo_expresion *expo,
                     nodo_expresion *exps,Ptr_tabla sust,
                     int es_else, int *sangria)
{/*--------------------------------------------------------------------*/
    nodo_argum     *arg;
    nodo_argum     *arg_aux;
    nodo_expresion *expr;
    cad12          nombre;
    int            posi;
    int            sangria_inic;

    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    assert(expo!=NULL);              /* j-tamara 29/May/1997 */
    sangria_inic = (*sangria);
    if (!strcmp(expo->clase,"VAR")) {
        agr(sust,expo->nombre,exps);
      }
    else {
        if ((*sangria) == 1){
            indente(ffunc,(*sangria));
            if (es_else)
                fprintf(ffunc,"else ");
            fprintf(ffunc,"if ( ");
            (*sangria)++;
        }
        if (sangria_inic == (*sangria))
            fprintf(ffunc," && \n\t");
        if (!strcmp(expo->nombre,"ZERO")) {
            fprintf(ffunc,"(");
            codigo_expresion(ffunc,exps);
            fprintf(ffunc," == 0)");
	} else if (!strcmp(expo->nombre,"SUC")) {
            fprintf(ffunc,"(");
            codigo_expresion(ffunc,exps);
            fprintf(ffunc," > 0)");
	} else if (!strcmp(expo->nombre,"TRUE"))
            codigo_expresion(ffunc,exps);
        else if (!strcmp(expo->nombre,"FALSE")) {
            fprintf(ffunc,"!");
            codigo_expresion(ffunc,exps);
        } else {
            fprintf(ffunc,"is_%s( ",expo->nombre);
            codigo_expresion(ffunc,exps);
            fprintf(ffunc," )");
        }
        for(arg = expo->argumentos,posi = 0;arg!=NULL;arg=arg->sig,posi++) {
            sprintf(nombre,"%c%d",arg->expr->tipo[0],posi);
            sprintf(nombre,"pr%d%s",posi,expo->nombre);
            expr = crear_expresion(nombre,arg->expr->tipo,"SELECT");
            arg_aux = crear_argumento(exps);
            expr->argumentos = arg_aux;
            sust_r_lado_izq(ffunc,arg->expr,expr,sust,es_else,sangria);
	}
   }

/*--------------------------------------------------------------------*/}

/************************************************************************
* void codigo_bool( FILE *ffunc, nodo_expresion *expr)                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo de la expresion booleana expr *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void codigo_bool( FILE *ffunc, nodo_expresion *expr)
{/*--------------------------------------------------------------------*/
    nodo_argum *arg;

    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    assert(expr!=NULL);              /* j-tamara 29/May/1997 */
    arg = expr->argumentos;
    if (!strcmp(expr->nombre,"TRUE"))
        fprintf(ffunc,"TRUE");
    else if (!strcmp(expr->nombre,"FALSE"))
        fprintf(ffunc,"FALSE");
    else if (!strcmp(expr->nombre,"NOT")) {
        fprintf(ffunc,"!(");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc,")");
    }
    else if (!strcmp(expr->nombre,"AND")) {
        fprintf(ffunc,"(");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc,") &&\n\t(");
        codigo_expresion(ffunc,arg->sig->expr);
        fprintf(ffunc,")");
    }
    else if (!strcmp(expr->nombre,"OR")) {
        fprintf(ffunc,"(");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc,") ||\n\t(");
        codigo_expresion(ffunc,arg->sig->expr);
        fprintf(ffunc,")");
    }
    else if ((!strcmp(expr->nombre,"EQ_BOOL")) ||
	     (!strcmp(expr->nombre,"EQ_NAT")) ||
             (!strcmp(expr->nombre,"EQUAL")) )  {
        fprintf(ffunc,"(");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc,") == (");
        codigo_expresion(ffunc,arg->sig->expr);
        fprintf(ffunc,")");
    }
    else if (!strcmp(expr->nombre,"GT")) {
        fprintf(ffunc,"(");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc,") > (");
        codigo_expresion(ffunc,arg->sig->expr);
        fprintf(ffunc,")");
    }
    else if (!strcmp(expr->nombre,"LT")) {
        fprintf(ffunc,"(");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc,") < (");
        codigo_expresion(ffunc,arg->sig->expr);
        fprintf(ffunc,")");
    }
    else if (!strcmp(expr->nombre,"GE")) {
        fprintf(ffunc,"(");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc,") >= (");
        codigo_expresion(ffunc,arg->sig->expr);
        fprintf(ffunc,")");
    }
    else if (!strcmp(expr->nombre,"LE")) {
        fprintf(ffunc,"(");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc,") <= (");
        codigo_expresion(ffunc,arg->sig->expr);
        fprintf(ffunc,")");
    }
    else {
        fprintf(ffunc,"%s",expr->nombre);
        if (strcmp(expr->clase,"VAR")) {
            fprintf(ffunc,"(");
            for (;arg!=NULL;arg=arg->sig) {
                codigo_expresion(ffunc,arg->expr);
                if (arg->sig != NULL)
                    fprintf(ffunc,", ");
	    }
            fprintf(ffunc,")");
	}
    }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void codigo_nat( FILE *ffunc, nodo_expresion *expr)                   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo de la expresion integer expr  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void codigo_nat( FILE *ffunc, nodo_expresion *expr)
{/*--------------------------------------------------------------------*/
    nodo_argum *arg;
    int        num;

    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    assert(expr!=NULL);              /* j-tamara 29/May/1997 */
    arg = expr->argumentos;
    if (!strcmp(expr->nombre,"ZERO"))
        fprintf(ffunc,"0");
    else if (!strcmp(expr->nombre,"pr0SUC")) {
        fprintf(ffunc,"(");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," - 1)");
    } else if (!strcmp(expr->nombre,"SUC")) {
        for( num = 1; !strcmp(arg->expr->nombre,"SUC");
                              num++,arg=arg->expr->argumentos);
	/* VTP. Better code generation 10/Oct/1998 */
	if (strcmp(arg->expr->nombre,"ZERO")==0)
	  {
	    fprintf(ffunc,"%d",num);
	  }
	else
	  {
	    codigo_expresion(ffunc,arg->expr);
	    fprintf(ffunc,"+ %d",num);
	  }
    } else if (!strcmp(expr->nombre,"PRED")) {
        for( num = 1; !strcmp(arg->expr->nombre,"PRED");
                              num++,arg=arg->expr->argumentos);
             codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc,"- %d",num);
    }
    else if (!strcmp(expr->nombre,"ADD")) {
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," + ");
        codigo_expresion(ffunc,arg->sig->expr);
    }
    else if (!strcmp(expr->nombre,"SUBS")) {
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," - ");
        codigo_expresion(ffunc,arg->sig->expr);
    }
    else if (!strcmp(expr->nombre,"MULT")) {
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," * ");
        codigo_expresion(ffunc,arg->sig->expr);
    }
    else if (!strcmp(expr->nombre,"DIV")) {
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," / ");
        codigo_expresion(ffunc,arg->sig->expr);
    }
    else if (!strcmp(expr->nombre,"MOD")) {
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," %c ", '%');
        codigo_expresion(ffunc,arg->sig->expr);
    }
    else {
        fprintf(ffunc,"%s",expr->nombre);
        if (strcmp(expr->clase,"VAR")) {
            fprintf(ffunc,"(");
            for (;arg!=NULL;arg=arg->sig) {
                codigo_expresion(ffunc,arg->expr);
                if (arg->sig != NULL)
                    fprintf(ffunc,", ");
	    }
            fprintf(ffunc,")");
	}
    }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void codigo_expresion( FILE *ffunc, nodo_expresion *expr)             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo de la expresion expr          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void codigo_expresion( FILE *ffunc, nodo_expresion *expr)
{/*--------------------------------------------------------------------*/
    nodo_argum *arg;

    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    if (expr == NULL)
       return;
    if (!strcmp(expr->clase,"IF")) {
        fprintf(ffunc,"( ");
        arg = expr->argumentos;
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," ) ? ");
        arg = arg->sig;
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," : ");
        arg = arg->sig;
        codigo_expresion(ffunc,arg->expr);
    }
    else {
       if (!strcmp(expr->tipo,"BOOL"))
           codigo_bool(ffunc,expr);
       else if (!strcmp(expr->tipo,"NAT"))
           codigo_nat(ffunc,expr);
       else {
           fprintf(ffunc,"%s",expr->nombre);
           if (strcmp(expr->clase,"VAR")) {
               fprintf(ffunc,"(");
               for (arg=expr->argumentos;arg!=NULL;arg=arg->sig) {
                   codigo_expresion(ffunc,arg->expr);
                   if (arg->sig != NULL)
                       fprintf(ffunc,", ");
	       }
               fprintf(ffunc,")");
           }
       }
    }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void codigo_lado_der( FILE *ffunc, nodo_tad *t,                       *
*                       nodo_select *sel,                               *
*                       nodo_expresion *lado_der,                       *
*                       cad20 tipo_izq,int sangria)                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo de la expresion lado_der.     *
* t: Es el tad sobre el cual se genera codigo.                          *
* sel: Es la selectora sobre la cual se genera codigo.                  *
* tipo_izq: Es el tipo del lado izquierdo, si se debe hacer la coercion *
*           de la expresion.                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void codigo_lado_der( FILE *ffunc, nodo_tad *t, nodo_select *sel,
                      nodo_expresion *lado_der,
                      cad20 tipo_izq,int sangria)
{/*--------------------------------------------------------------------*/
    nodo_argum *arg;

    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    assert(lado_der!=NULL);              /* j-tamara 29/May/1997 */
    arg = lado_der->argumentos;
    if ((!strcmp(lado_der->clase,"IF")) && (t->tipo != RESTRICCION)) {
        indente(ffunc,sangria);
        fprintf(ffunc,"if ( ");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," ) {\n");
        sangria+=1;
        indente(ffunc,sangria);
        fprintf(ffunc,"return( ");
        arg = arg->sig;
        if (!strcmp(arg->expr->clase,"ERROR"))
            fprintf(ffunc,"(%s)ERROR(\"%s\")",tipo_izq,sel->nombre);
          else
            codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," );\n");
        sangria-=1;
        indente(ffunc,sangria);
        fprintf(ffunc,"} else {\n");
        sangria+=1;
        indente(ffunc,sangria);
        fprintf(ffunc,"return( ");
        arg=arg->sig;
        if (!strcmp(arg->expr->clase,"ERROR"))
            fprintf(ffunc,"(%s)ERROR(\"%s\")",tipo_izq,sel->nombre);
          else
            codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," );\n");
        sangria-=1;
        indente(ffunc,sangria);
        fprintf(ffunc,"}\n");
    }
    else if ((!strcmp(lado_der->clase,"IF")) && (t->tipo == RESTRICCION) &&
             (!strcmp(arg -> sig -> expr -> clase, "ERROR"))) {
/*-- El tad es de tipo restriccion y dice IF xx then ERROR else yyy.
     En este caso se genera un assert(!xx)                           --*/
        indente(ffunc,sangria);
        fprintf(ffunc,"assert (! ");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," );\n");
        indente(ffunc,sangria);
        arg = arg->sig->sig;
        fprintf(ffunc,"return( ");
        if (!strcmp(arg->expr->clase,"ERROR"))
            fprintf(ffunc,"(%s)ERROR(\"%s\")",tipo_izq,sel->nombre);
        else
            codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," );\n");
      }
    else if ((!strcmp(lado_der->clase,"IF")) && (t->tipo == RESTRICCION) &&
             (!strcmp(arg -> sig -> sig-> expr -> clase, "ERROR"))) {
/*-- El tad es de tipo restriccion y dice IF xx then yyy else ERROR.
     En este caso se genera un assert(xx)                           --*/
        indente(ffunc,sangria);
        fprintf(ffunc,"assert ( ");
        codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," );\n");
        indente(ffunc,sangria);
        arg = arg->sig;
        fprintf(ffunc,"return( ");
        if (!strcmp(arg->expr->clase,"ERROR"))
            fprintf(ffunc,"(%s)ERROR(\"%s\")",tipo_izq,sel->nombre);
          else
            codigo_expresion(ffunc,arg->expr);
        fprintf(ffunc," );\n");
    }
    else {
        indente(ffunc,sangria);
        fprintf(ffunc,"return( ");
        if (!strcmp(lado_der->clase,"ERROR"))
            fprintf(ffunc,"(%s)ERROR(\"%s\")",tipo_izq,sel->nombre);
	  else
            codigo_expresion(ffunc,lado_der);
        fprintf(ffunc," );\n");
    }
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_generadora(FILE *fplant,FILE *ffunc,nodo_generadora *gen,  *
*                         nodo_tad *t, int autom)                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo de la generadora gen, y lo    *
* graba en el archivos ffunc (codigo).                                  *
* POST : Si autom = 1, el codigo se crar automaticamente. Si no, solo   *
*        se crea el esqueleto del programa.                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_generadora(FILE *fplant,FILE *ffunc, nodo_generadora *gen,
                        nodo_tad *t, int autom)
{/*--------------------------------------------------------------------*/
    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    generadora_comentario(fplant,ffunc,gen,t);
    encabezado_gen(ffunc,gen,t);
    fprintf(ffunc,"\n");
    copiar_plantilla(fplant,ffunc);
    if (autom)
        codigo_generadora(ffunc,gen,t);
    else
        fprintf(ffunc,"\n#error %s\n\n",mensajes[125]);
    copiar_plantilla(fplant,ffunc);

/*--------------------------------------------------------------------*/}

/************************************************************************
* void codigo_generadora( FILE *ffunc, nodo_generadora *gen,            *
*                         nodo_tad *t )                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo de la generadora gen del tad t*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void codigo_generadora( FILE *ffunc, nodo_generadora *gen,
                        nodo_tad *t )
{/*--------------------------------------------------------------------*/
    int             sangria;
    nodo_generadora *g;
    nodo_nombre     *nom;
    int             num_nom;

    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sangria = 2;
    indente(ffunc,sangria);
    fprintf(ffunc,"%s\t%c;\n\n",t->nombre,t->nombre[0]);
    indente(ffunc,sangria);
    fprintf(ffunc,"%c = (%s)malloc(sizeof(struct %s));\n",
                   t->nombre[0],t->nombre,t->nombre);
    indente(ffunc,sangria);
    fprintf(ffunc,"if (%c == NULL) return ((%s)ERROR(\"%s\"));\n",t->nombre[0],
                   t->nombre,gen->nombre);
    indente(ffunc,sangria);
    fprintf(ffunc,"%c->manta_uso = 0;\n",t->nombre[0]);
    indente(ffunc,sangria);
    fprintf(ffunc,"%c->manta_op = OP_%s;\n",t->nombre[0],gen->nombre);
    for(num_nom=0,nom=gen->dominio;nom!=NULL;nom= nom->sig,num_nom++){
        indente(ffunc,sangria);
        fprintf(ffunc,"%c->%s%d = %c%d;\n",t->nombre[0],
                       gen->nombre,num_nom,nom->nombre[0],num_nom);
    }
    for(g=t->iniciales;g!=NULL;g=g->sig)
        if (g != gen )
            for(num_nom=0,nom=g->dominio;nom!=NULL;
                nom= nom->sig,num_nom++){
                indente(ffunc,sangria);
                fprintf(ffunc,"%c->%s%d = ",t->nombre[0],
                           g->nombre,num_nom);
                if (strcmp(nom->nombre,"NAT")==0 || 
		    strcmp(nom->nombre,"BOOL")==0)
		  { fprintf(ffunc,"0;\n"); }
		else
		  {  fprintf(ffunc,"NULL;\n");}
            }
    for(g=t->constructoras;g!=NULL;g=g->sig)
        if (g != gen )
            for(num_nom=0,nom=g->dominio;nom!=NULL;
                nom= nom->sig,num_nom++){
                indente(ffunc,sangria);
                fprintf(ffunc,"%c->%s%d = ",t->nombre[0],
                           g->nombre,num_nom);
                if (strcmp(nom->nombre,"NAT")==0 || 
		    strcmp(nom->nombre,"BOOL")==0)
		  { fprintf(ffunc,"0;\n"); }
		else
		  {  fprintf(ffunc,"NULL;\n");}
            }
    indente(ffunc,sangria);
    fprintf(ffunc,"return( %c );\n",t->nombre[0]);

/*--------------------------------------------------------------------*/}

/************************************************************************
* void encabezado_sel(FILE *ffunc, nodo_select *sel)                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Copiar el encabezado de la funcion selectora sel en       *
* el archivo ffunc.                                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void encabezado_sel(FILE *ffunc, nodo_select *sel)
{/*--------------------------------------------------------------------*/
    nodo_dom *nom;
    int      cont;

    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    assert(sel!=NULL);              /* j-tamara 29/May/1997 */
    if (!tipo_primitivo(sel->codominio))
        fprintf(ffunc,"%s \t%s(",sel->codominio,sel->nombre);
    else if(!strcmp(sel->codominio,"NAT"))
        fprintf(ffunc,"int \t%s(",sel->nombre);
    else if (!strcmp(sel->codominio,"BOOL"))
        fprintf(ffunc,"BOOL \t%s(",sel->nombre);
    if (sel->dominio == NULL)
        fprintf(ffunc,"void");
    else {
        nom = sel->dominio;
        cont = 0;
        if (!tipo_primitivo(nom->nombre))
            fprintf(ffunc,"%s %c%d",nom->nombre,nom->nombre[0],cont);
        else if (!strcmp(nom->nombre,"NAT"))
            fprintf(ffunc,"int %c%d",nom->nombre[0],cont);
        else if (!strcmp(nom->nombre,"BOOL"))
            fprintf(ffunc,"BOOL %c%d",nom->nombre[0],cont);
        cont++;
        nom = nom->sig;
        for(; nom != NULL; cont++,nom=nom->sig)
            if (!tipo_primitivo(nom->nombre))
                fprintf(ffunc,", %s %c%d",nom->nombre,nom->nombre[0],cont);
            else if (!strcmp(nom->nombre,"NAT"))
                fprintf(ffunc,", int %c%d",nom->nombre[0],cont);
            else if (!strcmp(nom->nombre,"BOOL"))
                fprintf(ffunc,", BOOL %c%d",nom->nombre[0],cont);
    }
    fprintf(ffunc,")");

/*--------------------------------------------------------------------*/}

/************************************************************************
* void encabezado_gen(FILE *ffunc, nodo_generadora *gen,nodo_tad *t)    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Copiar el encabezado de la funcion generadora gen en      *
* el archivo ffunc.                                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void encabezado_gen(FILE *ffunc, nodo_generadora *gen, nodo_tad *t)
{/*--------------------------------------------------------------------*/
    nodo_nombre *nom;
    int          cont;

    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    if (!tipo_primitivo(t->nombre))
        fprintf(ffunc,"%s \t%s(",t->nombre,gen->nombre);
    else if (!strcmp(t->nombre,"NAT"))
        fprintf(ffunc,"int \t%s(",gen->nombre);
    else if (!strcmp(t->nombre,"BOOL"))
        fprintf(ffunc,"BOOL \t%s(",gen->nombre);
    if (gen->dominio == NULL)
        fprintf(ffunc,"void");
    else {
        nom = gen->dominio;
        cont = 0;
        if (!tipo_primitivo(nom->nombre))
            fprintf(ffunc,"%s %c%d",nom->nombre,nom->nombre[0],cont);
        else if (!strcmp(nom->nombre,"NAT"))
                fprintf(ffunc,"int %c%d",nom->nombre[0],cont);
        else if (!strcmp(nom->nombre,"BOOL"))
                fprintf(ffunc,"BOOL %c%d",nom->nombre[0],cont);
        cont++;
        nom = nom->sig;
        for(; nom != NULL; cont++,nom=nom->sig)
            if (!tipo_primitivo(nom->nombre))
                fprintf(ffunc,", %s %c%d",nom->nombre,nom->nombre[0],cont);
            else if (!strcmp(nom->nombre,"NAT"))
                fprintf(ffunc,", int %c%d",nom->nombre[0],cont);
            else if (!strcmp(nom->nombre,"BOOL"))
                fprintf(ffunc,", BOOL %c%d",nom->nombre[0],cont);

    }
    fprintf(ffunc,")");

/*--------------------------------------------------------------------*/}

/************************************************************************
* void selectora_comentario(FILE *fplant,FILE *ffunc, nodo_select *sel) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario de la selectora sel, y lo *
* graba en el archivo ffunc (codigo).                                   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void selectora_comentario(FILE *fplant,FILE *ffunc, nodo_select *sel)
{/*--------------------------------------------------------------------*/
    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,ffunc);
    encabezado_sel(ffunc,sel);
    fprintf(ffunc,"\n");
    copiar_plantilla(fplant,ffunc);
/*-- Colocar pre --*/
    copiar_plantilla(fplant,ffunc);
/*-- Colocar post --*/
    copiar_plantilla(fplant,ffunc);
    linea_fecha(fplant,ffunc);
    copiar_plantilla(fplant,ffunc);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void generadora_comentario(FILE *fplant,FILE *ffunc,                  *
*                            nodo_generadora *gen, nodo_tad *t)         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el comentario de la selectora sel, y lo *
* graba en el archivo ffunc (codigo).                                   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void generadora_comentario(FILE *fplant,FILE *ffunc,
                           nodo_generadora *gen, nodo_tad *t)
{/*--------------------------------------------------------------------*/

    assert(ffunc!=NULL);              /* j-tamara 29/May/1997 */
    copiar_plantilla(fplant,ffunc);
    encabezado_gen(ffunc,gen,t);
    fprintf(ffunc,"\n");
    copiar_plantilla(fplant,ffunc);
/*-- Colocar pre --*/
    copiar_plantilla(fplant,ffunc);
/*-- Colocar post --*/
    copiar_plantilla(fplant,ffunc);
    linea_fecha(fplant,ffunc);
    copiar_plantilla(fplant,ffunc);

/*--------------------------------------------------------------------*/}

/************************************************************************
* void linea_fecha(FILE *fplant,FILE *farch)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : La proxima linea de fplant esta en formato fecha                *
* POST: La ultima linea de farch se grabo segun el formato de fplant    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void linea_fecha(FILE *fplant,FILE *farch)
{/*--------------------------------------------------------------------*/
    texpr     linea1,linea2;
    time_t    tiempo;
    struct tm *tp;
    int       l;

    assert(farch!=NULL);              /* j-tamara 29/May/1997 */
    fgets(linea1,sizeof(texpr),fplant);
    formatear_linea(linea1,linea2);
    sprintf(linea1,linea2,nombre_usuario);
    l = strlen(linea1);
    linea1[l] = '\n';
    linea1[l+1] = '\0';
    fprintf(farch,linea1);
    copiar_plantilla(fplant,farch);
    tiempo = time(&tiempo);
    tp = localtime(&tiempo);
    fgets(linea1,sizeof(texpr),fplant);
    formatear_linea(linea1,linea2);
    l = strlen(linea2);
    strftime(linea1,sizeof(texpr),linea2,tp);
    linea1[l-1] = linea2[l-1];
    linea1[l] = '\n';
    linea1[l+1] = '\0';
    fprintf(farch,linea1);

/*--------------------------------------------------------------------*/}


/************************************************************************
  void lineas_def(FILE *fplant,FILE *farch)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 AUTOR : Vladimir Tamara Patino                          FECHA: 4/Oct/98
************************************************************************/
void lineas_def(FILE *fplant,FILE *farch,char *nom_arc)
{/*--------------------------------------------------------------------*/
    texpr     linea1,linea2;
    int       l;

    assert(farch!=NULL);              /* j-tamara 29/May/1997 */
    /* Toma linea #ifndef %s_h */
    fgets(linea1,sizeof(texpr),fplant);
    formatear_linea(linea1,linea2);
    sprintf(linea1,linea2,nom_arc);
    l=strlen(linea1);
    linea1[l] = '\n';
    linea1[l+1] = '\0';
    fprintf(farch,linea1);
    /* Toma linea #define %s_h */
    fgets(linea1,sizeof(texpr),fplant);
    formatear_linea(linea1,linea2);
    sprintf(linea1,linea2,nom_arc);
    l=strlen(linea1);
    linea1[l] = '\n';
    linea1[l+1] = '\0';
    fprintf(farch,linea1);

/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_case_gen(FILE *finter, int sangria, int op,                *
*                       nodo_generadora *gen,nodo_tad *t)               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : finter = archivo en el cual se genera el codigo                 *
*       sangria = numero de espacios dejados en la indentacion de la    *
*       funcion generada.                                               *
*       op = opcion del case que se genera.                             *
*       gen = generadora para la cual se genera el case.                *
* POST: En el archivo finter se ha generado el case para la opcion      *
*       correspondiente a la generadora gen del tad.                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_case_gen(FILE *finter, int sangria, int op,
                      nodo_generadora *gen,nodo_tad *t)
{/*--------------------------------------------------------------------*/
    nodo_nombre     *n;
    int             num;
    int             espacios;

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    assert(gen!=NULL);              /* j-tamara 29/May/1997 */
    espacios = 1;
    indente(finter,sangria);
    fprintf(finter,"case %d  : {\n",op);
    sangria++;
    indente(finter,sangria+espacios);
    fprintf(finter,"char expr[LONG_CAD];\n");
    indente(finter,sangria+espacios);
    if (!tipo_primitivo(t->nombre))
        fprintf(finter,"%s\t%c_obj;\n",t->nombre,t->nombre[0]);
    else if (!strcmp(t->nombre,"NAT"))
        fprintf(finter,"int \t%c_obj;\n",t->nombre[0]);
    else if (!strcmp(t->nombre,"BOOL"))
        fprintf(finter,"BOOL\t%c_obj;\n",t->nombre[0]);
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++) {
        indente(finter,sangria+espacios);
        if (!tipo_primitivo(n->nombre))
            fprintf(finter,"%s\t%c%d;\n",n->nombre,n->nombre[0],num);
        else if (!strcmp(n->nombre,"NAT"))
            fprintf(finter,"int\t%c%d;\n",n->nombre[0],num);
        else if (!strcmp(n->nombre,"BOOL"))
            fprintf(finter,"BOOL\t%c%d;\n",n->nombre[0],num);
    }
    fprintf(finter,"\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"sprintf(expr,\"%s(\");\n",gen->nombre);
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++) {
        if (num > 0) {
           indente(finter,sangria+espacios);
           fprintf(finter,"strcat(expr,\",\");\n");
        }
        indente(finter,sangria+espacios);
        fprintf(finter,"obj = leer_buscar_obj(expr,\"%s\");\n",n->nombre);
        indente(finter,sangria+espacios);
        fprintf(finter,"if (obj == NULL) break;\n");
        indente(finter,sangria+espacios);
        if (!tipo_primitivo(n->nombre)) {
            fprintf(finter,"strcat(expr,obj->nom_var);\n");
            indente(finter,sangria+espacios);
            fprintf(finter,"%c%d = (%s)obj->u.obj;\n",
                    n->nombre[0],num,n->nombre);
	} else {
            fprintf(finter,"strcat(expr,obj->nom_tipo);\n");
            indente(finter,sangria+espacios);
            fprintf(finter,"%c%d = obj->u.%s;\n",
                    n->nombre[0],num,n->nombre);
          }
    }
    indente(finter,sangria+espacios);
    fprintf(finter,"%c_obj = %s(",t->nombre[0],gen->nombre);
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++)
        if (num == 0)
            fprintf(finter,"%c%d",n->nombre[0],num);
        else
            fprintf(finter,", %c%d",n->nombre[0],num);
    fprintf(finter,");\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"if (tad_error) break;\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"strcat(expr,\")\");\n");
    for(num=0,n = gen->dominio; n != NULL; n=n->sig,num++) {
       if (!tipo_primitivo(n->nombre)) {
	   indente(finter,sangria+espacios);
	   fprintf(finter,"inc_uso(%c%d);\n",n->nombre[0],num);
       }
    }
    indente(finter,sangria+espacios);
    if (!tipo_primitivo(t->nombre))
        fprintf(finter,"leer_guardar_obj(%c_obj,expr,\"%s\");\n",
                        t->nombre[0],t->nombre);
    else if (!strcmp(t->nombre,"NAT"))
        fprintf(finter,"leer_guardar_obj(&%c_obj,expr\"NAT\");\n",
                        t->nombre[0]);
    else if (!strcmp(t->nombre,"BOOL"))
        fprintf(finter,"leer_guardar_obj(&%c_obj,expr\"BOOL\");\n",
                        t->nombre[0]);
    indente(finter,sangria+espacios);
    fprintf(finter,"salvar_escribir(expr,\"%s\");\n",t->nombre);
    indente(finter,sangria+espacios);
    if (!tipo_primitivo(t->nombre))
        fprintf(finter,"imprimir_obj_%s(%c_obj);\n",
                        t->nombre,t->nombre[0]);
    else if (!strcmp(t->nombre,"NAT"))
        fprintf(finter,"imprimir_obj_prim(&%c_obj,\"NAT\");\n",
                        t->nombre[0]);
    else if (!strcmp(t->nombre,"BOOL"))
        fprintf(finter,"imprimir_obj_prim(&%c_obj,\"BOOL\");\n",
                        t->nombre[0]);
    sangria--;
    indente(finter,sangria+espacios);
    fprintf(finter,"}\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"break;\n");
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_case_sel(FILE *finter, int sangria, int op,                *
*                       nodo_select *sel, nodo_tad *t)                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : finter = archivo en el cual se genera el codigo                 *
*       sangria = numero de espacios dejados en la indentacion de la    *
*       funcion generada.                                               *
*       op = opcion del case que se genera.                             *
*       sel = selectora para la cual se genera el case.                 *
* POST: En el archivo finter se ha generado el case para la opcion      *
*       correspondiente a la selectora sel del tad.                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_case_sel(FILE *finter, int sangria, int op,
                      nodo_select *sel/*, nodo_tad *t*/)
{/*--------------------------------------------------------------------*/
    nodo_dom        *n;
    int             num;
    int             espacios;

  /* Alvaro */

    assert(finter!=NULL);              /* j-tamara 29/May/1997 */
    assert(sel!=NULL);              /* j-tamara 29/May/1997 */
    espacios = 1;
    indente(finter,sangria);
    fprintf(finter,"case %d  : {\n",op);
    sangria++;
    indente(finter,sangria+espacios);
    fprintf(finter,"char expr[LONG_CAD];\n");
    indente(finter,sangria+espacios);
    if (!tipo_primitivo(sel->codominio))
        fprintf(finter,"%s\t%c_obj;\n",sel->codominio,sel->codominio[0]);
    else if (!strcmp(sel->codominio,"NAT"))
        fprintf(finter,"int\t%c_obj;\n",sel->codominio[0]);
    else if (!strcmp(sel->codominio,"BOOL"))
        fprintf(finter,"BOOL\t%c_obj;\n",sel->codominio[0]);
    for(num=0,n = sel->dominio; n != NULL; n=n->sig,num++) {
        indente(finter,sangria+espacios);
        if (!tipo_primitivo(n->nombre))
            fprintf(finter,"%s\t%c%d;\n",n->nombre,n->nombre[0],num);
        else if (!strcmp(n->nombre,"NAT"))
            fprintf(finter,"int\t%c%d;\n",n->nombre[0],num);
        else if (!strcmp(n->nombre,"BOOL"))
            fprintf(finter,"BOOL\t%c%d;\n",n->nombre[0],num);
    }
    fprintf(finter,"\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"sprintf(expr,\"%s(\");\n",sel->nombre);
    for(num=0,n = sel->dominio; n != NULL; n=n->sig,num++) {
        if (num > 0) {
           indente(finter,sangria+espacios);
           fprintf(finter,"strcat(expr,\",\");\n");
        }
        indente(finter,sangria+espacios);
        fprintf(finter,"obj = leer_buscar_obj(expr,\"%s\");\n",n->nombre);
        indente(finter,sangria+espacios);
        fprintf(finter,"if (obj == NULL) break;\n");
        indente(finter,sangria+espacios);
        if (!tipo_primitivo(n->nombre)) {
            fprintf(finter,"strcat(expr,obj->nom_var);\n");
            indente(finter,sangria+espacios);
            fprintf(finter,"%c%d = (%s)obj->u.obj;\n",
                    n->nombre[0],num,n->nombre);
        } else {
            fprintf(finter,"strcat(expr,obj->nom_tipo);\n");
            indente(finter,sangria+espacios);
            fprintf(finter,"%c%d = obj->u.%s;\n",
                    n->nombre[0],num,n->nombre);
	  }
    }
    indente(finter,sangria+espacios);
    fprintf(finter,"%c_obj = %s(",sel->codominio[0],sel->nombre);
    for(num=0,n = sel->dominio; n != NULL; n=n->sig,num++)
        if (num == 0)
            fprintf(finter,"%c%d",n->nombre[0],num);
        else
            fprintf(finter,", %c%d",n->nombre[0],num);
    fprintf(finter,");\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"if (tad_error) break;\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"strcat(expr,\")\");\n");
    for(num=0,n = sel->dominio; n != NULL; n=n->sig,num++) {
       if (!tipo_primitivo(n->nombre)) {
	   indente(finter,sangria+espacios);
	   fprintf(finter,"inc_uso(%c%d);\n",n->nombre[0],num);
       }
    }
    indente(finter,sangria+espacios);
    if (!tipo_primitivo(sel->codominio))
        fprintf(finter,"leer_guardar_obj(%c_obj,expr,\"%s\");\n",
                        sel->codominio[0],sel->codominio);
    else if (!strcmp(sel->codominio,"NAT"))
        fprintf(finter,"leer_guardar_obj(&%c_obj,expr,\"NAT\");\n",
                        sel->codominio[0]);
    else if (!strcmp(sel->codominio,"BOOL"))
        fprintf(finter,"leer_guardar_obj(&%c_obj,expr,\"BOOL\");\n",
                        sel->codominio[0]);
    indente(finter,sangria+espacios);
    fprintf(finter,"salvar_escribir(expr,\"%s\");\n",sel->codominio);
    indente(finter,sangria+espacios);
    if (!tipo_primitivo(sel->codominio))
        fprintf(finter,"imprimir_obj_%s(%c_obj);\n",
                        sel->codominio,sel->codominio[0]);
    else if (!strcmp(sel->codominio,"NAT"))
        fprintf(finter,"imprimir_obj_prim(&%c_obj,\"NAT\");\n",
                        sel->codominio[0]);
    else if (!strcmp(sel->codominio,"BOOL"))
        fprintf(finter,"imprimir_obj_prim(&%c_obj,\"BOOL\");\n",
                        sel->codominio[0]);
    sangria--;
    indente(finter,sangria+espacios);
    fprintf(finter,"}\n");
    indente(finter,sangria+espacios);
    fprintf(finter,"break;\n");
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_make(nodo_tad *t)                                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea el codigo del archivo que sirve de      *
* entrada a la aplicacion makefile, permitiendo compilar y ejecutar     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 12-Dic-1991	*
************************************************************************/

void crear_make(nodo_tad *t)
{/*--------------------------------------------------------------------*/
    FILE        *fmake;
    FILE        *fplant;
    cad20       nombre_arch;

    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    sprintf(nombre_arch,"%s/%s",direct[1],direct[15]);
    fplant = fopen(nombre_arch,"r");
    if (fplant == NULL)
        return;
    sprintf(nombre_arch,"%s/%s.mak",direct[7],t->nombre);
    fmake=fopen(nombre_arch,"w");
    FileOpen(fmake,nombre_arch);        /* j-tamara 29/May/1997 */
    copiar_plantilla_encab(fplant,fmake,t);
    crear_secciones(fmake,t);
    copiar_plantilla(fplant,fmake);
    fprintf(fmake,"\t$(CC) $(CFLAGEXE) -o %s $(OBJETOS) \n",
                   t->nombre);
    /*** VTP Mar/18/1997. En la anterior linea el makefile se generaba con
              -ltermcap (ya no) ***/
    copiar_plantilla(fplant,fmake);
    fclose(fmake);
    fclose(fplant);
/*--------------------------------------------------------------------*/}

/************************************************************************
* void crear_secciones(FILE *fmake, nodo_tad *t)                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Esta funcion crea las secciones de objetos, fuentes y     *
* encabezados del archivo make.                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 20-Ene-1991	*
************************************************************************/

void crear_secciones(FILE *fmake, nodo_tad *t)
{/*--------------------------------------------------------------------*/
    nodo_nombre *lista=NULL;                /* j-tamara 29/May/1997 */
    nodo_nombre *l;

    assert(fmake!=NULL);              /* j-tamara 29/May/1997 */
    assert(t!=NULL);              /* j-tamara 29/May/1997 */
    lista = lista_tipos(t);
    fprintf(fmake,"OBJETOS =\t manta.o manta_in.o %s_ma.o ",t->nombre);
    fprintf(fmake,"%s.o %s_in.o ",t->nombre,t->nombre);
    for(l = lista; l != NULL; l = l->sig)
        if (!tipo_primitivo(l->nombre))
            fprintf(fmake,"%s.o %s_in.o ",l->nombre,l->nombre);
    fprintf(fmake,"\n");
/*-- Creada la seccion de objetos --*/

    fprintf(fmake,"FUENTES =\t manta.c manta_in.c %s_ma.c ",t->nombre);
    fprintf(fmake,"%s.c %s_in.c ",t->nombre,t->nombre);
    for(l = lista; l != NULL; l = l->sig)
        if (!tipo_primitivo(l->nombre))
            fprintf(fmake,"%s.c %s_in.c ",l->nombre,l->nombre);
    fprintf(fmake,"\n");
/*-- Creada la seccion de fuentes --*/
 /* Alvaro
    fprintf(fmake,"ENCABEZADOS =\t manta.h manta_in.h ",t->nombre);
 */
 /* Puesta por Alvaro */
    fprintf(fmake,"ENCABEZADOS =\t manta.h manta_in.h ");

    fprintf(fmake,"%s.h %s_in.h ",t->nombre,t->nombre);
    for(l = lista; l != NULL; l = l->sig)
        if (!tipo_primitivo(l->nombre))
            fprintf(fmake,"%s.h %s_in.h ",l->nombre,l->nombre);
    fprintf(fmake,"\n");
/*-- Creada la seccion de encabezados --*/
    liberar_nombres(&lista);
/*--------------------------------------------------------------------*/}

