 
/* man_teoremas.c                                               */
/* Arhivo de manejo de Teoremas.                                */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>		/* VTP. 30/Abr/1997 */
#include "definic.h"
#include "os_dep.h"
#include "apimanta.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 8/May/1997 */



/************************************************************************
 void test_rep()
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
  PROPOSITO:    Prueba una representacion elegida por el usuario .      
  DESCRIPCION:  Muestra las representaciones en ambiente, y permite al  
               usuario elejir una.  Despues crea las conjeturas        
	       apropiadas y trata de probarlas
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 AUTOR: V. Tamara                                     FECHA: 5 Jun 98  
************************************************************************/
void test_rep()
{
  
  int c, f;  
  texpr mens_error;
  cad50 arcname;
  nodo_nombre *lis;
  texpr nombre;
  nodo_tad *p;
  FILE *fp;
  nodo_igualdad * conj;

  if ((lis=lista_rep_ambiente ("",""))==NULL)
    {
      escriba_mensaje(143);
      return;
    }
  
  do
    {
      admon_menu_scroll (menus_manta[REPRESENTS_AMBIENTE], lis, nombre);
      if (strcmp (nombre, "")==0 || strcmp (nombre, mensajes[150])==0)
	{
	  return;
	}
      p = existe_tad_amb (nombre);
      if (p == NULL)
	{
	  escriba_mensaje (146);
	  admon_menu_scroll (menus_manta[REPRESENTS_AMBIENTE], lis, nombre);
	}
    } while (p==NULL);

  strcpy(arcname,"conjrep.thm");
  fp=fopen(arcname,"wt");
  if (fp==NULL)
    {
      sprintf(mens_error,mensajes[196],arcname);
      escriba_cadena(mens_error,1);
      return;
    }
  if (conjeturasRep(fp,p,mens_error)!=0)
    {
      escriba_cadena(mens_error,1);
      fclose(fp);
      return;
    }
  fclose(fp);
  fp=fopen(arcname,"rt");
  if (fp==NULL)
    {
      sprintf(mens_error,mensajes[196],arcname);
      escriba_cadena(mens_error,1);
      return;
    }
  conj = analice_conjeturas (fp, mens_error);
  if (conj == NULL)
    {
      escriba_cadena(mens_error,1);
      fclose(fp);
      return;
    }
  probar_conjeturas (&c, &f, &conj);
  escriba_mensaje (25);	/* 25:Resultado */
  sprintf (mens_error, mensajes[78], c);/* 78:Conjeturas probadas: ... */
  escriba_mensaje_dinam (mens_error);
  sprintf (mens_error, mensajes[77], f);/* 77:Conjeturas no probadas:... */
  escriba_mensaje_dinam (mens_error);
  unlink(arcname);
}


/************************************************************************
* void manejar_teoremas()                                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Controla el proceso de administracion de teoremas.      *
* DESCRIPCION:  Despliega el menu de opciones de teoremas e invoca la   *
*       funcion correspondiente a la seleccion del usuario.             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 20 Feb 91 *
************************************************************************/

void
manejar_teoremas ()
{
/*--------------------------------------------------------------------*/

  int opcion;
  nodo_igualdad *conj;

  conj = NULL;
  opcion = admon_menu (menus_manta[TEOREMAS]);
  while (opcion != NO_OPCION &&
	 opcion != menus_manta[TEOREMAS]->num_op - 2)
    {
      switch (opcion)
	{
	case 0:		/* DIRECTORIO */
	  escriba_dir (direct[6], "*");
	  break;
	case 1:		/* NORMALIZAR */
	  forma_normal ();
	  break;
	case 2:		/* CONJETURAS */
	  edite_conjeturas (&conj);
	  break;
	case 3:		/* PROBAR REPRESENTACION */
	  test_rep ();
	  break;
	case 4:		/* PROBADAS */
	  conjeturas (1);
	  break;
	case 5:		/* NO PROBADAS */
	  conjeturas (0);
	  break;
	case 6:		/* SALVAR PROBADAS */
	  salve_conjeturas (1);
	  break;
	case 7:		/* SALVAR NO PROBADAS */
	  salve_conjeturas (0);
	  break;
	case 9:		/* AYUDA */
	  escriba_mensaje (154);
	  break;
	default:
	  escriba_mensaje (146);
	  break;
	}
      opcion = admon_menu (menus_manta[TEOREMAS]);
    }
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void conjeturas(int tipo)                                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Edita un archivo con conjeturas.                        *
*               tipo = 0, Conjeturas no probadas.                       *
*               tipo = 1, Conjeuras probadas.                           *
* DESCRIPCION:  Invoca comando vi, con opcion [read-only]               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 20 Feb 91 *
* MODIFICACIONES                                                        *
* VTP 14/Ene/97                                                         *
*	Antes borraba un archivo empleando system("rm ... ahora         *
*	se usa unlink (portable para DOS y UNIX)                        *
*       Empleba touch por una invocacion al sistema.  Se reemplazo por  *
*       una llamada a la funcion touch                                  *
************************************************************************/

void
conjeturas (int tipo)
{
/*--------------------------------------------------------------------*/
  texpr comando;
  nodo_demost *p;
  /* Alvaro
     int         opcion;
   */
  FILE *fp;
  nodo_nombre *lis;
  nodo_nombre *last, *ant_last;
  texpr nombre, dir, ext, nombre_arch;
  trio_edicion edicion[MAX_EDICION];

  assert (tipo == 0 || tipo == 1);	/* VTP. 30/Abr/1997 */
	/*** VTP. Decia:sprintf(comando,"%s/%s.the",direct[6],mensajes[113]);
						 system(comando); ***/
  sprintf (comando, "%s/%s.the", direct[6], mensajes[113]);
  transArch (comando);
  touch (comando);
	/*** VTP. sprintf(comando,"touch %s/%s.con",direct[6],mensajes[113]);
				 system(comando);  ***/
  sprintf (comando, "%s/%s.con", direct[6], mensajes[113]);
  transArch (comando);
  touch (comando);
  if (tipo)
    {
      strcpy (dir, direct[6]);
      dir[strlen (direct[6])] = '\0';	/* VTP. Abr/30/1997 */
      strcpy (ext, "the");
      ext[3] = '\0';		/* VTP. Abr/30/1997 */
      lis = lista_dir (dir, ext,0);
      for (ant_last = NULL, last = lis;
           last != NULL && strcmp (last->nombre, mensajes[113]);
           ant_last = last, last = last->sig);
      if (ant_last != NULL)
        {
          ant_last->sig = last->sig;
          last->sig = lis;
          lis = last;
        }
      p = proved;
    }
  else
    {
      strcpy (dir, direct[6]);
      dir[strlen (direct[6])] = '\0';	/* VTP. Abr/30/1997 */
      strcpy (ext, "con");
      ext[3] = '\0';		/* VTP. Abr/30/1997 */
      lis = lista_dir (dir, ext,0);
      for (ant_last = NULL, last = lis;
           last != NULL && strcmp (last->nombre, mensajes[113]);
           ant_last = last, last = last->sig);
      if (ant_last != NULL)
        {
          ant_last->sig = last->sig;
          last->sig = lis;
          lis = last;
        }
      p = unproved;
    }
  /* VTP. 10/Feb/1998. Se reorganizo y se dejo el do siguiente
     Asi despues de editar una probada o no probada permite editar otra */
  do
    {
      admon_menu_scroll (menus_manta[CONJ_DIRECTORIO], lis,
                         nombre);
      if (strcmp (nombre, "") && strcmp (nombre, mensajes[150]))
        {
          if (strcmp (nombre, mensajes[113]))
            {
              sprintf (nombre_arch, "%s/%s.%s", dir, nombre, ext);
              transArch (nombre_arch);
              edicion[0].modo = 1;
              edicion[0].linea = 1;
              strcpy (edicion[0].nombre, nombre_arch);
            }
          else
            {
              sprintf (comando, "%s/%s.%s", dir, mensajes[113], ext);
              transArch (comando);
              fp = fopen (comando, "w");
              FileOpen (fp, comando);	/* VTP. 30/Abr/1997 */
              escriba_arch (fp, p, tipo);
              fclose (fp);
              sprintf (nombre_arch, "%s/%s.%s", dir, mensajes[113], ext);
              transArch (nombre_arch);
              edicion[0].modo = 0;
              edicion[0].linea = 1;
              strcpy (edicion[0].nombre, nombre_arch);
            }
          llamar_al_editor (1, edicion, "");
          if (!strcmp (nombre, mensajes[113]))
            {
              sprintf (comando, "%s/%s.%s", dir, mensajes[113], ext);
              transArch (comando);
              unlink (comando);
                     /*** VTP. 18/Mar/1997. Decia
              sprintf(comando,"rm %s/%s.%s",dir,mensajes[113],ext);
              system(comando); ***/
            }
        }
    } while (strcmp (nombre, "") && strcmp (nombre, mensajes[150]));
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void forma_normal(void)                                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Permite editar expresiones y normalizarlas.             *
* DESCRIPCION:  Invoca la herramienta UNIX vi, para editar un archivo   *
*       con la expresion, luego invoca el normalizador.                 *
* MODIFICADA: Hernan Aponte                                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 20 Feb 91 *
* MODIFICACIONES                                                        *
* VTP 14/Ene/97                                                         *
*	Antes borraba un archivo empleando system("rm ... ahora         *
*	se usa unlink (portable para DOS y UNIX)                        *
************************************************************************/

void
forma_normal (void)
{
/*--------------------------------------------------------------------*/
  texpr mens_error;
/* Alvaro
   texpr         comando;
 */
  FILE *fp;
/* Alvaro
   texpr         cad;
 */
  int corregir;
  nodo_argum *arg = NULL;
  nodo_igualdad *conj = NULL;
  trio_edicion edicion[MAX_EDICION];

  esc_arch_edicion_signatura (tad_actual);
  editar_2w ("signa_ed.tad", "");
  edicion[0].modo = 0;
  edicion[0].linea = 1;
  strcpy (edicion[0].nombre, "signa_ed.tad");
  edicion[1].modo = 1;
  edicion[1].linea = 4;
  strcpy (edicion[1].nombre, "normaliz");
  llamar_al_editor (2, edicion, "temp");

  fp = fopen ("normaliz", "r");
  /* FileOpen (fp, "normaliz"); VTP. 3/Jun/1997 */
  for (corregir = 1, arg = NULL; fp != NULL && corregir;)
    {
      arg = analice_lista_exp (fp, mens_error);
      fclose (fp);
      fp=NULL;
      corregir = 0;
      if ((arg == NULL) && (strcmp (mens_error, "")))
	corregir = desea_corregir_edicion (2, mens_error);
      if (corregir)
	{
	  edicion[1].linea = num_ed;
	  llamar_al_editor (2, edicion, "temp");
	  fp = fopen ("normaliz", "r");		/* y esta instruccion que hace aqui */
	  FileOpen (fp, "normaliz");
	}
    }
  if (fp!=NULL)
    { fclose (fp); }
/*** VTP. Decia: system("rm -f normaliz"); ***/
  unlink ("normaliz");
/*** VTP. Decia: system("rm -f temp"); ***/
  unlink ("temp");
/*** VTP. Decia: system("rm -f signa_ed.tad"); ***/
  unlink ("signa_ed.tad");
  if (arg != NULL)
    {
      normalizar_lista (&arg, &conj);
      fp = fopen ("normaliz", "w");
      FileOpen (fp, "normaliz");
      escriba_arch_ig (fp, conj, mensajes[79]);
      fclose (fp);
      edicion[0].modo = 1;
      edicion[0].linea = 1;
      strcpy (edicion[0].nombre, "normaliz");
      llamar_al_editor (1, edicion, "");
/*** VTP. Decia:  system("rm -f normaliz");    ***/
      unlink ("normaliz");
    }
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void edite_conjeturas(nodo_igualdad *conj)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Permite editar conjeturas y demostrarlas.               *
* DESCRIPCION:  Invoca un editor de UNIX, para editar un archivo        *
*       con las conjeturas, luego invoca el probador de teoremas.       *
* MODIFICADA: Hernan Aponte                                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 20 Feb 91 *
* MODIFICACIONES                                                        *
* VTP 14/Ene/97                                                         *
*	Antes borraba un archivo empleando system("rm ... ahora         *
*	se usa unlink (portable para DOS y UNIX)                        *
************************************************************************/

void
edite_conjeturas (nodo_igualdad ** conj)
{
/*--------------------------------------------------------------------*/
  texpr mens_error;
/* Alvaro
   texpr         comando;
 */
  FILE *fp;
  int c, f;
  texpr cad;
  int corregir;
  trio_edicion edicion[MAX_EDICION];

  assert (conj != NULL);	/* VTP. 30/Abr/1997 */
  if ((*conj) != NULL)
    {
      fp = fopen ("conje_ed.con", "w");
      FileOpen (fp, "conje_ed.con");
      escriba_arch_ig (fp, (*conj), "");
      fclose (fp);
    }
  liberar_conjeturas (conj);
  esc_arch_edicion_signatura (tad_actual);
  editar_2w ("signa_ed.tad", "");

  edicion[0].modo = 0;
  edicion[0].linea = 1;
  strcpy (edicion[0].nombre, "signa_ed.tad");
  edicion[1].modo = 1;
  edicion[1].linea = 4;
  strcpy (edicion[1].nombre, "conje_ed.con");
  llamar_al_editor (2, edicion, "temp");

  fp = fopen ("conje_ed.con", "r");
/*  FileOpen (fp, "conje_ed.con"); No es necesario*/
  for (corregir = 1; fp != NULL && corregir;)
    {
      (*conj) = analice_conjeturas (fp, mens_error);
      fclose (fp);
      fp=NULL;
      corregir = 0;
      if (((*conj) == NULL) && (strcmp (mens_error, "")))
	corregir = desea_corregir_edicion (2, mens_error);
      if (corregir)
	{
	  edicion[1].linea = num_ed;
	  llamar_al_editor (2, edicion, "temp");
	  fp = fopen ("conje_ed.con", "r");
	  FileOpen (fp, "conje_ed.con");
	}
    }
  if (fp!=NULL)
    { fclose (fp); }
/*** VTP. Decia system("rm -f conje_ed.con"); ***/
  unlink ("conje_ed.con");
/*** VTP. Decia system("rm -f temp"); ***/
  unlink ("temp");
/*** VTP. Decia system("rm -f signa_ed.tad"); ***/
  unlink ("signa_ed.tad");
  if ((*conj) != NULL)
    {
      liberar_demost(&proved);
      liberar_demost(&unproved);
      probar_conjeturas (&c, &f, conj);
      escriba_mensaje (25);	/* 25:Resultado */
      sprintf (cad, mensajes[78], c);	/* 78:Conjeturas probadas: ... */
      escriba_mensaje_dinam (cad);
      sprintf (cad, mensajes[77], f);	/* 77:Conjeturas no probadas:... */
      escriba_mensaje_dinam (cad);
    }
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void salve_conjeturas(int tipo)                                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Salvar las conjeturas, con sus demostraciones.          *
*               Tipo = 1 , son conjeturas demostradas.                  *
*               Tipo = 0 , son conjeturas no demostradas.               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 20 Jun 91 *
* MODIF: Hernan Aponte                                                  *
************************************************************************/

void
salve_conjeturas (int tipo)
{
/*--------------------------------------------------------------------*/
  cad12 nombre_arch;
  texpr camino;
  nodo_demost *p;
  FILE *fp;

  assert (tipo == 1 || tipo == 0);	/* VTP. 30/Abr/1997 */

  lea_nombre (mensajes[76], nombre_arch);	/* 76:nombre del arhivo: */
  if (!strcmp (nombre_arch, ""))
    return;
  if (tipo)
    {
      sprintf (camino, "%s/%s.the", direct[6], nombre_arch);
      transArch (camino);
    }
  else
    {
      sprintf (camino, "%s/%s.con", direct[6], nombre_arch);
      transArch (camino);
    }
  fp = fopen (camino, "w");
  FileOpen (fp, camino);
  if (tipo)
    p = proved;
  else
    p = unproved;
  escriba_arch (fp, p, tipo);
  fclose (fp);
  escriba_mensaje (116);
  if (tipo)
    {
      liberar_demost (&proved);
      proved = NULL;
      ult_proved = NULL;
    }
  else
    {
      liberar_demost (&unproved);
      unproved = NULL;
      ult_unproved = NULL;
    }
/*--------------------------------------------------------------------*/
}

/************************************************************************
* void escriba_arch(FILE *fp,nodo_demost *dem, int tipo)        	*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Salvar la lista de demostraciones dem en el archivo fp. *
* PARAMETROS:   FILE           *fp      Archivo   		        *
*               nodo_demost    *dem     Lista de demostracines.         *
*               tipo = 1       Conjeturas probadas.                     *
*               tipo = 0       Conjeturas no probadas.                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 20 Nov 91 *
************************************************************************/

void
escriba_arch (FILE * fp, nodo_demost * dem, int tipo)
{
/*--------------------------------------------------------------------*/
  nodo_demost *d;
  nodo_historia *li, *ld, *ant_li, *ant_ld;
  texpr cad;
  int sangria;
  char c;

  assert (fp != NULL && (tipo == 0 || tipo == 1));	/* VTP. 30/Abr/1997 */

  sangria = 0;
  for (d = dem; d != NULL; d = d->sig)
    {
      if (tipo)
	fprintf (fp, mensajes[75]);
      else
	fprintf (fp, mensajes[74]);
      li = d->his_li;
      ld = d->his_ld;
      construya_exp (li->expr, cad,"");
      fprintf (fp, "\n%s = ", cad);
      construya_exp (ld->expr, cad,"");
      fprintf (fp, "%s\n\n", cad);
      ant_li = li;
      ant_ld = ld;
      li = li->sig;
      ld = ld->sig;
      while ((li != NULL) || (ld != NULL))
	{
	  if ((li != NULL) && (ld != NULL) &&
	      strcmp (li->nivel, "") && strcmp (ld->nivel, ""))
	    {
	      sangria = strlen (li->nivel);
	      c = li->nivel[sangria - 1];
	      switch (c)
		{
		case '1':			     /*-- Es un nodo con la informacion
											de la sustitucion que se hace, para
											la induccion, caso trivial --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[93]);
		  indente (fp, sangria);
		  construya_exp (li->expr, cad,"");
		  fprintf (fp, "%s %s -> ", mensajes[73], cad);
		  construya_exp (ld->expr, cad,"");
		  fprintf (fp, "%s\n", cad);
		  break;
		case '2':			     /*-- Es un nodo con la informacion de la
											hipotesis de induccion. --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[94]);
		  indente (fp, sangria);
		  fprintf (fp, "%s %s = %s\n",
			   mensajes[72],
			   li->regla->li_str,
			   li->regla->ld_str);
		  indente (fp, sangria);
		  construya_exp (li->expr, cad,"");
		  fprintf (fp, "%s %s -> ", mensajes[73], cad);
		  construya_exp (ld->expr, cad,"");
		  fprintf (fp, "%s\n", cad);
		  break;
		case '3':
/*-- Indica que comienza una prueba semantica --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[71]);
		  break;
		case '4':
/*-- Indica que la prueba fue existosa       --*/
		  fprintf (fp, mensajes[64]);
		  break;
		case '5':
/*-- Indica que la prueba fue existosa       --*/
		  fprintf (fp, mensajes[60]);
		  break;
		case '6':
/*-- Indica exito de la prueba semantica --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[91]);
		  break;
		case '7':
/*-- Indica falla de la prueba semantica --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[90]);
		  break;
		case '8':			     /*-- Indica exito en prueba induccion
											caso trivial --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[95]);
		  break;
		case '9':			     /*-- Indica exito en prueba induccion
											caso no trivial --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[96]);
		  break;
		case 'A':			     /*-- Indica falla en prueba induccion
											caso trivial --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[97]);
		  break;
		case 'B':			     /*-- Indica falla en prueba induccion
											caso no trivial --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[98]);
		  break;
		case 'C':
/*-- Indica comienzo de la prueba de induccion --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[92]);
		  break;
		case 'D':
/*-- Indica comienzo de la prueba de induccion --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[99]);
		  break;
		case 'E':
/*-- Indica comienzo de la prueba por casos --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[134]);
		  indente (fp, sangria);
		  fprintf (fp, mensajes[134]);
		  indente (fp, sangria);
		  fprintf (fp, mensajes[138]);
		  break;
		case 'F':
/*-- Indica exito prueba por casos  TRUE--*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[135]);
		  indente (fp, sangria);
		  fprintf (fp, mensajes[139]);
		  break;
		case 'G':
/*-- Indica exito prueba por casos  FALSE--*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[136]);
		  break;
		case 'H':
/*-- Indica falla prueba por casos --*/
		  indente (fp, sangria);
		  fprintf (fp, mensajes[137]);
		  break;
		}
	      li = li->sig;
	      ld = ld->sig;
	      sangria += 2;
	    }
	  else
	    {
	      if ((li != NULL) && !strcmp (li->nivel, "") &&
		  (li->regla != NULL))
		{
		  indente (fp, sangria);
		  fprintf (fp, "%s %s = %s\n", mensajes[70],
			   li->regla->li_str, li->regla->ld_str);
		}
	      if ((ld != NULL) && !strcmp (ld->nivel, "") &&
		  (ld->regla != NULL))
		{
		  indente (fp, sangria);
		  fprintf (fp, "%s %s = %s\n", mensajes[69],
			   ld->regla->li_str, ld->regla->ld_str);
		}
	      if ((li != NULL) && !strcmp (li->nivel, ""))
		{
		  indente (fp, sangria);
		  construya_exp (li->expr, cad,"");
		  fprintf (fp, "%s = ", cad);
		  ant_li = li;
		  li = li->sig;
		}
	      else
		{
		  indente (fp, sangria);
		  construya_exp (ant_li->expr, cad,"");
		  fprintf (fp, "%s = ", cad);
		}
	      if ((ld != NULL) && !strcmp (ld->nivel, ""))
		{
		  construya_exp (ld->expr, cad,"");
		  fprintf (fp, "%s\n\n", cad);
		  ant_ld = ld;
		  ld = ld->sig;
		}
	      else
		{
		  construya_exp (ant_ld->expr, cad,"");
		  fprintf (fp, "%s\n\n", cad);
		}
	    }
	}
    }
/*--------------------------------------------------------------------*/
}


/************************************************************************
* void escriba_arch_ig(FILE *fp,nodo_igualdad *ig,char *mensaje)        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Salvar la lista de igualdades ig en el archivo fp.      *
* PARAMETROS:   FILE           *fp      Archivo   		        *
*               nodo_igualdad  *ig      Lista de igualdades.            *
*               char           *mensaje Titulo del archivo (1ra.linea)  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 20 Feb 91 *
* MODIF: Hernan Aponte                                                  *
************************************************************************/

void
escriba_arch_ig (FILE * fp, nodo_igualdad * ig, char *mensaje)
{
/*--------------------------------------------------------------------*/
  texpr formato;

  assert (fp != NULL && ig != NULL && mensaje != NULL);		/* VTP. 30/Abr/1997 */

  if (strcmp (mensaje, ""))
    fprintf (fp, "----  %s ----\n", mensaje);
  for (; ig != NULL; ig = ig->sig)
    {
      strcpy (formato, "");
      formatee_ld_axioma (ig->lado_der, formato, strlen (ig->li_str) + 4);
      fprintf (fp, "%s = %s \n", ig->li_str, formato);
    }

/*--------------------------------------------------------------------*/
}

/************************************************************************
* void indente( FILE *fp, int sangria )                  		*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:    Coloca en el archivo fp, tantos espacios como sangria.  *
* PARAMETROS:   FILE           *fp      Archivo   		        *
*               int            sangria  Numero de espacios.             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 20 Feb 91 *
************************************************************************/

void
indente (FILE * fp, int sangria)
{
/*--------------------------------------------------------------------*/
  int i;
  texpr cad;

  assert (fp != NULL && sangria >= 0);	/* VTP. 30/Abr/1997 */

  strcpy (cad, "");
  for (i = 0; i < 2 * sangria; i++)
    strcat (cad, " ");
  fprintf (fp, "%s", cad);
/*--------------------------------------------------------------------*/
}
