  
/* SISTRE.C                                                             */
/* Implantacion del TAD Sistema de Reglas de Reescritura                */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "definic.h"
#include "os_dep.h"
#include "apimanta.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 8/May/1997 */


/************************************************************************
* nodo_expresion *contiene_ERROR( nodo_expresion *exp  )         	*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Retornar la posicion de una subexpresion ERROR en exp.    *
* DESCRIPCION : Recorre la expresion recursivamente, buscando una       *
*               expresion ERROR en los argumentos.
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 18-Jul-1990	*
************************************************************************/

nodo_expresion *
contiene_ERROR (nodo_expresion * exp)
{
/*--------------------------------------------------------------------*/
  nodo_expresion *e;
  nodo_argum *arg;

  assert (exp != NULL);		/* VTP. 8/May/1997 */
  if (!strcmp (exp->nombre, "ERROR"))
    {
      strcpy (exp->clase, "ERROR");
      return (exp);
    }
  if (!strcmp (exp->clase, "VAR"))
    return (NULL);
  if (strcmp (exp->clase, "IF"))
    {
      for (e = NULL, arg = exp->argumentos;
	   (arg != NULL) && (e == NULL); arg = arg->sig)
	e = contiene_ERROR (arg->expr);
    }
  else
    e = contiene_ERROR (exp->argumentos->expr);
  return (e);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_expresion *ldr( nodo_igualdad *regla )				*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Lado derecho de una regla					*
* DESCRIPCION : Retorna el apuntador, de la estructura de igualdad,     *
*               correspondiente al lado derecho de una regla		*
* PARAMETROS : 								*
*   nodo_igualdad *regla Apuntador a la regla. 		                *
* PRE! : regla								*
* POST! : ldr = Apuntador al lado derecho de regla			*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 25-Nov-1990	*
************************************************************************/

nodo_expresion *
ldr (nodo_igualdad * regla)
{
/*--------------------------------------------------------------------*/
  assert (regla != NULL);	/* VTP. 8/May/1997 */
  return (regla->lado_der);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_expresion *lir( nodo_igualdad *regla )				*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Lado izquierdo de una regla				*
* DESCRIPCION : Retorna el apuntador, de la estructura de igualdad,     *
*               correspondiente al lado izquierdo de una regla		*
* PARAMETROS : 					         		*
*   nodo_igualdad *regla Apuntador a la regla. 		                *
* PRE! : regla								*
* POST! : lir = Apuntador al lado izquierdo de regla			*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 25-Nov-1990	*
************************************************************************/

nodo_expresion *
lir (nodo_igualdad * regla)
{
/*--------------------------------------------------------------------*/
  assert (regla != NULL);	/* VTP. 8/May/1997 */
  return (regla->lado_izq);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int aplica( nodo_expresion *expri,nodo_expresion *exprd )             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Indica si expri ocurre en alguna parte de exprd.    	*
* DESCRIPCION : Busca, recursivamente, la primera subexpresion ( mas    *
*               externa, mas izquierda ) de exprd que unifique con expri*
* RESULTADO : Si aplica o no.                                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 25-Jun-1990	*
************************************************************************/

int
aplica (nodo_expresion * expri, nodo_expresion * exprd)
{
/*--------------------------------------------------------------------*/
  Ptr_tabla unif;
  /* Alvaro
     nodo_expresion *expaux;
   */
  nodo_argum *arg;
  int aplic;

  if ((expri == NULL) || (exprd == NULL))
    return (0);
  unif = umg1 (expri, exprd);
  if (unif != NULL)
    return (1);
  for (aplic = 0, arg = exprd->argumentos; (arg != NULL) && (!aplic);
       arg = arg->sig)
    aplic = aplica (expri, arg->expr);
  return (aplic);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int long_expr( nodo_expresion *expr)                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Hallar la longitud de la expresion expr.                  *
* La longitud de una expresion es el numero de simbolos (functores y    *
* variables) que esta tiene.                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 25-Jun-1990	*
************************************************************************/

int
long_expr (nodo_expresion * expr)
{
/*--------------------------------------------------------------------*/
  int l;
  nodo_argum *arg;

  l = 0;
  if (expr == NULL)
    return (l);
  for (l = 1, arg = expr->argumentos; arg != NULL; arg = arg->sig)
    {
      l += long_expr (arg->expr);
    }
  return (l);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* nodo_expresion *reescribalo( nodo_expresion *expr,                    *
*  		      nodo_igualdad *regla,short *aplico, int *num_if ) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Reescribe la expresion, con la regla. Aplica, si es 	*
* 	       posible, la regla en la parte mas externa, mas izquierda *
*	       de expr. Ademas, indica si fue posible aplicar la regla. *
* DESCRIPCION : Busca, recursivamente, la primera subexpresion ( mas    *
*               externa, mas izquierda ) que unifique con la parte 	*
* 		 izquierda de la regla. Si encuentra la subexpresion    *
*		 reescribe la expresion con la aplicacion de la regla,  *
*               si no encuentra ninguna subexpresion retorna la 	*
*		 expresion sin modificaciones.				*
* PARAMETROS : 				         			*
*   nodo_expresion *expr    Apuntador a la expresion			*
*   nodo_igualdad  *regla   Apuntador a la regla. 		        *
*   short          *aplico  Parametro de salida, con informacion sobre  *
* 			     la aplicacion de la regla.			*
* RESULTADO : La expresion luego de ser reescrita.                      *
* PRE! : expresion, regla, aplico 					*
* POST! : ( aplico = 1, Existe SubExpr = Subexpresion de Expr,		*
*           Existe UMG1(lir(regla),SubExpr ) or (aplico = 0, No existe  *
*           Subexpr = Subexpresion de Expr que unifique con lir(regla)) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 25-Nov-1990	*
************************************************************************/

nodo_expresion *
reescribalo (nodo_expresion * expr, nodo_igualdad * regla,
	     short *aplico, int *num_if)
{
/*--------------------------------------------------------------------*/
  Ptr_tabla unif;
  nodo_expresion *expaux;
  nodo_argum *arg;

  assert (regla != NULL);	/* VTP. 8/May/1997 */
  *aplico = 0;
  if (expr == NULL)
    return (expr);
  if (!strcmp (regla->lado_der->clase, "IF") &&
      !strcmp (expr->clase, "IF") && (*num_if > 2))
    return (expr);
  else if (!strcmp (regla->lado_der->clase, "IF") &&
	   !strcmp (expr->clase, "IF"))
    (*num_if)++;
  unif = umg1 (lir (regla), expr);
  if (unif != NULL)
    {
      if (!strcmp (regla->lado_izq->clase, "IF"))
	(*num_if)--;
      expaux = apl (ldr (regla), unif, 1, aplico);
      *aplico = 1;
      return (expaux);
    }
  else if (strcmp(expr->clase,"IF")!=0)
    {
      for (arg = expr->argumentos;
	   (arg != NULL) && !(*aplico); arg = arg->sig)
	{
	  expaux = reescribalo (arg->expr, regla, aplico, num_if);
	  arg->expr = expaux;
	}
    }
    else if (strcmp(expr->clase,"IF")==0)
      {
        arg=expr->argumentos;
        expaux=reescribalo(arg->expr,regla,aplico,num_if);
        arg->expr=expaux;
      }
  return (expr);
/*--------------------------------------------------------------------*/
}

/************************************************************************
* int existe_regla( nodo_historia *his, nodo_igualdad *r)               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO : Indica la relga r,existe en la historia his.       	*
* RESULTADO : Retorna 1 si existe la regla en la historia, 0 si no      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR : Cesar Augusto Munoz Hurtado		FECHA : 25-Jun-1990	*
************************************************************************/

int
existe_regla (nodo_historia * his, nodo_igualdad * r)
{
/*--------------------------------------------------------------------*/
  nodo_historia *p;

  for (p = his; p != NULL && p->regla != r; p = p->sig);
  return (p != NULL);
/*--------------------------------------------------------------------*/
}


nodo_igualdad* crea_if(int valor)
  {
    nodo_expresion *te;
    nodo_igualdad *nif;
    nif=(nodo_igualdad *)malloc(sizeof(nodo_igualdad));
    HayMem(nif,sizeof(nodo_igualdad));
    if (valor)
      {
	sprintf(nif->li_str,"%s TRUE %s X1 %s X2 %s",claves[8],claves[9],
		claves[10],claves[15]);
        strcpy(nif->ld_str,"X1");
      }
    else
      {
	sprintf(nif->li_str,"%s FALSE %s X1 %s X2 %s",claves[8],claves[9],
		claves[10],claves[15]);
        strcpy(nif->ld_str,"X2");
      }
    nif->sig=NULL;
    nif->lado_izq=(nodo_expresion *)malloc(sizeof(nodo_expresion));
    HayMem(nif->lado_izq,sizeof(nodo_expresion));
    strcpy(nif->lado_izq->nombre,claves[8]);
    nif->lado_izq->tipo[0]='\0';
    strcpy(nif->lado_izq->clase,"IF");
    nif->lado_izq->argumentos=(nodo_argum *)malloc(sizeof(nodo_argum));
    HayMem(nif->lado_izq->argumentos,sizeof(nodo_argum));
    te=(nodo_expresion *)malloc(sizeof(nodo_expresion));
    HayMem(te,sizeof(nodo_expresion));
    if (valor)
      {
        strcpy(te->nombre,"TRUE");
      }
    else
      {
        strcpy(te->nombre,"FALSE");
      }
    strcpy(te->tipo,"BOOL");
    strcpy(te->clase,"INIC");
    te->argumentos=NULL;
    nif->lado_izq->argumentos->expr=te;
    nif->lado_izq->argumentos->sig=(nodo_argum *)malloc(sizeof(nodo_argum));
    HayMem(nif->lado_izq->argumentos->sig,sizeof(nodo_argum));;
    te=(nodo_expresion *)malloc(sizeof(nodo_expresion));
    HayMem(te,sizeof(nodo_expresion));
    strcpy(te->nombre,"X1");
    strcpy(te->tipo,"");
    strcpy(te->clase,"VAR");
    te->argumentos=NULL;
    nif->lado_izq->argumentos->sig->expr=te;
    nif->lado_izq->argumentos->sig->sig=(nodo_argum *)malloc(sizeof(nodo_argum));
    HayMem(nif->lado_izq->argumentos->sig->sig,sizeof(nodo_argum));;
    te=(nodo_expresion *)malloc(sizeof(nodo_expresion));
    HayMem(te,sizeof(nodo_expresion));
    strcpy(te->nombre,"X2");
    strcpy(te->tipo,"");
    strcpy(te->clase,"VAR");
    te->argumentos=NULL;
    nif->lado_izq->argumentos->sig->sig->expr=te;
    nif->lado_izq->argumentos->sig->sig->sig=NULL;

    nif->lado_der=(nodo_expresion *)malloc(sizeof(nodo_expresion));
    HayMem(nif->lado_der,sizeof(nodo_expresion));
    if (valor)
      {
        strcpy(nif->lado_der->nombre,"X1");
      }
    else
      {
        strcpy(nif->lado_der->nombre,"X2");
      }
    strcpy(nif->lado_der->tipo,"\0");
    strcpy(nif->lado_der->clase,"VAR");
    nif->lado_der->argumentos=NULL;

    return nif;
  }



/************************************************************************
* void fnlo(nodo_expresion *expr,nodo_historia **prim,                  *
*           nodo_historia **ult)                                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:  Reescribe expr con el sistema de reglas dado por los      *
*          axiomas de los TAD's.                                        *
* DESCRIPCION:  Primero intenta reescribir con los axiomas del Tad      *
*       Actual, luego con los demas axiomas.                            *
*       Retorna en prim la historia de demostracion. Y en ult la ultima *
*       demostracion. (Es su forma normal)                              *
* PARAMETROS:   nodo_expresion *expr. Expresion que se quiere reescribir*
*               nodo_historia **prim. Lista de la cabeza demostracion.  *
*               nodo_historia **ult.  Ultimo de la demostracion.(forma  *
*                                     normal)                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: C.A. Munoz                                    FECHA: 13-3-1991 *
************************************************************************/

void
fnlo (nodo_expresion * expr, nodo_historia ** prim,
      nodo_historia ** ult)
{
/*--------------------------------------------------------------------*/
  texpr mens_error;
  nodo_historia *aux;
  int no_fin;			/* Indica hasta cuando se aplica el sistema */
  nodo_expresion *cex;

  assert (ult != NULL && prim != NULL);		/* VTP. 8/May/1997 */

  aux = crear_historia (expr, NULL, "");
  (*ult) = aux;
  (*prim) = aux;
  if (copiaNodoExpresion(&cex,expr,mens_error)!=0)
    {
      printf(mensajes[166]);
      return;
    }
  no_fin=rewriteOutermost(&cex,especiales,1,prim,ult,mens_error);
  if (no_fin<0)
    {
      printf("Error rewriting: %s",mens_error);
      elimNodoExpresion(&cex);
      return;
    }
  strcpy(expr->nombre,cex->nombre);
  strcpy(expr->clase,cex->clase);
  strcpy(expr->tipo,cex->tipo);
  elimLArgumento(&(expr->argumentos));
  expr->argumentos=cex->argumentos;
  return;

/*--------------------------------------------------------------------*/
}

