/* ANAL_SINT_AXS.C                                                       */
/* Analiza sintacticamente el archivo "axiom_ed.tad" que contiene los   */
/* axiomas de un TAD.                                                    */
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "definic.h"
#include "apimanta.h"
#include "os_dep.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 8/May/1997 */


/**************************************************************************
* short analice_argum_generadora(FILE *fp, texpr mens_error,              *
*      texpr str_axioma,nodo_argum **a,nodo_nombre *dom_generadora,       *
*      nodo_expresion *expr_lado_izq) 					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente los argumentos de una generadora  *
*              (inicial o constructora).                                  *
* DESCRIPCION: Analiza uno a uno los argumentos de la generadora de acuedo*
*        al dominio de esta.                                              *
* PARAMETROS:  							          *
*   FILE        *fp            	  Ap. al archivo que contiene los axiomas.*
*   texpr       mens_error        Comentario acerca de un eventual error. *
*   texpr       str_axioma        Segmento ya analizado del axioma actual *
*   nodo_argum  **a               Ap. a la lista de los argumentos anali- *
*                                 zados de la generadora.                 *
*   nodo_nombre *dom_generadora   Ap. a la lista que contiene el dominio  *
*				  de la generadora. 			  *
*   nodo_expresion *expr_lado_izq Ap. a la estructura que contiene la ex- *
*                                 presion del lado izquierdo del axioma   *
*                                 que ya ha sido "parseada".		  *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas, str_axioma = 		  *
*         "////<Nom.Generadora>",     //// expresion ya parseada,         *
*         dom_generadora apunt. a la lista del dominio de <Nom.Generadora>*
*         expr_lado_izq apunt. a la estructura que contiene el lado iz-   *
*	  quierdo ya parseado del axioma actual.			} *
* POST!: {analice_argum_generadora = 0, Si los argumentos de la generadora*
*            tienen algun error sintactico,mens_error contiene una expli- *
*            cacion acerca del error detectado, o                         *
*         analice_argum_generadora = 1, Si los argumentos de la generadora*
*	     son correctos str_axioma = 				  *
*	     "////<Nom.Geneadora>(<arg1>,..,<argn>)"                      *
* !OJO!: La expr_lado_izq se usa realmente cuando se esta analizando el   *
*        lado derecho del axioma para verificar que las variables que ocu-*
*        rren al lado der. corresponden a variables del lado izq.         *
* LLAMA A:                                                                *
*    void deme_token         		short analice_argum_tipo   	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
analice_argum_generadora (FILE * fp, texpr mens_error, texpr str_axioma,
			  nodo_argum ** a, nodo_nombre * dom_generadora,
			  nodo_expresion * expr_lado_izq)
{
/*----------------------------------------------------------------------*/

  int error;
  nodo_nombre *dc;
  nodo_argum *arg, *ult_arg;
  texpr nombre_gen;

  assert(nombre_gen!=NULL && token!=NULL && fp!=NULL && str_axioma!=NULL);
     /* VTP. 25/May/1997 */
  strcpy (nombre_gen, token);
  deme_token (fp);
  error = es_error_esperaba ("(", token, mens_error);
  if (!error)
    {
      strcat (str_axioma, "(");
      (*a) = NULL;
      ult_arg = NULL;
      deme_token (fp);
      for (dc = dom_generadora; (dc != NULL) && (!error); dc = dc->sig)
	{
	  arg = (nodo_argum *) malloc (sizeof (nodo_argum));
          HayMem(arg,sizeof(nodo_argum)); /* VTP. 25/May/1997 */
	  arg->sig = NULL;
	  if (analice_argum_tipo (fp, mens_error, str_axioma,
 				  dc->nombre, &(arg->expr), expr_lado_izq))
	    {
	      if ((*a) == NULL)
		(*a) = arg;
	      else
		ult_arg->sig = arg
		  ;
	      ult_arg = arg;
	      if (dc->sig != NULL)
		{
		  strcat (str_axioma, ",");
		}
	      deme_token (fp);
	      if (!strcmp (token, ",") && (dc->sig != NULL))
		deme_token (fp);
	      else if (strcmp (token, ")") || (dc->sig != NULL))
		{
		  error = 1;
		  sprintf (mens_error, mensajes[32], nombre_gen);
		}
	    }
	  else
	    {
	      error = 1;
	    }
	}
      if (!error)
	{
	  error = es_error_esperaba (")", token, mens_error);
	  if (!error)
	    strcat (str_axioma, token);
	}
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_argum_select(FILE *fp, texpr mens_error,texpr str_axioma, *
*       nodo_argum **a,nodo_dom *dom_select,nodo_expresion *expr_lado_izq)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente los argumentos de una selectora.  *
* DESCRIPCION: Analiza uno a uno los argumentos de la selectora  de acuedo*
*        al dominio de esta.                                              *
* PARAMETROS:  							          *
*   FILE        *fp            	  Ap. al archivo que contiene los axiomas.*
*   texpr       mens_error        Comentario acerca de un eventual error. *
*   texpr       str_axioma        Segmento ya analizado del axioma actual *
*   nodo_argum  **a               Ap. a la lista de los argumentos anali- *
*                                 zados de la selectora .                 *
*   nodo_nombre *dom_select       Ap. a la lista que contiene el dominio  *
*				  de la selectora . 			  *
*   nodo_expresion *expr_lado_izq Ap. a la estructura que contiene la ex- *
*                                 presion del lado izquierdo del axioma   *
*                                 que ya ha sido "parseada".		  *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas, str_axioma = 		  *
*         "////<Nom.Selectora >",     //// expresion ya parseada,         *
*         dom_select    apunt. a la lista del dominio de <Nom.Selectora>  *
*         expr_lado_izq apunt. a la estructura que contiene el lado iz-   *
*	  quierdo ya parseado del axioma actual.			} *
* POST!: {analice_argum_select     = 0, Si los argumentos de la selectora *
*            tienen algun error sintactico,mens_error contiene una expli- *
*            cacion acerca del error detectado, o                         *
*         analice_argum_select     = 1, Si los argumentos de la selectora *
*	     son correctos str_axioma = 				  *
*	     "////<Nom.Selectora>(<arg1>,..,<argn>)"                      *
* !OJO!: La expr_lado_izq se usa realmente cuando se esta analizando el   *
*        lado derecho del axioma para verificar que las variables que ocu-*
*        rren al lado der. corresponden a variables del lado izq.         *
* LLAMA A:                                                                *
*    void deme_token         		short analice_argum_tipo   	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
analice_argum_select (FILE * fp, texpr mens_error, texpr str_axioma,
     nodo_argum ** a, nodo_dom * dom_select, nodo_expresion * expr_lado_izq)
{
/*----------------------------------------------------------------------*/

  short error;
  nodo_dom *ds;
  nodo_argum *arg, *ult_arg;
  texpr nombre_select;


  assert(nombre_select!=NULL && token!=NULL && fp!=NULL && mens_error!=NULL &&
          str_axioma!=NULL); /* VTP. 25/May/1997 */
  strcpy (nombre_select, token);

  if (dom_select==NULL) /* Es una selectora sin parametros
			   no debe tener parentesis ni nada */
    {
      return 1;
    }
  deme_token (fp);
  if ((error=es_error_esperaba ("(", token, mens_error))!=0)
    {
      return 0;
    }
  strcat (str_axioma, "(");
  deme_token (fp);

  (*a) = NULL;
  ult_arg = NULL;
  for (ds = dom_select; (ds != NULL) && (!error); ds = ds->sig)
    {
      arg = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem(arg,sizeof(nodo_argum)); /* VTP. 25/May/1997 */
      arg->sig = NULL;
      if (analice_argum_tipo (fp, mens_error, str_axioma,
			      ds->nombre, &(arg->expr), expr_lado_izq))
	{
	  if ((*a) == NULL)
	    (*a) = arg;
	  else
	    ult_arg->sig = arg;
	  ult_arg = arg;
	  if (ds->sig != NULL)
	    {
	      strcat (str_axioma, ",");
	    }
	  deme_token (fp);
	  if (!strcmp (token, ",") && (ds->sig != NULL))
	    deme_token (fp);
	  else if (strcmp (token, ")") || (ds->sig != NULL))
	    {
	      error = 1;
	      sprintf (mens_error, mensajes[32], nombre_select);
	    }
	}
      else
	{
	  error = 1;
	}
    }
  if (!error)
    {
      error = es_error_esperaba (")", token, mens_error);
      strcat (str_axioma, token);
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_argum_tipo(FILE *fp, texpr mens_error,texpr str_axioma,   *
*    cad20 nombre_tipo,nodo_expresion **exp,nodo_expresion *expr_lado_izq)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente una expresion (argumento dentro de*
*              otra expresion) que debe ser de un determinado tipo.       *
* DESCRIPCION: Analiza la siguiente expresion del archivo verificando que *
*        sea sintacticamente correcta y que sea del tipo requerido:       *
*        <nombre_tipo>.                                                   *
* PARAMETROS:  							          *
*   FILE        *fp            	  Ap. al archivo que contiene los axiomas.*
*   texpr       mens_error        Comentario acerca de un eventual error. *
*   texpr       str_axioma        Segmento ya analizado del axioma actual *
*   cad20       nom_tipo	  Nombre del tipo del cual debe ser la ex-*
*				  presion.				  *
*   nodo_expresion **exp          Ap. a la estructura que se construye a  *
*				  medida que se "parsea" la expresion.    *
*   nodo_expresion *expr_lado_izq Ap. a la estructura que contiene la ex- *
*                                 presion del lado izquierdo del axioma   *
*                                 que ya ha sido "parseada".		  *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas, str_axioma = 		  *
*         "////",  //// expresion ya parseada, nom_tipo = NT              *
*         expr_lado_izq apunt. a la estructura que contiene el lado iz-   *
*	  quierdo ya parseado del axioma actual.			} *
* POST!: {analice_argum_generadora = 0, Si la expresion leida contiene    *
*            algun error sintactico o no es de tipo NT,mens_error contiene*
*    	     una explicacion acerca del error detectado, o   		  *
*         analice_argum_generadora = 1, Si la expresion leida es correcta *
*	     str_axioma = "////<Expresion>"                               *
* !OJO!: La expr_lado_izq se usa realmente cuando se esta analizando el   *
*        lado derecho del axioma para verificar que las variables que ocu-*
*        rren al lado der. corresponden a variables del lado izq.         *
* LLAMA A:                                                                *
*    void deme_token         		short es_variable_tipo            *
*    short es_inicial_tipo_dom  	short es_constructura_tipo_dom 	  *
*    short analice_argum_generadora	short es_selectora_tipo_dom  	  *
*    short analice_argum_select   	  				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
analice_argum_tipo (FILE * fp, texpr mens_error, texpr str_axioma,
		    cad20 nombre_tipo, nodo_expresion ** exp, 
		    nodo_expresion * expr_lado_izq)
{
/*----------------------------------------------------------------------*/

  int error;
  nodo_nombre *dom_generadora=NULL;
  nodo_dom *dom_select=NULL;

  assert(token!=NULL && nombre_tipo!=NULL && str_axioma!=NULL &&
         mens_error!=NULL && exp!=NULL); /* VTP. 25/May/1997 */

  error = 0;
  (*exp) = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem(*exp,sizeof(nodo_expresion)); /* VTP. 25/May/1997 */
  (*exp)->argumentos = NULL;
  strcpy ((*exp)->nombre, token);
  strcpy ((*exp)->tipo, nombre_tipo);
  strcat (str_axioma, token);
  if (!strcmp (token, ""))
    {
      error = 1;
      sprintf (mens_error, mensajes[114], mensajes[28]);
    }
  else if (!es_identificador (token) && strcmp (token, claves[8]) &&
	   strcmp (token, claves[11]) && strcmp (token, claves[12]))
    {
      error = 1;
      sprintf (mens_error, mensajes[36], mensajes[28], token);
    }
  else if (es_inicial_tipo_dom (token, nombre_tipo, &dom_generadora))
    {
      strcpy ((*exp)->clase, "INIC");
      if (dom_generadora != NULL)
	{
	  error = (!analice_argum_generadora (fp, mens_error, str_axioma,
		     &((*exp)->argumentos), dom_generadora, expr_lado_izq));
	}
    }
  else if (es_constructora_tipo_dom (token, nombre_tipo, &dom_generadora))
    {
      strcpy ((*exp)->clase, "CONSTR");
      error = (!analice_argum_generadora (fp, mens_error, str_axioma,
		     &((*exp)->argumentos), dom_generadora, expr_lado_izq));
    }
  else if (es_selectora_tipo_dom (token, nombre_tipo, &dom_select))
    {
      strcpy ((*exp)->clase, "SELECT");
      error = (!analice_argum_select (fp, mens_error, str_axioma,
			 &((*exp)->argumentos), dom_select, expr_lado_izq));
    }
  else if (!strcmp (token, claves[12]) && !strcmp (nombre_tipo, "BOOL"))
    {
      strcpy ((*exp)->clase, "EQUAL");
      error = (!analice_equal (fp, mens_error, str_axioma,
			       &((*exp)->argumentos), expr_lado_izq));
    }
  else if (strcmp(token,claves[11])==0)
    {
      strcpy ((*exp)->clase, "ERROR");
    }
  else if ((expr_lado_izq == NULL) ||
	   (es_variable_tipo (token, expr_lado_izq, nombre_tipo)))
    {
      strcpy ((*exp)->clase, "VAR");
    }
  else
    {
      error=1; /* VTP. 17/Jun/1997 Se quito un case que seguia */
      sprintf (mens_error, mensajes[35], token, nombre_tipo);
      /*
      switch (tad_ed->tipo)
	{
	case NORMAL:
	case PROD_CARTESIANO:
	  error = 1;
	  break;
	case RESTRICCION:
	  if (!strcmp (tad_ed->info_tipo->nombre, nombre_tipo))
	    {
	      error = (!analice_argum_tipo (fp, mens_error, str_axioma,
				       tad_ed->nombre, exp, expr_lado_izq));
	    }
	  else
	    {
	      error = 1;
	    }
	  break;
	case COMPLETACION:
	  error = !(analice_argum_tipo (fp, mens_error, str_axioma,
			    tad_ed->info_tipo->nombre, exp, expr_lado_izq));
	  break; */
	    /*** VTP. 5/Mar/1997 Decia EXTENSION en vez de NT_EXTENSION
	        pero el nombre renia con el definido en una libreria del
		DJGPP ***/
/*	case NT_EXTENSION:
	case VARIACION:
	  error = 1; */
	/***    error = !(analice_argum_tipo(fp,mens_error,str_axioma,
        			tad_ed->info_tipo->nombre,exp,expr_lado_izq)
		      ||
                      analice_argum_tipo(fp,mens_error,str_axioma,
        		     tad_ed->info_tipo->sig->nombre,exp,expr_lado_izq));
	    ********/
/*	  break;
	default:
	  error = 1;
	  break;
	}
      if (error)
	{
	  sprintf (mens_error, mensajes[35], token, nombre_tipo);
	} */
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}



/**************************************************************************
int exprIguales(nodo_expresion *n1,nodo_expresion *n2)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
  This function returns 1 if the expresions it receives are the same
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
  AUTHOR: Vladimir Tamara                         DATE: 30-Apr-98 
**************************************************************************/
int exprIguales(nodo_expresion *n1,nodo_expresion *n2)
{
  nodo_argum *na1,*na2;
  int dif;
  if (n1==NULL && n2==NULL)
    {
      return 1;
    }
  if (n1==NULL || n2==NULL)
    {
      return 0;
    }
  if (strcmp(n1->nombre,n2->nombre) || strcmp(n1->tipo,n2->tipo) 
      || strcmp(n1->clase,n2->clase) )
    {
      return 0;
    }
  dif=0;
  for (na1=n1->argumentos,na2=n2->argumentos;
       na1!=NULL && na2!=NULL && !dif; 
       na1=na1->sig,na2=na2->sig)
    {
      dif=!exprIguales(na1->expr,na2->expr);
    }
  
  return !dif;
  
}

/**************************************************************************
short variablesRepetidas(nodo_expresion *e,char *varlist)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
  This function returns 1 if the expresion e has any variable already
  in the variable list varlist, it also adds variables of expresion e to 
  varlist.  If the expression do not have repeated variables it returns 0
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
  AUTHOR: Vladimir Tamara                         DATE: 24-Apr-98 
**************************************************************************/
short variablesRepetidas(nodo_expresion *e,char *varlist)
{
  char idvar[100];
  nodo_argum *a;

  assert(varlist!=NULL);
  if (e==NULL)
    return 0;
  if (strcmp(e->clase,"VAR")==0)
    {
      sprintf(idvar,"| %s",e->nombre);
      if (strstr(varlist,idvar)!=NULL)
	{
	  return 1;
	}
      strcat(varlist,idvar);
      return 0;
    }
   for (a=e->argumentos;a!=NULL;a=a->sig)
    {
      if (variablesRepetidas(a->expr,varlist)!=0)
	{
	  return 1;
	}
    }
  return 0;
}


void 
cambiaPosfijoVariables(nodo_expresion *e,int agrega,char *pos)
{
  nodo_argum *ar;

  if (e==NULL)
    return;
  if (strcmp_nocase(e->clase,"VAR")==0)
    {
      if (agrega)
	{ strcat(e->nombre,pos);}
      else if (strcmp(e->nombre+strlen(e->nombre)-strlen(pos),pos)==0)/* Quitar posfijo */
	{ e->nombre[strlen(e->nombre)-strlen(pos)]='\0'; }
    }
  for (ar=e->argumentos;ar!=NULL;ar=ar->sig)
    cambiaPosfijoVariables(ar->expr,agrega,pos);
}



/* Retorna verdadero si un conjunto de axiomas es ambiguo
   Busca pares criticos.
   Todo par critico debe resolverse trivialmente es decir al instanciar
   uno en el otro los lados derechos deben coincidir
axs es la cabeza de la lista de axiomas para una selectora
mens_error es un buffer para retornar mensjaes de error
Por Vladimir Tamara
7/May/1998
*/
int axiomas_ambiguos(nodo_igualdad *axs,char *mens_error)
{
  
  nodo_igualdad *ax1,*ax2;
  short aplico;
  nodo_expresion *expaux1,*expaux2;
  Ptr_tabla unif;
  
  for (ax1=axs;ax1!=NULL;ax1=ax1->sig)
    for (ax2=ax1->sig;ax2!=NULL; ax2=ax2->sig)
      {
	cambiaPosfijoVariables(lir(ax1),1,"_1");
	cambiaPosfijoVariables(lir(ax2),1,"_2");
	unif=umg(lir(ax1),lir(ax2));
	if (unif!=NULL)
	  {
	    if ((expaux1 = apl (ldr (ax1), unif, 1, &aplico))==NULL)
	      {
		cambiaPosfijoVariables(lir(ax1),1,"_1");
		cambiaPosfijoVariables(lir(ax2),1,"_2");
		elimLTabla(&unif);
	      }
	    if ((expaux2 = apl (ldr (ax2), unif, 1, &aplico))==NULL)
	      {
		cambiaPosfijoVariables(lir(ax1),1,"_1");
		cambiaPosfijoVariables(lir(ax2),1,"_2");
		elimNodoExpresion(&expaux1);
		elimLTabla(&unif);
	      }
	    if (!exprIguales(expaux1,expaux2))
	      {
		sprintf(mens_error,mensajes[176],
			ax1->li_str,ax2->li_str);
		cambiaPosfijoVariables(lir(ax1),0,"_1");
		cambiaPosfijoVariables(lir(ax2),0,"_2");
		elimNodoExpresion(&expaux1);
		elimNodoExpresion(&expaux2);
		elimLTabla(&unif);
		return 1;
	      }
	    elimNodoExpresion(&expaux1);
	    elimNodoExpresion(&expaux2);
	  }
	cambiaPosfijoVariables(lir(ax1),0,"_1");
	cambiaPosfijoVariables(lir(ax2),0,"_2");
	elimLTabla(&unif);
      }
  return 0;
}


/**************************************************************************
* short analice_axiomas(texpr mens_error,nodo_igualdad **cabeza_axs,      *
*                       cad50 nom_select,short *tipo_error)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente los axiomas en el archivo         *
*              axiom_ed.tad y verificar que constituyan una buena 	  *
*              definicion.						  *
* DESCRIPCION: Lee cada axioma y lo verifica sintactica/. Si todos estan  *
*        correctos, invoca a la funcion que se encarga de verificar que   *
*        sea una definicion completa.                                     *
* PARAMETROS: 								  *
*  texpr  mens_error     	Para retornar un mensaje de informacion en*
*				caso de error.                		  *
*  nodo_igualdad **cabeza_axs   Para retornar un apuntador a la cabeza de *
*				la lista de axiomas.			  *
*  cad50         nom_select     Nombre de la selectora de la cual se estan*
*                               analizando los axiomas.                   *
*  short         *tipo_error    En caso de error retorna 1 si el error es *
*                               es sintactico y 0 si es semantico (el     *
*                               conjunto de axiomas no es "completo").    *
* V.GLOBALES: texpr  token      Almacena el ultimo token leido.           *
* RESULTADO:  short             0, Si hubo error (sintactico o semantico) *
*                               1, Sino.                                  *
* PRE!:  { "axiom_ed.tad" contine un conjunto de axiomas, nom_select=NS} *
* POST!: { analice_axiomas = 0, Si los axiomas de archivo tienen algun    *
*            error. Ademas mens_error contiene una explicacion sobre este *
*	     y *tipo_error= 0, si el error es sintactico y 1 si es semant.*
*          analice_axiomas = 1, Si los axiomas de archivo no tienen ningun*
*            error. Ademas *cabeza_axs apunta a la cabeza de la lista de  *
*	     axiomas.							} *
* LLAMA A: 								  *
*   void   deme_token      		short  analice_expr_izq           *
*   short  analice_expr_der             short  es_definicion_completa     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
analice_axiomas (texpr mens_error, nodo_igualdad ** cabeza_axs,
		 cad50 nom_select, short *tipo_error)
 {
/*----------------------------------------------------------------------*/

  FILE *fp;
  texpr str_axioma;
  short error;
  texpr tipo;
  nodo_igualdad *ig, *ult_ax;
  char varlist[1000];

  assert(mens_error!=NULL && str_axioma!=NULL && token!=NULL );
    /* VTP. 25/May/1997 */
  error = 0;
  (*cabeza_axs) = NULL;
  ult_ax = NULL;
  fp = fopen ("axiom_ed.tad", "r");
  FileOpen(fp,"axiom_ed.tad");
  num_ed = 1;
  deme_token (fp);
  while ((!feof (fp)) && (!error))
    {
      strcpy (str_axioma, token);
      ig = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
      HayMem(ig,sizeof(nodo_igualdad)); /* VTP. 25/May/1997 */
      ig->sig = NULL;
      if (analice_expr_izq (fp, mens_error, str_axioma, &(ig->lado_izq),
			    tipo, nom_select))
	{
	  /* VTP. 25/Abr/1998.  Error if axiom is  not left-linear */
	  varlist[0]='\0';
	  if (variablesRepetidas(ig->lado_izq,varlist))
	    {
	      sprintf(mens_error,mensajes[175],nom_select);
	      *tipo_error=7;
              fclose(fp);
	      return 0;
	    }
	  /* VTP. 31/ABR/1998. ERROR is not allowed at left side */
	  if (strstr(varlist,"| ERROR")!=NULL)
	    {
	      sprintf(mens_error,mensajes[177],nom_select);
	      *tipo_error=7;
              fclose(fp);
	      return 0;
	    }
	  deme_token (fp);
	  if (es_error_esperaba ("=", token, mens_error))
	    {
              fclose(fp);
	      return (0);
	    }
	  strcpy (ig->li_str, str_axioma);
	  strcpy (str_axioma, "");
	  deme_token (fp);
	  if (analice_expr_der (fp, mens_error, str_axioma, &(ig->lado_der),
				ig->lado_izq, tipo))
	    {
	      strcpy (ig->ld_str, str_axioma);
	      if ((*cabeza_axs) == NULL)
		(*cabeza_axs) = ig;
	      else
		ult_ax->sig = ig;
	      ult_ax = ig;
	      deme_token (fp);
	    }
	  else
	    error = 1;
	}
      else
	error = 1;
    }
  fclose (fp);
  if (error)
    {
      *tipo_error = 2;
    }
  else
    {
      if (!es_definicion_completa (*cabeza_axs, mens_error))
	{
	  error = 1;
	  *tipo_error = 3;
	}
    }

  /* VTP. 30/Abr/1998.  We have to test for non-ambiguity in axioms */
  if (!error && axiomas_ambiguos(*cabeza_axs,mens_error))
    {
      error=1;
      *tipo_error=7;
    }

  return (short)(!error);
/*----------------------------------------------------------------------*/
}


/**************************************************************************
short analice_axiomas_m32 (FILE *fp,char * mens_error,
                     nodo_tad *ted, short *tipo_error)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 Analizar sintacticamente los axiomas en el archivo fp
 y verificar que constituyan una buena 	definicion.
 ENTRADAS
      FILE *fp Archivo abierto y ubicado al comienzo
      char * mens_error  Buffer para retornar mensajes de error
      nodo_tad *ted      TAD siendo compilado
      short *tipo_error   Devuelve el tipo de error en esta variable
 RETORNADO
      0 si hay error
      !=0 si no hay error
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90
MODIFICACIONES:  Adaptado por VTP. 6/Jul/1997
**************************************************************************/

short
analice_axiomas_m32 (FILE *fp,char * mens_error,
                     nodo_tad *ted)
{
/*----------------------------------------------------------------------*/

  texpr str_axioma;
  short error;
  texpr tipo;
  nodo_igualdad *ig, *ult_ax;
  nodo_igualdad **cabeza_axs;
  nodo_select *sel;
  cad50 nom_select;
  char varlist[1500];

  assert(mens_error!=NULL);
  assert(fp!=NULL);
  assert(token!=NULL );

  error = 0;
  cabeza_axs = NULL;
  deme_token (fp);
  while (!feof(fp) && (!error))
    {
      sel=busque_selectora (token, ted->selectoras);
      if (sel==NULL) /* No es selectora del TAD */
        {
          error=1;
	  sprintf(mens_error,mensajes[173],token);
        }
      else
        {
          cabeza_axs=&(sel->axiomas);
          strcpy(nom_select,token);
        }
      while (!strcmp(nom_select,token) && (!feof (fp)) && (!error))
        {
          strcpy (str_axioma, token);
          ig = (nodo_igualdad *) malloc (sizeof (nodo_igualdad));
          HayMem(ig,sizeof(nodo_igualdad)); /* VTP. 25/May/1997 */
          ig->sig = NULL;
          if (analice_expr_izq (fp, mens_error, str_axioma, &(ig->lado_izq),
                    tipo, nom_select))
            {
	      /* VTP. 24/Abr/1998.  Error if axiom is  not left-linear */
	      varlist[0]='\0';
	      if (variablesRepetidas(ig->lado_izq,varlist))
		{
 		  sprintf(mens_error,mensajes[175],nom_select);
		  return 0;
		}
	      /* VTP. 31/Abr/1998. ERROR is not allowed at left side */ 
	      if (strstr(varlist,"| ERROR")!=NULL)
		{
		  sprintf(mens_error,mensajes[177],nom_select);
		  return 0;
		}
              deme_token (fp);
              if (es_error_esperaba ("=", token, mens_error))
                {
                  return (0);
                }
              strcpy (ig->li_str, str_axioma);
              strcpy (str_axioma, "");
              deme_token (fp);
              if (analice_expr_der (fp, mens_error, str_axioma, &(ig->lado_der),
                        ig->lado_izq, tipo))
                {
                  strcpy (ig->ld_str, str_axioma);
                  if ((*cabeza_axs) == NULL)
                    (*cabeza_axs) = ig;
                  else
                    {
                      for (ult_ax=*cabeza_axs;ult_ax->sig!=NULL;ult_ax=ult_ax->sig);
                      ult_ax->sig = ig;
                      /* assert(ig->sig==NULL); */
                    }
                  deme_token (fp);
                }
              else
                error = 1;
          }
          else
              error = 1;
        }
      if  (!error && !es_definicion_completa (*cabeza_axs, mens_error))
	{
	  error = 1;
	}
      if (!error && axiomas_ambiguos(*cabeza_axs,mens_error))
	{
	  error=1;
	}
    }

  if (!error)
    {
      /* We have to see if all the declared selectors have axioms */
      for (sel=ted->selectoras;sel!=NULL && sel->axiomas!=NULL
	  ;sel=sel->sig);
      error =(short)(sel!=NULL?1:0);
      if (error)
	{
	  sprintf(mens_error,mensajes[172],sel);
	}
    }

  return (short)(!error);
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* short analice_expr_der(FILE *fp, texpr mens_error,texpr str_axioma,     *
*      nodo_expresion **exp,nodo_expresion *expr_lado_izq,texpr tipo_izq) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente la parte derecha del axioma actual*
*              que debe ser del mismo tipo que el lado izquierdo.         *
* DESCRIPCION: Analiza uno a uno los siguientes tokens del archivo veri-  *
*        ficando que conformen una expresion sintacticamente correcta y   *
*   	 del tipo requerido: <tipo_izq>.				  *
* PARAMETROS:  							          *	
*   FILE        *fp            	  Ap. al archivo que contiene los axiomas.*
*   texpr       mens_error        Comentario acerca de un eventual error. *
*   texpr       str_axioma        Segmento ya analizado del axioma actual *
*   nodo_expresion **exp          Ap. a la estructura que se construye a  *
*				  medida que se "parsea" la expresion.    *
*   nodo_expresion *expr_lado_izq Ap. a la estructura que contiene la ex- *
*                                 presion del lado izquierdo del axioma   *
*                                 que ya ha sido "parseada".		  *
*   texpr       tipo_izq	  Nombre del tipo del cual debe ser la ex-*
*				  presion (tipo del lado izq).		  *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas, str_axioma = 		  *
*         "////",  //// expresion ya parseada, tipo_izq = TI              *
*         expr_lado_izq apunt. a la estructura que contiene el lado iz-   *
*	  quierdo ya parseado del axioma actual.			} *
* POST!: {analice_expr_der         = 0, Si la expresion leida contiene    *
*            algun error sintactico o no es de tipo TI,mens_error contiene*
*    	     una explicacion acerca del error detectado, o   		  *
*         analice_expr_der         = 1, Si la expresion leida es correcta *
*	     str_axioma = "////<Expresion>"                               *
* !OJO!: La expr_lado_izq se usa para verificar que las variables que ocu-*
*        rren al lado der. corresponden a variables del lado izq.         *
* LLAMA A:                                                                *
*    void deme_token         		short es_variable_tipo            *	
*    short es_inicial_tipo_dom  	short es_constructura_tipo_dom 	  *
*    short analice_argum_generadora	short es_selectora_tipo_dom  	  *
*    short analice_argum_select   	short analice_if                  *
*    short analice_equal						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
analice_expr_der (FILE * fp, texpr mens_error, texpr str_axioma,
		  nodo_expresion ** exp, nodo_expresion * expr_lado_izq,
		  texpr tipo_izq)
{
/*----------------------------------------------------------------------*/

  int error;

 /*** Alvaro
  nodo_nombre 	*tipo_der;
 ***/
  nodo_nombre *dom_generadora=NULL; /* VTP. 25/may/1997 */
  nodo_dom *dom_select=NULL; /* VTP. 25/may/1997 */

  assert(exp!=NULL && token!=NULL && tipo_izq!=NULL && str_axioma!=NULL); /* VTP. 25/May/1997 */
  error = 0;
  (*exp) = (nodo_expresion *) malloc (sizeof (nodo_expresion));
  HayMem(*exp,sizeof(nodo_expresion)); /* VTP. 25/May/1997 */
  (*exp)->argumentos = NULL;
  strcpy ((*exp)->nombre, token);
  strcpy ((*exp)->tipo, tipo_izq);
  strcat (str_axioma, token);
  if ((expr_lado_izq == NULL) && (!strcmp (tipo_izq, "")))
    determine_tipo_teo (tipo_izq);
  if (!strcmp (token, ""))
    {
      error = 1;
      sprintf (mens_error, mensajes[114], mensajes[28]);
    }
  else if (!es_identificador (token) && strcmp (token, claves[8]) &&
	   strcmp (token, claves[11]) && strcmp (token, claves[12]))
    {
      error = 1;
      sprintf (mens_error, mensajes[36], mensajes[28], token);
    }
  else if (!strcmp (token, claves[11]))
    {
      strcpy ((*exp)->clase, "ERROR");
    }
  else if (es_inicial_tipo_dom (token, tipo_izq, &dom_generadora))
    {
      strcpy ((*exp)->clase, "INIC");
      if (dom_generadora != NULL)
	{
	  error = (!analice_argum_generadora (fp, mens_error, str_axioma,
		     &((*exp)->argumentos), dom_generadora, expr_lado_izq));
	}
    }
  else if (es_constructora_tipo_dom (token, tipo_izq, &dom_generadora))
    {
      strcpy ((*exp)->clase, "CONSTR");
      error = (!analice_argum_generadora (fp, mens_error, str_axioma,
		     &((*exp)->argumentos), dom_generadora, expr_lado_izq));
    }
  else if (es_selectora_tipo_dom (token, tipo_izq, &dom_select))
    {
      strcpy ((*exp)->clase, "SELECT");
      error = (!analice_argum_select (fp, mens_error, str_axioma,
			 &((*exp)->argumentos), dom_select, expr_lado_izq));
    }
  else if (!strcmp (token, claves[8]))
    {
      strcpy ((*exp)->clase, "IF");
      strcat (str_axioma, "(");
      error = (!analice_if (fp, mens_error, str_axioma, &((*exp)->argumentos),
			    expr_lado_izq, tipo_izq));
    }
  else if ((!strcmp (token, claves[12])) && (!strcmp (tipo_izq, "BOOL")))
    {
      strcpy ((*exp)->clase, "EQUAL");
      error = (!analice_equal (fp, mens_error, str_axioma, &((*exp)->argumentos),
			       expr_lado_izq));
    }
  else if ((expr_lado_izq == NULL) ||
	   es_variable_tipo (token, expr_lado_izq, tipo_izq))
    {
      strcpy ((*exp)->clase, "VAR");
    }
  else
    {
      error=1;
      sprintf (mens_error, mensajes[35], token, tipo_izq);

      /* VTP. 1/Jul/1997.
         Se quito un switch/case que hacia entrar en recursion
      switch (tad_ed->tipo)
	{
	case NORMAL:
	case PROD_CARTESIANO:
	  error = 1;
	  break;
	case RESTRICCION:
	  if (!strcmp (tad_ed->info_tipo->nombre, tipo_izq))
	    {
	      error = (!analice_expr_der (fp, mens_error, str_axioma, exp,
					  expr_lado_izq, tad_ed->nombre));
	    }
	  else
	    {
	      error = 1;
	    }
	  break;
	case COMPLETACION:
	  error = !(analice_expr_der (fp, mens_error, str_axioma,
			    exp, expr_lado_izq, tad_ed->info_tipo->nombre));
	  break;
	     VTP. 5/Mar/1997 Decia EXTENSION en vez de NT_EXTENSION
	        pero el nombre renia con el definido en una libreria del
		DJGPP
	case NT_EXTENSION:
	case VARIACION:
	  error = 1;
	  *******
            error = !(analice_expr_der(fp,mens_error,str_axioma,exp,
				expr_lado_izq,tad_ed->info_tipo->nombre)
		      ||
            	      analice_expr_der(fp,mens_error,str_axioma,exp,
		      		expr_lado_izq,tad_ed->info_tipo->sig->nombre));
	    *********
	  break;
	default:
	  error = 1;
	  break;
	}
        if (error)
	{
	  sprintf (mens_error, mensajes[35], token, tipo_izq);
	}

        */
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_equal(FILE *fp,texpr mens_error,texpr str_axioma,         *
*		      nodo_argum **a,nodo_expresion *expr_lado_izq)       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente una expresion EQUAL(...,...).     *
* DESCRIPCION: Analiza cada uno de los argumentos del equal y verifica que*
*        sean del mismo tipo.                                             *
* PARAMETROS:  							          *
*   FILE        *fp            	  Ap. al archivo que contiene los axiomas.*
*   texpr       mens_error        Comentario acerca de un eventual error. *
*   texpr       str_axioma        Segmento ya analizado del axioma actual *
*   nodo_argum  **a               Ap. a la lista de los argumentos anali- *
*                                 zados del equal.                        *
*   nodo_expresion *expr_lado_izq Ap. a la estructura que contiene la ex- *
*                                 presion del lado izquierdo del axioma   *
*                                 que ya ha sido "parseada".		  *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas, str_axioma = "////EQUAL",	  *
*         //// expresion ya parseada, expr_lado_izq apunt. a la estructura*
* 	  que contiene el lado izquierdo ya parseado del axioma actual.	} *
* POST!: {analice_equal = 0, Si los argumentos del equal tienen algun     *
*            error sintactico,mens_error contiene una explicacion acerca  *
*            del error detectado, o                         		  *
*         analice_equal = 1, Si los argumentos del equal son correctos,   *
*            str_axioma = "////EQUAL(arg1>,<arg2>)".                    } *
* !OJO!: Una expresion EQUAL(...,...) solo puede aparecer al lado derecho *
*        del axioma.                                                      *
*        Se requiere que el primer argumento del equal permita deducir su *
*        tipo (para poder verificar la igual de tipos de los dos args.).  *
* LLAMA A: void   deme_token                                              *
*          short  determine_tipo_equal   short analice_expr_der           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
analice_equal (FILE * fp, texpr mens_error, texpr str_axioma,
	       nodo_argum ** a, nodo_expresion * expr_lado_izq)
{
/*----------------------------------------------------------------------*/

  short error;
  nodo_argum *arg;
  nodo_expresion *arg1, *arg2;
  cad20 tipo_eq;

  assert(mens_error!=NULL && str_axioma!=NULL ); /* VTP. 25/May/1997 */
  deme_token (fp);
  error = es_error_esperaba ("(", token, mens_error);
  if (error)
    return (0);
  strcat (str_axioma, "(");
  if (determine_tipo_equal (fp, mens_error, expr_lado_izq, tipo_eq))
    {
      arg = (nodo_argum *) malloc (sizeof (nodo_argum));
      HayMem(arg,sizeof(nodo_argum)); /* VTP. 25/May/1997 */
      arg->sig = NULL;
      (*a) = arg;
      if (analice_expr_der (fp, mens_error, str_axioma, &(arg->expr),
			    expr_lado_izq, tipo_eq))
	{
	  arg = (nodo_argum *) malloc (sizeof (nodo_argum));
          HayMem(arg,sizeof(nodo_argum)); /* VTP. 25/May/1997 */
	  arg->sig = NULL;
	  (*a)->sig = arg;
	  strcat (str_axioma, ",");
	  deme_token (fp);
	  error = es_error_esperaba (",", token, mens_error);
	  if (error)
	    return (0);
	  deme_token (fp);
	  if (analice_expr_der (fp, mens_error, str_axioma, &(arg->expr),
				expr_lado_izq, tipo_eq))
	    {
	      deme_token (fp);
	      error = es_error_esperaba (")", token, mens_error);
	      if (error)
		return (0);
	      strcat (str_axioma, token);
	      if ((arg->expr->tipo)[0]=='\0')
		{ 
		  strcpy(arg->expr->tipo,tipo_eq);
		  strcpy((*a)->expr->tipo,tipo_eq);
		}
	    }
	  else
	    {
	      error = 1;
	    }
	}
      else
	{
	  error = 1;
	}
    }
  else
    {
      error = 1;
    }
  if (!error)
    {
      arg1 = (*a)->expr;
      arg2 = (*a)->sig->expr;
      if (!strcmp (arg1->tipo, ""))
	strcpy (arg1->tipo, arg2->tipo);
      if (!strcmp (arg2->tipo, ""))
	strcpy (arg2->tipo, arg1->tipo);
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_expr_booleana(FILE *fp,texpr condicion,texpr mens_error,  *
*                       nodo_expresion **e,nodo_expresion *expr_lado_izq) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente una expresion de tipo booleano.   *
* DESCRIPCION: Analiza uno a uno los siguientes tokens del archivo veri-  *
*        ficando que conformen una expresion sintacticamente correcta y   *
*   	 del tipo booleano.                   				  *
* PARAMETROS:  						                  *
*   FILE        *fp            	  Ap. al archivo que contiene los axiomas.*
*   texpr       condicion         Segmento ya analizado de la expresion   *
*                                 booleana actual.                        *
*   texpr       mens_error        Comentario acerca de un eventual error. *
*   nodo_expresion **e            Ap. a la estructura que se construye a  *
*				  medida que se "parsea" la expresion.    *
*   nodo_expresion *expr_lado_izq Ap. a la estructura que contiene la ex- *
*                                 presion del lado izquierdo del axioma   *
*                                 que ya ha sido "parseada".		  *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas, condicion = "////",  ////      *
*	  expresion booleana ya parseada, expr_lado_izq apunt. a la es-   *
*   	  tructura que contiene el lado izquierdo ya parseado del axioma  *
*         actual.							} *
* POST!: {analice_expr_booleana    = 0, Si la expresion leida contiene    *
*            algun error sintactico o no es booleana, mens_error contiene *
*    	     una explicacion acerca del error detectado, o   		  *
*         analice_expr_booleana    = 1, Si la expresion leida es correcta *
*	     condicion  = "<Expresion Booleana>"                          *
* LLAMA A: void   deme_token             short  analice_expr_booleana     *
*	   short  analice_equal          short  es_inicial_tipo           *
*          short   es-selectora_tipo_dom short  analice_argum_select      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
analice_expr_booleana (FILE * fp, texpr condicion, texpr mens_error,
		       nodo_expresion ** e, nodo_expresion * expr_lado_izq)
{
/*----------------------------------------------------------------------*/

  int error;
  nodo_dom *dom_select=NULL;
  /*** Alvaro
    ,*ds;
  ***/
  texpr cadena, nom_tad;
  nodo_tad *p;

  assert(fp!=NULL && token!=NULL); /* VTP. 25/May/1997 */
  error = 0;
  deme_token (fp);
  if (!strcmp (token, "("))
    {
      if (analice_expr_booleana (fp, condicion, mens_error, e, expr_lado_izq))
	{
	  deme_token (fp);
	  error = es_error_esperaba (")", token, mens_error);
	}
      else
	{
	  error = 1;
	}
    }
  else
    {
      (*e) = (nodo_expresion *) malloc (sizeof (nodo_expresion));
      HayMem(*e,sizeof(nodo_expresion)); /* VTP. 25/May/1997 */
      (*e)->argumentos = NULL;
      strcpy ((*e)->nombre, token);
      strcpy ((*e)->tipo, "BOOL");
      strcat (condicion, token);
      if (!strcmp (token, ""))
	{
	  error = 1;
	  sprintf (mens_error, mensajes[114], mensajes[28]);
	}
      else if (!es_identificador (token) && strcmp (token, claves[8]) &&
	       strcmp (token, claves[11]) && strcmp (token, claves[12]))
	{
	  error = 1;
	  sprintf (mens_error, mensajes[36], mensajes[28], token);
	}
      else if (!strcmp (token, claves[12]))
	{
	  strcpy ((*e)->clase, "EQUAL");
	  error = (!analice_equal (fp, mens_error, condicion,
				   &((*e)->argumentos), expr_lado_izq));
	}
      else if (es_inicial_tipo (token, "BOOL"))
	{
	  strcpy ((*e)->clase, "INIC");
	}
      else if (es_selectora_tipo_dom (token, "BOOL", &dom_select))
	{
	  strcpy ((*e)->clase, "SELECT");
	  error = (!analice_argum_select (fp, mens_error, condicion,
			   &((*e)->argumentos), dom_select, expr_lado_izq));
	}
      else if (es_prefijo ("EQ_", token))
	{
	  strcpy (cadena, token);
	  parta_expresion (cadena, nom_tad, '_');
	  p = existe_tad (nom_tad);
	  if (p != NULL &&
	      es_selectora_con_codom_tipo (token, "BOOL", p->selectoras,
					   &dom_select))
	    {
	      strcpy ((*e)->clase, "SELECT");
	      error = (!analice_argum_select (fp, mens_error, condicion,
			   &((*e)->argumentos), dom_select, expr_lado_izq));
	    }
	  else
	    {
	      error = 1;
	      sprintf (mens_error, mensajes[35], token, "BOOL");
	    }
	}
      else if (es_identificador (token))
	{
/*-- Se supone que es una variable Booleana --*/
	  strcpy ((*e)->clase, "VAR");
	}
      else if (strcmp(token,"ERROR")==0)
	{
	  strcpy((*e)->clase,"ERROR");
	}
      else
	{
	  error = 1;
	  strcpy (mens_error, mensajes[33]);
	}
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_expr_izq(FILE *fp, texpr mens_error,texpr str_axioma,     *
*                   nodo_expresion **e,texpr tipo_codom,cad50 nom_select) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente el lado izquierdo de un axioma.   *
* DESCRIPCION: Analiza uno a uno los siguientes tokens del archivo veri-  *
*        ficando que conformen una expresion sintacticamente correcta y   *
*   	 cuyo funtor mas externo sea <nom_select>.
* PARAMETROS:  							          *	
*   FILE        *fp            	  Ap. al archivo que contiene los axiomas.*
*   texpr       mens_error        Comentario acerca de un eventual error. *
*   texpr       str_axioma        Segmento ya analizado del axioma actual *
*   nodo_expresion **e            Ap. a la estructura que se construye a  *
*				  medida que se "parsea" la expresion.    *
*   texpr       tipo_codom 	  Nombre del tipo (codominio) de la selec-*
*				  tora <nom_select>.              *
*   cad50       nom_select 	  Nombre de la selectora que debe ser el  *
*				  funtor mas externo del lado izq. del    *
*				  axioma.				  *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas, nom_select = NS y 		  *
*	  str_axioma = "NS".					        } *
* POST!: {analice_expr_izq = 0, Si la expresion leida contiene algun error*
*            sintactico, mens_error contiene una explicacion acerca del   *
*            error detectado, o   		  			  *
*         analice_expr_izq = 1, Si la expresion leida es correcta,        *
*	     str_axioma = "////<Expresion>" , (*e) apunta a la estructura *
*	     parseada de <Expresion> y tipo_codom contiene el nombre del  *
* 	     tipo o codominio de NS.					} *
* LLAMA A: short  es_selectora_dom_codom   short  analice_argum_select    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
analice_expr_izq (FILE * fp, texpr mens_error, texpr str_axioma,
		  nodo_expresion ** e, texpr tipo_codom, cad50 nom_select)
{
/*----------------------------------------------------------------------*/
  int error;
  nodo_dom *tipo_dom=NULL;

  assert(token!=NULL && mens_error!=NULL && str_axioma!=NULL &&
         nom_select!=NULL && tipo_codom!=NULL); /* VTP. 25/May/1997 */
  error = es_error_esperaba ("", token, mens_error);
  if (error)
    return (0);
  if ((!strcmp (str_axioma, nom_select)) &&
      (es_selectora_dom_codom (str_axioma,tad_ed->selectoras, 
			       &tipo_dom, tipo_codom)))
    {
      (*e) = (nodo_expresion *) malloc (sizeof (nodo_expresion));
      HayMem(*e,sizeof(nodo_expresion)); /* VTP. 25/May/1997 */
      strcpy ((*e)->nombre, str_axioma);
      strcpy ((*e)->tipo, tipo_codom);
      strcpy ((*e)->clase, "SELECT");
      error = (!analice_argum_select (fp, mens_error, str_axioma,
				      &((*e)->argumentos), tipo_dom, NULL));
    }
  else
    {
      error = 1;
      sprintf (mens_error, mensajes[34], nom_select);
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short analice_if(FILE *fp,texpr mens_error,texpr str_axioma,            *
*            nodo_argum **a,nodo_expresion *expr_lado_izq,texpr tipo_izq) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Analizar sintacticamente una expresion                     *
*              IF <exp_booleana> THEN <expr de tipo tipo_izq>             *
*                ELSE <expr de tipo tipo_izq>             		  *
* DESCRIPCION: Analiza uno a uno los siguientes tokens del archivo veri-  *
*        ficando que conformen una expresion IF ... THEN ... ELSE ...	  *
*	 sintacticamente correcta del tipo requerido: <tipo_izq>.	  *
* PARAMETROS:  							          *	
*   FILE        *fp            	  Ap. al archivo que contiene los axiomas.*
*   texpr       mens_error        Comentario acerca de un eventual error. *
*   texpr       str_axioma        Segmento ya analizado del axioma actual *
*   nodo_expresion **exp          Ap. a la estructura que se construye a  *
*				  medida que se "parsea" la expresion.    *
*   nodo_expresion *expr_lado_izq Ap. a la estructura que contiene la ex- *
*                                 presion del lado izquierdo del axioma   *
*                                 que ya ha sido "parseada".		  *
*   texpr       tipo_izq	  Nombre del tipo del cual debe ser la ex-*
*				  presion (tipo del lado izq).		  *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas, str_axioma = "////",  ////     *
*         expresion ya parseada, tipo_izq = TI, expr_lado_izq apunt. a la *
*	  estructura que contiene el lado izquierdo ya parseado del axioma*
*	  actual.							} *
* POST!: {analice_if = 0, Si la expresion leida contiene algun error sin- *
*	     tactico o no es de tipo TI, mens_error contiene una explica- *
*	     cion acerca del error detectado, o   		  	  *
*         analice_if = 1, Si la expresion leida es correcta               *
*	     str_axioma = "////IF <exp_booleana> THEN <expr tipo_izq>     *
*                          ELSE <expr tipo_izq>"			  *
* LLAMA A: void   deme_token             short  analice_expr_booleana     *
*          short  analice_expr_der                                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
analice_if (FILE * fp, texpr mens_error, texpr str_axioma, nodo_argum ** a,
	    nodo_expresion * expr_lado_izq, texpr tipo_izq)
{
/*----------------------------------------------------------------------*/

  short error;
  texpr condicion, expr_ax;
  nodo_argum *arg;
  nodo_expresion *arg1, *arg2;

  assert(a!=NULL && str_axioma!=NULL && fp!=NULL); /* VTP. 25/May/1997 */
  (*a) = (nodo_argum *) malloc (sizeof (nodo_argum));
  HayMem(*a,sizeof(nodo_argum)); /* VTP. 25/May/1997 */
  (*a)->sig = NULL;
  arg = (*a);
  strcpy (condicion, "");
  if (analice_expr_booleana (fp, condicion, mens_error, &(arg->expr),
			     expr_lado_izq))
    {
      strcat (condicion, ",");
      strcat (str_axioma, condicion);
      deme_token (fp);
      error = es_error_esperaba (claves[9], token, mens_error);
      if (!error)
	{
	  strcpy (expr_ax, "");
	  (arg->sig) = (nodo_argum *) malloc (sizeof (nodo_argum));
          HayMem(arg->sig,sizeof(nodo_argum)); /* VTP. 25/May/1997 */
	  arg = arg->sig;
	  arg->sig = NULL;
	  deme_token (fp);
	  if (analice_expr_der (fp, mens_error, expr_ax, &(arg->expr),
				expr_lado_izq, tipo_izq))
	    {
	      deme_token (fp);
	      error = es_error_esperaba (claves[10], token, mens_error);
	      if (!error)
		{
		  strcat (expr_ax, ",");
		  strcat (str_axioma, expr_ax);
		  strcpy (expr_ax, "");
		  (arg->sig) = (nodo_argum *) malloc (sizeof (nodo_argum));
                  HayMem(arg->sig,sizeof(nodo_argum)); /* VTP. 25/May/1997 */
		  arg = arg->sig;
		  arg->sig = NULL;
		  deme_token (fp);
		  if (analice_expr_der (fp, mens_error, expr_ax, &(arg->expr),
					expr_lado_izq, tipo_izq))
		    {
		      strcat (expr_ax, ")");
		      strcat (str_axioma, expr_ax);
		    }
		  else
		    {
		      error = 1;
		    }
		}
	    }
	  else
	    {
	      error = 1;
	    }
	}
    }
  else
    {
      error = 1;
    }
  if (!error)
    {
      arg1 = (*a)->sig->expr;
      arg2 = (*a)->sig->sig->expr;
      if (!strcmp (arg1->tipo, ""))
	strcpy (arg1->tipo, arg2->tipo);
      if (!strcmp (arg2->tipo, ""))
	strcpy (arg2->tipo, arg1->tipo);
    }
  if (!error)
    {
      deme_token (fp);
      error = es_error_esperaba (claves[15], token, mens_error);
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short determine_tipo_equal(FILE *fp,texpr mens_error,			  *
*                            nodo_expresion *expr_lado_izq,cad20 tipo_eq) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar el tipo del primer argumento de un EQUAL.       *
* DESCRIPCION: Analiza el siguiente token del archivo para determinar el  *
*	 tipo al cual pertenece.					  *
* PARAMETROS:  							          *
*   FILE        *fp            	  Ap. al archivo que contiene los axiomas.*
*   texpr       mens_error        Comentario acerca de un eventual error. *
*   nodo_expresion *expr_lado_izq Ap. a la estructura que contiene la ex- *
*                                 presion del lado izquierdo del axioma   *
*                                 que ya ha sido "parseada".		  *
*   cad20	tipo_eq		  Contiene el tipo del token analizado.	  *
* V.GLOBALES: texpr  token        Contiene el ultimo token leido.         *
* RESULTADO:  short               0, Si se detecto algun error. 1, Sino.  *
* PRE!:  {fp apunt. al archivo de axiomas y expr_lado_izq apunt. a la es- *
* 	  tructura que contiene el lado izquierdo ya parseado del axioma  *
*	  actual.							} *
* POST!: {determine_tipo_equal = 0, Si no se puede determinar el tipo del *
*	     token leido y mens_error contiene una explicacion acerca del *
*	     error detectado, o   		  			  *
*         determine_tipo_equal = 1, Si se determino el tipo TT del token  *
*            leido y tipo_eq = TT.					} *
* !OJO!: La expr_lado_izq se usa para verificar si el token es una varia- *
*	 ble que ocurre al lado izquierdo. 				  *
* LLAMA A:                                                                *
*    void  deme_token         		short es_variable                 *
*    short es_generadora_algun_tipo     short es_selectora_algun_tipo	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
determine_tipo_equal (FILE * fp, texpr mens_error,
		      nodo_expresion * expr_lado_izq, cad20 tipo_eq)
{
/*----------------------------------------------------------------------*/

  short error;

  assert(mens_error!=NULL && fp!=NULL && token!=NULL); /* VTP. 25/May/1997 */
  error = 0;
  deme_token (fp);
  if (!strcmp (token, ""))
    {
      error = 1;
      sprintf (mens_error, mensajes[114], mensajes[28]);
    }
  else if (!strcmp (token, claves[11]))
    {
      strcpy (tipo_eq, "");
      /* VTP. 25/Jun/1998 Antes EQUAL(ERROP...) era error ahora es ERROR
	 error = 1;
	 strcpy (mens_error, mensajes[30]);	 */
    }
  else
    {
      if (!es_variable (token, expr_lado_izq, tipo_eq))
	{
	  if (!es_generadora_algun_tipo (token, tipo_eq))
	    {
	      if (!es_selectora_algun_tipo (token, tipo_eq))
		{
		  if (expr_lado_izq == NULL)
		    strcpy (tipo_eq, "");
		  else if (!es_identificador (token))
		    {
		      error = 1;
		      sprintf (mens_error, mensajes[36], mensajes[28], token);
		    }
		  else
		    {
		      error = 1;
		      sprintf (mens_error, mensajes[29], token);
		    }
		}
	    }
	}
    }
  return (short)(!error);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_variable(cad20 nombre,nodo_expresion *exp,cad20 tipo)          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si <nombre> es una variable conocida.           *
* DESCRIPCION: Verifica que <nombre> sea una variable que aparece en la   *
*	 estructura apuntada por <exp> y en tal caso retorna en <tipo> el *
*        tipo de la variable.                                             *
* PARAMETROS: cad20  nombre         Nombre de la "pretendida" variable.   *
*             nodo_expresion *exp   Ap. a la estructura "parseada" del    *
*				    lado izq. del axioma actual.	  *
*             cad20  tipo           Tipo de la variable <nombre> en la es-*
*                                   tructura apunt. por <exp>.            *
* RESULTADO:  short  1, Si <nombre> es una variable en la estructura apun.*
*	                por <exp>, o					  *
*		     0, Sino.						  *
* PRE!:  { nombre = NV, exp apunt. a EPI la estructura "parseada" del lado*
*          izquierdo del axioma.                                        } *
* POST!: { es_variable = 1, Si NV es una variable en EPI y tipo = TV  es  *
*                           el tipo de la variable NV en EPI.             *
*	   es_variable = 0, Sino.					} *
* LLAMA A: short  es_variable                                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
es_variable (cad20 nombre, nodo_expresion * exp, cad20 tipo)
{
/*----------------------------------------------------------------------*/

  short es_var;
  nodo_argum *a;

  es_var = 0;
  if (exp != NULL)
    {
      if (!strcmp (exp->nombre, nombre))
	{
	  if (!strcmp (exp->clase, "VAR"))
	    {
	      strcpy (tipo, exp->tipo);
	      es_var = 1;
	    }
	}
      else
	{
	  for (a = exp->argumentos; (!es_var) && (a != NULL); a = a->sig)
	    {
	      es_var = es_variable (nombre, a->expr, tipo);
	    }
	}
    }
  return (es_var);
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* short es_variable_tipo(cad20 nombre,nodo_expresion *exp,cad20 tipo)     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si <nombre> es una variable de tipo <tipo>      *
*  	 conocida.           						  *
* DESCRIPCION: Verifica que <nombre> sea una variable de tipo <tipo> que  *
*	 aparece en la estructura apuntada por <exp>.			  *
* PARAMETROS: cad20  nombre         Nombre de la "pretendida" variable.   *
*             nodo_expresion *exp   Ap. a la estructura "parseada" del    *
*				    lado izq. del axioma actual.	  *
*             cad20  tipo           Tipo "pretendido" de la var. <nombre> *
* RESULTADO:  short  1, Si <nombre> es una variable de tipo <tipo> en la  *
*		        estructura apun. por <exp>, o			  *
*		     0, Sino.						  *
* PRE!:  { nombre = NV, exp apunt. a EPI la estructura "parseada" del lado*
*          izquierdo del axioma, tipo = TV                              } *
* POST!: { es_variable_tipo = 1, Si NV es una variable de tipo TV en EPI o*
*          es_variable_tipo = 0, Sino.                                  } *
* LLAMA A: short  es_variable_tipo                                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 10-Feb-90 *
**************************************************************************/

short
es_variable_tipo (cad20 nombre, nodo_expresion * exp, cad20 tipo)
{
/*----------------------------------------------------------------------*/

  short es_var;
  nodo_argum *a;

  es_var = 0;
  if (exp != NULL)
    {
      if (!strcmp (exp->nombre, nombre))
	{
	  if ((!strcmp (exp->tipo, tipo)) && (!strcmp (exp->clase, "VAR")))
	    {
	      es_var = 1;
	    }
	}
      else if (strcmp (exp->clase, "VAR"))
	{
	  for (a = exp->argumentos; (!es_var) && (a != NULL); a = a->sig)
	    {
	      es_var = es_variable_tipo (nombre, a->expr, tipo);
	    }
	}
    }
  return (es_var);
/*----------------------------------------------------------------------*/
}
