 
/* ARCHIVOS.C                                                                             */
/* Contiene las rutinas de manejo de archivos TADs                              */

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "definic.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 28/May/1997 */


/**************************************************************************
* short arch_modificado(cad20 nombre_arch)                                        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si el archivo fue modificado durante la ultima  *
*              edicion.                                                   *
* DESCRIPCION: Compara las fechas de la ultima edicion y de la ultima mo- *
*        dificacion del archivo <nom_arch> para determinar si fue o no    *
*        modificado durante la ultima edicion.                            *
* PARAMETROS:                                                                               *
*         cad20      nombre_arch                  Nombre del archivo a determinar si*
*                                         fue modificado o no.              *
* RESULTADO:  short           1, Si el archivo fue modificado durante la*
*                                  ultima edicion.                        *
*                               0, Sino.                                          *
* PRE!:  { nom_arch = NA nombre de un  archivo de edicion.              } *
* POST!: { arch_modificado = 0 , Si fecha de la ultima modificacion de NA *
*                           es menor que la fecha del ultimo acceso a NA. *
*          arch_modificado = 1 , Sino                                   } *
* LLAMA A: int    stat                                                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
arch_modificado (cad20 nombre_arch)
{
/*----------------------------------------------------------------------*/

  struct stat *buf;

  buf = (struct stat *) malloc (sizeof (struct stat));
  HayMem(buf,sizeof(struct stat));              /* v-tamara 28/May/1997 */
  if (stat (nombre_arch, buf) < 0)
    {
      printf (mensajes[65]);
      exit (1);
    }
  if ((buf->st_atime) <= (buf->st_mtime))
    return (1);
  else
    return (0);

/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short busque_arch_tad(cad20 nom)                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Buscar el archivo correspondiente al tad con nombre <nom>. *
* DESCRIPCION: Busca en el directorio D_ADT el archivo <nom>.adt. Si lo   *
*        encuentra carga la informacion de ese tad sobre la version exis- *
*        tente en el ambiente (si existia alguna).                        *
* PARAMETROS:                                                                               *
*    cad20          nom             Nombre del tad del cual se quiere buscar  *
*                                       el archivo.                                         *
* V.GLOBALES:                                                             *
*    nodo_tad       *tad                Apunt. a la cabeza de la lista de los tads*
*                                       del ambiente.                                       *
*    nodo_tad       *ult_tad  Apunt. al ultimo tad de la lista de tads  *
*                                       del ambiente.                                       *
* RESULTADO:  short           1, Si encontro el archivo.                *
*                                       0, Sino.                                  *
* PRE!:  { nom = NT,                                                      *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.                                                *
*          D_ADT = <a1.adt,...,am.adt>, m >= 0                                  } *
* POST!: { busque_arch_tad = 1,                                                             *
*              Si NT = ak, 1 <= k <= n                                            *
*                  tad -> <t1,...,tn,t(n+1)> <- ult_tad tq.             *
*                t(n+1).nombre = ak  y ti.nombre <> ak para todo i <= n   *
*            busque_arch_tad = 0, Sino.                                         } *
* LLAMA A:                                                                *
*         void      nombre_archivo                          void      lea_arch_tad          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

short
busque_arch_tad (cad20 nom)
{
/*----------------------------------------------------------------------*/

  cad20 nom_arch;
  FILE *fp;
  nodo_tad *q, *p;


  if (!strcmp (nom, ""))
    return (0);
  if (!tipo_primitivo (nom))
    {
      nombre_archivo (nom, nom_arch);
      if ((fp = fopen (nom_arch, "r")) == NULL)
	{
	  return (0);
	}
      else
	{
	  /* Suprimir la posible version del tad existente en el ambiente */
	  for (q = tad; (q->sig != NULL) && (strcmp (q->sig->nombre, nom));
	       q = q->sig)
	    ;
	  if (q->sig != NULL)
	    {
	      p = q->sig;
	      if (q->sig == ult_tad)
		ult_tad = q;
	      q->sig = q->sig->sig;
	      free (p);
	    }
	  /* Se lee el tad del archivo  */
	  lea_arch_tad (fp);
	  fclose (fp);
	  return (1);
	}

    }
/*** Alvaro return
***/
  return 1;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void escriba_arch_generadoras(FILE *fp,nodo_generadora *cabeza)       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribe en el archivo fp la lista de generadoras encabezada*
*              por <cabeza>.                                              *
* DESCRIPCION: Recorre la lista apuntada por cabeza escribiendo el dominio*
*          de cada operacion generadora.                                                    *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_generadora     *cabeza             Ap. a la primera operacion genera-*
*                                                 dora de la lista.               *
* PRE!:  { fp-> [///], cabeza-> <g1,...,gn>,  n >= 0.                           } *
* POST!: { fp-> [///g1.nombre DOMINIO <g1->dominio> FIN  ...                      *
*                     gn.nombre DOMINIO <gn->dominio> FIN                         *
*                FIN]                                                                     } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
escriba_arch_generadoras (FILE * fp, nodo_generadora * cabeza)
{
/*----------------------------------------------------------------------*/

  for (; cabeza != NULL; cabeza = cabeza->sig)
    {
      fprintf (fp, "  %s\n", cabeza->nombre);
      fprintf (fp, "  %s\n", cabeza->nombre_fte);
      fprintf (fp, "    DOMINIO\n");
      escriba_arch_nombres (fp, cabeza->dominio, "    ");
    }
  fprintf (fp, "FIN\n");
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void escriba_arch_igualdades(FILE *fp, nodo_igualdad *cabeza)                   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribe en el archivo fp la lista de igualdades  encabezada*
*              por <cabeza>.                                              *
* DESCRIPCION: Recorre la lista apuntada por cabeza escribiendo la infor- *
*          macion analizada de cada igualdad.                                     *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_igualdad       *cabeza             Ap. a la primera igualdad de la   *
*                                                 lista.                                    *
* PRE!:  { fp-> [///], cabeza-> <i1,...,in>,  n >= 0.                   } *
* POST!: { fp-> [///                                                                        *
*                  i1->li_str=i1->ld_str                                                    *
*                     analisis(i1->lado_izq) analisis(i1->lado_der)     *
*                  ...                                                      *
*                  in->li_str=in->ld_str                                                    *
*                     analisis(in->lado_izq) analisis(in->lado_der)FIN]           *
* LLAMA A: void       escriba_arch_tipo_clase                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
escriba_arch_igualdades (FILE * fp, nodo_igualdad * cabeza)
{
/*----------------------------------------------------------------------*/


  for (; cabeza != NULL; cabeza = cabeza->sig)
    {
/*        fprintf(fp,"    %s=%s\n",cabeza->li_str,cabeza->ld_str); */
      fprintf (fp, "    ");
      escriba_arch_expresion (fp, cabeza->lado_izq);
      fprintf (fp, "=");
      escriba_arch_expresion (fp, cabeza->lado_der);
      fprintf (fp, "\n");
      escriba_arch_tipo_clase (fp, cabeza->lado_izq);
      escriba_arch_tipo_clase (fp, cabeza->lado_der);
    }
  fprintf (fp, "  FIN\n");
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void escriba_arch_nombres(FILE *fp, nodo_nombre *cabeza,cad12 indent)   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribe en el archivo fp la lista de nombres encabezada            *
*              por <cabeza>.                                              *
* DESCRIPCION: Recorre la lista apuntada por cabeza los nombres indentados*
*          con <indent> en el archivo.                                            *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_nombre         *cabeza             Ap. al primer nombre de la lista. *
*         cad12               indent              Cadena de blancos usada para in-  *
*                                                 dentar.                                   *
* PRE!:  { fp-> [///], cabeza-> <m1,...,mn>,  n >= 0.                           } *
* POST!: { fp-> [///<indent>m1 ... <indent>mn FIN]                              } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
escriba_arch_nombres (FILE * fp, nodo_nombre * cabeza, cad12 indent)
{
/*----------------------------------------------------------------------*/

  texpr linea;


  for (; cabeza != NULL; cabeza = cabeza->sig)
    {
      strcpy (linea, indent);
      strcat (linea, cabeza->nombre);
      fprintf (fp, "  %s\n", linea);
    }
  strcpy (linea, indent);
  strcat (linea, "FIN");
  fprintf (fp, "%s\n", linea);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void escriba_arch_selectoras(FILE *fp,nodo_select *cabeza)                      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribe en el archivo fp la lista de selectoras  encabezada*
*              por <cabeza>.                                              *
* DESCRIPCION: Recorre la lista apuntada por cabeza escribiendo la info.  *
*          de cada operacion selectora.                                           *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_select         *cabeza             Ap. a la primera operacion selec- *
*                                                 tora de la lista.               *
* PRE!:  { fp-> [///], cabeza-> <s1,...,sn>,  n >= 0.                           } *
* POST!: { fp-> [///                                                                        *
*                  s1.nombre                                                      *
*                    CARACTERIZADORA/.                                            *
*                    DOMINIO <s1->dominio> FIN                                    *
*                    CODOMINIO s1->codominio                                      *
*                    AXIOMAS <s1->axiomas>                                        *
*                ...                                                                        *
*                  sn.nombre                                                      *
*                    CARACTERIZADORA/.                                            *
*                    DOMINIO <sn->dominio> FIN                                    *
*                    CODOMINIO sn->codominio                                      *
*                    AXIOMAS <sn->axiomas>                                        *
*                  FIN]                                                                   } *
* LLAMA A: void     escriba_arch_igualdades                                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
escriba_arch_selectoras (FILE * fp, nodo_select * cabeza)
{
/*----------------------------------------------------------------------*/

  nodo_dom *q;

  for (; cabeza != NULL; cabeza = cabeza->sig)
    {
      fprintf (fp, "  %s %s %s\n", cabeza->nombre, cabeza->nombre_fte,
	       (cabeza->caracterizadora) ? "CARACTERIZADORA" : ".");
      fprintf (fp, "    DOMINIO\n");
      for (q = cabeza->dominio; q != NULL; q = q->sig)
	fprintf (fp, "      %s %s\n", q->nombre,
		 (q->induccion) ? "INDUCCION" : ".");
      ;
      fprintf (fp, "    FIN\n");
      fprintf (fp, "    CODOMINIO\n");
      fprintf (fp, "      %s\n", cabeza->codominio);
      fprintf (fp, "  AXIOMAS\n");
      escriba_arch_igualdades (fp, cabeza->axiomas);
    }
  fprintf (fp, "FIN\n");
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void escriba_arch_tad(cad20 nom,nodo_tad *p)                                    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribir en el archivo <nom>.adt la informacion del tad p. *
* DESCRIPCION: Escribe en el archivo XXX/<nom>.adt la informacion       *
*          del tad apuntado por p, donde XXX es el nombre del directorio    *
*          para salvar TADS del ambiente                                                  *
* PARAMETROS:                                                             *
*         cad20               nom             Nombre del tad a ser salvado en   *
*                                                 memoria secundaria.             *
*         nodo_tad        *p            Ap. al tad que debe salvarse.     *
* PRE!:  { nom = NT,                                                      *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2. p -> tk, 1 <= k <= n.              } *
*          D_ADT = <a1.adt,...,am.adt>, m >= 0                                  } *
* POST!: { D_ADT = <a1.adt,...,ai.adt,...,am1.adt>, m1 = m, o, m1 = m+1,  *
*            1 <= i <= m1, tq ai = tk.nombre                                    } *
* LLAMA A:                                        void escriba_arch_nombres       *
*         void escriba_arch_generadoras           void escriba_arch_selectoras    *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
escriba_arch_tad (cad20 nom, nodo_tad * p)
{
/*----------------------------------------------------------------------*/

  cad16 nom_arch;
  FILE *fp;

  nombre_archivo (nom, nom_arch);
  fp = fopen (nom_arch, "w");
  FileOpen(fp,nom_arch);                /* v-tamara 28/May/1997 */
  fprintf (fp, "%s\n", p->nombre);
  fprintf (fp, "STATUS\n");
  fprintf (fp, "%d\n", p->status);
  fprintf (fp, "TIPO\n");
  fprintf (fp, "%d\n", p->tipo);
  fprintf (fp, "INFO_TIPO\n");
  escriba_arch_nombres (fp, p->info_tipo, "");
  fprintf (fp, "PARAMETROS\n");
  escriba_arch_nombres (fp, p->parametros, "");
  fprintf (fp, "INICIALES\n");
  escriba_arch_generadoras (fp, p->iniciales);
  fprintf (fp, "CONSTRUCTORAS\n");
  escriba_arch_generadoras (fp, p->constructoras);
  fprintf (fp, "SELECTORAS\n");
  escriba_arch_selectoras (fp, p->selectoras);
  fprintf (fp, "FIN\n");
  fclose (fp);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void escriba_arch_tipo_clase(FILE *fp,nodo_expresion *e)              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Escribe en el archivo fp la informacion analizada de la ex-*
*              presion e.                                                 *
* DESCRIPCION: Recorre en preorden el arbol sintactico de la expresion a- *
*          puntada por e escribiendo  el tipo y la clase de cada subexpre-  *
*          sion hallada.                                                                    *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_expresion      *e                  Ap. a la cabeza de la expresion.  *
* PRE!:  { fp-> [///], e-> <exp>                                                } *
* POST!: { fp-> [///analisis(<exp>)], donde                                       *
*                    analisis(<f(a1,...,an)>) <-                                  *
*                     <Tipo de f> <Clase de f> analisis(<a1>)...analisis(<an>)*
*                    para n >= 0.                                                         } *
* LLAMA A: void escriba_arch_tipo_clase                                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
escriba_arch_tipo_clase (FILE * fp, nodo_expresion * e)
{
/*----------------------------------------------------------------------*/

  nodo_argum *a;

  assert(e!=NULL);                      /* v-tamara 28/May/1997 */
  fprintf (fp, "      %s %s\n", e->tipo, e->clase);
  for (a = e->argumentos; (a != NULL); a = a->sig)
    {
      escriba_arch_tipo_clase (fp, a->expr);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* int existe_archivo(texpr nombre, texpr dir)                             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Indica si un archivo con nombre <nombre> existe en disco,  *
*              en el directorio <dir>                                     *
* POST : existe_archivo = 1 , si <nombre> es un nombre de un archivo      *
*                       = 0, de lo contrario                              *
**************************************************************************/

int
existe_archivo (texpr nombre, texpr dir)
{
/*----------------------------------------------------------------------*/

  FILE *fp;
  texpr comando;

  sprintf (comando, "%s/%s", dir, nombre);
  fp = fopen (comando, "r");
  if (fp == NULL)
    return (0);
  fclose (fp);
  return (1);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void lea_arch_generadoras(FILE *fp, nodo_generadora **cabeza)                   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer del archivo fp una lista de generadoras.                      *
* DESCRIPCION: Lee los siguientes tokens del archivo creando una lista de *
*          generadoras con esa informacion, que queda apuntada por *cabeza. *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_generadora     **cabeza  Ap. a la primera operacion genera-*
*                                                 dora de la lista.               *
* PRE!:  { fp-> [///g1.nombre DOMINIO <g1->dominio> FIN  ...                      *
*                     gn.nombre DOMINIO <gn->dominio> FIN                         *
*                FIN\\\]   n >= 0.                                              } *
* POST!: { *cabeza-> <g1,...,gn>                                                } *
* LLAMA A:          void  lea_arch_nombres                                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
lea_arch_generadoras (FILE * fp, nodo_generadora ** cabeza)
{
/*----------------------------------------------------------------------*/

  texpr linea;
  nodo_generadora *q, *ult_generad;

  *cabeza = NULL;
  ult_generad = NULL;
  for (fscanf (fp, "%s", linea); strcmp (linea, "FIN"); fscanf (fp, "%s", linea))
    {
      q = (nodo_generadora *) malloc (sizeof (nodo_generadora));
      HayMem(q,sizeof (nodo_generadora));               /* v-tamara 28/May/1997 */
      if (ult_generad == NULL)
	*cabeza = q;
      else
	ult_generad->sig = q
	  ;
      ult_generad = q;
      strcpy (q->nombre, linea);
      fscanf (fp, "%s", linea);
      strcpy (q->nombre_fte, linea);
      q->sig = NULL;
      fscanf (fp, "%s", linea);	/* DOMINIO */
      lea_arch_nombres (fp, &(q->dominio));
    }
/*----------------------------------------------------------------------*/
}

/* Reads a string without spaces from a file, skips initial spaces */
void
readStringWithoutSpaces(char *line,int size, FILE *fp)
{
  int c,i;
  while ((c=fgetc(fp))!=EOF && isspace(c));
  i=0;
  while (c!=EOF && !isspace(c) && i<size-1)
    { 
      line[i]=(char)c;
      i++;
      c=fgetc(fp);
    }
  line[i]='\0';
  if (i>=size-1)
    {
      printf("An ADT file contains an expression too long: [%s]\n"
	     ,line);
      printf("\nChange TMAX_EXPR and recompile\n");
      exit(1);
    }
}


/**************************************************************************
* void lea_arch_igualdades(FILE *fp,nodo_igualdad **cabeza)             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer del archivo fp una lista de igualdades.             *
*              por <cabeza>.                                              *
* DESCRIPCION: Lee los siguientes tokens del archivo creando una lista de *
*          igualdades con esa informacion, que queda apuntada por *cabeza.  *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_igualdad       **cabeza  Ap. a la primera igualdad de la   *
*                                                 lista.                                    *
* PRE!:  { fp-> [///                                                                        *
*                  i1->li_str=i1->ld_str                                                    *
*                     analisis(i1->lado_izq) analisis(i1->lado_der)     *
*                  ...                                                      *
*                  in->li_str=in->ld_str                                                    *
*                     analisis(in->lado_izq) analisis(in->lado_der)     *
*                  FIN],                n >= 0                                            } *
* PRE!:  { *cabeza-> <i1,...,in>                                                } *
* LLAMA A:                                                                                  *
* void    transf_iguald_estruct                   void      lea_arch_tipo_clase   *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
lea_arch_igualdades (FILE * fp, nodo_igualdad ** cabeza)
{
/*----------------------------------------------------------------------*/

  nodo_igualdad *q=NULL, *ult_exp;          /* v-tamara 28/May/1997 */
  texpr linea;

  *cabeza = NULL;
  ult_exp = NULL;
  readStringWithoutSpaces(linea,MAX_TEXPR,fp);
  while (strcmp (linea, "FIN")!=0)
    {
      /*      printf("%s\n",linea); */
      transf_iguald_estruct (linea, &q);
      reemplace_claves (q->lado_der);
      lea_arch_tipo_clase (fp, q->lado_izq);
      lea_arch_tipo_clase (fp, q->lado_der);
      if (ult_exp == NULL)
	*cabeza = q;
      else
	ult_exp->sig = q;
      ult_exp = q;
      readStringWithoutSpaces(linea,MAX_TEXPR,fp);
    }
/*--------------------------------------------------------------------*/
}

/**************************************************************************
* void lea_arch_nombres(FILE *fp, nodo_nombre **cabeza)                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer del archivo fp una lista de nombres.                          *
* DESCRIPCION: Lee los siguientes tokens del archivo creando una lista de *
*          nombres con esa informacion, que queda apuntada por *cabeza.           *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_nombre         **cabeza  Ap. al primer nombre de la lista. *
* POST!: { fp-> [///m1 ... mnFIN\\\], n >= 0                                    } *
* PRE!:  { cabeza-> <m1,...,mn>.                                                } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
lea_arch_nombres (FILE * fp, nodo_nombre ** cabeza)
{
/*----------------------------------------------------------------------*/

  texpr linea;
  nodo_nombre *q, *ult_nom;

  *cabeza = NULL;
  ult_nom = NULL;
  for (fscanf (fp, "%s", linea); strcmp (linea, "FIN"); fscanf (fp, "%s", linea))
    {
      q = (nodo_nombre *) malloc (sizeof (nodo_nombre));
      HayMem(q,sizeof (nodo_nombre));           /* v-tamara 28/May/1997 */
      if (ult_nom == NULL)
	*cabeza = q;
      else
	ult_nom->sig = q
	  ;
      ult_nom = q;
      strcpy (q->nombre, linea);
      q->sig = NULL;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void lea_arch_selectoras(FILE *fp, nodo_select **cabeza)              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer del archivo fp una lista de selectoras.                       *
* DESCRIPCION: Lee los siguientes tokens del archivo creando una lista de *
*          selectoras con esa informacion, que queda apuntada por *cabeza.  *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_select         *cabeza             Ap. a la primera operacion selec- *
*                                                 tora de la lista.               *
* POST!: { fp-> [///                                                                        *
*                  s1.nombre                                                      *
*                    CARACTERIZADORA/.                                            *
*                    DOMINIO <s1->dominio> FIN                                    *
*                    CODOMINIO s1->codominio                                      *
*                    AXIOMAS <s1->axiomas>                                        *
*                ...                                                                        *
*                  sn.nombre                                                      *
*                    CARACTERIZADORA/.                                            *
*                    DOMINIO <sn->dominio> FIN                                    *
*                    CODOMINIO sn->codominio                                      *
*                    AXIOMAS <sn->axiomas>                                        *
*                  FIN\\\]                        n >= 0.                       } *
* PRE!:  { *cabeza-> <s1,...,sn>                                                } *
* LLAMA A: void     lea_arch_igualdades                                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
lea_arch_selectoras (FILE * fp, nodo_select ** cabeza)
{
/*----------------------------------------------------------------------*/

  texpr linea;
  nodo_select *q, *ult_select;
  nodo_dom *n, *ult_nom;

  *cabeza = NULL;
  ult_select = NULL;
  for (fscanf (fp, "%s", linea); strcmp (linea, "FIN"); fscanf (fp, "%s", linea))
    {
      q = (nodo_select *) malloc (sizeof (nodo_select));
      HayMem (q,sizeof (nodo_select));          /* v-tamara 28/May/1997 */
      if (ult_select == NULL)
	*cabeza = q;
      else
	ult_select->sig = q
	  ;
      ult_select = q;
      strcpy (q->nombre, linea);
      fscanf (fp, "%s", linea);
      strcpy (q->nombre_fte, linea);
      fscanf (fp, "%s", linea);
      if (strcmp (linea, "CARACTERIZADORA"))
	q->caracterizadora = 0;
      else
	q->caracterizadora = 1
	  ;
      q->sig = NULL;
      q->dominio = NULL;
      ult_nom = NULL;
      fscanf (fp, "%s", linea);
      for (fscanf (fp, "%s", linea); strcmp (linea, "FIN");
	   fscanf (fp, "%s", linea))
	{
	  n = (nodo_dom *) malloc (sizeof (nodo_dom));
          HayMem(n,sizeof (nodo_dom));          /* v-tamara 28/May/1997 */
	  if (ult_nom == NULL)
	    q->dominio = n;
	  else
	    ult_nom->sig = n
	      ;
	  ult_nom = n;
	  strcpy (n->nombre, linea);
	  fscanf (fp, "%s", linea);
	  if (strcmp (linea, "INDUCCION"))
	    n->induccion = 0;
	  else
	    n->induccion = 1
	      ;
	  n->sig = NULL;
	}
      fscanf (fp, "%s", linea);	/* CODOMINIO */
      fscanf (fp, "%s", linea);	/* CONJUNTO CODOMINIO */
      strcpy (q->codominio, linea);
      fscanf (fp, "%s", linea);	/* AXIOMAS       */
      lea_arch_igualdades (fp, &(q->axiomas));
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void lea_arch_tad(FILE *fp)                                                     *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer del archivo fp la informacion de un tad.                      *
* DESCRIPCION: Lee los tokens del archivo almacenando esa informacion en  *
*          un nuevo nodo_tad en el ambiente.                                      *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2                                       *
*          D_ADT = <a1.adt,...,am.adt>, m >= 0                                    *
*            fp -> ak,  1 <= k <= m.                                            } *
* POST!: { tad -> <t1,t2,...,tn,t(n+1)> <- ult_tad, donde               *
*            t(n+1) es el tad cuya info. esta en ak.                            } *
* LLAMA A:                                        void lea_arch_nombres           *
*         void lea_arch_generadoras     void lea_arch_selectoras        *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
lea_arch_tad (FILE * fp)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;
  texpr linea;

  cree_nodo_tad ();
  p = ult_tad;
  p->modif = 0;
  fscanf (fp, "%s", p->nombre);
  fscanf (fp, "%s", linea);	/* STATUS        */
  fscanf (fp, "%s", linea);
  p->status = (short) atoi (linea);
  fscanf (fp, "%s", linea);	/* TIPO          */
  fscanf (fp, "%s", linea);
  p->tipo = (short) atoi (linea);
  fscanf (fp, "%s", linea);	/* INFO_TIPO     */
  lea_arch_nombres (fp, &(p->info_tipo));
  fscanf (fp, "%s", linea);	/* PARAMETROS    */
  lea_arch_nombres (fp, &(p->parametros));
  fscanf (fp, "%s", linea);	/* INICIALES     */
  lea_arch_generadoras (fp, &(p->iniciales));
  fscanf (fp, "%s", linea);	/* CONSTRUCTORAS */
  lea_arch_generadoras (fp, &(p->constructoras));
  fscanf (fp, "%s", linea);	/* SELECTORAS    */
  lea_arch_selectoras (fp, &(p->selectoras));
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void lea_arch_tipo_clase(FILE *fp,nodo_expresion *e)                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer del archivo fp un analisis de una expresion.        *
* DESCRIPCION: Lee los siguientes tokens del archivo creando el arbol sin-*
*          tactico de una expresion con esa informacion, que queda apuntado *
*          por e.                                                                           *
* PARAMETROS:                                                             *
*         FILE                *fp                 Ap. al archivo de info. del tad.  *
*         nodo_expresion      *e                  Ap. a la cabeza de la expresion.  *
* POST!: { fp-> [///analisis(<exp>)\\\\], donde                                   *
*                    analisis(<f(a1,...,an)>) <-                                  *
*                     <Tipo de f> <Clase de f> analisis(<a1>)...analisis(<an>)*
*                    para n >= 0.                                                         } *
* PRE!:  { fp-> [///], e-> <exp>                                                } *
* LLAMA A: void lea_arch_tipo_clase                                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
lea_arch_tipo_clase (FILE * fp, nodo_expresion * e)
{
/*----------------------------------------------------------------------*/

  nodo_argum *a;
  texpr linea;

  assert(e!=NULL);                      /* v-tamara 28/May/1997 */
  fscanf (fp, "%s", linea);
  strcpy (e->tipo, linea);
  fscanf (fp, "%s", linea);
  strcpy (e->clase, linea);
  for (a = e->argumentos; (a != NULL); a = a->sig)
    {
      lea_arch_tipo_clase (fp, a->expr);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void nombre_archivo(cad20 nom,cad20 nom_arch)                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Producir el nombre de un archivo para salvar un tad.       *
* DESCRIPCION: Genera el nombre D_ADT/<nom>.adt si <nom> es BOOL o NAT,   *
*              en otro caso genera el nombre <XXX>/<nom>.adt, donde       *
*              XXX es el contenido de "direct[2]".                        *
*              retorna la respuesta en <nom_arch>                         *
*        Es usada para construir el nombre del archivo en que se debe sal-*
*        var un tad.                                                      *
* PARAMETROS:                                                                               *
*         cad20     nom                 Cadena con el nombre de un tad.                     *
*         cad20     nom_arch  Cadena con el nombre del archivo en el que*
*                                       se debe salvar el tad.                              *
* PRE!:  { nom = NT                                                                       } *
* POST!: { (nom_arch = D_ADT/NT.adt ,(NT=BOOL o NT=NAT)) o                *
*          (nom_arch = XXX/NT.adt , (NT <>BOOL , NT<>NAT) , XXX=direct[2] *
*        }                                                                *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
* MODIFICADA: Hernan Aponte
**************************************************************************/

void
nombre_archivo (cad20 nom, cad20 nom_arch)
{
/*----------------------------------------------------------------------*/
/* VTP. 9/Jul/1997.  Se quito el if siguiente porque obligaba
                     a tener un directorio llamado D_ADT,
                     ahora se obliga a que en el directorio de
                     TADs esten los tipos primitivos */
/*  if (tipo_primitivo (nom))
    {
      sprintf (nom_arch, "%s/%s.adt", "D_ADT", nom);
    }
  else
    {  */
  sprintf (nom_arch, "%s/%s.adt", direct[2],nom);
/*    }
  ;*/
/*----------------------------------------------------------------------*/
}


/*
void
nombre_archivoR (nodo_represent *nr, cad20 nom_arch)
{
  sprintf (nom_arch, "%s/%s_%s.%s", direct[3],
           nr->tad_fuente,nr->tad_objeto,nr->nom_funcion);

}
*/
void
escriba_arch_expresion (FILE * fp, nodo_expresion * e)
{
  nodo_argum *a;
  int k;

  assert(e!=NULL);              /* v-tamara 28/May/1997 */
  k = es_palabra_clave (e->nombre, claves);
  if (k != -1)
    fprintf (fp, "%s", claves_manta[k]);
  else
    fprintf (fp, "%s", e->nombre);
  if (e->argumentos != NULL)
    {
      fprintf (fp, "(");
      for (a = e->argumentos; a != NULL; a = a->sig)
	{
	  escriba_arch_expresion (fp, a->expr);
	  if (a->sig != NULL)
	    fprintf (fp, ",");
	}
      fprintf (fp, ")");
    }
}

int
es_palabra_clave (cad12 nombre, texpr clav[MAX_CLAVES])
{
  int i;

  for (i = 0; i < MAX_CLAVES; i++)
    {
      if (!strcmp (nombre, clav[i]))
	{
	  if ((i > 7) && (i < 13))
	    return i;
	  else
	    return -1;
	}
    }
  return -1;
}


void
reemplace_claves (nodo_expresion * e)
{
  nodo_argum *a;
  int k;

  assert(e!=NULL);                      /* v-tamara 28/May/1997 */
  k = es_palabra_clave (e->nombre, claves_manta);
  if (k != -1)
    strcpy (e->nombre, claves[k]);
  if (e->argumentos != NULL)
    for (a = e->argumentos; a != NULL; a = a->sig)
      reemplace_claves (a->expr);
}
