

/* COMPLETACION.C                                                       */
/* Arhivo de manejo de completaciones de tipos.                         */


#include <stdio.h> 
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#include "definic.h"


/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 29/May/1997 */

/**************************************************************************
* void completacion(cad12 nom_tad_c)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar un tad de tipo COMPLETACION.                       *
* DESCRIPCION: Pide el nombre del tad que se va a completar y el elemento *
*        de completacion, genera automatica/. la signatura del nuevo tad  *
*	 <nom_tad_c> y solicita al usuario que extienda las operaciones se*
*	 lectoras del tad padre definiendolas sobre el nuevo elemento.	  *
* PARAMETROS:                                                             *
*    cad12	nom_tad_c	Nombre del nuevo tad de tipo COMPLETACION.*
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	COMPLETACION				  *
* PRE!:  { nom_tad_c = NT,                                                *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { tad -> <t1,...,tn,t(n+1)> <- ult_tad = tad_ed, donde		  *
*		t(n+1) = <p> + { <nom_elem_com> }, t(n+1).nombre = NT, o  *	
*          tad -> <t1,...,tn> <- ult_tad, Si la generacion del nuevo tad  *
*	   	no fue exitosa.						  *
* LLAMA A:nodo_tad  *pida_tad_ambiente					  *
*   void  pida_elem_completacion 	 void  genere_copia_tad		  *
*   void  informacion_tipo_completacion  void  genere_inicial_completacion*
*   void  genere_selectoras_completacion void  edite_igualdad		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
completacion (cad12 nom_tad_c)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;
  cad12 nom_elem_com,nomtad;
  texpr mens_error;
/*** Alvaro
  nodo_nombre	*ult_nom;
  ***/
  p = pida_tad_ambiente (mensajes[44]);
  if (p != NULL)
    {
      if (p->iniciales==NULL && p->constructoras==NULL)
	{
	  escriba_mensaje(189);
	  return;
	} 
      pida_elem_completacion (nom_tad_c, p->nombre, nom_elem_com);
      if (strcmp (nom_elem_com, ""))
	{
	  /* El nombre del elemento para completar no debe estar definido */
	  if (existsSymbol(symtable,nom_elem_com,nomtad))
	    {
	      sprintf(mens_error,mensajes[180],nom_elem_com,nomtad);
	      escriba_cadena(mens_error,1);
	      return;
	    }
	  tad_ed = crear_tad ();
	  genere_copia_tad (p, p->nombre, tad_ed, nom_tad_c);
	  informacion_tipo_completacion (tad_ed, p->nombre, nom_elem_com);
	  genere_inicial_completacion (nom_elem_com);
	  if (genere_selectoras_completacion (p))
	    {
	      tad_ed->status = pida_status ();
	    }
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void informacion_tipo_completacion(nodo_tad *p,cad12 nom_tad_origen,    *
*				     cad12 nom_elem_com)	          *	
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Asignar tipo COMPLETACION a un tad.  			  *
* DESCRIPCION: Asigna COMPLETACION al tipo de p, <nom_tad_origen> a la    *
*	 primera componente y <nom_elem_com> a la segunda componente de la*
*	 informacion asociada al tipo.					  *
* PARAMETROS:                                                             *
*    nodo_tad	*p		Ap. al tad que es de tipo COMPLETACION.	  *
*    nodo_tad	*tad_origen	Ap. al tad padre de la completacion.      *
*    cad12	nom_elem_com	Nombre del elemento de completacion.	  *
* PRE!:  { p-> T, nom_tad_origen = NTO, nom_elem_com = NEC 		} *
* POST!: { T.tipo = COMPLETACION, T.info_tipo = <NTO , NEC>		} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
informacion_tipo_completacion (nodo_tad * p, cad12 nom_tad_origen,
			       cad12 nom_elem_com)
{
/*----------------------------------------------------------------------*/

  nodo_nombre *n;

  assert(p!=NULL);              /* v-tamara 29/May/1997 */
  p->tipo = COMPLETACION;
  n = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem(n,sizeof (nodo_nombre));               /* v-tamara 29/May/1997 */
  strcpy (n->nombre, nom_tad_origen);
  p->info_tipo = n;
  n->sig = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem(n->sig,sizeof (nodo_nombre));          /* v-tamara 29/May/1997 */
  n = n->sig;
  strcpy (n->nombre, nom_elem_com);
  n->sig = NULL;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_axs_compl(FILE *fp,nodo_select *s)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar los lados izqs. de los axiomas para definir una    *
*	       selectora en el elemento de completacion.		  *
* DESCRIPCION: Genera lados izq. para definir a s en el elemento de comple*
*        tacion. Los lados izqs. se construyen poniendo el elemento de com*
*	 pletacion en los distintos argumentos de TDI y variables en los  *
*	 demas argumentos.						  *
* PARAMETROS:                                                             *
*   FILE	*fp		Ap. Al archivo de edicion de axiomas.     *
*   nodo_select	*s 		Ap. a la selectora que se va a extender.  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	COMPLETACION				  *
* PRE!:  { fp-> [////], s-> S, tad_ed-> TC = T + { C } para algun tad T } *
* POST!: { fp-> [////li1,...,lik], donde k = # de argumentod del TDI de s,*
*		y para j = 1,...,k:                			  *
*	        lij = s(v1,...,v(j-1),C,v(j+1),...,vk) con vm variable de *
*		tipo correspondiente al argumento m-esimo de s.		} *
* LLAMA A:     void   genere_plantilla_selectora			  *
*	void   copie_lista_expresion_apunt 	void   construya_exp	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
genere_axs_compl (FILE * fp, nodo_select * s)
{
/*----------------------------------------------------------------------*/

  nodo_expresion *plant_li_ax=NULL, *li_ax=NULL;        /* v-tamara 29/May/1997 */
  nodo_argum *arg, *arg_tdi=NULL;                       /* v-tamara 29/May/1997 */
  char cont;
  texpr exp_li_ax;

  cont = '1';
  genere_plantilla_selectora (s, &plant_li_ax, &cont);
  for (arg = plant_li_ax->argumentos; arg != NULL; arg = arg->sig)
    {
      if (!strcmp (arg->expr->tipo, tad_ed->nombre))
	{
	  copie_lista_expresion_apunt (plant_li_ax, &li_ax, arg, &arg_tdi);
	  strcpy (arg_tdi->expr->nombre, tad_ed->info_tipo->sig->nombre);
	  strcpy (arg_tdi->expr->tipo, tad_ed->nombre);
	  construya_exp (li_ax, exp_li_ax,"");
	  fprintf (fp, "%s=\n", exp_li_ax);
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_inicial_completacion(cad12 nom_elem_com)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Poner al elemento de completacion como una nueva inicial   *
*	       del tad.							  *
* DESCRIPCION: Recorre la lista de iniciales del tad_ed (heredadas del pa-*
*	 dre) y adiciona al final la nueva inicial <nom_elem_com>.	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	COMPLETACION				  *
* PRE!:  { nom_elem_com = C, tad_ed->iniciales = <i1,...,in>, n >= 0    } *
* POST!: { tad_ed->iniciales = <i1,...,in,C>.                           } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
genere_inicial_completacion (cad12 nom_elem_com)
{
/*----------------------------------------------------------------------*/

  nodo_generadora *inic, *inicial;


  inic = (nodo_generadora *) malloc (sizeof (nodo_generadora));
  HayMem(inic,sizeof (nodo_generadora));                /* v-tamara 29/May/1997 */
  strcpy (inic->nombre, nom_elem_com);
  strcpy (inic->nombre_fte, nom_elem_com);
  inic->dominio = NULL;
  inic->sig = NULL;
  if (tad_ed->iniciales == NULL)
    tad_ed->iniciales = inic;
  else
    {
      for (inicial = tad_ed->iniciales; inicial->sig != NULL;
	   inicial = inicial->sig)
	;
      inicial->sig = inic;
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_selectoras_completacion(nodo_tad *tad_origen,		  *
*				      cad12 nom_elem_com)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar las selectoras de un tad de tipo COMPLETACION.     *
* DESCRIPCION: El tad_ed hereda las selectoras del <tad_origen> que deben *
*        ser extendidas definiendolas sobre <nom_elem_com>. Esto se hace  *
*	 generando lados izqs. de los axiomas que logren tal definicion y *
*	 solicitando al usuario los lados derechos respectivos.  	  *
*	 Si este proceso tiene exito, se solicita el status del tad y se  *
*	 adiciona a la lista de tads del ambiente.		  	  *
* PARAMETROS:                                                             *
*   nodo_tad	*tad_origen	Ap. al tad padre de la completacion.      *
*   cad12	nom_elem_com	Nombre del elemento de completacion.	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	COMPLETACION				  *
* PRE!:  { tad_origen-> T, nom_elem_com = C, tad_ed-> TC = T + { C },     *
*	   T.selectoras = <s1,...,sm>, m >= 0, y para i = 1,...,m :	  *
*	   si.axiomas = <ai1,...,aiki>, ki >= 0				  *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { tad -> <t1,...,tn,TC)> <- ult_tad = tad_ed, donde		  *
*               TC.selectoras = <s1,...,sm>, tq. para i = 1,...,m :	  *
*	        si.axiomas = <ai1,...,aiki,bj1,...,bjl>, para l >= 0	  *
*		tq. bj1,...,bjl extiende la definicion de si en   C,   o  *
*          tad -> <t1,...,tn> <- ult_tad, Si la generacion del nuevo tad  *
*	   	no fue exitosa.						} *
* !OJO!: El predicado de igualdad del tad_ed no esta siendo generado.     *
* LLAMA A: 			void  copie_lista_selectoras		  *
*   short es_prefijo 		short construya_axs_selec_sum_direc	  *
*   short pida_status 		void  esc_nom_tad_actual		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

int
genere_selectoras_completacion (nodo_tad * tad_origen)
{
/*----------------------------------------------------------------------*/

  nodo_select *s, *ult_select=NULL;     /* v-tamara 29/May/1997 */
  short correcto;
  char *adic;
  nodo_select *seq;

  assert(tad_ed!=NULL);                 /* v-tamara 29/May/1997 */

  /* VTP. 11/Jun/1997 Se agregaron las siguientes 3 lineas. Ver Modificaciones */
  adic=(char *)malloc(sizeof(char)*(strlen(tad_ed->nombre)+3));
  HayMem(adic,sizeof(char)*(strlen(tad_ed->nombre)+3));
  sprintf(adic,"_%s",tad_ed->nombre);

  copie_lista_selectoras (tad_origen->selectoras, &(tad_ed->selectoras),
			&ult_select, tad_origen->nombre, tad_ed->nombre, "",
			  tad_origen, tad_ed->nombre);
  /* Suprimir la igualdad */
  s = tad_ed->selectoras;
  while ((s != NULL) && (s->sig != NULL))
    {
      if (es_prefijo ("EQ_", s->sig->nombre))
	{
	  s->sig = s->sig->sig;
	}
      else
	{
	  s = s->sig;
	}
    }
  if ((tad_ed->selectoras != NULL) &&
      es_prefijo ("EQ_", tad_ed->selectoras->nombre))
    {
      tad_ed->selectoras = tad_ed->selectoras->sig;
    }
  /* VTP. 6/May/1998.  Equality administration fixed */
  seq=NULL;
  if (tad_ed->iniciales!=NULL || tad_ed->constructoras!=NULL)
    {
      adic_select_eq (tad_ed, &seq);
    }
  for (s = tad_ed->selectoras, correcto = 1; 
       s != NULL && s!=seq && correcto;
       s = s->sig)
    {
      correcto = construya_axs_selec_sum_direc (tad_origen, s,adic,NULL);
    }
  if (correcto && s==seq && seq!=NULL)
    {
      correcto = construya_axiomas (seq, 0, "", tad_ed->nombre,NULL);
    }
  if (correcto)
    {
      ult_tad->sig = tad_ed;
      ult_tad = tad_ed;
      tad_actual = tad_ed;
    }
  else
    {
      liberar_tad(&tad_ed); /* VTP. 6/May/1998  Fixing memory admin. */
    }
  free(adic);
  return correcto;
/*----------------------------------------------------------------------*/
}

