 /* DEF_TIPOS.C                                                  */
/* Arhivo de definicion de tipos.                               */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include "definic.h"


/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 7/May/1997 */


/**************************************************************************
* void copie_tad(cad12 nombre)						  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar un tad con un nuevo nombre.                         *
* DESCRIPCION: Pide un tad del ambiente y adiciona al ambiente un tad     *
*        identico pero con un nuevo <nombre>.                             *
* PARAMETROS:                                                             *
*	cad12		nombre			Nombre del nuevo tad	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, tad_actual -> ti,  1 <= i <= n       } *
* POST!: { p -> tk, 1 <= k <= n, t(n+1) es identico a tk salvo el nombre  *
*	    tad -> <t1,...,tn,t(n+1)> <- ult_tad, tad_actual -> t(n+1)    *
* !OJO!: Usa funciones de FENIX.                                          *
* LLAMA A:                                                                *
*   nodo_tad	*pida_tad_ambiente    	void	genere_copia_tad 	  *
*   void 	esc_nom_tad_actual      void 	cree_nodo_tad		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Ene-90 *
**************************************************************************/

void
copie_tad (cad12 nombre)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;

  assert (nombre != NULL);	/* VTP. 7/May/1997 */

  p = pida_tad_ambiente (mensajes[47]);		/* 47:Copiar TAD */
  if (p != NULL)
    {
      cree_nodo_tad ();
      genere_copia_tad (p, p->nombre, ult_tad, nombre);
      /* VTP. Mar/8/1998 Ahora si pide el estado del tipo*/
      ult_tad->status = pida_status ();
      tad_actual = ult_tad;
      esc_nom_tad_actual ();
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void cree_nodo_tad()							  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Crear una nueva estructura para un tad.                    *
* DESCRIPCION: Pide espacio de memoria para una estructura de tipo nodo_  *
*        tad y la pone como ultima en la lista de tads del ambiente (queda*
* 	 apuntada por ult_tad) e inicializa en NULL todos los campos de   *
*	 tipo apuntador la nueva estructura. Ademas deja la nueva estruc- *
*	 tura creada como tad actual (apuntado por tad_actual).           *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad		Apunt. a la cabeza de la lista de los tads*
*				del ambiente.				  *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 0                                       } *
* POST!: { tad -> <t1,t2,...,tn,t(n+1)> <- ult_tad  } y todos             *
*	   los campos de tipo apuntador de t(n+1) estan en NULL         } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Ene-90 *
**************************************************************************/

void
cree_nodo_tad ()
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;

  p = crear_tad ();
  if (tad == NULL)
    tad = p;
  else
    ult_tad->sig = p;
  ;
  ult_tad = p;
  tad_actual = p;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void instancie_parametro(cad12 nombre)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Instanciar un parametro un tad.                            *
* DESCRIPCION: Pide un tad T del ambiente, un parametro P de este y un nom*
*	 bre IP para instanciar P y adiciona al ambiente un nuevo tad     *
*	 <nombre> que es identico a T excepto que P esta instanciado con  *
*	 IP. Si IP es el nombre de un tad en el ambiente o en el directo- *
*	 rio, se suprime IP de la lista de parametros del nuevo tad.      *
* PARAMETROS:                                                             *
*	cad12		nombre			Nombre del nuevo tad	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*ult_tad	Apunt. al ultimo tad de la lista de tads  *
*				del ambiente.				  *
*    nodo_tad	*tad_actual	Apunt. al tad actual de la lista de tads  *
*				del ambiente.				  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, tad_actual -> ti,  1 <= i <= n,	  *
*	   nombre = NT							} *
* POST!: { tad -> <t1,...,tn,t(n+1)> <- ult_tad, tad_actual -> t(n+1)     *
*          p -> tk, 1 <= k <= n, nom_param = P, nombre_nuevo_param = IP,  *
*	   t(n+1) es identico a tk salvo en el nombre y que 		  *
*	   tk.parametros = <..pPq..> mientras que:			  *
*	   t(n+1).parametros = <..pq..>,   Si IC es un tad conocido o	  *
*	   t(n+1).parametros = <..pICq..>, Sino.			  *
* !OJO!: Usa funciones de FENIX.                                          *
* LLAMA A:                                                                *
*   nodo_tad	*pida_tad_ambiente    	void	genere_copia_tad 	  *
*   void 	esc_nom_tad_actual      void 	cree_nodo_tad		  *
**************************************************************************/

void
instancie_parametro (cad12 nombre)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p, *p_inst;
  cad50 nom_param, nuevo_nombre_param;

  assert (nombre != NULL);	/* VTP. 7/May/1997 */
  p = pida_tad_ambiente (mensajes[46]);		/* 46:TAD generico */
  if (p != NULL)
    {
      tad_actual = p;
      if (p->parametros == NULL)
	{
	  escriba_mensaje (107);	/* 107:Este TAD no es generico */
	  return;
	}
      admon_menu_scroll (menus_manta[PARAMETROS], p->parametros, nom_param);
      if (strcmp (nom_param, ""))
	{
	  p_inst = pida_tad_ambiente (mensajes[45]);	/* 45:Instanciar con */
	  if (p_inst == NULL)
	    return;
	  /* VTP. 1/May/1998.  Instance with parametric types is not allowed*/
	  if (p_inst->parametros!=NULL ||
	      (p_inst->iniciales==NULL && p_inst->constructoras==NULL)) 
	    {
	      escriba_mensaje(178);
	      return;
	    }
	  strcpy (nuevo_nombre_param, p_inst->nombre);
	  if (strcmp (nuevo_nombre_param, ""))
	    {
	      cree_nodo_tad ();
	      genere_copia_tad (p, p->nombre, ult_tad, nombre);
	      cambiar_nombre (ult_tad, nom_param, nuevo_nombre_param);
	      tad_actual = ult_tad;
/*******    Suprimir el param. si es tad existente ***/
	      p = existe_tad (nuevo_nombre_param);
	      if (p != NULL)
		{
		  suprima_param_tad (tad_actual, nuevo_nombre_param);
		}
              /* VTP. 8/Mar/1998 Se agrego la pedida del estado del Tipo */
              tad_actual->status = pida_status ();
	      esc_nom_tad_actual ();
	    }
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void nuevo_tad()						   	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Controlar el proceso de definicion de nuevos tipos.     	  *
* DESCRIPCION: Despliega el menu de opciones de definicion de nuevos tads *
*	 e invoca a la funcion correspondiente a la seleccion del usuario.*
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, 					} *
* POST!: { tad -> <t1,t2,...,tn,nt1,...,ntm> <- ult_tad, m >= 0 donde     *
*	   nt1,...,ntm son los nuevos tads definidos por el usuario.      *
* LLAMA A:                                                                *
*    void  lea_nombre_nuevo_tad 	void  edite_tad_nuevo		  *
*    void  producto_cartesiano		void  restriccion		  *
*    void  suma_directa			                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 30-Abr-90 *
**************************************************************************/

void
nuevo_tad ()
{
/*----------------------------------------------------------------------*/

  cad12 nombre;
  int opcion;
  texpr merr;
  texpr tadname;

  lea_nombre_nuevo_tad (nombre);
  /* VTP. 4/May/1998. Just good names */
  if (strcmp (nombre, ""))    {
      if (existsSymbol(symtable,nombre,tadname))
	{
	  sprintf(merr,mensajes[180],nombre,tadname);
	  escriba_cadena(merr,1);
	  return;
	}
      if (existsSymbolParameter(symtable,nombre,tadname))
	{
	  sprintf(merr,mensajes[181],nombre,tadname);
	  escriba_cadena(merr,1);
	  return;
	}
      
      /* Muestra las opciones que hay para crear TADs */
      opcion = admon_menu (menus_manta[NUEVO_TAD]);
      /* VTP. 10/Feb/1998 Se quito un while que habia y se dejo if
         Asi despues de crear un tipo nuevo vuelve al menu de tipos */
      if (opcion != NO_OPCION &&
	     opcion != menus_manta[NUEVO_TAD]->num_op - 2)
	{
	  switch (opcion)
	    {
	    case 0:		/* INDUCTIVA */
	      edite_tad_nuevo (nombre);
	      break;
	    case 1:		/* INSTANCIACION */
	      instancie_parametro (nombre);
	      break;
	    case 2:		/* COPIA */
	      copie_tad (nombre);
	      break;
	    case 3:		/* PRODUCTO CARTESIANO */
	      prod_cart_implicito (nombre);
	      break;
	    case 4:		/* RESTRICCION */
	      restriccion (nombre);
	      break;
	    case 5:		/* NT_EXTENSION Nombre cambiado por VTP */
	      suma_directa (nombre);
	      break;
	    case 6:
	      nuevo_represent (nombre);
	      break;
	    case 7:		/* SALIR */
	      break;
	    case 8:		/* AYUDA */
	      escriba_mensaje (159);
	      break;
	    default:
	      escriba_mensaje (146);
	      break;
	    }
	  /* VTP. 4/May/1998. Administration of symbol table*/
	  if (ult_tad!=NULL && updateSymbols(&symtable,ult_tad,merr)!=0)
	    {
	      escriba_cadena(merr,1);
	    }
	}
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void suma_directa(cad12 nombre)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar la suma directa o extension de uno o dos tads.     *
* DESCRIPCION: Permite al usuario escoger el modo de generacion de la ex- *
*	 tension: Suma Directa, Completacion o Variacion e invoca a la fun*
*	 cion apropiada de acuerdo a la escogencia.  			  *
* PARAMETROS:                                                             *
*	cad12		nombre			Nombre del nuevo tad	  *
* PRE!:  { tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2, tad_actual -> ti,  1 <= i <= n       } *
* POST!: { tad -> <t1,...,tn,t(n+1)> <- ult_tad, tad_actual -> t(n+1)     *
*	    t(n+1) es un tad generado como extension de tads en el ambient*
*	   o,			   	  				  *
*          tad -> <t1,t2,...,tn> <- ult_tad.                            } *
* LLAMA A: 								  *
*		void	extension                                	  *
*		void 	completacion 		void 	variacion 	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
suma_directa (cad12 nombre)
{
/*----------------------------------------------------------------------*/
  int opcion;

  assert (nombre != NULL);	/* VTP. 7/May/1997 */
  do
    {
      opcion = admon_menu (menus_manta[TIPO_SUM_DIREC]);
      switch (opcion)
	{
	case 0:		/* DEF. POR SUMA DIRECTA */
	  extension (nombre);
	  break;
	case 1:		/* DEF. POR COMPLETACION */
	  completacion (nombre);
	  break;
	case 2:		/* DEF. POR VARIACION */
	  variacion (nombre);
	  break;
	case 3:		/* SALIR */
	  break;
	case 4:		/* AYUDA */
	  escriba_mensaje (158);
	  break;
	case NO_OPCION:
	  break;
	default:
	  escriba_mensaje (146);
	  break;
	}
    }
  while (opcion > 3);
/*----------------------------------------------------------------------*/
}
