/* PROY_TAD.C                                                           */
/* Manipula partes de un TAD.                                           */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "definic.h"
#include "apimanta.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* j-tamara 29/May/1997 */

/**************************************************************************
* void adic_select_eq(nodo_tad *p,nodo_select **select_eq)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Adicionar el predicado de igualdad a un tad.               *
* DESCRIPCION: Crea una nueva selectora de igualdad de <p>:               *
*        EQ_<p>: <p> (I) x <p> (I) -> BOOL                          	  *
*        y (*select_eq) apunt. a la estructura que la contiene.           *
* PARAMETROS:                                  	                          *
*   nodo_tad		*p		Ap. al tad al que se le adiciona  *
*					la igualdad.			  *
*   nodo_select		**select_eq	Ap. a la estructura que contiene  *
*					la nueva selectora EQ_<p>.	  *
* PRE!:  { p-> P                                                        } *
* POST!: { (*select_eq)-> S , donde S es el predicado de igualdad de P:   *
*        	EQ_<p>: <p> (I) x <p> (I) -> BOOL                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

void
adic_select_eq (nodo_tad * p, nodo_select ** select_eq)
{
/*----------------------------------------------------------------------*/

  nodo_select *ult_select, *s;

  assert(p!=NULL);                      /* j-tamara 29/May/1997 */
  for (ult_select = p->selectoras;
       (ult_select != NULL) && (ult_select->sig != NULL);
       ult_select = ult_select->sig)
    {
      ult_select->caracterizadora = 0;
    }
  s = (nodo_select *) malloc (sizeof (nodo_select));
  HayMem(s,sizeof (nodo_select));               /* j-tamara 29/May/1997 */
  s->sig = NULL;
  strcpy (s->nombre, "EQ_");
  strcat (s->nombre, p->nombre);
  strcpy (s->nombre_fte, "EQ");
  strcpy (s->codominio, "BOOL");
  s->caracterizadora = 1;
  s->dominio = (nodo_dom *) malloc (sizeof (nodo_dom));
  HayMem(s->dominio,sizeof (nodo_dom));             /* j-tamara 29/May/1997 */
  strcpy (s->dominio->nombre, p->nombre);
  s->dominio->induccion = 1;
  s->dominio->sig = (nodo_dom *) malloc (sizeof (nodo_dom));
  HayMem(s->dominio->sig,sizeof (nodo_dom));            /* j-tamara 29/May/1997 */
  strcpy (s->dominio->sig->nombre, p->nombre);
  s->dominio->sig->induccion = 1;
  s->dominio->sig->sig = NULL;
  s->axiomas=NULL;
  if (p->selectoras == NULL)
    p->selectoras = s;
  else
    ult_select->sig = s
      ;
  (*select_eq) = s;
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* nodo_select *busque_selectora(texpr nombre, nodo_select *cab_ops)       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Retorna el nodo de la selectora con nombre, y que aparce   *
*              en la lista cab_ops. Si no existe retorna NULL             *
**************************************************************************/

nodo_select *
busque_selectora (texpr nombre, nodo_select * cab_ops)
{
/*----------------------------------------------------------------------*/
  nodo_select *s;

  for (s = cab_ops; s != NULL && strcmp (nombre, s->nombre); s = s->sig);
  return (s);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_constructora_tipo(texpr token,cad20 tipo)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una constructora de un tipo.    *
* DESCRIPCION: Busca el tad <tipo> y busca a <token> en su lista de cons- *
*	 tructoras.							  *
* PARAMETROS:                                                             *
*   texpr		token		Nombre de la supuesta operacion.  *
*   cad20		tipo		Codominio de la supuesta operacion*
* RESULTADO:  short	1, Si <token> es una constructora de <tipo>	  *
*			0, Sino.					  *
* PRE!:  { token = TO, tipo = T                                         } *
* POST!: { es_constructora_tipo = 1, Si TO es una constructora de T, o	  *
*          es_constructora_tipo = 0, Sino.                              } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_constructora_tipo (texpr token, cad20 tipo)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;

  p = existe_tad (tipo);
  if (p != NULL)
    return (es_generadora (token, p->constructoras));
  else
    return (0)
      ;
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* short es_constructora_tipo_dom(cad20 nombre,cad20 tipo,		  *
*				 nodo_nombre **dominio)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una constructora de un tipo. En *
*	       caso afirmativo retornar su dominio.			  *
* DESCRIPCION: Busca el tad <tipo> y busca a <nombre> en su lista de cons *
*	 tructoras.							  *
* PARAMETROS:                                                             *
*   texpr		nombre		Nombre de la supuesta operacion.  *
*   cad20		tipo		Codominio de la supuesta operacion*
*   nodo_nombre		**dominio	Ap. al dominio de la operacion.   *
* V.GLOBALES:                                                             *
*   nodo_tad		*tad_actual	Apunt. al tad en edicion.         *
* RESULTADO:  short	1, Si <nombre> es una constructora de <tipo>	  *
*			0, Sino.					  *
* PRE!:  { nombre = TO, tipo = T                                        } *
* POST!: { es_constructora_tipo_dom = 1, Si TO es una constructora de T y *
*		(*dominio) ap. a la lista de su dominio.  o		  *
*          es_constructora_tipo_dom = 0, Sino.                          } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_constructora_tipo_dom (cad20 nombre, cad20 tipo, nodo_nombre ** dominio)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;
/*** Alvaro
  nodo_generadora 	*cabeza;
 ***/
  short si_es;

  assert(tad_ed!=NULL);                 /* j-tamara 29/May/1997 */
  if (!strcmp (tipo, tad_ed->nombre))
    {
      if (es_generadora_con_codom_tipo (nombre,
					tad_ed->constructoras, dominio))
	{
	  si_es = 1;
	}
      else
	{
	  si_es = 0;
	  /* Codigo util cuando se implemente herencia.
	     switch (tad_ed->tipo) {
	     case RESTRICCION:
	     case COMPLETACION:
	     si_es =
	     (es_constructora_tipo_dom(nombre,tad_ed->info_tipo->nombre,
	     dominio));
	     break;
	     case NT_EXTENSION: * VTP. 7/Mar/1997 *
	     case VARIACION:
	     si_es =
	     ((es_constructora_tipo_dom(nombre,tad_ed->info_tipo->nombre,
	     dominio))
	     ||
	     (es_constructora_tipo_dom(nombre,
	     tad_ed->info_tipo->sig->nombre,dominio)));
	     break;
	     default:
	     si_es = 0;
	     break;
	     }
	   */
	}
    }
  else if (((p = existe_tad (tipo)) != NULL) &&
	   (es_generadora_con_codom_tipo (nombre,
					  p->constructoras, dominio)))
    {
      si_es = 1;
    }
  else
    {
      si_es = 0;
    }
  return (si_es);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_generadora(cad20 nombre,nodo_generadora *cabeza)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una generadora de una lista.    *
* DESCRIPCION: Busca a <nombre> en la lista de generadoras ap. por <cabeza*
* PARAMETROS:                                                             *
*   cad20		nombre		Nombre de la supuesta operacion.  *
*   nodo_generadora     *cabeza         Ap. a la lista de generadoras.    *
* RESULTADO:  short	1, Si <nombre> es una generadora de la lista.	  *
*			0, Sino.					  *
* PRE!:  { nombre = TO, cabeza -> LISTA.                                } *
* POST!: { es_generadora = 1, Si TO es una generadora de LISTA.		  *
*          es_generadora = 0, Sino.                                     } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_generadora (cad20 nombre, nodo_generadora * cabeza)
{
/*----------------------------------------------------------------------*/

  for (; ((cabeza != NULL) && strcmp (cabeza->nombre, nombre));
       cabeza = cabeza->sig)
    ;
  if (cabeza == NULL)
    return (0);
  else
    return (1)
      ;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_generadora_algun_tipo(cad20 nombre,cad20 tipo)    		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una generadora y de que tipo    *
* DESCRIPCION: Busca a <nombre> en la lista de generadoras de tad_ed. Sino*
*	busca a <nombre> en la lista de generadoras de cada uno de los    *
*	tads en el ambiente. Si lo encuentra retorna el nombre del tad don*
*	de lo encontro.							  *
* PARAMETROS:                                                             *
*   cad20		nombre		Nombre de la supuesta operacion.  *
*   cad20		tipo		Codominio de la operacion.	  *
* V.GLOBALES:                                                             *
*   nodo_tad		*tad_actual	Apunt. al tad en edicion.         *
* RESULTADO:  short	1, Si <nombre> es una generadora.            	  *
*			0, Sino.					  *
* PRE!:  { nombre = TO                                                  } *
* POST!: { es_generadora = 1, Si TO es una generadora de algun tad T	  *
*		(tad_ed o uno de los tads del ambiente).	   	  *
*		tipo = T.nombre						  *
*          es_generadora = 0, Sino.                                     } *
* LLAMA A: 	short	es_generadora					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_generadora_algun_tipo (cad20 nombre, cad20 tipo)
{
/*----------------------------------------------------------------------*/

  short es_generad;
  nodo_tad *p;

  es_generad = 0;
  if (tad_ed!=NULL &&
      (es_generadora (nombre, tad_ed->iniciales) ||
      es_generadora (nombre, tad_ed->constructoras)))
    {
      strcpy (tipo, tad_ed->nombre);
      es_generad = 1;
    }
  else
    {
      for (p = tad; (p != NULL) && (!es_generad); p = p->sig)
	{
	  es_generad = (short) (es_generadora (nombre, p->iniciales) ||
				es_generadora (nombre, p->constructoras));
	  strcpy (tipo, p->nombre);
	}
    }
  return (es_generad);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_generadora_con_codom_tipo(cad20 nombre,nodo_generadora *cabeza,*
*				     nodo_nombre **dominio)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una generadora de una lista. En *
*	       caso afirmativo retornar su dominio.			  *
* DESCRIPCION: Busca a <nombre> en la lista de generadoras apunt. por <ca *
*	beza>. Si lo encuentra retorna a (*dominio) apunt. a su dominio.  *
* PARAMETROS:                                                             *
*   cad20		nombre		Nombre de la supuesta operacion.  *
*   nodo_generadora     *cabeza         Ap. a la lista de generadoras.    *
*   nodo_nombre		**dominio	Ap. al dominio de la generadora.  *
* RESULTADO:  short	1, Si <nombre> es una generadora de la lista.	  *
*			0, Sino.					  *
* PRE!:  { nombre = TO, cabeza -> LISTA.                                } *
* POST!: { es_generadora = 1, Si TO es una generadora de LISTA.		  *
*			(*dominio) ap. al dominio de TO.		  *
*          es_generadora = 0, Sino.                                     } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_generadora_con_codom_tipo (cad20 nombre, nodo_generadora * cabeza,
			      nodo_nombre ** dominio)
{
/*----------------------------------------------------------------------*/

  for (; ((cabeza != NULL) && strcmp (cabeza->nombre, nombre));
       cabeza = cabeza->sig)
    ;
  if (cabeza == NULL)
    return (0);
  else
    {
      (*dominio) = cabeza->dominio;
      return (1);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_inicial_tipo(texpr token,cad20 tipo)			  	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una inicial de un tipo.    	  *
* DESCRIPCION: Busca el tad <tipo> y busca a <token> en su lista de inicia*
*	 les.      							  *
* PARAMETROS:                                                             *
*   texpr		token		Nombre de la supuesta operacion.  *
*   cad20		tipo		Codominio de la supuesta operacion*
* RESULTADO:  short	1, Si <token> es una inicial de <tipo>	  	  *
*			0, Sino.					  *
* PRE!:  { token = TO, tipo = T                                         } *
* POST!: { es_inicial_tipo = 1, Si TO es una inicial de T, o	  	  *
*          es_inicial_tipo = 0, Sino.                              	} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_inicial_tipo (texpr token, cad20 tipo)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;

  p = existe_tad (tipo);
  if (p != NULL)
    return (es_generadora (token, p->iniciales));
  else
    return (0);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_inicial_tipo_dom(cad20 nombre,cad20 tipo,nodo_nombre **dominio)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una inicial de un tipo. En 	  *
*	       caso afirmativo retornar su dominio.			  *
* DESCRIPCION: Busca el tad <tipo> y busca a <nombre> en su lista de ini- *
*	 ciales.   							  *
* PARAMETROS:                                                             *
*   cad20 		nombre		Nombre de la supuesta operacion.  *
*   cad20		tipo		Codominio de la supuesta operacion*
*   nodo_nombre		**dominio	Ap. al dominio de la operacion.   *
* V.GLOBALES:                                                             *
*   nodo_tad		*tad_actual	Apunt. al tad en edicion.         *
* RESULTADO:  short	1, Si <nombre> es una inicial de <tipo>	  	  *
*			0, Sino.					  *
* PRE!:  { nombre = TO, tipo = T                                        } *
* POST!: { es_inicial_tipo_dom = 1, Si TO es una inicial de T y 	  *
*		(*dominio) ap. a la lista de su dominio.  o		  *
*          es_inicial_tipo_dom = 0, Sino.                               }
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_inicial_tipo_dom (cad20 nombre, cad20 tipo, nodo_nombre ** dominio)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;
/*** Alvaro
  nodo_generadora	*cabeza;
 ***/
  short si_es;

  assert(tad_ed!=NULL);
  if (!strcmp (tipo, tad_ed->nombre))
    {
      if (es_generadora_con_codom_tipo (nombre, tad_ed->iniciales, dominio))
	{
	  si_es = 1;
	}
      else
	{
	  si_es = 0;
/*-- Util si hay herencia
	    switch (tad_ed->tipo) {
	      case RESTRICCION:
	      case COMPLETACION:
	 	si_es =
		 (es_inicial_tipo_dom(nombre,tad_ed->info_tipo->nombre,
			  	      dominio));
	   	break;
	      case NT_EXTENSION: *** VTP. 7/Mar/1997 ***
	      case VARIACION:
	 	si_es =
		 ((es_inicial_tipo_dom(nombre,tad_ed->info_tipo->nombre,
		                       dominio))
		  ||
		  (es_inicial_tipo_dom(nombre,tad_ed->info_tipo->sig->nombre,
		                       dominio)));
	   	break;
	      default:
		si_es = 0;
		break;
	    } --*/
	}
    }
  else if (((p = existe_tad (tipo)) != NULL) &&
	   (es_generadora_con_codom_tipo (nombre, p->iniciales, dominio)))
    {
      si_es = 1;
    }
  else
    {
      si_es = 0;
    }
  return (si_es);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_parametro(cad20 nombre,nodo_nombre *cabeza)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es un parametro de una lista.      *
* DESCRIPCION: Busca a <nombre> en la lista de parametros ap. por <cabeza>*
* PARAMETROS:                                                             *
*   cad20		nombre		Nombre del supuesto parametro.	  *
*   nodo_nombre        	*cabeza         Ap. a la lista de parametros.     *
* RESULTADO:  short	1, Si <nombre> es un parametro de la lista.	  *
*			0, Sino.					  *
* PRE!:  { nombre = TO, cabeza -> LISTA.                                } *
* POST!: { es_parametro = 1, Si TO es un parametro de LISTA.		  *
*          es_parametro = 0, Sino.                                     } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_parametro (cad20 nombre, nodo_nombre * cabeza)
{
/*----------------------------------------------------------------------*/

  for (; ((cabeza != NULL) && strcmp (cabeza->nombre, nombre));
       cabeza = cabeza->sig)
    ;
  if (cabeza == NULL)
    return (0);
  else
    return (1)
      ;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_selectora(cad20 nombre,nodo_select *cabeza)			  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una selectora de una lista.     *
* DESCRIPCION: Busca a <nombre> en la lista de selectoras ap. por <cabeza>*
* PARAMETROS:                                                             *
*   cad20		nombre		Nombre de la supuesta operacion.  *
*   nodo_select		*cabeza         Ap. a la lista de selectoras.     *
* RESULTADO:  short	1, Si <nombre> es una selectora de la lista.	  *
*			0, Sino.					  *
* PRE!:  { nombre = TO, cabeza -> LISTA.                                } *
* POST!: { es_selectora = 1, Si TO es una selectora de LISTA.		  *
*          es_selectora = 0, Sino.                                      } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_selectora (cad20 nombre, nodo_select * cabeza)
{
/*----------------------------------------------------------------------*/

  for (; ((cabeza != NULL) && strcmp (cabeza->nombre, nombre));
       cabeza = cabeza->sig)
    ;
  if (cabeza == NULL)
    return (0);
  else
    return (1)
      ;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_selectora_algun_tipo(cad20 nombre,cad20 tipo)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una selectora de algun tipo.    *
* DESCRIPCION: Determina si <nombre> es una selectora del tad_ed. Sino    *
*	 recorre la lista de tads en el ambiente buscando alguno que tenga*
*	 una selectora <nombre>. Si lo encuentra retorna en <tipo> el co- *
*	 dominio de la selectora.					  *
*	 tructoras.							  *
* PARAMETROS:                                                             *
*   cad20		nombre		Nombre de la supuesta operacion.  *
*   cad20		tipo		Codominio de la operacion.	  *
* RESULTADO:  short	1, Si <nombre> es una selectora de algun tipo.    *
*			0, Sino.					  *
* PRE!:  { nombre = TO                                                  } *
* POST!: { es_selectora  = 1, Si TO es una selectora  de algun tad T	  *
*		(tad_ed o uno de los tads del ambiente).	   	  *
*		tipo = codominio de TO.					  *
*          es_selectora  = 0, Sino.                                     } *
* LLAMA A: 	short	es_selectora_dom_codom				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_selectora_algun_tipo (cad20 nombre, cad20 tipo)
{
/*----------------------------------------------------------------------*/

  short es_select;
  nodo_tad *p;
  nodo_dom *tipo_dom=NULL;              /* j-tamara 29/May/1997 */

  assert(tad_ed!=NULL);                 /* j-tamara 29/May/1997 */
  es_select = 0;
  if (es_selectora_dom_codom (nombre, tad_ed->selectoras, &tipo_dom, tipo))
    {
      es_select = 1;
    }
  else
    {
      for (p = tad; (p != NULL) && (!es_select); p = p->sig)
	{
	  es_select = (es_selectora_dom_codom (nombre, p->selectoras, &tipo_dom,
					       tipo));
	}
    }
  return (es_select);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_selectora_con_codom_tipo(cad20 nombre,cad20 tipo,		  *
*				   nodo_select *cabeza,nodo_dom **dominio)*
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una selectora de un tipo en una *
*	       lista. En caso afirmativo, retornar su dominio.            *
* DESCRIPCION: Busca a <nombre> en la lista de selectoras ap. por <cabeza>*
*   	Si la encuentra verifica que su codominio sea <tipo> y retorna    *
*	a (*dominio) apuntando al dominio de la selectora.		  *
* PARAMETROS:                                                             *
*   cad20		nombre		Nombre de la supuesta operacion.  *
*   cad20		tipo		Codominio de la operacion.	  *
*   nodo_select		*cabeza         Ap. a la lista de selectoras.     *
*   nodo_dom   		**dominio	Ap. al dominio de la selectora.   *
* RESULTADO:  short	1, Si <nombre> es una selectora de la lista.	  *
*			0, Sino.					  *
* PRE!:  { nombre = TO, tipo = T, cabeza -> LISTA.                      } *
* POST!: { es_selectora_con_codom_tipo = 1, Si TO es una selectora con co-*
*	 				    dominio T de LISTA.		  *
*          es_selectora_con_codom_tipo = 0, Sino.                       } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_selectora_con_codom_tipo (cad20 nombre, cad20 tipo,
			     nodo_select * cabeza, nodo_dom ** dominio)
{
/*----------------------------------------------------------------------*/

  for (; ((cabeza != NULL) && strcmp (cabeza->nombre, nombre));
       cabeza = cabeza->sig)
    ;
  if ((cabeza == NULL) || (strcmp (cabeza->codominio, tipo)))
    return (0);
  else
    {
      (*dominio) = cabeza->dominio;
      return (1);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_selectora_dom_codom(cad20 nombre,nodo_select *cabeza,	  *
*			       nodo_dom **tipo_dom,cad20 tipo_codom)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una selectora de una lista. En  *
*	caso afirmativo, retornar su dominio y su tipo.            	  *
* DESCRIPCION: Busca a <nombre> en la lista de selectoras ap. por <cabeza>*
*   	Si la encuentra, retorna a (*dominio) apuntando al dominio de la  *
*	selectora y en <tipo_codom> su codominio.			  *
* PARAMETROS:                                                             *
*   cad20		nombre		Nombre de la supuesta operacion.  *
*   nodo_select		*cabeza         Ap. a la lista de selectoras.     *
*   nodo_dom   		**tipo_dom	Ap. al dominio de la selectora.   *
*   cad20		tipo_codom	Codominio de la operacion.	  *
* RESULTADO:  short	1, Si <nombre> es una selectora de la lista.	  *
*			0, Sino.					  *
* PRE!:  { nombre = TO, cabeza -> LISTA.                      		} *
* POST!: { es_selectora_dom_codom = 1, Si TO es una selectora de la LISTA,*
*					(*tipo_dom) ap. su dominio y 	  *
*					tipo_codom = a su codominio	  *
*	 				    dominio T de LISTA.		  *
*          es_selectora_dom_codom = 0, Sino.                       	} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_selectora_dom_codom (cad20 nombre, nodo_select * cabeza,
			nodo_dom ** tipo_dom, cad20 tipo_codom)
{
/*----------------------------------------------------------------------*/

  for (; ((cabeza != NULL) && strcmp (cabeza->nombre, nombre));
       cabeza = cabeza->sig)
    ;
  if (cabeza == NULL)
    return (0);
  else
    {
      (*tipo_dom) = cabeza->dominio;
      strcpy (tipo_codom, cabeza->codominio);
      return (1);
    }
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short es_selectora_tipo_dom(cad20 nombre,cad20 tipo,nodo_dom **dominio) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si un nombre es una selectora de un tipo de al- *
*	       gun tad. En caso afirmativo, retornar su dominio.          *
* DESCRIPCION: Busca a <nombre> en la lista de selectoras de tad_ed. Si no*
*   	la encuentra, la busca en la selectoras de los tads en el ambiente*
*	verificando que su codominio sea <tipo>. Si la encuentra, retorna *
*	a (*dominio) apuntando al dominio de la selectora.		  *
* PARAMETROS:                                                             *
*   cad20		nombre		Nombre de la supuesta operacion.  *
*   cad20		tipo		Codominio de la operacion.	  *
*   nodo_dom   		**dominio	Ap. al dominio de la selectora.   *
* RESULTADO:  short	1, Si <nombre> es una selectora con codom. <tipo> *
*			0, Sino.					  *
* PRE!:  { nombre = TO, tipo = T					} *
* POST!: { es_selectora_tipo_dom = 1, Si TO es una selectora con codominio*
*			T de tad_ed o de algun tad del ambiente           *
*			(*dominio) apunt. a su dominio.			  *
*          es_selectora_tipo_dom = 0, Sino.                       	} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
es_selectora_tipo_dom (cad20 nombre, cad20 tipo, nodo_dom ** dominio)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;
  short es_select;
  texpr merr,ntad;
/*** Alvaro
  nodo_nombre *n;
 ***/

 
  es_select = 0;
  if (es_selectora_con_codom_tipo (nombre, tipo, tad_ed->selectoras, dominio))
    {
      es_select = 1;
    }
  else
    {
      for (p = tad; (p != NULL) && (!es_select); p = p->sig)
	es_select = es_selectora_con_codom_tipo (nombre, tipo, 
						 p->selectoras, dominio);
    }
  if (!es_select && existsSymbol(symtable,nombre,ntad)==SYM_SELECTOR &&
      (p=tipoAmbiente(&tad,ntad,merr))!=NULL)
    {
      es_select = es_selectora_con_codom_tipo (nombre, tipo, 
					       p->selectoras, dominio);
    }
  return (es_select);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short num_parametros(nodo_tad *t)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Contar el numero de parametros de un tad.                  *
* DESCRIPCION: Recorre la lista de parametros de <t>, contandolos.        *
* PARAMETROS:                                                             *
*   nodo_tad		*t	Tad al que se le cuentan los parametros.  *
* RESULTADO:  short		Numero de parametros del tad.             *
* PRE!:  { t-> T, T.parametros = <p1,...,pn>,  n >= 0.                  } *
* POST!: { num_parametros = n.                                          } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
num_parametros (nodo_tad * t)
{
/*----------------------------------------------------------------------*/

  short i;
  nodo_nombre *p;

  for (i = 0, p = t->parametros; (p != NULL); i++, p = p->sig)
    ;
  return (i);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short tiene_argums_induccion(nodo_select *s)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Determinar si una selectora tiene algun argumento de induc-*
*	       cion.							  *
* DESCRIPCION: Recorre la lista del dominio de <s> hasta agotarla o encon-*
*	trar un argumento de induccion. 				  *
* PARAMETROS:                                                             *
*   nodo_select		*s		Ap. a la selectora.		  *
* RESULTADO:  short		1, Si <s> tiene algun argumento de inducc.*
*				0, Sino.				  *
* PRE!:  { s-> S, S.dominio = <d1,...,dn>,  n >=0.                      } *
* POST!: { tiene_argums_induccion = 1, Si existe i,  1 <= i <= n, tq.     *
*				   di es un argumento del TDI de induccion*
*          tiene_argums_induccion = 0, Sino.                            } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
tiene_argums_induccion (nodo_select * s)
{
/*----------------------------------------------------------------------*/

  short tiene;
  nodo_dom *d;

  assert(s!=NULL);                      /* j-tamara 29/May/1997 */
  for (d = s->dominio, tiene = 0; (d != NULL) && (!tiene); d = d->sig)
    {
      if (d->induccion)
	tiene = 1
	  ;
    }
  return (tiene);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* short tiene_igualdad_definida(nodo_tad *p)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:  Determinar si un tad tiene definido el presicado de igualdad*
* DESCRIPCION: Recorre la lista de las selectoras de <p> hasta agotarla o *
*	encontrar el predicado de igualdad.				  *
* PARAMETROS:                                                             *
*   nodo_tad   		*p		Ap. al tad.		  	  *
* RESULTADO:  short		1, Si <p> tiene definida la igualdad.     *
*				0, Sino.				  *
* PRE!:  { p-> P, P.selectoras = <s1,...,sn>,  n >=0.                   } *
* POST!: { tiene_igualdad_definida =  1, Si existe i,  1 <= i <= n, tq.   *
*				   si es un argumento de igualdad EQ_<p>  *
*          tiene_igualdad_definida = 0, Sino.                           } *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 20-Feb-90 *
**************************************************************************/

short
tiene_igualdad_definida (nodo_tad * p)
{
/*----------------------------------------------------------------------*/

  cad50 nom_eq;
  nodo_select *s;

  strcpy (nom_eq, "EQ_");
  strcat (nom_eq, p->nombre);
  for (s = p->selectoras;
       (s != NULL) && (strcmp (s->nombre, nom_eq));
       s = s->sig)
    ;
  if (s != NULL)
    {
      if (strcmp (s->nombre_fte, "EQ"))
	strcpy (s->nombre_fte, "EQ");
      return (1);
    }
  else
    return (0)
      ;
/*----------------------------------------------------------------------*/
}
