/* MAN_REPRESENT.C                                              */
/* Archivo de manejo de representaciones.                       */
 
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include "definic.h"
#include "os_dep.h"
#include "apimanta.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* v-tamara 25/May/1997 */



/**************************************************************************
* short lea_nombre_nueva_represent(cad12 nom_funcion,cad12 nom_tad_fte,   *
*				  cad12 nom_tad_obj,int fila,int columna) *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Leer el nombre de una nueva representacion.                *
* DESCRIPCION: Lee cadenas de caracteres hasta encontrar una secuencia que*
*	no corresponda al nombre de una representacion existente (en el   *
*	ambiente o en memoria secundaria).				  *
* PARAMETROS:                                                             *
*    cad12	nom_funcion	Nombre de la funcion de representacion.   *
*    cad12	nom_tad_fte	Nombre del tad fuente de la representacion*
*    cad12	nom_tad_obj	Nombre del tad objeto de la representacion*
*    int	fila		Fila en la cual debe leer la cadena.      *
*    int	columna		Columna a partir de la cual debe leer la  *
*				cadena.      				  *
* PRE!:  { represent -> <r1,r2,...,rn> <- ult_represent, donde r1,...,rn  *
*	   son las representaciones en el ambiente, n >= 2,               *
*	   fila = F, columna = C					} *
* POST!: { nom_funcion = NF, nom_tad_fte = NTF, nom_tad_obj = NTO, tq.    *
*	   NF <> ti.nom_funcion,NTF <> ti.tad_fuente,NTO <> ti.tad_objeto *
*	   para todo 1 <= i <= n.       				} *
* LLAMA A:                                                                *
*    void	lea_cadena		void	espere_tecla              *
* MODIF: Hernan Aponte                                                    *
**************************************************************************/

short
lea_nombre_nueva_represent (cad12 nom_funcion, nodo_tad ** tad_fte,
			    nodo_tad ** tad_obj)
{
/*----------------------------------------------------------------------*/

  assert (nom_funcion != NULL);	/* VTP. 7/May/1997 */

  (*tad_fte) = pida_tad_ambiente (mensajes[101]);	/* 101:Tad fuente */
  if ((*tad_fte) == NULL)
    { return (0); }
  if (!esTipoBasico(*tad_fte))
    {
      escriba_mensaje(183);
      return 0;
    }
  (*tad_obj) = pida_tad_ambiente (mensajes[100]);	/* 100:Tad objeto */
  if (((*tad_obj) == NULL))
    { return (0); }
  if (!esTipoBasico(*tad_obj))
    {
      escriba_mensaje(183);
      return 0;
    }
  if (*tad_fte==*tad_obj || 
      strcmp((*tad_fte)->nombre,(*tad_obj)->nombre)==0)
    {
      escriba_mensaje(188);
      return 0;
    }
  return 1;
}

/**************************************************************************
* nodo_nombre *lista_rep_ambiente(void)                                  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 PROPOSITO:   Retornar una lista con los nombres de las representaciones
              en el ambiente
**************************************************************************/

nodo_nombre *
lista_rep_ambiente (char *dominio,char *codominio)
{
/*----------------------------------------------------------------------*/

  nodo_nombre *l;
  nodo_tad *p;

  l = NULL;
  for (p = tad; p != NULL; p = p->sig)
    {
      if (p->tipo==REPRESENTACION && 
	  (dominio[0]=='\0' || strcmp(p->info_tipo->nombre,dominio)==0) &&
	  (codominio[0]=='\0' || 
	   strcmp(p->info_tipo->sig->nombre,codominio)==0))
	{
	  insertar_nombre_ord (&l, p->nombre);
	}
    }
  return (l);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
void componga_represents (cad12 nom)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
Pide dos representaciones al usuario y las compone
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
AUTOR: V. Tamara                                         FECHA: 5/Jun/1998
**************************************************************************/
void componga_represents (cad12 nom)
{
  texpr mens_error,nombre;
  nodo_nombre *lis;
  nodo_tad *r1,*r2,*nt=NULL;

  if ((lis=lista_rep_ambiente ("",""))==NULL)
    {
      escriba_mensaje(143);
      return;
    }
  do
    {
      admon_menu_scroll (menus_manta[REPRESENTS_AMBIENTE], lis, nombre);
      if (strcmp (nombre, "")==0 || strcmp (nombre, mensajes[150])==0)
	{
	  return;
	}
      r1 = existe_tad_amb (nombre);
      if (r1 == NULL)
	{
	  escriba_mensaje (146);
	  admon_menu_scroll (menus_manta[REPRESENTS_AMBIENTE], lis, nombre);
	}
    } while (r1==NULL);

  if ((lis=lista_rep_ambiente (r1->info_tipo->sig->nombre,""))==NULL)
    {
      escriba_mensaje(143);
      return;
    }
  do
    {
      admon_menu_scroll (menus_manta[REPRESENTS_AMBIENTE], lis, nombre);
      if (strcmp (nombre, "")==0 || strcmp (nombre, mensajes[150])==0)
	{
	  return;
	}
      r2 = existe_tad_amb (nombre);
      if (r2 == NULL)
	{
	  escriba_mensaje (146);
	  admon_menu_scroll (menus_manta[REPRESENTS_AMBIENTE], lis, nombre);
	}
    } while (r2==NULL);

  if (componeRep(nom,r1,r2,&nt,mens_error)!=0)
    {
      escriba_cadena(mens_error,1);
      return;
    }
  if (adicTipoAmbiente(&tad,&symtable,nt,mens_error)!=0)
    {
      escriba_cadena(mens_error,1);
      return;
    }
  nt->status = pida_status ();
  tad_actual = nt;
  esc_nom_tad_actual ();
}


/**************************************************************************
 void edite_represent_nueva(cad12 nom)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Crear una nueva representacion inductivamente.             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
V. Tamara       5/Jun/1998
**************************************************************************/
short
edite_represent_nueva (cad12 nom_funcion)
{
/*----------------------------------------------------------------------*/

  nodo_tad *nt;
  FILE *fp;
  texpr mens_error;
  trio_edicion edicion[MAX_EDICION];
  int correcto;
  nodo_tad *tad_fte=NULL,*tad_obj=NULL;

  assert(nom_funcion!=NULL);

  if (!lea_nombre_nueva_represent (nom_funcion, &tad_fte, &tad_obj))
    {
      return 0;
    }
  
  fp=fopen("repre_ed.rep","wt");
  if (fp==NULL)
    {
      escriba_mensaje(196);
      return 0;
    }
  nt=NULL;
  if (nuevaRep(nom_funcion,tad_fte,tad_obj,fp,&nt,mens_error)!=0)
    {
      escriba_cadena(mens_error,1);
      return 0;
    }
  fclose(fp);
  /* Ya se escribio en el archivo, ahora permitimos editar al usuario */
  esc_arch_edicion_signatura (tad_obj);
  editar_2w ("signa_ed.tad", "");
  edicion[0].modo = 0;
  edicion[0].linea = 1;
  strcpy (edicion[0].nombre, "signa_ed.tad");
  edicion[1].modo = 1;
  edicion[1].linea = 1;
  strcpy (edicion[1].nombre, "repre_ed.rep");
  do
    {
      mens_error[0]='\0';
      llamar_al_editor (2, edicion, "temp");
      fp=fopen("repre_ed.rep","rt");
      if (fp==NULL)
	{correcto=0;}
      else
	{correcto=cambiosRep(&nt,fp,mens_error)==0;}
        fclose(fp);
    }  while (!correcto && desea_corregir_edicion(1,mens_error));

  unlink ("repre_ed.rep");
  unlink ("temp");
  unlink ("signa_ed.tad");
  if (!correcto)
    {
      return 0;
    }

  if (adicTipoAmbiente(&tad,&symtable,nt,mens_error)!=0)
    {
      escriba_cadena(mens_error,1);
      return 0;
    }
  else
    {
      nt->status = pida_status ();
      tad_actual = nt;
      esc_nom_tad_actual ();
    }
  return 1;
}



/**************************************************************************
* void nuevo_represent()					   	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Controlar el proceso de definicion de nuevas represents.   *
* DESCRIPCION: Despliega el menu de opciones de definicion de nuevas repre*
*	 sentaciones e invoca a la funcion correspondiente a la seleccion *
*	 del usuario.							  *
* PRE!:  { represent -> <r1,r2,...,rn> <- ult_represent, donde r1,...,rn  *
*	   son las representaciones en el ambiente, n >= 2,		} *
* POST!: { represent -> <r1,r2,...,rn,nr1,...,nrm> <- ult_represent,m >= 0*
*	   donde nr1,...,nrm son las nuevas representaciones definidas por*
*	   el usuario.      						  *
* LLAMA A:                              void  pite                        *
*    void  edite_represent_nueva  	void componga_represents	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 18-Jun-90 *
**************************************************************************/

void
nuevo_represent (char *nom_fun)
{
/*----------------------------------------------------------------------*/
  int opcion;

  assert(nom_fun!=NULL);
  do
    {
      opcion = admon_menu (menus_manta[NUEVA_REPRESENT]);
      switch (opcion)
	{
	case 0:		/* DEF. INDUCTIVA */
	  edite_represent_nueva (nom_fun);
	  break;
	case 1:		/* DEF. POR COMPOSICION */
	  componga_represents (nom_fun);
	  break;
	case 2:		/* SALIR */
	  break;
	case 3:		/* AYUDA */
	  escriba_mensaje (160);
	  break;
	case NO_OPCION:
	  break;
	default:
	  escriba_mensaje (146);
	  break;
	}
    }
  while (opcion > 2);
/*----------------------------------------------------------------------*/
}




