
/* RESTRICCION.C                                                */
/* Arhivo de manejo de restriccion de tipos.                    */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "definic.h"
#include "apimanta.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* j-tamara 29/May/1997 */

/**************************************************************************
* nodo_tad *tad_restriccion(nodo_tad *tad_p)                              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Retorna el tad que fue restringido originalmente para      *
*              producir tad_p.                                            *
**************************************************************************/

nodo_tad *
tad_restriccion (nodo_tad * tad_p)
{
/*----------------------------------------------------------------------*/
  nodo_tad *t;

  for (t = tad_p; t != NULL && t->tipo == RESTRICCION;
       t = existe_tad (t->info_tipo->nombre));
  return (t);
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_selectora_restr(nodo_tad *tad_padre,cad12 nom_pred_r)	  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar las selectoras de un tad de tipo RESTRICCION.      *
* DESCRIPCION: El tad_ed hereda las selectoras del <tad_padre> que deben  *
*        ser definidas solo sobre los elementos que cumplan el predicado  *
* 	 de restriccion. Esto se hace generando lados izqs. de los axiomas*
*	 usando lema de forma normal,  plantillas de lados derechos garan-*
*	 tizando que la definicion solo se hace para los argumentos del   *
*	 TDI cumplen la restriccion y solicitando al usuario que complete *
*	 estos lados derechos.						  *
*	 Si este proceso tiene exito, se solicita el status del tad y se  *
*	 adiciona a la lista de tads del ambiente.		  	  *
* PARAMETROS:                                                             *
*   nodo_tad	*tad_padre	Ap. al tad padre de la restriccion.       *
*   cad12	nom_pred_r	Nombre del predicado de restriccion.	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	RESTRICCION				  *
* PRE!:  { tad_padre-> T, nom_pred_r = P, tad_ed-> TC = ( T : P ),        *
*	   T.selectoras = <s1,...,sm>, m >= 0, y para i = 1,...,m :	  *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { tad -> <t1,...,tn,TC)> <- ult_tad = tad_ed, donde		  *
*               TC.selectoras = <P,s1,...,sm>, tq. para i = 1,...,m :	  *
*	        si.axiomas = <ai1,...,aiki>, para ki >= 0	  	  *
*		conjunto de axiomas que definen a si en argumentos que cum*
*	        pren la resticcion e indefinida en los demas,   o  	  *
*          tad -> <t1,...,tn> <- ult_tad, Si la generacion del nuevo tad  *
*	   	no fue exitosa.						} *
* !OJO!: El predicado de igualdad del tad_ed no esta siendo generado.     *
* LLAMA A:    			short es_prefijo 			  *
*   short construya_axiomas 	short construya_axs_formas_normales	  *
*   short pida_status 		void  esc_nom_tad_actual		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
genere_selectora_restr (nodo_tad * tad_padre, cad12 nom_pred_r)
{
/*----------------------------------------------------------------------*/
  nodo_select *s, *sel;
  nodo_dom *d;
  short correcto;
  nodo_tad *t;
  texpr mens_error;

  assert(tad_ed!=NULL);                 /* j-tamara 29/May/1997 */
  s = (nodo_select *) malloc (sizeof (nodo_select));
  HayMem(s,sizeof (nodo_select));       /* j-tamara 29/May/1997 */
  strcpy (s->nombre, nom_pred_r);
  strcpy (s->nombre_fte, s->nombre);
  d = (nodo_dom *) malloc (sizeof (nodo_dom));
  HayMem(d,sizeof (nodo_dom));  /* j-tamara 29/May/1997 */
  t = tad_restriccion (tad_padre);
  strcpy (d->nombre, t->nombre);
  d->induccion = 1;
  d->sig = NULL;
  s->dominio = d;
  strcpy (s->codominio, "BOOL");
  s->caracterizadora = 1;
  s->sig = tad_ed->selectoras;
  tad_ed->selectoras = s;

  /* Suprime la selectora de igualdad del tad padre */
  sel = tad_ed->selectoras;
  while ((sel != NULL) && (sel->sig != NULL))
    {
      if (es_prefijo ("EQ_", sel->sig->nombre))
	{
	  sel->sig = sel->sig->sig;
	}
      else
	{
	  sel = sel->sig;
	}
    }

  if (tad_padre->tipo == RESTRICCION)
    {
      correcto = construya_axiomas (s, 1, tad_padre->info_tipo->sig->nombre,
				    tad_padre->nombre,NULL);
      s->sig = s->sig->sig;
    }
  else
    correcto = construya_axs_formas_normales (tad_padre, s,NULL);
  
  /*  for (sel = s->sig; (sel != NULL) && correcto; sel = sel->sig)
      correcto = construya_axiomas (sel, 1, tad_ed->info_tipo->sig->nombre,
      tad_ed->nombre);
      */
  /* Ahora modificamos todos los
     axiomas de selectoras del nuevo TAD, excepto el predicado */
  for (sel = s->sig; (sel != NULL) && correcto; sel = sel->sig)
    {
      /* Los parametros de cualquier selectora del tipo restringido
	 no son inductivos */
      cambiaParametrosInduccion(sel,0);
      if (genereAxiomaRestriccion(t->nombre,sel,nom_pred_r,mens_error)!=0)
	{
	  escriba_cadena(mens_error,1);
	  elimNodoTipo(&tad_ed);
	  correcto=0;
	}
    }
  
  if (correcto)
    {
      tad_ed->status = pida_status ();
      ult_tad->sig = tad_ed;
      ult_tad = tad_ed;
      tad_actual = tad_ed;
    }
  esc_nom_tad_actual ();
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void informacion_tipo_restr(nodo_tad *p,cad12 nom_tad_origen,           *
*			      cad12 nom_pred_r)				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Asignar tipo RESTRICCION a un tad.  			  *
* DESCRIPCION: Asigna RESTRICCION al tipo de p, <nom_tad_origen> a la     *
*	 primera componente y <nom_pred_r> a la segunda componente de la  *
*	 informacion asociada al tipo.					  *
* PARAMETROS:                                                             *
*    nodo_tad	*p		Ap. al tad que es de tipo RESTRICCION.	  *
*    nodo_tad	*tad_origen	Ap. al tad padre de la RESTRICCION.       *
*    cad12	nom_pred_r	Nombre del elemento de RESTRICCION.	  *
* PRE!:  { p-> T, nom_tad_origen = NTO, nom_pred_r = NPR 		} *
* POST!: { T.tipo = RESTRICCION, T.info_tipo = <NTO , NPR>		} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
informacion_tipo_restr (nodo_tad * p, cad12 nom_tad_origen, cad12 nom_pred_r)
{
/*----------------------------------------------------------------------*/
  nodo_nombre *n;

  assert(p!=NULL);              /* j-tamara 29/May/1997 */
  p->tipo = RESTRICCION;
  n = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem(n,sizeof (nodo_nombre));       /* j-tamara 29/May/1997 */
  strcpy (n->nombre, nom_tad_origen);
  p->info_tipo = n;
  n->sig = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem(n->sig,sizeof (nodo_nombre));  /* j-tamara 29/May/1997 */
  n = n->sig;
  strcpy (n->nombre, nom_pred_r);
  n->sig = NULL;
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* void restriccion(cad12 nom_tad_r)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar un tad de tipo RESTRICCION.                        *
* DESCRIPCION:Pide el nombre del tad que se va a restringir y el predicado*
*        de restriccion, genera automatica/. la signatura del nuevo tad   *
*	 <nom_tad_r> y solicita al usuario que defina los axiomas del pre-*
*	 dicado de restriccion.                                           *
* PARAMETROS:                                                             *
*    cad12	nom_tad_r	Nombre del nuevo tad de tipo RESTRICCION. *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	RESTRICCION				  *
* PRE!:  { nom_tad_r = NT,                                                *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { tad -> <t1,...,tn,t(n+1)> <- ult_tad = tad_ed, donde		  *
*		t(n+1) = ( <p> : <nom_pred_r> ), t(n+1).nombre = NT, o    *	
*          tad -> <t1,...,tn> <- ult_tad, Si la generacion del nuevo tad  *
*	   	no fue exitosa.						  *
* LLAMA A:nodo_tad  *pida_tad_ambiente					  *
*   void  pida_predicado_restr		void  genere_copia_tad		  *
*   void  informacion_tipo_restr    	void  genere_selectora_restr      *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
restriccion (cad12 nom_tad_r)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p;
  cad12 nom_pred_r;
  texpr mens_error;
  cad12 nomtad;
/*** Alvaro
  nodo_nombre	*ult_nom;
 ***/

  p = pida_tad_ambiente (mensajes[68]);
  if (p != NULL)
    {
      /* Solo pueden restringirse tipos basicos */
      if (p->parametros!=NULL ||
	  (p->iniciales==NULL && p->constructoras==NULL))
	{
	  escriba_mensaje(183);
	  return;
	}
      pida_predicado_restr (p->nombre, nom_pred_r);
      if (strcmp (nom_pred_r, ""))
	{
	  if (existsSymbol(symtable,nom_pred_r,nomtad))
	    {
	      sprintf(mens_error,mensajes[180],nom_pred_r,nomtad);
	      escriba_cadena(mens_error,1);
	      return;
	    }
	  tad_ed = crear_tad ();
          /*VTP. 8/Mar/1988 TOCA MODIFICAR PARA QUE GENERE UN
            NUEVO TIPO POR RESTRICCION */
	  genere_copia_tad (p, "", tad_ed, "");
	  strcpy (tad_ed->nombre, nom_tad_r);
	  informacion_tipo_restr (tad_ed, p->nombre, nom_pred_r);
	  elimLGeneradora(&(tad_ed->iniciales));
	  elimLGeneradora(&(tad_ed->constructoras));
	  cambiar_nombre_sel(tad_ed,nom_tad_r);
	  genere_selectora_restr (p, nom_pred_r);
	}
    }
/*----------------------------------------------------------------------*/
}
