/* VARIACION.C                                                  */
/* Arhivo de manejo de variaciones de tipos.                    */
 
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "definic.h"
#include "apimanta.h"

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* j-tamara 29/May/1997 */

/**************************************************************************
* short genere_select_conversion(nodo_tad *t1,nodo_tad *t2,		  *
* 			         nodo_select **cab_select)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar la selectora de traduccion de un tad de tipo 	  *
*	       VARIACION.     	  				          *
* DESCRIPCION: Se construye la selectora de traduccion de <t1> a <t2> ,   *
*	 junto con sus axiomas y retona (*cab_select) apuntando a dicha   *
*	 estructura.							  *
* PARAMETROS:                                                             *
*   	nodo_tad	*t1	Ap. al primer tad padre de la VARIACION.  *
*   	nodo_tad	*t2	Ap. al segundo tad padre de la VARIACION. *
* V.GLOBALES:                                                             *
*    	nodo_tad	*tad_ed Apunt. al tad de que se esta generando por*
*			  	VARIACION				  *
* PRE!:  { t1-> T1, t2-> T2, tad_ed-> TC = T1 U T2, 	  		  *
*	   TC.selectoras = <s1,...,sm>, m >= 0                     	} *
* POST!: { genere_select_conversion = 1, Si				  *
*	        TC.selectoras = <s1,...,sm,T1-T2>			  *
*		con T1-T2 la traduccion de T1 a T2 , junto con sus axiomas*
*          genere_select_conversion = 2, Si la edicion de los axiomas de  *
*	   	T1-T2 no fue exitosa.					} *
* LLAMA A: short  construya_axs_formas_normales				  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

short
genere_select_conversion (nodo_tad * t1, nodo_tad * t2,
			  nodo_select ** cab_select, FILE *fp2)
{
/*----------------------------------------------------------------------*/
  nodo_select *s;
  nodo_dom *d;

  s = (nodo_select *) malloc (sizeof (nodo_select));
  HayMem(s,sizeof (nodo_select));               /* j-tamara 29/May/1997 */
  s->sig = NULL;
  (*cab_select) = s;
  sprintf (s->nombre, "%s_%s", t1->nombre, t2->nombre);
  strcpy (s->nombre_fte, s->nombre);
  d = (nodo_dom *) malloc (sizeof (nodo_dom));
  HayMem(d,sizeof (nodo_dom));                  /* j-tamara 29/May/1997 */
  strcpy (d->nombre, t1->nombre);
  d->induccion = 1;
  d->sig = NULL;
  s->dominio = d;
  strcpy (s->codominio, t2->nombre);
  s->caracterizadora = 0;
  if (construya_axs_formas_normales (t1, s,fp2))
    return (1);
  else
    return (0)
      ;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void genere_selectoras_variacion(nodo_tad *tad_origen1,        	  *
*				   nodo_tad *tad_origen2)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar las selectoras de un tad de tipo VARIACION.     	  *
* DESCRIPCION: El tad_ed hereda las selectoras de <tad_origen1> y de <tad_*
*	 origen2>. Esto es, el conjunto de las selectoras de tad_ed es la *
*  	 union disyunta de las selectoras de sus padres, junto con las ope*
*	 raciones de traduccion que convierten la reprentacion de un obje-*
*	 to en <tad_origen1> a la equivalente en <tad_origen2> y viceversa*
* PARAMETROS:                                                             *
*   nodo_tad	*tad_origen1	Ap. al primer tad padre de la VARIACION.  *
*   nodo_tad	*tad_origen2	Ap. al segundo tad padre de la VARIACION. *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	VARIACION				  *
* PRE!:  { tad_origen1-> T1, tad_origen2-> T2, tad_ed-> TC = T1 + T2, 	  *
*	   T1.selectoras = <s11,...,s1m1>, m1 >= 0,                    	  *
*	   T2.selectoras = <s21,...,s2m2>, m2 >= 0,			  *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { tad -> <t1,...,tn,TC)> <- ult_tad = tad_ed, donde		  *
*          	TC.selectoras = <T1-T2,T2-T1,s11,...,s1m1,s21,...,s2m2>,  *
*		con T1-T2 la traduccion de T1 a T2 y			  *
*		    T2-T1 la traduccion de T2 a T1 ,  o  		  *
*          tad -> <t1,...,tn> <- ult_tad, Si la generacion del nuevo tad  *
*	   	no fue exitosa.						} *
* LLAMA A: 								  *
*   void  copie_lista_selectoras void  genere_select_conversion		  *
*   short es_prefijo 		 short construya_axs_selec_sum_direc	  *
*   short pida_status 		 void  esc_nom_tad_actual		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

int
genere_selectoras_variacion (nodo_tad * tad_origen1, nodo_tad * tad_origen2)
{
/*----------------------------------------------------------------------*/

  int correcto;
  nodo_select *seq; 
  
  /* VTP. 6/May/1998.  Equality administration fixed */
  seq=NULL;
  if (tad_ed->iniciales!=NULL || tad_ed->constructoras!=NULL)
    {
      adic_select_eq (tad_ed, &seq);
    }
  /* Selectoras de conversion */
  correcto=genere_select_conversion (tad_origen1, tad_origen2,
				     &(tad_ed->selectoras->sig),NULL);
  if (correcto)
    {
      correcto=genere_select_conversion (tad_origen2, tad_origen1, 
					 &(tad_ed->selectoras->sig->sig),NULL);
    }
  /* Finalmente selectora de igualdad */
  if (correcto && seq!=NULL)
    {
      correcto = construya_axiomas (seq, 0, "", tad_ed->nombre,NULL);
      if (correcto) /* Reordenamos la signatura */
	{
	  tad_ed->selectoras=tad_ed->selectoras->sig;
	  tad_ed->selectoras->sig->sig=seq;
	  seq->sig=NULL;
	}
    }
  
  if (correcto)
    {
      ult_tad->sig = tad_ed;
      ult_tad = tad_ed;
      tad_actual = tad_ed;
    }
  else
    {
      liberar_tad(&tad_ed); /* VTP. 6/May/1998  Fixing memory admin. */
    }
  
  return correcto;
/*----------------------------------------------------------------------*/
}

/**************************************************************************
* void informacion_tipo_variacion(nodo_tad *p,cad12 nom_tad_origen1,	  *
*				  cad12 nom_tad_origen2)		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Asignar tipo VARIACION a un tad.  			  *
* DESCRIPCION: Asigna VARIACION al tipo de p, <nom_tad_origen1> a la      *
*	 primera componente y <nom_tad_origen2> a la segunda componente de*
*	 la informacion asociada al tipo.				  *
* PARAMETROS:                                                             *
*    nodo_tad	*p		Ap. al tad que es de tipo VARIACION.	  *
*    cad12	nom_tad_origen1 Nombre del primer tad componente de la    *
*				variacion.   				  *
*    cad12	nom_tad_origen2 Nombre del segundo tad componente de la   *
*				variacion.   				  *
* PRE!:  { p-> T, nom_tad_origen1 = NTO1, nom_tad_origen2 = NTO2.	} *
* POST!: { T.tipo = VARIACION, T.info_tipo = <NTO1 , NTO2>		} *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
informacion_tipo_variacion (nodo_tad * p, cad12 nom_tad_origen1,
			    cad12 nom_tad_origen2)
{
/*----------------------------------------------------------------------*/
  nodo_nombre *n;

  assert(p!=NULL);                      /* j-tamara 29/May/1997 */
  p->tipo = VARIACION;
  n = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem(n,sizeof (nodo_nombre));       /* j-tamara 29/May/1997 */
  strcpy (n->nombre, nom_tad_origen1);
  p->info_tipo = n;
  n->sig = (nodo_nombre *) malloc (sizeof (nodo_nombre));
  HayMem(n->sig,sizeof (nodo_nombre));          /* j-tamara 29/May/1997 */
  n = n->sig;
  strcpy (n->nombre, nom_tad_origen2);
  n->sig = NULL;
/*----------------------------------------------------------------------*/
}


int genere_iniciales_variacion(nodo_tad *tad_ed,nodo_tad *p, nodo_tad *q)
{
  /* Primera inicializadora convierte p's en tad_ed */
  tad_ed->iniciales=(nodo_generadora *)malloc(sizeof(nodo_generadora));
  HayMem(tad_ed->iniciales,sizeof(nodo_generadora));
  sprintf(tad_ed->iniciales->nombre,"%s_%s",tad_ed->nombre,p->nombre);
  tad_ed->iniciales->nombre_fte[0]='\0';
  tad_ed->iniciales->dominio=(nodo_nombre *)malloc(sizeof(nodo_nombre));
  HayMem(tad_ed->iniciales->dominio,sizeof(nodo_nombre));
  strcpy(tad_ed->iniciales->dominio->nombre,p->nombre);

  /* Segunda inicializadora convierte q's en tad_ed */
  tad_ed->iniciales->sig=(nodo_generadora *)malloc(sizeof(nodo_generadora));
  HayMem(tad_ed->iniciales->sig,sizeof(nodo_generadora));
  sprintf(tad_ed->iniciales->sig->nombre,"%s_%s",tad_ed->nombre,q->nombre);
  tad_ed->iniciales->sig->nombre_fte[0]='\0';
  tad_ed->iniciales->sig->dominio=(nodo_nombre *)malloc(sizeof(nodo_nombre));
  HayMem(tad_ed->iniciales->sig->dominio,sizeof(nodo_nombre));
  strcpy(tad_ed->iniciales->sig->dominio->nombre,q->nombre);

  /* No hay mas */
  tad_ed->iniciales->sig->sig=NULL;
  return 1;
}
/**************************************************************************
* void variacion(cad12 nom_tad_v)					  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Generar un tad de tipo VARIACION.                          *
* DESCRIPCION: Pide los nombre de los tads T1 y T2 que se va a sumar, gene*
*	 ra automatica/. la signatura del nuevo tad <nom_tad_v> y solicita*
*	 al usuario que extienda las operaciones de traduccion  de T1 a T2*
*	 y viceversa.				  			  *
* PARAMETROS:                                                             *
*    cad12	nom_tad_v	Nombre del nuevo tad de tipo VARIACION.	  *
* V.GLOBALES:                                                             *
*    nodo_tad	*tad_ed     	Apunt. al tad de que se esta generando por*
*			  	VARIACION				  *
* PRE!:  { nom_tad_v = NT,                                                *
*          tad -> <t1,t2,...,tn> <- ult_tad, donde t1,...,tn son los tads *
*          en el ambiente, n >= 2.					} *
* POST!: { tad -> <t1,...,tn,t(n+1)> <- ult_tad = tad_ed, donde		  *
*	        p-> ti, q-> tj,  1 <= i,j <= n,				  *
*		t(n+1) = <p> U <q>, t(n+1).nombre = NT, o  		  *	
*          tad -> <t1,...,tn> <- ult_tad, Si la generacion del nuevo tad  *
*	   	no fue exitosa.						  *
* LLAMA A:nodo_tad  *pida_tad_ambiente					  *
*   void  informacion_tipo_variacion  void  genere_param_sum_direc 	  *
*   void  genere_iniciales_sum_direc  void  genere_constructoras_sum_direc*
*   void  genere_selectoras_variacion void  edite_igualdad		  *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Carolina Ruiz Carrizosa                         FECHA: 15-Abr-90 *
**************************************************************************/

void
variacion (cad12 nom_tad_v)
{
/*----------------------------------------------------------------------*/

  nodo_tad *p, *q;

  /* Solo pueden hacerse variaciones de tipos basicos diferentes */
  p = pida_tad_ambiente (mensajes[52]);
  if (p != NULL)
    {
      /* The first has to be a basic type */
      if (p->parametros!=NULL ||
	  (p->iniciales==NULL && p->constructoras==NULL))
	{
	  escriba_mensaje(183);
	  return;
	}
      q = pida_tad_ambiente (mensajes[51]);
      if (q != NULL)
	{
	/* Different types to make direct sum */
	if (q==p)
	  {
	    escriba_mensaje(188);
	    return;
	  }
	/* The second has to be a basic type */
	if (q->parametros!=NULL ||
	    (q->iniciales==NULL && q->constructoras==NULL))
	  {
	    escriba_mensaje(183);
	    return;
	  }
	
	tad_ed = crear_tad ();
	strcpy (tad_ed->nombre, nom_tad_v);
	informacion_tipo_variacion (tad_ed, p->nombre, q->nombre);
	genere_param_sum_direc (p, q);
	genere_iniciales_variacion(tad_ed,p,q);
	if (genere_selectoras_variacion (p, q))
	  {
	    /* The next doesn't make sense
	       genere_iniciales_sum_direc (p, q);
	       genere_constructoras_sum_direc (p, q);*/
	    tad_ed->status = pida_status ();
	  }
	}
    }
/*----------------------------------------------------------------------*/
}
