
/**
 * ManTa 3.1 Theorem Prover
 * Version: 3.1b4 - 1999
 * Author: Vladimir Tamara
 * See http://wwwisc.uniandes.edu.co/~manta
 **/


#include <stdio.h>
#include "definic.h"
#include "apimanta.h"
#include "os_dep.h"
#include <assert.h>
#include <stdlib.h>
#include <time.h>


#define maxInFiles 100

/****************
  PROTOTYPES
 ****************/

/* Test syntax of conjectures */
int testConj(char *arcnam);

/* Proves conjectures */
int proveConj(char *,char *,char *);


/*****************
    MAIN   
 *****************/

int main(int argc, char *argv[])
{

  int rc,i,numInFiles,error;
  char proved[200],notproved[200],configFile[200];
  char inFile[maxInFiles][100];
  char workDir[1000];
  
  
  printf(" ManTa Theorem Prover  v 3.1b4\n");
  printf(" Universidad de los Andes.  \n"
         " Software Construction Group.   1999\n"
         " See http://wwwisc.uniandes.edu.co/~manta\n\n");

  strcpy(configFile,"manta.ini\0");
  proved[0]='\0';
  notproved[0]='\0';
  numInFiles=0;
  
  for (error=0,i=1;!error && i<argc;i++)
    {
      if (argv[i][0]=='-')
	{
	  switch (argv[i][1]) 
	    {
	    case 'p': 
	      if (i+1<argc)
		{ 
		  strcpy(proved,argv[i+1]); 
		  transArch(proved);  
		  i++;
		}
	      else
		{ error=1; }
	      break;
	    case 'n': 
	      if (i+1<argc)
		{ 
		  strcpy(notproved,argv[i+1]); 
		  transArch(notproved);  
		  i++;
		}
	      else
		{ error=1; }
	      break;
	    case 'd': 
	      if (i+1<argc)
		{ 
		  strcpy(workDir,argv[i+1]); 
		  transArch(workDir);
		  if (chdir(workDir)!=0) {
		    printf("Could'nt change to directory %s\n",workDir);
		    error=1;
		  }
		  i++;
		}
	      else
		{ error=1; }
	      break;
	    case 'i': 
	      if (i+1<argc)
		{ 
		  strcpy(configFile,argv[i+1]); 
		  transArch(configFile);  
		  i++;
		}
	      else
		{ error=1; }
	      break;
	    default:
	      error=1;
	      break;
	    }
	}
      else /* It must be an input file */
	{
	  if (numInFiles>maxInFiles)
	    { 
	      printf("Can't process more than %i files\n",maxInFiles);
	      rc=3;
	      goto exit;
	    }
	  strcpy(inFile[numInFiles],argv[i]);
	  transArch(inFile[numInFiles]);
	  numInFiles++;
	}
    }
  if (numInFiles==0)
    { error=1; }
  
  if (error)
    {
      printf("Syntax is: \n"
	     " manprove [-d <dir>] [-i <confFile>] [-p <proved>] [-n <not_proved>] "
	     "<inFile> [<inFile>]*\n"
	     "<dir> is the working directory\n"
             "<confFile> is the configuration file (by default manta.ini)\n"
             "<proved> is a file for succesfull proofs\n"
             "<not_proved> is a file for failed proofs\n"
             "<inFile> is a text file with conjectures to prove\n");
      return 1;
    }

  if (iniMantaAPI(configFile)<0)
    {
      printf("Initialization Failed (sure you have configuration file %s ?)\n",
	     configFile);
      return 1;
    }

  for (i=0;i<numInFiles;i++)
    {
      if (testConj(inFile[i])!=0)
	{ 
	  rc=-1; 
	  goto exit;
	}
      rc=proveConj(inFile[i],proved,notproved);
    }

 exit:
  finMantaAPI();
  return rc;
}


/***************************** FUNCTIONS DEFINITION *************************/


/**
 * Proves conjectures 
 * @param arcname Name of text file with conjectures (separated by \n)
 * @return 0 iff everything is OK
 **/
int proveConj(char *arcnam,char *proved,char *notproved)
{
  int nnop,np; /* Amount of proved and unproved conjectures */
  nodo_demost *p,*nop;
  FILE *fp;
  texpr merr;
  clock_t start, end;

  transArch(arcnam);
  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Error opening conjetures file %s",arcnam);
      return -1;
    }
  merr[0]='\0';

  start = clock();
  if (probarConjeturas(fp,&np,&p,&nnop,&nop,merr)!=0)
    {
      printf("Problems trying to prove conjetures in %s",arcnam);
      fclose(fp);
      return -1;
    }
  end = clock();
  fclose(fp);
  printf("File %s\n",arcnam);
  printf("Proved: %i\n",np);
  printf("Unproved: %i\n",nnop);
  printf("Time: %f\n", (float)(end - start)/CLOCKS_PER_SEC);
  np=0;

  if (proved[0]!='\0')
    {
      fp = fopen (proved, "a");
      if (fp==NULL)
	{ 
	  printf("Problems opening file %s",proved); 
	  np=-2;
	  goto exit;
	}
      escriba_arch (fp, p,1);
      fclose (fp);
    }
  if (notproved[0]!='\0')
    {
      fp = fopen (notproved, "a");
      if (fp==NULL)
	{ 
	  printf("Problems opening file %s",notproved); 
	  np=-2;
	  goto exit;
	}
      escriba_arch (fp, nop,0);
      fclose (fp);
    }

 exit:

  liberar_demost(&p);
  liberar_demost(&nop);
  return np;
}


/**
 * Test syntax of conjectures
 * @param arcname Name of text file with conjectures (separated by \n)
 * @return 0 iff everything is OK
 **/
int testConj(char *arcnam)
{
  FILE *fp;
  texpr merr;
  nodo_igualdad *conj;

  fp=fopen(arcnam,"rt");
  if (fp==NULL)
    {
      printf("Conjetures file \"%s\" not found\n",arcnam);
      return -1;
    }
  conj = analice_conjeturas(fp,merr);
  if (conj==NULL)
    {
      printf("Couldn't read conjeture\n%s\n",merr);
      return -1;
    }
  fclose(fp);
  elimLIgualdad(&conj);
  return 0;
}



