
/**
 * ManTa Rewrite Motor
 * Version: 3.1b4 - 1999
 * Author: Vladimir Tamara
 * See details at http://wwwisc.uniandes.edu.co/~manta
 **/

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "apimanta.h"
#include "os_dep.h"



/****************
  PROTOTYPES
 ****************/

/* Finds the normal form of expressions in a file */
int rewrite_file(char *inFile,char *outFile);


/*****************
    MAIN   
 *****************/

int main(int argc,char *argv[])
{

  int rc,i,error;
  char configFile[200];
  char inFile[200];
  char outFile[200];
  char workDir[1000];
  
  printf(" ManTa Rewrite Motor  v 3.1b4\n");
  printf(" Universidad de los Andes.  \n"
         " Software Construction Group.   1999\n"
         " See http://wwwisc.uniandes.edu.co/~manta\n\n");


  strcpy(configFile,"manta.ini\0");
  for (i=1,error=0;!error && i<argc && argv[i][0]=='-';i++)
    {
      switch (argv[i][1]) 
	{
	case 'd': 
	  if (i+1<argc)
	    { 
	      strcpy(workDir,argv[i+1]); 
	      transArch(workDir);
	      if (chdir(workDir)!=0) {
		printf("Could'nt change to directory %s\n",workDir);
		error=1;
	      }
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	case 'i': 
	  if (i+1<argc)
	    { 
	      strcpy(configFile,argv[i+1]); 
	      transArch(configFile);  
	      i++;
	    }
	  else
	    { error=1; }
	  break;
	default:
	  error=1;
	  break;
	}
    }

  if (error || i!=argc-2) 
    {
      printf("Syntax is: \n"
	     " mancomp [-d <dir>] [-i <confFile>] <inFile> <outFile>\n"
	     "<dir> is the working directory\n"
	     "<confFile> is the configuration file (by default manta.ini)\n"
             "<inFile> is a text with a ManTa specification\n"
	     "<outFile> is the name of the generated bytecode\n");
      return 1;
    }
  strcpy(inFile,argv[i]);
  strcpy(outFile,argv[i+1]);

  transArch(inFile);
  transArch(outFile);

  if (iniMantaAPI(configFile)<0)
    {
      printf("Initialization Failed (sure you have configuration file %s ?)\n",
	     configFile);
      return 1;
    }

  if ((rc=rewrite_file(inFile,outFile))!=0)
    {
      printf("Problems rewriting expressions\n");
      goto exit;
    }
  
  printf("Succesfull rewriting\n");
      
 exit:
  finMantaAPI();
  return rc;
}


/*************************
    FUNCTIONS DEFINITIONS
 *************************/

/**
 * This function finds the normal form of expressions in a file
 * @param inFile Name of file with expressions to rewrite
 * @param outFile Name of file to write normal forms
 * @return 0 iff everything is OK
 **/
int rewrite_file(char *inFile,char *outFile)
{
  FILE *fp;
  nodo_argum *arg = NULL;
  nodo_igualdad *conj = NULL;
  texpr mens_error;

  fp=fopen(inFile,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read file \"%s\"\n",inFile);
      return -1;
    }
  if ((arg = analice_lista_exp (fp, mens_error))==NULL)
    {
      printf("Wrong expression:\n%s\n",mens_error);
      fclose (fp);
      return -1;
    }
  fclose(fp);
  normalizar_lista (&arg, &conj);

  fp = fopen (outFile, "wt");
  if (fp==NULL)
    {
      printf("Couldn't write file \"%s\"\n",outFile);
      return -1;
    }
  escriba_arch_ig (fp, conj, mensajes[79]);
  fclose (fp);
  
  return 0;
}

