/***********************************************************
 MODULO CON FUNCIONES DEPENDIENTES DEL SISTEMA OPERATIVO
 PLATAFORMA: BC ++ 5.0 o 4.5x
 Por Vladimir Tamara
 Inicio: 13/Mar/1997
 Modificacion mas reciente: 13/Mar/1997
 ***********************************************************
 Los simbolos de compilacion empleados son:
  DJGPP - DJGPP es GNU C++ para DOS 32 bits
  __CYGWIN32__ -- CYGNUS es GNU C++ para Windows 95
  __BORLANDC__ - Borland C++ 5.0 para Windows 95/
                 Windows 3.1      y DOS
***********************************************************/


#include "os_dep.h"
#include <assert.h>
#include <string.h>
/*** VTP. 15/Ene/97.  Se incluyen funciones para hacer:
    - copia (copy, cp)
    - touch
    - directorio (dir, ls)
***/

/* A este codigo fuente se le han aplicado estas tecnicas preventivas: */
/* 1. assert al comienzo de toda funcion para verificar precondiciones */
/* 2. HayMem despues de cada malloc para verificar que se asigne memoria
   y para inicializar cada zona de memoria reservada en 0 */
/* 3. Los apuntadores liberados han sido dejados apuntando a NULL */
/* 4. Todo apuntador local a funciones no inicializado ha sido inicializado
   en NULL */
/* 5. Despues de cada fopen se agrego la funcion FileOpen */
/* j-tamara 29/May/1997 */


#include <time.h>
#include <dos.h>
#include <dir.h>
#include <io.h>
/* En BC 5.0 la funcion setftime esta en io.h */

/**************************************************************************
* int copia_archivo(char *fuente,char *destino)                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Copiar el archivo fuente en el destino                     *
* DESCRIPCION:                                                            *
*   [1] Localiza un bloque lo ms grande posible                       *
*   [2] Abre archivo fuente para lectura y destino para escritura (b)     *
*   [3] Copia bloques del archivo fuente en el destino                    *
*   [4] Cierra archivos abiertos y libera memoria                         *
* R.R.  Puede emplear compilacion condicional para hacerla mas eficiente  *
*       en diversos SO                                                    *
* PRE!:  fuente y destino son cadenas                                     *
* POST!: el archivo fuente es copiado en destino o se retorna un codigo   *
*        de error (numero negativo)                                       *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Vladimir Tamara Patino (VTP)                    FECHA: 13-Ene-97 *
**************************************************************************/
int
copia_archivo (char *fuente, char *destino)
{
/*----------------------------------------------------------------------*/

  char *buf;			/* Buffer de copia */
  long tambuf = 0x80000;	/* Aspiramos a casi 500KB para el buffer */
  long ocubuf;			/* Lo ocupado del buffer */
  FILE *in, *out;

  assert (fuente != NULL && destino != NULL);

  /* [1] Localizamos memoria */
  for (buf = malloc (sizeof (char) * tambuf);
       buf == NULL && tambuf > 0;
       tambuf -= 0x1000)
    {
      buf = malloc (sizeof (char) * tambuf);
    }

  if (tambuf <= 0)
    {
      return -1;
    }
  /* [2] Abrimos archivos */
  assert (buf != NULL);
  if ((in = fopen (fuente, "rb")) == NULL)
    {
      return -1;
    }
  if ((out = fopen (destino, "wb")) == NULL)
    {
      return -1;
    }

  /* [3] Copiamos bloques hasta que no haya ms por copiar */
  assert (in != NULL && out != NULL);

  do
    {
      ocubuf = fread (buf, sizeof (char), tambuf, in);
      if (ocubuf > 0)
	{
	  fwrite (buf, sizeof (char), ocubuf, out);
	}
    }
  while (ocubuf > 0);

  /* [4] Cerramos archivos y liberamos memoria */

  fclose (in);
  fclose (out);

  free (buf);

  return 0;

/*----------------------------------------------------------------------*/ 
}


/**************************************************************************
* nodo_nombre *lista_dir(cad20 dir, cad20 ext)                          *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   Retorna una lista con los nombres de los archivos del      *
*              directorio dir, cuya extension sea ext.                    *
* MODIFICACIONES:
* Hernan Aponte
* VTP 18/Dic/96
*         Esta funcion empleaba system para pedir el directorio a UNIX,
*       tal metodo no funciona en DOS.   Tuvo que emplearse compilacion
*       condicional y las instrucciones findfirst y findnext ofrecidas
*       con tal fin por djgpp
*         La compilacion condicional emplea el simbolo DJGPP que solo
*         funcionara con tal compilador
* VTP. 3/Jun/1997 Se agrego el parametro conext, que indica si
*      deben retornarse o no las extensiones
**************************************************************************/

nodo_nombre *
lista_dir (cad20 dir, cad20 ext, int conext)
{
/*----------------------------------------------------------------------*/

  texpr comando;
  texpr nombre;
  nodo_nombre *l;

  struct ffblk entdir;
/*** VTP. Agregado para obtener directorios en DOS */
  int done;

  l = NULL;

  sprintf (comando, "%s/*.%s", dir, ext);
  transArch(comando);
  done = findfirst (comando, &entdir, FA_RDONLY | FA_HIDDEN | FA_SYSTEM | FA_ARCH);
  while (!done)
    {
      if (strcmp(entdir.ff_name+strlen(entdir.ff_name)-strlen(ext),ext)==0)
	{
	  formatear_nombre (entdir.ff_name, nombre,conext);
	  insertar_nombre_ord (&l, nombre);
	}
      done = findnext (&entdir);
    }

  return (l);
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* void formatear_nombre(texpr lin1, texpr lin2,int conext)                           *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : lin1 es un nombre de archivo con camino y extension.              *
* POS : contiene unicamente el nombre del archivo que aparece en lin1,    *
*       es decir sin camino ni comentarios.                               *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Cesar Munoz                                     FECHA: 28-Nov-91 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* MODIFICACIONES:
* VTP. 3/Jun/1997.  Agregada a este modulo, con la opcion de dejar o no la
*                   extension del archivo
**************************************************************************/

void formatear_nombre(texpr lin1, texpr lin2,int conext)
{/*----------------------------------------------------------------------*/
	 int prim,ult; /* Primer y ultimo caracter imprimible de lin */
	 int j,i;
	 assert(lin1!=NULL && lin2!=NULL && strlen(lin1)<MAX_TEXPR
				&& strlen(lin2)<MAX_TEXPR); /* VTP. 23/Abr/1997 */

	 for (prim = strlen(lin1); prim > 0 && lin1[prim] != '/'
                                   && lin1[prim]!= '\\';
              prim--);
	 if (lin1[prim] == '/' || lin1[prim] == '\\')
		  prim++;
         if (conext)
         {
             ult=strlen(lin1)-1;
         }
         else
         {
             for (ult = prim; lin1[ult] != '.' && lin1[ult] != '\0'; ult++);
             if (lin1[ult] == '.')
                      ult--;
         }

	 for (j=prim,i=0; j<=ult; lin2[i]=lin1[j],i++,j++);
	 lin2[i]='\0';
/*----------------------------------------------------------------------*/}


/**************************************************************************
* void nombre_autor(texpr nombre)                                         *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PRE : True                                                              *
* POS : nombre == Nombre del usuario del sistema                          *
* MODIFICACIONES:             
*   VTP. 13/Mar/1997
*   En DOS no hay nombres de usuario, se retornara vacio
**************************************************************************/

void
nombre_autor (texpr nombre)
{
/*----------------------------------------------------------------------*/

  nombre[0]=' ';
  nombre[1] = '\0';
  return;
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* int touch(char *nomarc)                                                 *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   A los archivos especificados en nomarc (pueden usarse      *
*              wildcards *, ?), les cambia la fecha y hora a la actual    *
* DESCRIPCION:                                                            *
*   Si es UNIX emplea el touch del sistema                                *
*   Si es DOS emplea el siguiente algoritmo                               *
*    [1]  Se obtiene hora y fecha actual                                  *
*    [2] Se obtiene la via de los archivos                                *
*    [3] A cada archivo se le cambia fecha y hora por la actual           *
* PRE!:  nomarc es cadena                                                 *
* POST!: Cambio fecha de archivos que "unificara" con nomarc              *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Vladimir Tamara Patino (VTP)                    FECHA: 15-Ene-97 *
**************************************************************************/
int 
touch (char *nomarc)
{
/*----------------------------------------------------------------------*/

  texpr comando;

  time_t tnow;			/* Hora actual */
  struct tm tmnow;		/* Hora actual */
  struct ftime ft;		/* Fecha actual */
  FILE *f;			/* Archivo por tocar */
  struct ffblk entdir;		/* Entrada de directorio */
  texpr nombre;			/* Cadena para almacenar nombres de archivos */
  int done;			/* Indica si se termino de procesar directorio */
  int ci;			/* Contador */
  char drive[MAXDRIVE], path[MAXDIR], file[MAXFILE], ext[MAXEXT];
  int rspl;			/* Resultado retornado por funcion fnsplit */
  texpr nomini;			/* Via por agregar a los nombres de archivos */

  /* [1]  Se obtiene hora y fecha actual */

  tnow = time (NULL);
  tmnow = *localtime (&tnow);

  ft.ft_year = tmnow.tm_year - 80;
  ft.ft_month = tmnow.tm_mon + 1;
  ft.ft_day = tmnow.tm_mday;
  ft.ft_hour = tmnow.tm_hour;
  ft.ft_min = tmnow.tm_min;
  ft.ft_tsec = tmnow.tm_sec / 2;


  /* El nombre de directorio debe cambiarse de UNIX a DOS */
  sprintf (comando, "%s", nomarc);
  for (ci = 0; ci < strlen (comando); ci++)
    {
      if (comando[ci] == '/')
	{
	  comando[ci] = '\\';
	}
    }

  /* [2] Se obtiene la via de los archivos */
  drive[0] = path[0] = file[0] = ext[0] = nomini[0] = '\0';
  rspl = fnsplit (comando, drive, path, file, ext);
  if (rspl | DRIVE)
    {
      sprintf (nomini, "%s:", drive);
    }
  if (rspl | DIRECTORY)
    {
      sprintf (nomini, "%s%s", nomini, path);
    }

  /* [3] A cada archivo se le cambia fecha y hora por la actual */
  done = findfirst (comando, &entdir, FA_RDONLY | FA_HIDDEN | FA_SYSTEM | FA_ARCH);
  if (done)
    {				/* Touch aplicado a un archivo que no existe lo crea */
      f = fopen (comando, "w");
      /* No se verifica porque el nombre podria ser con Wildcards */
      if (f != NULL)
	{
	  fclose (f);
	}
    }

  while (!done)
    {
      sprintf (nombre, "%s\\%s", nomini, entdir.ff_name);
      if ((f = fopen (nombre, "r+b")) != NULL)
	{
	  setftime (fileno (f), &ft);
	  fclose (f);
	}
      done = findnext (&entdir);
    }

  return 0;
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* void transArch(char *nomarc)                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   El nombre de archivo nomarc es transformado del formato de *
*              UNIX al del sistema operativo que se este usando           *
* PRE!:  nomarc es cadena                                                 *
* POST!: Convierte nomarc al formato del s.o empleado                     *
*        Por ahora si el simbolo DJGPP existe cambia / por \\             *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Vladimir Tamara Patino (VTP)                    FECHA: 15-Ene-97 *
**************************************************************************/
void
transArch (char *nomarc)
{
/*----------------------------------------------------------------------*/
  char *c;
  for (c = nomarc; *c != '\0'; c++)
    {
      *c = (*c == '/') ? (char) '\\' : *c;
    }
/*----------------------------------------------------------------------*/
}


/**************************************************************************
* void transArch2(char *nomarc)                                            *
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* PROPOSITO:   En Cygnus pone nomarc entre comillas "
*              Usado para editar bien archivos con edit
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
* AUTOR: Vladimir Tamara Patino (VTP)                    FECHA: 24-Jun-97 *
**************************************************************************/
void
transArch2 (char *nomarc)
{
/*----------------------------------------------------------------------*/
  assert(nomarc!=NULL);
/*----------------------------------------------------------------------*/
}


/**************************************************************************
  int strcasecmp (char *s1,char *s2)
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
  La funcion de comparacion de dos cadenas sin tener en cuenta
  capitalizacion es strcmpi, aunque el estndar es strcasecmp
*~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~*
 AUTOR: Vladimir Tamara Patino (VTP)
**************************************************************************/
int strcasecmp (char *s1,char *s2)
{
  while (*s1!='\0' && *s2!='\0' && toupper(*s1)==toupper(*s2))
    {
      s1++;
      s2++;
    }
  return toupper(*s1)-toupper(*s2);
}

