
/**
 * Pretty Printer of ManTa 3.1 files
 * Version: 1.0 - 1998
 * Author: Vladimir Tamara and ManTa System
 **/

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include "est2abs.h"
#include "apimanta.h"
#include "os_dep.h"

#include "AbsType.h"
#include "String.h"


/****************
  PROTOTYPES
 ****************/

/* Prints a String */
void printString(String cadt);

/* Loads a type written in a text file (not in environment)  */
nodo_tad *loadTextType(char *name);


/*****************
    MAIN   
 *****************/

int main(int argc,char *argv[])
{

  nodo_tad *et;
  char merr[2000]; /* Error messages */
  char arcnam[500];
  AbsType ast=NULL;
  String cadt=NULL;  
  int rc=0;

  printf("/**\n");
  printf(" * Pretty Printer of ManTa (3.1) Files v 1.0b1\n");
  printf(" * Universidad de los Andes.  \n"
         " * Software Construction Group.   1998\n"
	 " **/\n\n");

  if (argc<2)
    {
      printf("Sintax is: \n"
             " prp <file>\n"
             "<file> is the ManTa file to print\n");
      return 1;
    }
  if (iniMantaAPI("manta.ini")<0)
    {
      printf("Initialization Failed (sure you hava manta.ini\?)\n");
      return 1;
    }

  sprintf(arcnam,argv[1]);
  et=loadTextType(arcnam);
  if (et==NULL || adicElemLTipo(&tad,et,ADIC_REP,merr)!=0)
    {
      printf("Error adding type %s\n",arcnam);
      rc=2;
      goto exit;
    }
  if (nodo_tad2AbsType(&ast,et)!=0)
    {
      printf("Error translating file. Probably ther isn't enough memory\n");
      rc=3;
      goto exit;
    }
  if ((cadt=prettyAbsType(ast))==NULL)
    {
      printf("Not enough Memory to print ADT\n");
      rc=3;
      goto exit;
      };
  printString(cadt);

 exit:
  if (ast!=NULL)
    { borrar_obj_AbsType(&ast); }
  if (cadt!=NULL)
    { borrar_obj_String(&cadt); }
  finMantaAPI();
  return rc;
}


/* Prints the contents of a file in stdout */
int imprimeArchivo(char *aname)
{
  FILE *fp;
  char lin[1000];

  printf("Printing file: %s\n",aname);
  fp=fopen(aname,"rt");
  if (fp==NULL)
    {
      printf("Couldn't read from %s\n",aname);
      return -1;
    }
  fgets(lin,1000,fp);
  while (!feof(fp))
    {
      printf("%s",lin);
      fgets(lin,1000,fp);
    }
  fclose(fp);

  return 0;
}


void printString(String cadt)
{
  if (!isEmptyList_String(cadt))
    {
      printf("%c",pr0charOfNAT(head_String(cadt)));
      printString(tail_String(cadt));
    }
}

/* Loads a type written in a text file (not in environment)  */
nodo_tad *loadTextType(char *name)
{

  FILE *f;
  texpr msg;
  nodo_tad *tad=NULL;
  assert(name!=NULL);

  transArch(name);
  f=fopen(name,"rb");
  if (f==NULL)
    {
      printf("File not found. \n");
      return NULL;
    }
  msg[0]='\0';
  if (cargaTipoTexto(f,&tad,msg))
    {
      printf("Error loading file. Message:\n");
      printf("%s\n",msg);
    }
  fclose(f);
  return tad;
}

