{(*===========================================================================
                              Mplexer.mll
==============================================================================
Preprocessor for ManTa
============================================================================*)

exception StmtLexicalError 
exception EndOfFile

let string_literal_str=ref "";;

let rec one_nat =function
   0 -> "ZERO"
 | n -> "SUC("^(one_nat (n-1))^")"
;;

let one_char s = 
  "charOfNAT("^
  (one_nat (Char.code (String.get s 1)))^
  ")"
;;

let one_esp_char s =
  "charOfNAT("^
  (match (String.get s 2) with
    'n' -> one_nat 10
  | 'r' -> one_nat 13
  | 't' -> one_nat 9
  | 'b' -> one_nat 8
  | '"' -> one_nat  (Char.code '"')
  | '\\' -> one_nat (Char.code '\\')
  | '\'' -> one_nat (Char.code '\'')
  | '0'..'9' -> one_nat (int_of_string (String.sub s 2 ((String.length s)-3)))
  | _   -> raise StmtLexicalError)^
  ")"
;; 


let other_thing s =
  s
;;

let rec string_literal s = 
  if s="" then 
    "nil_String"
  else
    "cons_String("^
    (one_char ("'"^(String.sub s 0 1)^"'"))^
    ","^
    (string_literal (String.sub s 1 ((String.length s)-1)))^
    ")";
;;

}

rule get_token = parse 

	  "/*"          	{ "/*"^(comment lexbuf) }

	| ['a'-'z' 'A'-'Z' '_'] ['a'-'z' 'A'-'Z' '_' '0'-'9']*
	                        { other_thing (Lexing.lexeme lexbuf) }

	| "'" '\\' _ "'"   	{ one_esp_char (Lexing.lexeme lexbuf) }

	| "'" '\\' ['0'-'9'] ['0'-'9']? ['0'-'9']? "'" 
				{ one_esp_char (Lexing.lexeme lexbuf) }

	| "'" _ "'"      	{ one_char (Lexing.lexeme lexbuf) }

        | "\""                  { string_literal_str:="";string lexbuf }

        | ['0'-'9']*   		{ one_nat (int_of_string (Lexing.lexeme lexbuf)) }

	| eof               	{ raise EndOfFile }

	| _			{ other_thing (Lexing.lexeme lexbuf) }

and comment = parse 
	  "*/"          	{ "*/" }
	| eof               	{ raise StmtLexicalError }
	| _			{ let i=(Lexing.lexeme lexbuf) in i^(comment lexbuf)}

and string = parse 
	  "\\\""		{ string_literal_str:=((!string_literal_str)^"\"");string lexbuf }
	| "\\n"			{ string_literal_str:=((!string_literal_str)^"\n");string lexbuf }
	| "\\r"			{ string_literal_str:=((!string_literal_str)^"\r");string lexbuf }
	| "\\t"			{ string_literal_str:=((!string_literal_str)^"\t");string lexbuf }
	| "\\b"			{ string_literal_str:=((!string_literal_str)^"\b");string lexbuf }
	| "\\'"			{ string_literal_str:=((!string_literal_str)^"'");string lexbuf }
	| "\\\\"		{ string_literal_str:=((!string_literal_str)^"\\");string lexbuf }
	| '\\' ['0'-'9'] ['0'-'9']? ['0'-'9']? 
	  { let t=(Lexing.lexeme lexbuf) in let ss=(String.make 1 (Char.chr (int_of_string (String.sub t 1 ((String.length t)-1) ))))
            in string_literal_str:=((!string_literal_str)^ss);string lexbuf }
	| "\""          	{ string_literal (!string_literal_str) }
	| eof               	{ raise StmtLexicalError }
	| _			{ string_literal_str:=((!string_literal_str)^(Lexing.lexeme lexbuf));string lexbuf }



