
(*
  ManTa 3.3
  Preprocessor
  Written by Vladimir Tmara
  1998
*)

open Mplexer
open Lexing
;;


exception LexicalError of string * string * int * string


let fromLexbuf readername lexerfun sourcename lexbuf = 
let rec repeat_lexer lf lb res=
try
  let res2=res^(lf lb) in
  (repeat_lexer lf lb res2)
with
    EndOfFile -> res
  | e -> raise e

in
  
   try
     repeat_lexer lexerfun lexbuf ""
   with
     _ ->
       	  (* dans le cas de cette erreur les valeurs lex_start pos
             sont negatives (car le debut du token n'est plus dans
             le buffer). Voir le code des  libraries caml "lexing.ml" *)
          let pos =  lexbuf.lex_start_pos+lexbuf.lex_abs_pos in
          raise (LexicalError(readername,sourcename,pos,"???")) 
;;


let fromInput readername lexerfun =
   fromLexbuf readername lexerfun "stdin" (Lexing.from_channel stdin)
;;

(****************************************
 MAIN
*****************************************)

let main ()=
	print_string "ManTa preprocessor.  Version 1.0b2\n";
	print_string "Written by Vladimir Tamara Patino. 1998. Santafe de Bogota (COLOMBIA)\n";
 	print_string "  This software includes the Objective Caml run-time system\n  which is copyright 1996, 1997 INRIA.\n\n";
	flush stdout;
	if (Array.length Sys.argv)<>3 then 
        begin
	  print_string "Syntax:\n   mpp <in_file> <out_file>\n";
	  print_string "<in_file> is a text file with a ManTa specification\n";
	  print_string "<out_file> is the name of the generated text\n";
	  print_string "  e.g.  mpp NAT.txt NAT.mpp\n\n";
	end
	else 
 	  let 
              infile=(open_in Sys.argv.(1)) and
              outfile=(open_out Sys.argv.(2)) 
          in  let 
              tout=(fromLexbuf "ManTa preprocessor" get_token Sys.argv.(1) (Lexing.from_channel infile))
          in
	      output_string outfile "/* Modified by ManTa 3.3 preprocessor.  Version 1.0 */\n\n";
	      output_string outfile tout;
	      flush outfile;
;;

main ();;
